﻿<#
    Skript, um alle Datenbankobjekte als .sql Datei zu exportieren
    Erfordert Powershell Version 5 ("altes Powershell", welches auf allen PCs vorhanden ist - nicht kompatibel mit 6+ für Visual Studio Code)

    Das Script muss von einem Benutzer ausgeführt werden, welcher mindestens db_owner Rechte in der Datenbank besitzt.

    Das Script erstellt im Zielverzeichnis einen Ordner mit dem Datenbanknamen und einer Unterstruktur
        - einen Ordner pro Objekttyp der Datenbank
        - einen Ordner pro Schema
        - eine Datei pro Datenbankobjekt

    Zielstellung:
        - kundenspezifische Anpassungen der OCT Datenbank werden im Bitbucket pro einzelnem Datenbankobjekt sichtbar
        - Anpassungen können bei Bedarf pro Objekt neu eingespielt werden

    Anwendung:
        - Das Script wird erstmalig ausgeführt nach
            - dem initialen Setup der Datenbank
            - der Aktivierung aller Module vom Logikserver 
            -> Sicherung des Initialzustandes vor kundenspezifischen Anpassungen
        - dieser Ordner muss transportiert und ins Bitbucket eingecheckt werden
        - nach Abschluss der Anpassungsarbeiten wird das Script erneut ausgeführt (und überschreibt dabei den initialen Ordner)
        - diese Version des Ordners wird wieder transportiert und eingecheckt
    
    Falls das Skript aufgrund von strikten PowerShell Ausführungsrechten nicht gestartet werden kann, muss folgender Befehl als Administrator ausgeführt werden:
    Set-ExecutionPolicy -ExecutionPolicy RemoteSigned -Scope CurrentUser
    ggf. in den Dateieigenschaften die Eigenschaft "die Datei stammt von einem anderen Computer..." aktivieren
#>


<# KONFIGURATION #>

# SQL Server Name (ggf. mit Instanz)
$ServerName = 'W10'

# Datenbankname
$Database = 'OCT'

# Zielverzeichnis, in dem die .sql Dateien abgelegt werden
$DirectoryToSaveTo = $PSScriptRoot + '\ExportDBStructure\' + $Database

# Vorhandenes Zielverzeichnis löschen
$Cleanup = $true

# Zeilen auskommentieren, wenn SQL Server Login verwendet werden soll
#$srv.ConnectionContext.LoginSecure = $false
#$srv.ConnectionContext.Login = "user"
#$srv.ConnectionContext.Password = "pw"


<# SKRIPT - AB HIER NICHTS MEHR ANPASSEN #>


if (($Cleanup -eq $true) -and (Test-Path($DirectoryToSaveTo))) {
    Remove-Item $DirectoryToSaveTo -Recurse -Force
}

# SMO Assemblys laden
$v = [System.Reflection.Assembly]::LoadWithPartialName( 'Microsoft.SqlServer.SMO')
if ((($v.FullName.Split(','))[1].Split('='))[1].Split('.')[0] -ne '9') {
    [System.Reflection.Assembly]::LoadWithPartialName('Microsoft.SqlServer.SMOExtended') | out-null
}
[System.Reflection.Assembly]::LoadWithPartialName('Microsoft.SqlServer.SmoEnum') | out-null
set-psdebug -strict
$ErrorActionPreference = "stop"
$My = 'Microsoft.SqlServer.Management.Smo'


# Serververbindung herstellen
$srv = new-object ("$My.Server") $ServerName
if ($srv.ServerType-eq $null) {
   Write-Error "Sorry, but I couldn't find Server '$ServerName' "
   return
}

# Verbindungsoptionen
$scripter = new-object ("$My.Scripter") $srv
$scripter.Options.ToFileOnly = $true
$scripter.Options.ExtendedProperties = $true # erweiterte Eigenschaften
$scripter.Options.DRIAll = $true # Constraints
$scripter.Options.Indexes = $true # Indizes
$scripter.Options.Triggers = $true # Trigger
$scripter.Options.Encoding = [System.Text.UTF8Encoding] $False # Kodierung (UTF8NoBom)

$all = [long]  [Microsoft.SqlServer.Management.Smo.DatabaseObjectTypes]::View `
    + [Microsoft.SqlServer.Management.Smo.DatabaseObjectTypes]::Table `
    + [Microsoft.SqlServer.Management.Smo.DatabaseObjectTypes]::StoredProcedure `
    + [Microsoft.SqlServer.Management.Smo.DatabaseObjectTypes]::UserDefinedFunction`
    + [Microsoft.SqlServer.Management.Smo.DatabaseObjectTypes]::Schema


$d = new-object System.Data.Datatable
# Objekte auslesen und diverse Systemobjekte rausfiltern
$d = $srv.databases[$Database].EnumObjects([long]0x1FFFFFFF -band $all) | Where-Object {$_.Schema -ne 'sys'-and $_.Schema -ne "information_schema" -and $_.DatabaseObjectTypes -ne 'ServiceBroker' -and $_.name}
# jedes Objekt als Datei rausschreiben
$d | FOREACH-OBJECT {
    # Dateiname festlegen
    if ($_.Schema -ne "") {
        $Folder = "$($DirectoryToSaveTo)\$($_.DatabaseObjectTypes)\$($_.Schema)"
        $Filename = $_.Schema + '.' + ($_.name -replace '[\\\/\:\.]','-')
    } else {
        $Folder = "$($DirectoryToSaveTo)\$($_.DatabaseObjectTypes)"
        $Filename = $_.name -replace '[\\\/\:\.]','-'
    }

    # Ordner erstellen
    if (!(Test-Path -path $Folder)) {
        Try {
            New-Item $Folder -type directory | out-null
        } 
        Catch [system.exception] {
            Write-Error "error while creating '$Folder' $_"
            return
         } 
    }

    $scripter.Options.Filename = "$Folder\$Filename.sql";

    # Datei rausschreiben
    $UrnCollection = new-object ('Microsoft.SqlServer.Management.Smo.urnCollection')
    $URNCollection.add($_.urn)
    $scripter.enumscript($URNCollection)
}

# alle Dateien bearbeiten und DROP hinzufügen
$files = Get-ChildItem $DirectoryToSaveTo -Recurse -Filter *.sql
foreach ($file in $files) {
    $ObjectType = ""
    if ($file.DirectoryName -like '*\Table\*') {
        $ObjectType = "TABLE"
    }
    
    if ($file.DirectoryName -like '*\StoredProcedure\*') {
        $ObjectType = "PROCEDURE"
    }
    
    if ($file.DirectoryName -like '*\UserDefinedFunction\*') {
        $ObjectType = "FUNCTION"
    }
    
    if ($file.DirectoryName -like '*\View\*') {
        $ObjectType = "VIEW"
    }
    $DropSQL = 'DROP ' + $ObjectType + ' IF EXISTS ' + ($file.Name -replace ".sql","") + ';'    

    if ($ObjectType -ne "") {
        $FileContent = (Get-Content $file.FullName | Select-Object -Skip 3)
        $FileContent[0] = $DropSQL
        Set-Content -Path $file.FullName -Value $FileContent

     }
}
