
DROP FUNCTION IF EXISTS dbo.sx_pf_Determine_TransactionUsername;
GO
/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Funktion returns the Name of an existing active User in OCT Usermanamgent or the string 403.

	Determine the name of user, for whom a transaction is done.
	The resulting name may be
		- an existing and active user of the OCT Usermanagement (including the special one with name "public")
		- an login who exists not in Usermanagment, but in the database as user with the role FactorySerivce or db_owner (priviledged user)
		- the string 'public' if no business user were found an a user public is declared
		- the string '403' if no authorized user was found

	Rules:
		- User with the Role FactoryService or db_owner are allowed to act in own name or in foreign name
		- to signalize that they act in own name, they pass in the username 'SQL'
		- User with other roles can act only as themself (the user which is connected)
		- even a user with role sysadmin can't pass in a foreign username

	Keep in Mind:
		- if a user with role db_owner is also also owner of the database he will be not appear with his login, instead with username 'dbo'
		- a user may be db_owner and has the role FactoryService
		- a user may be sysadmin, than he is not an db_owner
		- a user may be internal OCTUser and also db_owner
		- in OCT Usermanagement 
			- never a user with Name '403' must be created - this user would always been blocked
			- a User with the Name of the FactoryService must not be created (but he may be there, due to later db role assignment)
			- a User with the role db_owner may exist in the OCT Usermangement

	TODO for later editions where XLS Client must no longer be supported
	-- Return User with an role to determine if FactoryService is acting for itself or not

	Don't use the function IS_ROLEMEMBER on SQL Server, its buggy

	Testcall 
		SELECT dbo.sx_pf_Determine_TransactionUsername ('SQL')
		SELECT dbo.sx_pf_Determine_TransactionUsername ('W10\admin')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_Determine_TransactionUsername',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'FUNCTION', 'sx_pf_Determine_TransactionUsername','PARAMETER',NULL)

*/

CREATE FUNCTION dbo.sx_pf_Determine_TransactionUsername 
	(
		@Username AS NVARCHAR(255)=''
	)

RETURNS NVARCHAR (255) 
AS
	BEGIN 
			DECLARE  
				 @TransactionUsername	NVARCHAR (255) = '403'
				,@IsPrivilegedUser		INT = 0
				,@UserBlocked			INT = 0
				,@UserIsActiveOCTUser	INT = 0;
		
		-- trim Username
			SET @Username = LTRIM(RTRIM(@Username));


		-- 1. Determine if a privileged User is acting - the dbo is never member of any role, thats why its tested separat
			IF IS_MEMBER('pf_PlanningFactoryService') = 1 OR IS_MEMBER('db_owner')= 1 OR USER_NAME()='dbo'
				BEGIN
					SET @IsPrivilegedUser = 1;
				END;
			
		-- 2. If privileged user is acting, every Username is accepted, she string 'SQL' signalise they are acting as themself 
			IF  @IsPrivilegedUser = 1
				IF @Username = 'SQL'
					BEGIN
						SET @TransactionUsername = ORIGINAL_LOGIN()
					END
				ELSE
					BEGIN
						SET @TransactionUsername = @Username;
					END
			ELSE
				-- else only the Original_Login() is accepted - this happens to enable Working with the Excel Client or direct in SSMS (if user is GRANTEd to Execute SPs)
				BEGIN
					SET @TransactionUsername = ORIGINAL_LOGIN()
				END

		-- 3. Check if the user is an active OCT User
			IF (SELECT Count(Userkey) FROM system.trUser WHERE Username = @TransactionUsername AND Status = 'Active') = 1 
				BEGIN 
					SET @UserIsActiveOCTUser = 1;
				END

			-- privileged users are always exepted when they act for thereself  (even if they exists in usermanagement as inactive or don't exists there)		
			IF @IsPrivilegedUser = 1 AND @Username = 'SQL'
				BEGIN
					SET @TransactionUsername = @TransactionUsername -- do nothing, accept the user
				END

			-- When privileged user is acting for a User, only active users are accepted
			IF @IsPrivilegedUser = 1 AND @Username <> 'SQL' AND @UserIsActiveOCTUser = 0
				BEGIN
					SET @TransactionUsername = '403'
					SET	@UserBlocked = 1
				END

			-- When a non privileged User is connected directly, he is blocked if he is not an active user 
			IF @IsPrivilegedUser = 0 AND @UserIsActiveOCTUser = 0
				BEGIN
					SET @TransactionUsername = '403'
					SET	@UserBlocked = 1
				END
				
		-- 4. If the user was blocked, check if a public user exists, which can be used for the transaction
			IF @UserBlocked = 1
				BEGIN
					IF EXISTS
						(
						SELECT Userkey FROM system.trUser WHERE Username = 'public' AND Status = 'Active'
						)
						SET @TransactionUsername = 'public'
				END

		RETURN @TransactionUsername
	END
GO

ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_Determine_TransactionUsername TO pf_PlanningFactoryService;

GRANT EXECUTE ON dbo.sx_pf_Determine_TransactionUsername TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'dbo'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'sx_pf_Determine_TransactionUsername'			-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
 
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;


-- SET documententation             *************************************************************************
 
-- SET function documentation
    SET @value = N'Function and determine the business username for the transaction - 
		only for active users and the priviledged database roles FactoryService and db_owner an username is returnd.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'The name of the user, in which name the transaction shall happen. Pass in "SQL" if the Service shall act for himself.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 


GO