
DROP FUNCTION IF EXISTS system.fGetColumns;
GO
/*
	Table Function to GET for the requested table or view
		- all ColumnNames
		- per column the property if the column is calculated
		- per column if the column is part of the key
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Function
		SELECT * FROM system.fGetColumns ('planning','tAPI_Log')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'FUNCTION', 'fGetColumns',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'FUNCTION', 'fGetColumns','PARAMETER',NULL)
*/

CREATE FUNCTION system.fGetColumns (
	@Schema NVARCHAR(50),
	@Table NVARCHAR(50)
)
RETURNS TABLE 
AS
RETURN 
(
	-- for tables
	SELECT
		c.Name				AS ColName,
		c.is_computed		AS IsCalculatedKey,
		CASE 
			WHEN EXISTS(
						SELECT 1
						FROM sys.computed_columns cc
						WHERE 
								cc.object_id = t.object_id
							AND cc.definition COLLATE DATABASE_DEFAULT LIKE N'%![' + c.Name + N'!]%' ESCAPE N'!'
					) THEN 1 
			ELSE 0 
		END					AS IsPartOfKey
	FROM sys.columns c
		JOIN sys.tables t 
			ON t.object_id = c.object_id
		JOIN sys.schemas s 
			ON s.schema_id = t.schema_id
		LEFT JOIN sys.computed_columns cc 
			ON		cc.column_id = c.column_id 
				AND cc.object_id = t.object_id
	WHERE 
			s.name COLLATE DATABASE_DEFAULT = @Schema
		AND t.name COLLATE DATABASE_DEFAULT = @Table
	
	UNION ALL

	-- for Views
	SELECT
		c.name			AS ColName,
		c.is_computed	AS IsCalculatedKey,
		CASE
			WHEN EXISTS(
						SELECT 1
						FROM sys.computed_columns cc
						WHERE cc.object_id = v.object_id
						AND cc.definition COLLATE DATABASE_DEFAULT LIKE N'%![' + c.Name + N'!]%' ESCAPE N'!'
						) THEN 1 
			ELSE 0 
		END				AS IsPartOfKey
	FROM sys.columns c
	JOIN sys.views v 
		ON v.object_id = c.object_id	
	JOIN sys.schemas s 
		ON s.schema_id = v.schema_id
	LEFT JOIN sys.computed_columns cc
		ON 
				cc.column_id = c.column_id 
			AND cc.object_id = v.object_id
	WHERE 
			v.name COLLATE DATABASE_DEFAULT = @Table
		AND s.name COLLATE DATABASE_DEFAULT = @Schema
)
GO

-- Explicit GRANT for Factory Service
GRANT SELECT ON system.fGetColumns TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'fGetColumns'	-- enter function name
    ,@SX_Owner			NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the function from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = Demo object - out of shipment process
															-- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
 
-- KEEP this default constants     *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'FUNCTION'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;


-- SET documententation             *************************************************************************
 
-- SET function documentation
	SET @value = N'Function to GET for a requested table or view all ColumnNames and the properties if the column is calculated and if this Column is part of the key.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Schema';
    SET @value      = N'Parameter schema.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Table';
    SET @value      = N'Parameter table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO

