DROP PROCEDURE IF EXISTS agent.pCreateJSONView;
GO

/*
	Procedure to create a helper table for the agent to resolve a JSON Column to single columns
	- the JSON Column must be named "CustomValuesJSON"
	- the resulting table will be named agent.vJSON_[schema]_[table]
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = agent.pCreateJSONView
				 @Username	= 'SQL'
				,@Schema	= 'global'
				,@Table		= 'tCompanies'
				,@UseRowKey = 1
		PRINT @RC

		SELECT * FROM agent.vJSON_global_tCompanies

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pCreateJSONView',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pCreateJSONView','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pCreateJSONView
    (
        @Username     NVARCHAR(255)      
      , @Schema AS    NVARCHAR(50)
      , @Table AS     NVARCHAR(50)
      , @UseRowKey    INT          = 0
      , @ProcessID    NVARCHAR(50) = '0' 
      , @SubProcessID NVARCHAR(50) = '0' 
    )
AS
BEGIN
    DECLARE @KeyCol NVARCHAR(50);
    DECLARE @ColName NVARCHAR(50);
    DECLARE @ColType INT;
    DECLARE @SQLDrop NVARCHAR(MAX);
    DECLARE @SQLHeader NVARCHAR(MAX);
    DECLARE @SQL NVARCHAR(MAX);
    DECLARE @SQLFooter NVARCHAR(MAX);
    DECLARE @ResultCode INT = 501;
    DECLARE @CurrentTimestamp DATETIME;
    DECLARE @ErrorMessage NVARCHAR(2000);

    -- Temp Tabelle mit JSON Spalten erstellen
    DECLARE @JSONColsTbl NVARCHAR(MAX) = N'[##' + CAST(NEWID() AS NVARCHAR(MAX)) + N']';

    BEGIN TRY
        BEGIN TRANSACTION pCreateJSONView;
        EXEC ('CREATE TABLE ' + @JSONColsTbl + ' (ColName NVARCHAR(50), ColType int)');
        EXEC ('INSERT INTO ' + @JSONColsTbl + ' SELECT DISTINCT b.[Key], b.Type FROM ' + @Schema + '.' + @Table + ' a CROSS APPLY OPENJSON(a.CustomValuesJSON) b');
        EXEC ('SELECT 1 FROM ' + @JSONColsTbl);

        -- Schlüsselspalte auslesen
        SELECT
            @KeyCol = ColName
        FROM system.fGetColumns(@Schema, @Table)
        WHERE
            (
            @UseRowKey = 0
            AND IsCalculatedKey = 1
            )
            OR
                (
                @UseRowKey = 1
                AND ColName = 'RowKey'
                );

        -- Skript definieren
        SET @SQLDrop = N'DROP VIEW IF EXISTS agent.vJSON_' + @Schema + N'_' + @Table;
        SET @SQLHeader = N'CREATE VIEW agent.vJSON_' + @Schema + N'_' + @Table + N' AS
			SELECT
			a.['          + @KeyCol + N']';
        SET @SQLFooter = N'
		FROM ' + @Schema + N'.' + @Table + N' a
		';
        SET @SQL = N'';

        EXEC ('SELECT 1 FROM ' + @JSONColsTbl);
        IF @@ROWCOUNT > 0
        BEGIN
            SET @SQLFooter = @SQLFooter + N' OUTER APPLY OPENJSON (a.CustomValuesJSON)
			WITH (';

            -- JSON Spalten durchgehen und Skript dynamisch ergänzen
            EXEC ('DECLARE MyCursor CURSOR FOR' + ' SELECT ColName, ColType' + ' FROM ' + @JSONColsTbl + ' WHERE ColType IN (1,2,3) ORDER BY 1');
            OPEN MyCursor;
            FETCH MyCursor
            INTO
                @ColName
              , @ColType;

            WHILE @@FETCH_STATUS = 0
            BEGIN
                SET @SQL = @SQL + N',ISNULL([' + @ColName + N'],'''') ' + CASE @ColType
                                                                            WHEN 1 THEN 'COLLATE DATABASE_DEFAULT'
                                                                            ELSE ''
                                                                        END + N' AS [' + @ColName + N'_INT]';

                SET @SQLFooter = @SQLFooter + N'
								[' + @ColName + N'] ' + CASE @ColType
                                                          WHEN 1 THEN 'nvarchar(255)'
                                                          WHEN 2 THEN 'float'
                                                          WHEN 3 THEN 'bit'
                                                      END + N',';

                FETCH MyCursor
                INTO
                    @ColName
                  , @ColType;
            END;

            CLOSE MyCursor;
            DEALLOCATE MyCursor;

            SET @SQL = @SQLHeader + @SQL + LEFT(@SQLFooter, LEN(@SQLFooter) - 1) + N')';
        END;
        ELSE
        BEGIN
            -- keine Custom Spalten
            SET @SQL = @SQLHeader + @SQL + @SQLFooter;
        END;

        --PRINT @SQL;

        EXEC (@SQLDrop);
        EXEC (@SQL);

        COMMIT TRANSACTION pCreateJSONView;
        SET @ResultCode = 200;
    END TRY
    BEGIN CATCH
        DECLARE @Error_state INT = ERROR_STATE();
        SET @ErrorMessage = ERROR_MESSAGE();

        ROLLBACK TRANSACTION pCreateJSONView;

        IF @Error_state <> 10
        BEGIN
            SET @ResultCode = 500;
            PRINT 'Rollback due to not executable command.';
        END;
        ELSE IF @ResultCode IS NULL
                OR @ResultCode / 100 = 2
        BEGIN
            SET @ResultCode = 500;
        END;

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry
            @Username = @Username
          , @ModuleCode = 'Integration'
          , @ProcessName = 'Agent Execution'
          , @ProcessID = @ProcessID
          , @SubProcessName = 'agent.pSync'
          , @SubProcessID = @SubProcessID
          , @SubSubProcessName = 'agent.pCreateJSONView'
          , @EventCODE = 'Error'
          , @Timestamp = @CurrentTimestamp
          , @ReturnCODE = @ResultCode
          , @EventMessage = @ErrorMessage;
    END CATCH;

    EXEC ('DROP TABLE IF EXISTS ' + @JSONColsTbl);

    RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'agent'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'pCreateJSONView'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create a helper table for the agent to resolve a JSON Column to single columns
					- the JSON Column must be named "CustomValuesJSON"
					- the resulting table will be named agent.vJSON_[schema]_[table]';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'the users name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Schema';
    SET @value      = N'the schemas name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Table';
    SET @value      = N'name of the table';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@UseRowKey';
    SET @value      = N'use row key';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


	