
DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValues;
GO

/*
	Procedure to delete Integration Values 
	this is only an SWITCH procedure - it switches to an special procedure if defined or uses a default procedure
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9

	Testcall
		EXEC agent.pDeleteIntegrationValues
			 @Username			= 'SQL'
			,@Solution			= 'FIN'
			,@CompanyKeys		= '1|44'
			,@PeriodFrom		= 202001
			,@PeriodTo			= 202012
			,@MasterDataOnly	= 0
			,@ProcessID			= '0'
			,@SubProcessID		= '0'
			,@IsDeltaloadBool	= 0

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValues','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pDeleteIntegrationValues
   @Username        NVARCHAR(255)      
  ,@Solution        NVARCHAR(50)
  ,@CompanyKeys     NVARCHAR(MAX)      -- delimited by ; e.g. "1|42;1|43"
  ,@PeriodFrom      INT
  ,@PeriodTo        INT
  ,@MasterDataOnly  BIT          = 0
  ,@ProcessID       NVARCHAR(50) = '0' 
  ,@SubProcessID    NVARCHAR(50) = '0' 
  ,@IsDeltaloadBool BIT          = 0

WITH EXECUTE AS 'dbo'

AS
	BEGIN
	-- Standard declaration for logging
		DECLARE 
				@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username									,N'NULL')	+ N''','''
				+ ISNULL(@Solution									,N'NULL')	+ N''','''											 
				+ ISNULL(@CompanyKeys								,N'NULL')	+ N''','
				+ ISNULL(CAST(@PeriodFrom		AS NVARCHAR(255))	,N'NULL')	+ N','
				+ ISNULL(CAST(@PeriodTo			AS NVARCHAR(255))	,N'NULL')	+ N','
				+ ISNULL(CAST(@MasterDataOnly	AS NVARCHAR(255))	,N'NULL')	+ N','''
				+ ISNULL(@ProcessID									,N'NULL')	+ N''','''
				+ ISNULL(@SubProcessID								,N'NULL')	+ N''','''
				+ ISNULL(CAST(@IsDeltaloadBool AS NVARCHAR(255))	,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';
			
		DECLARE		
				 @SQLCommand		NVARCHAR(MAX)	= N''
				,@CompanyKey		NVARCHAR(100)	= N''
				,@DataSourceKey		INT				= NULL
				,@CompanyID			NVARCHAR(50)	= N''
				,@ProcName			NVARCHAR(50)	= N'';

		BEGIN TRY

			-- Check if a specific delete procedure for the given module exists
			-- If there is no proc for module, module is set to generic
			IF (OBJECT_ID('agent.pDeleteIntegrationValues' + @Solution) IS NULL)
				BEGIN
					SET @ProcName = 'agent.pDeleteIntegrationValuesGeneric';
				END
			ELSE 
				BEGIN
					SET @ProcName = 'agent.pDeleteIntegrationValues' + @Solution;
				END

			PRINT CONCAT('Procedure: ',@ProcName,' selected for execution.')

			-- Declare cursor to iterate over CompanyKeys
			DECLARE CompanyKeyCursor CURSOR FOR

				SELECT value AS CompanyKey
				FROM STRING_SPLIT(@CompanyKeys, ';');

			OPEN CompanyKeyCursor;
			FETCH CompanyKeyCursor INTO @CompanyKey;
			WHILE @@FETCH_STATUS = 0
			BEGIN
				PRINT CONCAT('Starting delete for CompanyKey: ',@CompanyKey);

				SELECT TOP 1 @DataSourceKey = Value FROM STRING_SPLIT(@CompanyKey, '|'); -- the first due to TOP
				SELECT		 @CompanyID		= Value FROM STRING_SPLIT(@CompanyKey, '|'); -- the last due to all are assigend and last survives
		
				PRINT CONCAT('Determined Datasource: ',@DataSourceKey,' from CompanyKey.');
				PRINT CONCAT('Determined CompanyID: ',@CompanyID,' from CompanyKey.');

				-- dynamically execute delete procedure for the selected Solution, Company and Period
				SET @SQLCommand
					= N'EXEC ' 
						+ @ProcName 
						+ N'  @MasterDataOnly	= '		+ CAST(@MasterDataOnly		AS NCHAR(1)) 
						+ N', @DataSourceKey	= '		+ CAST(@DataSourceKey		AS NCHAR(10))
						+ N', @CompanyID		= N'''  + @CompanyID				+ N'''' 
						+ N', @PeriodFrom		= '		+ CAST(@PeriodFrom			AS NCHAR(6))
						+ N', @PeriodTo			= '		+ CAST(@PeriodTo			AS NCHAR(6)) 
						+ N', @Username			= N'''	+ @Username					+ N''''
						+ N', @ProcessID		= N''' 	+ @ProcessID				+ N'''' 
						+ N', @SubProcessID		= N'''	+ @SubProcessID 			+ N''''
						+ N', @IsDeltaloadBool	= ' 	+ CAST(@IsDeltaloadBool		AS NCHAR(1)) 
						+ N', @Solution			= N'''  + @Solution					+ N'''' 
						+ N';';

				PRINT CONCAT('Execute Command: "',@SQLCommand,'"');
				EXEC (@SQLCommand);

				FETCH CompanyKeyCursor
				INTO @CompanyKey;
			END;
			CLOSE CompanyKeyCursor;
			DEALLOCATE CompanyKeyCursor;

			SET @ResultCode = 200;

    END TRY
    BEGIN CATCH
        DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE 
				IF @ResultCode IS NULL OR @ResultCode/100 = 2
					BEGIN
						SET @ResultCode = 500;	
					END;
    END CATCH;

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
    RETURN @ResultCode;
END;

GO

-- Procedure executes dynamic SQL, but uses EXECUTE AS 'dbo' - therefore no special GRANT 
-- NO GRANT

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'						 -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pDeleteIntegrationValues'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'					 -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'						 -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1								 -- 0 = Demo object - out of shipment process
																		 -- STANDARD OBJECTS
																		 -- 1 = shiped from saxess standard without modification
																		 -- 2 = shiped from saxess standard modified FOR customer from saxess
																		 -- 3 = shiped from saxess standard modified FOR customer from partner
																		 -- 4 = shiped from saxess standard modified FROM customer themself for own needs
																		 -- CUSTOM OBJECTS
																		 -- 10 = shiped from saxess as customer specific object
																		 -- 11 = shiped from partner	as customer specific object
																		 -- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';							 -- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete integration values - switches between executing a special delete procedure or deleting generic over a default procedure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
	SET @level2name = N'@Username';
    SET @value      = N'User name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@Solution';
    SET @value      = N'Name of the solution, e.g. FIN.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys as semikolon separated list, e.g. 1|22;1|33';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodFrom';
    SET @value      = N'PeriodFrom, e.g. 202201';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodTo';
    SET @value      = N'PeriodTo, e.g. 202212';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@MasterDataOnly';
    SET @value      = N'MasterDataOnly';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'String to name the process - only for logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'String to name the subprocess - only for logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsDeltaloadBool';
    SET @value      = N'Flag if Delta Load is used.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
