
DROP PROCEDURE IF EXISTS agent.pDeleteIntegrationValuesGeneric;
GO

/*
	Procedure Delete Integration Values Generic
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 	Testcall
		EXEC agent.pDeleteIntegrationValuesGeneric
			 @DataSourceKey		= Fin
			,@CompanyID			= '1'
			,@PeriodFrom		= 202001
			,@PeriodTo			= 202012
			,@MasterDataOnly	= 0
			,@Username			= 'SQL'
			,@ProcessID			= '0'
			,@SubProcessID		= '0'
			,@IsDeltaloadBool	= 0

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValuesGeneric',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pDeleteIntegrationValuesGeneric','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pDeleteIntegrationValuesGeneric
    @DataSourceKey   int
  , @CompanyID       nvarchar(50) = NULL
  , @PeriodFrom      int          = NULL
  , @PeriodTo        int          = NULL
  , @MasterDataOnly  bit
  , @Username        nvarchar(255)       -- used for logging
  , @ProcessID       nvarchar(50) = N'0' -- used for logging
  , @SubProcessID    nvarchar(50) = N'0' -- used for logging
  , @IsDeltaloadBool bit          = 0
  , @Solution        nvarchar(50)
AS
BEGIN
    DECLARE @ResultCode int = 501;
    DECLARE @CurrentTimestamp datetime;
    DECLARE @EventMessage nvarchar(2000);
    DECLARE @Table nvarchar(100);
    DECLARE @SQLDelete nvarchar(MAX) = N'';
    DECLARE @SQLWhere nvarchar(MAX) = N'';
    DECLARE @SQLExec nvarchar(MAX) = N'';

    BEGIN TRY
        DECLARE TableCursor CURSOR FOR
        SELECT TABLE_NAME
        FROM INFORMATION_SCHEMA.TABLES
        WHERE
            TABLE_TYPE = 'BASE TABLE'
            AND TABLE_SCHEMA = 'integration'
            AND TABLE_NAME LIKE 't' + @Solution + '%';

        OPEN TableCursor;
        FETCH TableCursor
        INTO @Table;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            -- clear variables
            SET @SQLDelete = N'';
            SET @SQLWhere = N' WHERE 1=1';
            SET @SQLExec = N'';


            SET @SQLDelete = N'DELETE FROM integration.' + @Table;
            IF COL_LENGTH('integration.' + @Table, 'DataSourceKey') IS NOT NULL
                SET @SQLWhere = @SQLWhere + N' AND DataSourceKey = ' + CONVERT(nvarchar, @DataSourceKey);


            IF COL_LENGTH('integration.' + @Table, 'CompanyID') IS NOT NULL
               AND @CompanyID IS NOT NULL
                SET @SQLWhere = @SQLWhere + N' AND CompanyID = N''' + @CompanyID + N'''';

            IF COL_LENGTH('integration.' + @Table, 'FinancialYear') IS NOT NULL
               AND COL_LENGTH('integration.' + @Table, 'FinancialPeriod') IS NOT NULL
               AND @PeriodFrom IS NOT NULL
               AND @PeriodTo IS NOT NULL
                SET @SQLWhere
                    = @SQLWhere + N' AND FinancialYear * 100 + FinancialPeriod BETWEEN '
                      + CONVERT(nvarchar, @PeriodFrom) + N' AND ' + CONVERT(nvarchar, @PeriodTo);


            SET @SQLExec = @SQLDelete + @SQLWhere;
            EXEC (@SQLExec);

            FETCH TableCursor
            INTO @Table;
        END;
        CLOSE TableCursor;
        DEALLOCATE TableCursor;

        SET @ResultCode = 200;

    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @EventMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCODE = 'Integration'
                                        , @ProcessName = 'Data Deletion'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = 'agent.pDeleteIntegrationValues'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = 'agent.pDeleteIntegrationValuesGeneric'
                                        , @EventCode = 'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @EventMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pDeleteIntegrationValuesGeneric' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Delete Integration Values Generic';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@DataSourceKey';
    SET @value      = N'DataSourceKey.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'CompanyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodFrom';
    SET @value      = N'PeriodFrom';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PeriodTo';
    SET @value      = N'PeriodTo';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@MasterDataOnly';
    SET @value      = N'MasterDataOnly';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Username';
    SET @value      = N'the users name';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsDeltaloadBool';
    SET @value      = N'IsDeltaloadBool';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	GO
