
DROP PROCEDURE IF EXISTS agent.pMaintenanceIndexes;
GO

/*
	Procedure for Maintenance Indexes
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceIndexes',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceIndexes','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pMaintenanceIndexes
    @Username     nvarchar(255)
  , @Module       nvarchar(50)       -- List delimited by ;
  , @ProcessID    nvarchar(50) = '0' -- used for logging
  , @SubProcessID nvarchar(50) = '0' -- used for logging
AS
BEGIN
    BEGIN TRY

        DECLARE @ResultCode int = 501;
        DECLARE @MaxRowKey int = 0;
        DECLARE @StatusMessage nvarchar(2000);
        DECLARE @CurrentTimestamp datetime;

        DECLARE @Schema nvarchar(255);
        DECLARE @Table nvarchar(255);
        DECLARE @Index nvarchar(255);

        DECLARE @SQL nvarchar(MAX);


        -- Create cursor to iterate over module integration tables
        DECLARE IndexCursor CURSOR FOR
            SELECT s.name   AS SCHEMA_NAME
                 , t.name   AS TABLE_NAME
                 , ind.name AS INDEX_NAME
            FROM sys.indexes     AS ind
                JOIN sys.tables  AS t
                  ON ind.object_id = t.object_id
                JOIN sys.schemas AS s
                  ON t.schema_id = s.schema_id
            WHERE
                ind.is_primary_key = 0
                AND t.name LIKE 't' + @Module + '%'
                AND ind.type IN (5, 6);

        -- Iterate over IndexCursor
        OPEN IndexCursor;
        FETCH IndexCursor
        INTO @Schema
           , @Table
           , @Index;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            SET @SQL = N'ALTER INDEX [' + @Index + N'] ON ' + @Schema + N'.[' + @Table + N'] REORGANIZE  WITH ( LOB_COMPACTION = ON, COMPRESS_ALL_ROW_GROUPS = OFF )';
            EXEC (@SQL);

            FETCH IndexCursor
            INTO @Schema
               , @Table
               , @Index;
        END;
        CLOSE IndexCursor;
        DEALLOCATE IndexCursor;

        SET @ResultCode = 200;
        RETURN @ResultCode;
    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @StatusMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = 'agent'
                                        , @ProcessName = 'Maintenance'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = 'agent.pMaintenanceIndexes'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = ''
                                        , @EventCODE = 'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pMaintenanceIndexes' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for Maintenance Indexes';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'Module';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

