
DROP PROCEDURE IF EXISTS agent.pMaintenanceJSONViews;
GO

/*
	Procedure for Recreation of all JSON Views of selected module in integration and global schema
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = agent.pMaintenanceJSONViews
			 @Username		= 'SQL'
			,@Module		= 'FIN'
			,@ProcessID		= '1'
			,@SubProcessID	= '2'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceJSONViews',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceJSONViews','PARAMETER',NULL)
*/
 
CREATE PROCEDURE agent.pMaintenanceJSONViews
    @Username     NVARCHAR(255)
   ,@Module       NVARCHAR(50)			
   ,@ProcessID    NVARCHAR(50) = N'0'	-- used for logging
   ,@SubProcessID NVARCHAR(50) = N'0'	-- used for logging
AS
BEGIN
    BEGIN TRY

        DECLARE  @ResultCode		INT				= 501
				,@GlobalResultcode  INT				= 0
				,@Tablecounter		INT				= 0
				,@StatusMessage		NVARCHAR(2000)	= N''
				,@CurrentTimestamp	DATETIME
				,@Table				NVARCHAR(255)	= N'';

        -- =============================================
        -- Recreate JSON views for tables in integration schema which contain a column named CustomValuesJSON for module @Module
        -- =============================================
        -- Create cursor to iterate over module integration tables
        DECLARE IntegrationTableCursor CURSOR FOR
            SELECT 
				t.TABLE_NAME COLLATE DATABASE_DEFAULT
            FROM INFORMATION_SCHEMA.TABLES      AS t
                JOIN INFORMATION_SCHEMA.COLUMNS AS c
                  ON CONCAT(
							 t.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
							,t.TABLE_SCHEMA		COLLATE DATABASE_DEFAULT
							,t.TABLE_NAME		COLLATE DATABASE_DEFAULT
							)
					= CONCAT(
							  c.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
							 ,c.TABLE_SCHEMA	COLLATE DATABASE_DEFAULT
							 ,c.TABLE_NAME		COLLATE DATABASE_DEFAULT
							 )
            WHERE
					t.TABLE_TYPE	COLLATE DATABASE_DEFAULT	= N'BASE TABLE'
                AND t.TABLE_SCHEMA	COLLATE DATABASE_DEFAULT	= N'integration'
				AND t.TABLE_NAME	COLLATE DATABASE_DEFAULT	LIKE N't' + @Module + N'%'
                AND c.COLUMN_NAME	COLLATE DATABASE_DEFAULT	= N'CustomValuesJSON';

        -- Iterate over IntegrationTableCursor
        OPEN IntegrationTableCursor;
        FETCH IntegrationTableCursor
        INTO @Table;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            EXEC @ResultCode = agent.pCreateJSONView	 @Username	= @Username
														,@Schema	= N'integration'
														,@Table		= @Table
														,@UseRowKey = 1;

			IF @ResultCode > @GlobalResultcode SET @GlobalResultcode = @ResultCode;
			SET @Tablecounter = +1;

            FETCH IntegrationTableCursor
            INTO @Table;
        END;
        CLOSE IntegrationTableCursor;
        DEALLOCATE IntegrationTableCursor;

        -- =============================================
        -- Recreate JSON views for tables in global schema for module @Module
        -- =============================================
        -- Create cursor to iterate over module integration tables
        DECLARE IntegrationTableCursor CURSOR FOR
            SELECT 
				t.TABLE_NAME COLLATE DATABASE_DEFAULT
            FROM INFORMATION_SCHEMA.TABLES      AS t
                JOIN INFORMATION_SCHEMA.COLUMNS AS c
                  ON CONCAT(
								 t.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
								,t.TABLE_SCHEMA		COLLATE DATABASE_DEFAULT
								,t.TABLE_NAME		COLLATE DATABASE_DEFAULT
							) 
				  = CONCAT(
								 c.TABLE_CATALOG	COLLATE DATABASE_DEFAULT
								,c.TABLE_SCHEMA		COLLATE DATABASE_DEFAULT
								,c.TABLE_NAME		COLLATE DATABASE_DEFAULT
							)
            WHERE
					t.TABLE_TYPE	COLLATE DATABASE_DEFAULT	= N'BASE TABLE'
                AND t.TABLE_SCHEMA	COLLATE DATABASE_DEFAULT	= N'global'
                AND t.TABLE_NAME	COLLATE DATABASE_DEFAULT	LIKE N't' + @Module + N'%'
                AND c.COLUMN_NAME	COLLATE DATABASE_DEFAULT	= N'CustomValuesJSON';

        -- Iterate over IntegrationTableCursor
        OPEN IntegrationTableCursor;
        FETCH IntegrationTableCursor
        INTO @Table;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            EXEC @ResultCode = agent.pCreateJSONView	 @Username	= @Username
														,@Schema	= N'global'
														,@Table		= @Table
														,@UseRowKey = 1;


			IF @ResultCode > @GlobalResultcode SET @GlobalResultcode = @ResultCode;
			SET @Tablecounter = +1;

            FETCH IntegrationTableCursor
            INTO @Table;
        END;
        CLOSE IntegrationTableCursor;
        DEALLOCATE IntegrationTableCursor;

        -- =============================================
        -- Recreate JSON view for global.tCompanies
        -- =============================================
        EXEC @ResultCode = agent.pCreateJSONView	 @Username	= @Username
													,@Schema	= N'global'
													,@Table		= N'tCompanies'
													,@UseRowKey = 1;

		IF @ResultCode > @GlobalResultcode SET @GlobalResultcode = @ResultCode;
		SET @Tablecounter = +1;

		IF @Tablecounter = 0 SET @GlobalResultcode = 200;
		RETURN @GlobalResultcode;

    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @StatusMessage = ERROR_MESSAGE();

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = N'agent'
                                        , @ProcessName = N'Maintenance'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = N'agent.pMaintenanceJSONViews'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = N''
                                        , @EventCODE = N'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;


END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pMaintenanceJSONViews'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to recreate the views for Tables which contain a CustomValuesJSON columns - for the passend Module and in the integration and global schema.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'Module';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO

