
DROP PROCEDURE IF EXISTS agent.pMaintenanceRecreateViews;
GO

/*
	Procedure for Maintenance to recreate all Views of a given Module. The Procedure just recreates the View with the existing
	definition and changes nothing. 
	It is done, because the View defintion contains an vjson.* column and the * columns are only new refreshed, when the view is recreated.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = agent.pMaintenanceRecreateViews
				 @Username	= 'SQL'
				,@Module	= 'FIN'	
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceRecreateViews',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pMaintenanceRecreateViews','PARAMETER',NULL)
*/
 
CREATE PROCEDURE agent.pMaintenanceRecreateViews
   @Username     NVARCHAR(255)
  ,@Module       NVARCHAR(50)		
  ,@ProcessID    NVARCHAR(50) = N'0' 
  ,@SubProcessID NVARCHAR(50) = N'0' 

AS
BEGIN

    DECLARE 
		 @ResultCode		INT = 501
		,@StatusMessage		NVARCHAR(2000)
		,@CurrentTimestamp	DATETIME		
		,@View				NVARCHAR(255)
		,@ViewDefinition	NVARCHAR(MAX);

    BEGIN TRY
        BEGIN TRANSACTION pMaintenanceRecreateViews;
        -- Create cursor to iterate over module result views
        DECLARE ViewCursor CURSOR FOR
            SELECT
                N'result.' + v.TABLE_NAME AS VIEW_NAME
            FROM INFORMATION_SCHEMA.VIEWS AS v
            WHERE
                v.TABLE_SCHEMA = N'result'
                AND v.TABLE_NAME LIKE N'v' + @Module + N'%';

        -- Iterate over ViewCursor
        OPEN ViewCursor;
        FETCH ViewCursor
        INTO
            @View;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            EXEC sp_refreshview @viewname = @View

            FETCH ViewCursor INTO @View;
        END;
        CLOSE ViewCursor;
        DEALLOCATE ViewCursor;
			   
        COMMIT TRANSACTION pMaintenanceRecreateViews;
		SET @ResultCode = 200;
        RETURN @ResultCode;
    END TRY
    BEGIN CATCH
        DECLARE @Error_state INT = ERROR_STATE();
        SET @StatusMessage = ERROR_MESSAGE();

        ROLLBACK TRANSACTION pMaintenanceRecreateViews;

        IF @Error_state <> 10
        BEGIN
            SET @ResultCode = 500;
			PRINT CONCAT(N'Error when creating view ',@View, ': ', @StatusMessage);
        END;
        ELSE IF @ResultCode IS NULL
                OR @ResultCode / 100 = 2
        BEGIN
            SET @ResultCode = 500;
        END;

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = N'agent'
                                        , @ProcessName = N'Maintenance'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = N'agent.pMaintenanceRecreateViews'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = ''
                                        , @EventCODE = N'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;

    RETURN @ResultCode;
END;

GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pMaintenanceRecreateViews'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for Maintenance to recreate all Views of a given Module. The Procedure just recreates the View with the existing
					definition and changes nothing. 
					It is done, because the View defintion contains an vjson.* column and the * columns are only new refreshed, when the view is recreated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Module';
    SET @value      = N'CODE of the Module, eg. FIN';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID - not needed, keep empty.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID - not needed, keep empty.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
