
DROP PROCEDURE IF EXISTS agent.pPostBulkload;
GO

/*
	Procedure for POST Bulkload
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pPostBulkload',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pPostBulkload','PARAMETER',NULL)
*/
 
CREATE PROCEDURE agent.pPostBulkload
    @Username     nvarchar(255)
  , @Modules      nvarchar(MAX)      -- List delimited by ;
  , @CompanyKeys  nvarchar(MAX)      -- List delimited by ;
  , @ProcessID    nvarchar(50) = '0' -- used for logging
  , @SubProcessID nvarchar(50) = '0' -- used for logging
AS
BEGIN
    BEGIN TRY
        DECLARE @ResultCode int = 404;
        DECLARE @MaxRowKey int = 0;
        DECLARE @StatusMessage nvarchar(2000);
        DECLARE @CurrentTimestamp datetime;
        DECLARE @Module nvarchar(50);
        DECLARE @ProcName nvarchar(50);


        -- Write log entry for begin of procedure execution
        IF @ProcessID = '0'
        BEGIN
            SELECT @MaxRowKey = MAX(RowKey)
            FROM system.tProcessLogs;
            SET @ProcessID = @MaxRowKey + 1;
        END;

        SET @CurrentTimestamp = GETUTCDATE();

        -- Create cursor to iterate over @Modules
        DECLARE ModuleCursor CURSOR FOR
            SELECT value AS Module
            FROM STRING_SPLIT(@Modules, ';')
            WHERE value <> '';

        -- Iterate over ModuleCursor
        OPEN ModuleCursor;
        FETCH ModuleCursor
        INTO @Module;
        WHILE @@FETCH_STATUS = 0
        BEGIN
            -- Check if specific PostBulkload procedure for given module exists.
            -- If procedure exists it will be executed.
            IF (OBJECT_ID('agent.pPostBulkload' + @Module) IS NOT NULL)
            BEGIN
                SET @ProcName = N'agent.pPostBulkload' + @Module;
                EXEC @ResultCode = @ProcName @Username = @Username
                                           , @Module = @Module
                                           , @CompanyKeys = @CompanyKeys
                                           , @ProcessID = @ProcessID
                                           , @SubProcessID = @SubProcessID;
            END;

            -- Execute maintenance procedure to create JSON views based on CustomValuesJSON column.
            EXEC @ResultCode = agent.pMaintenanceJSONViews @Username = @Username
                                                         , @Module = @Module
                                                         , @ProcessID = @ProcessID
                                                         , @SubProcessID = @SubProcessID;

            -- Recreate result views to include new or deleted CustomValuesJSON columns.
            EXEC @ResultCode = agent.pMaintenanceRecreateViews @Username = @Username
                                                         , @Module = @Module
                                                         , @ProcessID = @ProcessID
                                                         , @SubProcessID = @SubProcessID;

            -- Reorganize all columnstore indexes of given module.
            EXEC @ResultCode = agent.pMaintenanceIndexes @Username = @Username
                                                       , @Module = @Module
                                                       , @ProcessID = @ProcessID
                                                       , @SubProcessID = @SubProcessID;
            
            FETCH ModuleCursor
            INTO @Module;
        END;
        CLOSE ModuleCursor;
        DEALLOCATE ModuleCursor;

        RETURN @ResultCode;
    END TRY
    BEGIN CATCH
        SET @ResultCode = 500;
        SET @StatusMessage = ERROR_MESSAGE();
		PRINT @StatusMessage

        SET @CurrentTimestamp = GETUTCDATE();
        EXEC system.pPOST_ProcessLogEntry @Username = @Username
                                        , @ModuleCode = 'Integration'
                                        , @ProcessName = 'Post bulkload'
                                        , @ProcessID = @ProcessID
                                        , @SubProcessName = 'agent.pPostBulkload'
                                        , @SubProcessID = @SubProcessID
                                        , @SubSubProcessName = 'agent.pPostBulkload'
                                        , @EventCODE = 'Error'
                                        , @Timestamp = @CurrentTimestamp
                                        , @ReturnCODE = @ResultCode
                                        , @EventMessage = @StatusMessage;
    END CATCH;

    RETURN @ResultCode;
END;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pPostBulkload'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for POST Bulkload';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Modules';
    SET @value      = N'Modules';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO
