
DROP PROCEDURE IF EXISTS agent.pSync;
GO

/*
	Procedure for Sync
 
	Gerd Tautenhahn for Saxess Software GmbH
	10/2022 for OCT 5.8

	Testcall
		DECLARE  @RC INT
				,@Username		NVARCHAR(255)	='SQL'
				,@Solution		NVARCHAR(50)	='FIN'
				,@ResultOnly	INT				=0
				,@CompanyKeys	NVARCHAR(MAX)	='1|1;1|2' -- List must be ; separated
				,@ProcessID		NVARCHAR(50)	='0'
				,@SubProcessID	NVARCHAR(50)	='0'


		EXECUTE @RC = agent.pSync
			 @Username
			,@Solution
			,@ResultOnly
			,@CompanyKeys
			,@ProcessID
			,@SubProcessID

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'psync',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'psync','PARAMETER',NULL)
	
*/

CREATE PROCEDURE agent.pSync
    @Username     nvarchar(255)
  , @Solution     nvarchar(50)
  , @ResultOnly   int          = 0
  , @CompanyKeys  nvarchar(MAX)      -- delimited by ;
  , @ProcessID    nvarchar(50) = '0' -- used for logging
  , @SubProcessID nvarchar(50) = '0' -- used for logging
AS
BEGIN
    DECLARE @SQLCommand nvarchar(255);
    DECLARE @ResultCode int = 501;
    DECLARE @MaxRowKey int = 0;
    DECLARE @CurrentTimestamp datetime;

    -- write log entry for begin of agent execution
    IF @ProcessID = '0'
    BEGIN
        SELECT @MaxRowKey = MAX(RowKey)
        FROM system.tProcessLogs;
        SET @ProcessID = @MaxRowKey + 1;
    END;

    SET @CurrentTimestamp = GETUTCDATE();

    -- protect input parameters
    DECLARE @Module nvarchar(50) = @Solution; -- Set Variable Module, will be used in the future. Changes have to be made to OCT to set parameter @Module instead of @Solution
    SET @Username = dbo.sx_pf_pProtectString(@Username);
    SET @Module = dbo.sx_pf_pProtectString(@Module);
    SET @ResultOnly = dbo.sx_pf_pProtectInt(@ResultOnly);

	-- update mappingpriority of companies
	WITH Companies AS (
		SELECT
			*
		  , ROW_NUMBER() OVER (PARTITION BY COALESCE(NULLIF(CompanyIDResult, ''), CompanyID) ORDER BY MappingPriority) AS NewPriority
		FROM global.tCompanies
	)
	UPDATE Companies
	SET MappingPriority = NewPriority;

    -- execute sync procedure for the selected solution
    IF EXISTS
        (
        SELECT *
        FROM sysobjects
        WHERE
            id = OBJECT_ID(N'[agent].[pSync' + @Module + ']')
            AND OBJECTPROPERTY(id, N'IsProcedure') = 1
        )
    BEGIN
        SET @SQLCommand = N'agent.pSync' + @Module;
        EXEC @ResultCode = @SQLCommand @Username
                                     , @ResultOnly
                                     , @CompanyKeys
                                     , @ProcessID
                                     , @SubProcessID;
    END;
    ELSE
        -- if no agent procedure exists for module, nothing is executed and ResultCode 404 is returned
        SET @ResultCode = 404;

    -- write log entry for end of agent execution
    SET @CurrentTimestamp = GETUTCDATE();

    RETURN @ResultCode;
END;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pSync'			    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for Sync';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Solution';
    SET @value      = N'Solution';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ResultOnly';
    SET @value      = N'ResultOnly';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	