DROP PROCEDURE IF EXISTS agent.pSyncDimensionToGlobal;
GO

/*
	Procedure for syncing dimension to global 

	Nutzt dynamisches SQL !
 
	Gerd Tautenhahn for Saxess Software GmbH
	last modified: 10/2022 for OCT 5.8

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pSyncDimensionToGlobal',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'agent', 'PROCEDURE', 'pSyncDimensionToGlobal','PARAMETER',NULL)
*/

CREATE PROCEDURE agent.pSyncDimensionToGlobal
	@Username		NVARCHAR(255),		
	@CompanyKeys	NVARCHAR(MAX),
	@TableName		NVARCHAR(100),
	@ProcessID		NVARCHAR(50) = '0',	
	@SubProcessID	NVARCHAR(50) = '0'	
AS
BEGIN
	DECLARE @SQL NVARCHAR(MAX);
	DECLARE @SQLKeyAND NVARCHAR(MAX) = '';
	DECLARE @SQLKeyCOMMA NVARCHAR(MAX) = '';
	DECLARE @SQLAllFieldsCOMMA NVARCHAR(MAX) = '';
	DECLARE @SQLAllSourceFieldsCOMMA NVARCHAR(MAX) = '';
	DECLARE @SQLPropertyOR NVARCHAR(MAX) = '';
	DECLARE @SQLPropertyCOMMA NVARCHAR(MAX) = '';
	DECLARE @ResultCode INT = 501;
	DECLARE @ColName NVARCHAR(50);
	DECLARE @CurrentTimestamp	DATETIME;
	DECLARE @ErrorMessage		NVARCHAR(2000);
	DECLARE @ErrorMessageDuplicates		NVARCHAR(2000) = '';
	DECLARE @TableNameOrigin NVARCHAR(100) = REPLACE(@TableName, '#', '')
	DECLARE @Duplicates INT = -1;

	BEGIN TRY

		BEGIN TRANSACTION pSyncDimensionToGlobal;

		-- Cursor for key columns
		DECLARE KeyCursor CURSOR FOR
			SELECT ColName
			FROM system.fGetColumns('global', @TableNameOrigin)
			WHERE IsPartOfKey = 1

		-- Cursor for regular columns
		DECLARE ColCursor CURSOR FOR
			SELECT ColName
			FROM system.fGetColumns('integration', @TableNameOrigin)
			WHERE IsPartOfKey = 0
			AND IsCalculatedKey = 0
			AND ColName <> 'RowKey'
			AND ColName NOT IN (
				SELECT ColName
				FROM system.fGetColumns('global', @TableNameOrigin)
				WHERE IsPartOfKey = 1
			)

		-- generate lists of columns needed inside the script
		-- handle key fields
		OPEN KeyCursor
		FETCH KeyCursor INTO @ColName
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @SQLKeyAND += ' AND Target.' + @ColName + ' = Source.' + @ColName;
			SET @SQLKeyCOMMA += @ColName + ',';

			SET @SQLAllFieldsCOMMA += ' ' + @ColName + ',';

			SET @SQLAllSourceFieldsCOMMA += ' Source.' + @ColName + ',';

			FETCH KeyCursor INTO @ColName
		END
		CLOSE KeyCursor
		DEALLOCATE KeyCursor

		-- handle property fields
		OPEN ColCursor
		FETCH ColCursor INTO @ColName
		WHILE @@FETCH_STATUS = 0
		BEGIN
			SET @SQLPropertyOR += ' OR Target.' + @ColName + ' <> Source.' + @ColName;

			SET @SQLPropertyCOMMA += ' Target.' + @ColName + ' = Source.' + @ColName + ',';

			SET @SQLAllFieldsCOMMA += ' ' + @ColName + ',';

			SET @SQLAllSourceFieldsCOMMA += ' Source.' + @ColName + ',';

			FETCH ColCursor INTO @ColName
		END
		CLOSE ColCursor
		DEALLOCATE ColCursor

		-- remove unnecessary comma
		SET @SQLKeyCOMMA = LEFT(@SQLKeyCOMMA, LEN(@SQLKeyCOMMA) - 1);
		SET @SQLAllFieldsCOMMA = LEFT(@SQLAllFieldsCOMMA, LEN(@SQLAllFieldsCOMMA) - 1);
		SET @SQLAllSourceFieldsCOMMA = LEFT(@SQLAllSourceFieldsCOMMA, LEN(@SQLAllSourceFieldsCOMMA) - 1);
		SET @SQLPropertyCOMMA = LEFT(@SQLPropertyCOMMA, LEN(@SQLPropertyCOMMA) - 1);
		
		-- check for duplicates
		SET @SQL = '
SELECT @DuplicateCount = COUNT(1)
FROM (
	SELECT 1 AS Counter
	FROM integration.' + @TableName + '
	WHERE CAST(DataSourceKey AS NVARCHAR(50)) + N''|'' + CompanyID IN (
		SELECT value
		FROM STRING_SPLIT(N''' + @CompanyKeys + ''','';'')
	)
	GROUP BY ' + @SQLKeyCOMMA + '
	HAVING COUNT(1) > 1
) a;'

		EXEC sp_executesql @SQL, N'@DuplicateCount INT OUT', @Duplicates OUT;
		IF @Duplicates > 0
		BEGIN
			SET @ErrorMessageDuplicates = 'ERROR: found duplicates in table integration.' + REPLACE(@TableName, '#', '') + ': ' + CAST(@Duplicates AS NVARCHAR(10))
			RAISERROR (@ErrorMessageDuplicates, 11, 1)
		END

		SET @SQL = '
UPDATE Target
SET ' + @SQLPropertyCOMMA + '
FROM integration.' + @TableName + ' AS Source
JOIN global.' + @TableNameOrigin + ' AS Target
ON 1 = 1' + @SQLKeyAND + '
WHERE (1 = 0' + @SQLPropertyOR + ')
AND CAST(Source.DataSourceKey AS NVARCHAR(50)) + N''|'' + Source.CompanyID IN (
	SELECT value
	FROM STRING_SPLIT(N''' + @CompanyKeys + ''','';'')
);


INSERT INTO global.' + @TableNameOrigin + '
(' + @SQLAllFieldsCOMMA + ', OwnerCODE, UpdatedFlag)
SELECT ' + @SQLAllSourceFieldsCOMMA + ', ''INTEGRATION'', 1
FROM integration.' + @TableName + ' AS Source
LEFT JOIN global.' + @TableNameOrigin + ' AS Target
ON 1 = 1' + @SQLKeyAND + '
WHERE Target.RowKey IS NULL
AND CAST(Source.DataSourceKey AS NVARCHAR(50)) + N''|'' + Source.CompanyID IN (
	SELECT value
	FROM STRING_SPLIT(N''' + @CompanyKeys + ''','';'')
);';

		PRINT CONCAT(FORMAT(GETDATE(), ' yyyy/MM/dd hh:mm:ss', 'en-US'), ' - Updating global.', @TableNameOrigin)

		EXEC sp_executesql @SQL;
		
		SET @ResultCode = 200;

		COMMIT TRANSACTION pSyncDimensionToGlobal;
	END TRY
	BEGIN CATCH
		SET @ResultCode = 500;
		SET @ErrorMessage = ERROR_MESSAGE();
		
		ROLLBACK TRANSACTION pSyncDimensionToGlobal;

		SET @CurrentTimestamp = GETUTCDATE();
		EXEC system.pPOST_ProcessLogEntry @Username = @Username, @ModuleCODE = 'Integration', @ProcessName = 'Agent Execution',
		@ProcessID = @ProcessID, @SubProcessName = 'agent.pSync', @SubProcessID = @SubProcessID, @SubSubProcessName = 'agent.pSyncDimensionToGlobal', @EventCode = 'Error', 
		@Timestamp = @CurrentTimestamp, @ReturnCODE = @ResultCode, @EventMessage = @ErrorMessage;

		RAISERROR (@ErrorMessage, 11, 1)
	END CATCH

	RETURN @ResultCode;
END;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'agent'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pSyncDimensionToGlobal'     -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure for syncing dimension to global';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyKeys';
    SET @value      = N'CompanyKeys';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TableName';
    SET @value      = N'TableName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProcessID';
    SET @value      = N'ProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcessID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

