﻿  
DROP PROCEDURE IF EXISTS control.pGET_RightPointSummary;
GO

/*
	Procedure to calculate the Rights Credits of an Cluster
		- Every User with Cluserwrite Rights gets 2 C Points
		- Every User with Clusterread Rights gets 1 C Points
		- Every User with an Write Right on Factory Level gets 2 F Points per Factory
		- Every User with an Read Right on Factory Level gets 1 Point per Factory
		- Every User with an Write Right on an ProductLine gets 2 PL Points
		- Every User with an Read Rights on an ProductLine gets 1 PL Point 

	Its used to monitor Point count and see unusual values due to failures in Right import

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 	Testcall
		EXEC control.pGET_RightPointSummary

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'pGET_RightPointSummary',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'pGET_RightPointSummary','PARAMETER',NULL)
*/
 
CREATE PROCEDURE control.pGET_RightPointSummary

AS

SELECT
	 
	 Points.UserName
	,SUM(Points.C_Points) AS C_Points
	,SUM(Points.F_Points) AS F_Points
	,SUM(Points.PL_Points) AS PL_Points

	,SUM(Points.C_Points
		+Points.F_Points
		+Points.PL_Points) AS Full_Points

	,SUM( SUM(Points.C_Points )
		+ SUM(Points.F_Points )
		+ SUM(Points.PL_Points)) OVER () AS TotalPoints
FROM 
	(
		SELECT 
			 vUR.UserName
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS C_Points
			,0 AS F_Points
			,0 AS PL_Points
	
		FROM system.trUserRights vUR

		WHERE vUR.FactoryID = '' AND vUR.ProductLineID = ''

	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS F_Points
			,0 AS PL_Points
	
		FROM system.trUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID = ''


	UNION ALL

		SELECT 
			 vUR.UserName
			,0 AS C_Points
			,0 AS F_Points
			,CASE
				WHEN vUR.[Right] = 'Write' THEN 2
				WHEN vUR.[Right] = 'Read' THEN 1
				ELSE 0
			 END AS PL_Points
	
		FROM system.trUserRights vUR

		WHERE vUR.FactoryID <> '' AND vUR.ProductLineID <> ''

	) Points

GROUP BY Points.UserName


GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'control'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGET_RightPointSummary'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to calculate the Rights Credits of an Cluster
         
         - Every User with Cluserwrite Rights gets 2 C Points
         - Every User with Clusterread Rights gets 1 C Points
         - Every User with an Write Right on Factory Level gets 2 F Points per Factory
         - Every User with an Read Right on Factory Level gets 1 Point per Factory
         - Every User with an Write Right on an ProductLine gets 2 PL Points
         - Every User with an Read Rights on an ProductLine gets 1 PL Point';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
GO
