﻿
DROP PROCEDURE IF EXISTS control.spParamTables;
GO

/*
	Procedure to auto create param tables from all products in ProductLine PARAM in Factory ZT

	Uses dynamic SQL !

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		EXEC control.spParamTables '1-1'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'spParamTables',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'PROCEDURE', 'spParamTables','PARAMETER',NULL)

*/
  
CREATE PROCEDURE control.spParamTables
	@ProductID NVARCHAR(255)
AS

BEGIN
	SET NOCOUNT ON
	
	-- Standard declaration for logging
	DECLARE 
				@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			,@ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@ProductID				,N'NULL')					+ N''''

			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N''
			,@Username			NVARCHAR(255)	= N'SQL'

	DECLARE  @TableDelete				NVARCHAR(MAX)	= ''
			,@SQLDrop					NVARCHAR(MAX)	= ''
			,@ProductKey				BIGINT			= 0;
			
	-- START TRANSACTION			***********************************************************************************
	BEGIN TRY
		BEGIN TRANSACTION spParamTables;
		
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user don`t exists', 16, 10);
				END;

	-- DELETE existing param.tPivot table
	SET @TableDelete = 'param.[tPivot' + @ProductID+ ']';
	SET @SQLDrop = 'DROP TABLE IF EXISTS ' + @TableDelete;

	EXECUTE sp_executesql @SQLDrop;

	-- CREATE new table
	
	SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = 'ZT' AND ProductLineID = 'PARAM' AND ProductID = @ProductID;

	IF @ProductKey > 0
		BEGIN
				-- CREATE TABLE -------------------------------------------------------------------------------------------------------------------
				DECLARE	 @TableSchema			NVARCHAR(255)	= 'param.'
						,@TablePreName			NVARCHAR(255)	= 'tPivot'																						
						,@StringBasic			NVARCHAR(MAX)	= 'RowKey BIGINT IDENTITY(1,1),FactoryID NVARCHAR(255),ProductLineID NVARCHAR(255),ProductID NVARCHAR(255),TimeID INT'
						,@StringColumns			NVARCHAR(MAX)	= ''	
						,@SQLCreate				NVARCHAR(MAX)	= ''
						,@TableName				NVARCHAR(MAX)	= '';

				SELECT @TableName = @TableSchema + '[' + @TablePreName + @ProductID + ']';

				-- Gets all ValueSeriesIDs from a Product in ProductlLine 'PARAM' in Factory 'ZT', ValueSeriesIDs will become column headers
				-- All Text Values will become NVARCHAR(4000), all Numeric Columns Type Money

				-- Collect definition of all columns
				SELECT @StringColumns = COALESCE(@StringColumns + ',', '') +  
						IIF(
							dVS.[IsNumeric]=0
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] NVARCHAR(4000)'
								,'[' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] MONEY'
							)
				FROM planning.tdValueSeries dVS 
				WHERE dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- create the table
				SET @SQLCreate = 'CREATE TABLE ' 
							+ @TableName 
							+ '(' 
							+ @StringBasic 
							+ @StringColumns
							+ ' CONSTRAINT [PK_' + @TablePreName + @ProductID +'] PRIMARY KEY NONCLUSTERED (RowKey))';
			
				EXECUTE sp_executesql @SQLCreate;

				-- Fill Table -------------------------------------------------------------------------------------------------------
			
				DECLARE  @STRING_PivotList					NVARCHAR(MAX)  
						,@STRING_SelectColums				NVARCHAR(MAX)  
						,@STRING_Insert						NVARCHAR(MAX);

				-- Get string for dynamic SQL used in PIVOT command (...ValueSeriesID IN ([...]))
				SELECT @STRING_PivotList = COALESCE(@STRING_PivotList + ',', '') +  '[' + CONVERT(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- Get string for dynamic SQL - ColumnNames
				SELECT @STRING_SelectColums = COALESCE(@STRING_SelectColums + ',', '') 
					+ 'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''') AS [' + convert(NVARCHAR(255),ValueSeriesID) + ']'
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- Get string for dynamic SQL used in INSERT command
				SELECT @STRING_Insert = COALESCE(@STRING_Insert + ',', '')
							+IIF(
								 dVS.[IsNumeric]=0
								,'COALESCE([' + CONVERT(NVARCHAR(255),ValueSeriesID) + '],'''')'
								,'COALESCE(CAST([' + CONVERT(NVARCHAR(255),dVS.ValueSeriesID) + '] AS MONEY) ' + ',0) '
							)
				FROM	planning.tdValueSeries dVS
				WHERE	dVS.ProductKey = @ProductKey
				ORDER BY dVS.ValueSeriesNo;

				-- Select Product Values to #table to not use dynamic SQL on real table
				DROP TABLE IF EXISTS #ParamValues;
				
				CREATE TABLE #ParamValues
					(
						 FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,TimeID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
						,Value				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NOT NULL
					);

				INSERT INTO #ParamValues
					SELECT 
						 fV.FactoryID
						,fV.ProductLineID
						,fV.ProductID
						,fV.ValueSeriesID
						,fV.TimeID
						,CASE
							WHEN dVS.[IsNumeric] = 0 THEN fV.ValueText
							WHEN dVS.[IsNumeric] = 1 THEN CONVERT(NVARCHAR(255),CAST(fV.ValueInt AS MONEY)/dVS.Scale,2) --warum Parameter 2 im Convert ?
						 END AS Value
					FROM planning.tfValues fV
							LEFT JOIN planning.tdValueSeries dVS
								ON fV.ValueSeriesKey = dVS.ValueSeriesKey
							LEFT JOIN planning.tdProducts dP 
								ON fV.ProductKey = dP.ProductKey
					WHERE fV.ProductKey = @ProductKey
					ORDER BY dVS.ValueSeriesNo;

				DECLARE @SQLText	NVARCHAR(MAX) = 'INSERT INTO ' + @TableName + '
					SELECT	 PivotT.FactoryID,PivotT.ProductLineID,PivotT.ProductID,PivotT.TimeID
							,' + @STRING_Insert + '													
					FROM #ParamValues
					PIVOT
						(MAX(Value) FOR ValueSeriesID IN (' + @STRING_PivotList + ')) AS PivotT';

				EXECUTE sp_executesql @SQLText;

				SELECT @EffectedRows = COUNT(1)
				FROM #ParamValues;

				-- job well done
				SET @ResultCode = 200;
		END
	ELSE
		BEGIN
			SET @ResultCode = 404;
		END
		
			COMMIT TRANSACTION spParamTables;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode

END
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'control'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spParamTables'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to auto create param tables from all products in ProductLine PARAM in Factory ZT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID of an Product in Factory ZT / ProductLine PARAM';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
