DROP PROCEDURE IF EXISTS  dbo.sx_pf_COPY_Factory;
GO

/*
	COPY Operation for a Factory
	Not yet existing factory is copied when Source Factory exists

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 11/2022 for OCT 5.8

	Test call
	1. Empty  source or target FactoryID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists Source Factory => 404 Not Found
	5. Target Factory key already exists => 403 Forbidden
	6. Factory created > 201 Created

	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) = 'SQL';
			,@SourceFactoryID			NVARCHAR(255) = 'ZT'
			,@TargetFactoryID			NVARCHAR(255) = 'ZT2'
			,@TargetFactoryNameShort	NVARCHAR(255) = 'Neue Factory';

	EXECUTE @RC = dbo.sx_pf_COPY_Factory 
		 @Username
		,@SourceFactoryID
		,@TargetFactoryID
		,@TargetFactoryNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Factory',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_Factory
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetFactoryNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey		INT				= 0;
	DECLARE @SourceFactoryKey		INT				= 0;
	DECLARE @SourceFactoryNameShort NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetFactoryNameShort	IS NULL SET @TargetFactoryNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Factory;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetFactoryNameShort = dbo.sx_pf_pProtectString	(@TargetFactoryNameShort);
		
		IF	@SourceFactoryID	= N'' 
			OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	
					
		-- STEP 1.2 - Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Format key already exists', 16, 10);
		END;

		-- STEP 3 - Copy data
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
		-- dFactory
		INSERT INTO planning.tdFactories
		OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
		SELECT @TargetFactoryID			AS FactoryID
			, @TargetFactoryNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName 
		FROM planning.tdFactories 
		WHERE FactoryID = @SourceFactoryID; 

		SET @EffectedRows = @@ROWCOUNT;	

		SELECT TOP 1 @TargetFactoryKey = [Key]
		FROM @Keys;
		
		--dProductLine 
		INSERT INTO planning.tdProductLines
		SELECT @TargetFactoryKey AS FactoryKey
			, ProductLineID
			, @TargetFactoryID	 AS FactoryID
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3 
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6 
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9 	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12 
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18 
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21 	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM planning.tdProductLines 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;	

		--dProduct 
		INSERT INTO planning.tdProducts
		SELECT PL.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, ProductID
			, dP.ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts dP
				LEFT JOIN (
					SELECT ProductLineKey, ProductLineID 
					FROM planning.tdProductLines 
					WHERE FactoryKey = @TargetFactoryKey) PL 
				ON PL.ProductLineID = dP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries  
		SELECT P.ProductKey		AS ProductKey
			, P.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dVS.ProductID
			, dVS.ProductLineID 
			, @TargetFactoryID	AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries dVS
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dVS.ProductID AND P.ProductLineID = dVS.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime  
		SELECT 
			 P.ProductKey AS ProductKey
			, P.ProductLineKey AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, dT.ProductID
			, dT.ProductLineID 
			, @TargetFactoryID AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime dT
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = dT.ProductID AND P.ProductLineID = dT.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues 
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, VS.ProductLineKey		AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, fV.ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
				LEFT JOIN (
					SELECT ProductKey, ValueSeriesKey, ProductLineKey, ProductID, ValueSeriesID, ProductLineID 
					FROM planning.tdValueSeries 
					WHERE FactoryKey = @TargetFactoryKey) VS 
				ON fV.ValueSeriesID = VS.ValueSeriesID AND fV.ProductID = VS.ProductID AND fV.ProductLineID = VS.ProductLineID
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gFactories
		INSERT INTO planning.tgFactories 
		SELECT 
			 @TargetFactoryKey	AS FactoryKey
			,@TargetFactoryID	AS FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM planning.tgFactories
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--gProductLines
		INSERT INTO planning.tgProductLines
		SELECT 
			 PL.ProductLineKey	AS ProductLineKey 
			,@TargetFactoryKey	AS FactoryKey
			,gPL.ProductLineID
			,@TargetFactoryID	AS FactoryID
			,PropertyID
			,PropertyName
			,CommentUser
			,CommentDev
			,Unit
			,ValueText
			,ValueInt
			,Scale
			,IsROSystemProperty
			,FormatID
		FROM planning.tgProductLines gPL
			LEFT JOIN (
				SELECT ProductLineKey, ProductLineID 
				FROM planning.tdProductLines 
				WHERE FactoryKey = @TargetFactoryKey) 
			PL ON PL.ProductLineID = gPL.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO planning.tgProducts
		SELECT 
			  P.ProductKey		AS ProductKey
			, P.ProductLineKey	AS ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, gP.ProductID
			, gP.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
				FROM planning.tdProducts 
				WHERE FactoryKey = @TargetFactoryKey) P 
			ON P.ProductID = gP.ProductID AND P.ProductLineID = gP.ProductLineID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rPreferences
		INSERT INTO planning.trPreferences
		SELECT 
			 @TransactUserKey	AS UserKey 
			,Username			AS UserName 
			,@TargetFactoryID	AS FactoryID
			,ProductLineID
			,SettingName 
			,ValueInt 
			,ValueText
		FROM planning.trPreferences 
		WHERE FactoryID = @SourceFactoryID;

		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT 
				 @TargetFactoryKey
				,P.ProductLineKey
				,P.ProductKey
				,@TargetFactoryID		AS FactoryID
				,tF.ProductLineID		AS ProductLineID
				,tF.ProductID			AS ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
				,tF.UploadedBy
				,tF.UploadedAt
				,tF.Filepath
			FROM planning.tFiles tF
				LEFT JOIN (
					SELECT ProductKey, ProductLineKey, ProductID, ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey) P 
				ON P.ProductID = tF.ProductID AND P.ProductLineID = tF.ProductLineID
			WHERE FactoryID = @SourceFactoryID;

		--tTabs - they can exist on multiple levels ! Tabs on Factories without Productlines and on Productlines without Products may exist
		INSERT INTO planning.tTabs
			SELECT 
				 @TargetFactoryKey
				,COALESCE(PL.ProductLineKey,-1)	AS ProductLineKey
				,COALESCE(P.ProductKey,-1)		AS ProductKey
				,@TargetFactoryID				AS FactoryID
				,COALESCE(tT.ProductLineID,'')	AS ProductLineID
				,COALESCE(tT.ProductID,'')		AS ProductID
				,tT.TabID
				,tT.Orderindex
				,tT.TabName
				,tT.TabHint
				,tT.TabText
				,tT.PresentationCODE
				,tT.Datasource
				,tT.IsVisibleBOOL
				,tT.LayoutJSON
				,tT.ParameterJSON

			FROM planning.tTabs tT

				-- Determine new ProductKeys
				LEFT JOIN (
					SELECT 
						 ProductKey
						,ProductLineKey
						,ProductID
						,ProductLineID 
					FROM planning.tdProducts 
					WHERE FactoryKey = @TargetFactoryKey
					) P 
				ON 
						COALESCE(P.ProductID,'')		= tT.ProductID 
					AND COALESCE(P.ProductLineID,'')	= tT.ProductLineID

				-- Determine new ProductLineKeys
				LEFT JOIN (
					SELECT 
						 ProductLineKey
						,ProductLineID 
					FROM planning.tdProductlines
					WHERE FactoryKey = @TargetFactoryKey
					) PL
				ON COALESCE(PL.ProductLineID,'')	= tT.ProductLineID

			WHERE FactoryID = @SourceFactoryID;

		--fStatements CopyInformation
		INSERT INTO planning.tfStatements
		SELECT ProductKey
			, dp.ProductLineKey
			, @TargetFactoryKey AS FactoryKey
			, ProductID
			, dp.ProductLineID
			, @TargetFactoryID	AS FactoryID
			, N'COPY'			AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
				+ N'" / ProductLine <' + dp.ProductLineID + N'>"' + dpl.Nameshort
				+ N'" / Product <' + ProductID + N'>"' + dp.Nameshort + '"' AS Statement
			, @TransactUsername AS UserName 
			, N''				AS PCName
			, N''				AS ProcessorCode
			, N''				AS IPAddresses
			, GETUTCDATE()		AS Timestamp 
			, 0					AS IsDeleted 
			, 0					AS IsResolved
		FROM planning.tdProducts dp LEFT JOIN planning.tdProductLines dpl 
			ON dp.ProductLinekey= dpl.ProductLineKey
		WHERE dp.FactoryKey = @TargetFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Right materialization
		EXEC system.spMaterialize_trUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_Factory TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Factory TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_Factory'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for a Factory';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryNameShort';
    SET @value      = N'TargetFactoryNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
