
DROP PROCEDURE IF EXISTS dbo.sx_pf_COPY_Product;
GO

/*
	COPY Operation for a Product
	 - creates a new Product from an Template
	 - * as Target ProductID is supported, the next free numeric ID is used then
		
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8

	Test call
	1. Empty source or target FactoryID / ProductLineID / ProductID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine / Product => 404 Not Found
	5. Not exists target Factory / ProductLine => 404 Not Found
	6. Target Product key already exists => 403 Forbidden
	7. Product copied => 201 Created

	DECLARE  @RC						INT
			,@Username					NVARCHAR(255) = 'SQL'
			,@SourceFactoryID			NVARCHAR(255) = 'WSB'
			,@SourceProductLineID		NVARCHAR(255) = 'KRED1'
			,@SourceProductID			NVARCHAR(255) = '0'
			,@TargetFactoryID			NVARCHAR(255) = 'WSB'
			,@TargetProductLineID		NVARCHAR(255) = 'KRED1'
			,@TargetProductID			NVARCHAR(255) = '*'
			,@TargetProductNameShort	NVARCHAR(255) = 'Test';

	EXECUTE @RC = dbo.sx_pf_COPY_Product 
			   @Username
			  ,@SourceFactoryID
			  ,@SourceProductLineID
			  ,@SourceProductID
			  ,@TargetFactoryID
			  ,@TargetProductLineID
			  ,@TargetProductID
			  ,@TargetProductNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Product',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_Product','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_Product
	@Username				NVARCHAR(255),
	@SourceFactoryID		NVARCHAR(255),
	@SourceProductLineID	NVARCHAR(255),
	@SourceProductID		NVARCHAR(255),
	@TargetFactoryID		NVARCHAR(255),
	@TargetProductLineID	NVARCHAR(255),
	@TargetProductID		NVARCHAR(255),
	@TargetProductNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @TargetProductKey			INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@SourceProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineID,		N'NULL') + N''',''' 
			+ ISNULL(@TargetProductID,			N'NULL') + N''',''' 
			+ ISNULL(@TargetProductNameShort,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username				IS NULL SET @Username				= N'';
	IF @SourceFactoryID			IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID		IS NULL SET @SourceProductLineID	= N'';
	IF @SourceProductID			IS NULL SET @SourceProductID		= N'';
	IF @TargetFactoryID			IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID		IS NULL SET @TargetProductLineID	= N'';
	IF @TargetProductID			IS NULL SET @TargetProductID		= N'';
	IF @TargetProductNameShort	IS NULL SET @TargetProductNameShort = N'';
	
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_Product;

		-- STEP 0.2 - Protect input parameters	
		SET @Username				= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID		(@SourceProductLineID);
		SET @SourceProductID		= dbo.sx_pf_pProtectID		(@SourceProductID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID		(@TargetProductLineID);
		SET @TargetProductID		= dbo.sx_pf_pProtectID		(@TargetProductID);
		SET @TargetProductNameShort = dbo.sx_pf_pProtectString	(@TargetProductNameShort);

		IF	@SourceFactoryID		= N'' 
			OR @SourceProductLineID = N'' 
			OR @SourceProductID		= N''
			OR @TargetFactoryID		= N'' 
			OR @TargetProductLineID = N'' 
			OR @TargetProductID		= N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @TargetFactoryID, @TargetProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @SourceFactoryID, @SourceProductLineID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey, @SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		SELECT @SourceProductKey = ProductKey, @SourceProductNameShort = NameShort FROM planning.tdProducts WHERE FactoryKey = @SourceFactoryKey AND ProductLineKey = @SourceProductLineKey AND ProductID = @SourceProductID;
		
		IF	@SourceFactoryKey			= 0 
			OR @SourceProductLineKey	= 0 
			OR @SourceProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		SELECT @TargetProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @TargetFactoryKey AND ProductLineKey = @TargetProductLineKey AND ProductID = @TargetProductID;
		
		IF	@TargetFactoryKey		 = 0 
			OR @TargetProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductKey <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Product key already exists', 16, 10);
		END;
		
		-- STEP 1.4 - If ProductID * is submitted, search the next free numeric ProductID
		IF @TargetProductID = '*' 
		BEGIN
			SELECT @TargetProductID = COALESCE(MAX(CAST(ProductID AS BIGINT)), 0) + 1 
			FROM planning.tdProducts  
			WHERE ProductLineKey = @TargetProductLineKey 
				AND TRY_CAST(ProductID AS BIGINT) IS NOT NULL;
			
			-- if it there is no others Products in ProductLine`s
			IF @TargetProductID = '*' 
				SELECT @TargetProductID = 1;
		END

		-- STEP 3 - Copy data		
		--dProduct
		DECLARE @TargetProductKeyTable AS TABLE (KeyNumber BIGINT NOT NULL);

		INSERT INTO planning.tdProducts
		OUTPUT INSERTED.ProductKey INTO @TargetProductKeyTable
		SELECT @TargetProductLineKey	AS ProductLineKey
			, @TargetFactoryKey			AS FactoryKey
			, @TargetProductID			AS ProductID
			, @TargetProductLineID		AS ProductLineID
			, @TargetFactoryID			AS FactoryID
			, TimeType
			, @TargetProductNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion
			, GlobalAttribute1 , GlobalAttribute2 , GlobalAttribute3 , GlobalAttribute4 , GlobalAttribute5 , GlobalAttribute6 , GlobalAttribute7
			, GlobalAttribute8 , GlobalAttribute9 , GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductKey = KeyNumber 
		FROM @TargetProductKeyTable;

		--gProducts Inhalt
		INSERT INTO planning.tgProducts
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries
		SELECT
		      @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime
		SELECT
			 @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime
		WHERE ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, @TargetProductKey		AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID 
				FROM planning.tdValueSeries 
				WHERE ProductKey = @TargetProductKey) VS 
			ON VS.ValueSeriesID = fV.ValueSeriesID 
		WHERE fV.ProductKey = @SourceProductKey;

		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT
				 @TargetFactoryKey
				,@TargetProductLineKey
				,@TargetProductKey
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID
				,@TargetProductID		AS ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
				,tF.UploadedBy
				,tF.UploadedAt
				,tF.Filepath
			FROM planning.tFiles tF
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID
				AND ProductID		= @SourceProductID;

		--fStatements CopyInformation speichern
		INSERT INTO planning.tfStatements
		SELECT @TargetProductKey	AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, @TargetProductID		AS ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, N'COPY'				AS ActionType
			, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
				+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort 
				+ N'"/ Product <' + @SourceProductID + N'>"' + @SourceProductNameShort + N'"' AS Statement
			, @TransactUsername		AS UserName
			, N''					AS PCName 
			, N''					AS ProcessorCode 
			, N''					AS IPAddresses
			, GETUTCDATE()			AS Timestamp
			, 0						AS IsDeleted 
			, 0						AS IsResolved;
		
		SET @EffectedRows += @@ROWCOUNT;
		SET @ResultCode = 201;
			
		COMMIT TRANSACTION sx_pf_COPY_Product;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_Product;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_Product TO pf_PlanningFactoryService;   
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_Product TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_Product'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for a Product';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductID';
    SET @value      = N'SourceProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductID';
    SET @value      = N'TargetProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductNameShort';
    SET @value      = N'TargetProductNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
