
DROP PROCEDURE IF  EXISTS dbo.sx_pf_COPY_ProductLine;
GO

/*
	COPY Operation for a ProductLine
	Not yet existing ProductLine is copied when Source ProductLine is available

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified 09/2022 for OCT 5.8
 
	Test call
	1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine => 404 Not Found
	5. Not exists target Factory => 404 Not Found
	6. Target ProductLine key already exists => 403 Forbidden
	7. ProductLine copied => 201 Created

	DECLARE  @RC							INT
			,@Username						NVARCHAR(255) = 'SQL'
			,@SourceFactoryID				NVARCHAR(255) = 'ZT'
			,@SourceProductLineID			NVARCHAR(255) = 'ZT'
			,@TargetFactoryID				NVARCHAR(255) = 'ZT'
			,@TargetProductLineID			NVARCHAR(255) = 'ZT32'
			,@TargetProductLineNameShort	NVARCHAR(255) = 'Neue ProductLine32';

	EXECUTE  @RC = dbo.sx_pf_COPY_ProductLine 
			 @Username
			,@SourceFactoryID
			,@SourceProductLineID
			,@TargetFactoryID
			,@TargetProductLineID
			,@TargetProductLineNameShort;

	PRINT @RC;

	Testcall Documentation
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_ProductLine',NULL,NULL)
	UNION ALL
	SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_COPY_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_COPY_ProductLine
	@Username					NVARCHAR(255),
	@SourceFactoryID			NVARCHAR(255),
	@SourceProductLineID		NVARCHAR(255),
	@TargetFactoryID			NVARCHAR(255),
	@TargetProductLineID		NVARCHAR(255),
	@TargetProductLineNameShort NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUserKey			INT				= 0;
	DECLARE @SourceFactoryKey			INT				= 0;
	DECLARE @SourceProductLineKey		INT				= 0;
	DECLARE @SourceFactoryNameShort		NVARCHAR(255)	= N'';
	DECLARE @SourceProductLineNameShort NVARCHAR(255)	= N'';
	DECLARE @SourceProductNameShort		NVARCHAR(255)	= N'';
	DECLARE @TargetFactoryKey			INT				= 0;
	DECLARE @TargetProductLineKey		INT				= 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,					  N'NULL') + N''',''' 
			+ ISNULL(@SourceFactoryID,			  N'NULL') + N''',''' 
			+ ISNULL(@SourceProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetFactoryID,			  N'NULL') + N''','''
			+ ISNULL(@TargetProductLineID,		  N'NULL') + N''',''' 
			+ ISNULL(@TargetProductLineNameShort, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username					IS NULL SET @Username					= N'';
	IF @SourceFactoryID				IS NULL SET @SourceFactoryID			= N'';
	IF @SourceProductLineID			IS NULL SET @SourceProductLineID		= N'';
	IF @TargetFactoryID				IS NULL SET @TargetFactoryID			= N'';
	IF @TargetProductLineID			IS NULL SET @TargetProductLineID		= N'';
	IF @TargetProductLineNameShort	IS NULL SET @TargetProductLineNameShort = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_COPY_ProductLine;

		-- STEP 0.2 - Protect input parameters
		SET @Username					= dbo.sx_pf_pProtectString	(@Username);
		SET @SourceFactoryID			= dbo.sx_pf_pProtectID		(@SourceFactoryID);
		SET @SourceProductLineID		= dbo.sx_pf_pProtectID		(@SourceProductLineID);
		SET @TargetFactoryID			= dbo.sx_pf_pProtectID		(@TargetFactoryID);
		SET @TargetProductLineID		= dbo.sx_pf_pProtectID		(@TargetProductLineID);
		SET @TargetProductLineNameShort = dbo.sx_pf_pProtectString	(@TargetProductLineNameShort);

		IF	@SourceFactoryID				= N'' 
			OR @SourceProductLineID			= N'' 
			OR @TargetFactoryID				= N''
			OR @TargetProductLineID			= N'' 
			OR @TargetProductLineNameShort	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source', 16, 10);

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @SourceFactoryKey = FactoryKey,@SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey = ProductLineKey,@SourceProductLineNameShort = Nameshort FROM planning.tdProductLines WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		
		IF	@SourceFactoryKey		 = 0 
			OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Determine TargetKeys	
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- STEP 3 - Copy data		
		
		DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
		
		--dProductLine
		INSERT INTO planning.tdProductLines
		OUTPUT INSERTED.ProductLineKey INTO @Keys([Key]) 
		SELECT @TargetFactoryKey			AS FactoryKey
			, @TargetProductLineID			AS ProductLineID
			, @TargetFactoryID				AS FactoryID
			, @TargetProductLineNameShort	AS NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, DefaultTemplate 
			, GlobalAttributeSource1,  GlobalAttributeAlias1,  GlobalAttributeSource2,  GlobalAttributeAlias2,  GlobalAttributeSource3,  GlobalAttributeAlias3  
			, GlobalAttributeSource4,  GlobalAttributeAlias4,  GlobalAttributeSource5,  GlobalAttributeAlias5,  GlobalAttributeSource6,  GlobalAttributeAlias6  
			, GlobalAttributeSource7,  GlobalAttributeAlias7,  GlobalAttributeSource8,  GlobalAttributeAlias8,  GlobalAttributeSource9,  GlobalAttributeAlias9  	
			, GlobalAttributeSource10, GlobalAttributeAlias10, GlobalAttributeSource11, GlobalAttributeAlias11, GlobalAttributeSource12, GlobalAttributeAlias12  
			, GlobalAttributeSource13, GlobalAttributeAlias13, GlobalAttributeSource14, GlobalAttributeAlias14, GlobalAttributeSource15, GlobalAttributeAlias15 
			, GlobalAttributeSource16, GlobalAttributeAlias16, GlobalAttributeSource17, GlobalAttributeAlias17, GlobalAttributeSource18, GlobalAttributeAlias18  
			, GlobalAttributeSource19, GlobalAttributeAlias19, GlobalAttributeSource20, GlobalAttributeAlias20, GlobalAttributeSource21, GlobalAttributeAlias21  	
			, GlobalAttributeSource22, GlobalAttributeAlias22, GlobalAttributeSource23, GlobalAttributeAlias23, GlobalAttributeSource24, GlobalAttributeAlias24 
			, GlobalAttributeSource25, GlobalAttributeAlias25	
		FROM planning.tdProductLines
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		SELECT TOP 1 @TargetProductLineKey = [Key] 
		FROM @Keys;
		
		--dProduct
		INSERT INTO planning.tdProducts 
		SELECT @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		 AS FactoryKey
			, ProductID
			, @TargetProductLineID	 AS ProductLineID
			, @TargetFactoryID		 AS FactoryID
			, TimeType
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ResponsiblePerson
			, ImageName
			, Status
			, Template
			, TemplateVersion 
			, GlobalAttribute1,  GlobalAttribute2,  GlobalAttribute3,  GlobalAttribute4,  GlobalAttribute5,  GlobalAttribute6,  GlobalAttribute7  
			, GlobalAttribute8,  GlobalAttribute9,  GlobalAttribute10, GlobalAttribute11, GlobalAttribute12, GlobalAttribute13, GlobalAttribute14 
			, GlobalAttribute15, GlobalAttribute16, GlobalAttribute17, GlobalAttribute18, GlobalAttribute19, GlobalAttribute20, GlobalAttribute21  
			, GlobalAttribute22, GlobalAttribute23, GlobalAttribute24, GlobalAttribute25 
		FROM planning.tdProducts 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dValueSeries
		INSERT INTO planning.tdValueSeries 
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dVS.ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, ValueSeriesID
			, ValueSeriesNo
			, NameShort
			, NameLong
			, CommentUser
			, CommentDev
			, ImageName
			, [IsNumeric]
			, VisibilityLevel
			, ValueSource
			, ValueListID 
			, ValueFormatID
			, Unit
			, Scale
			, Effect
			, EffectParameter
		FROM planning.tdValueSeries dVS
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dVS.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--dTime
		INSERT INTO planning.tdTime 
		SELECT 
			P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, dT.ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, TimeID
			, FormatID
		FROM planning.tdTime dT
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts
				WHERE ProductLineKey = @TargetProductLineKey) P 
			ON P.ProductID = dT.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--fValues
		INSERT INTO planning.tfValues
		SELECT VS.ValueSeriesKey	AS ValueSeriesKey
			, VS.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, fV.ProductID
			, @TargetProductLineID	AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, fV.ValueSeriesID
			, TimeID
			, ValueFormula
			, ValueInt
			, ValueText
			, ValueComment
		FROM planning.tfValues fV
			LEFT JOIN (
				SELECT ValueSeriesKey, ProductKey, ValueSeriesID, ProductID 
				FROM planning.tdValueSeries
				WHERE ProductLineKey = @TargetProductLineKey) VS 
			ON fV.ValueSeriesID=VS.ValueSeriesID AND fV.ProductID = VS.ProductID
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		--gProductLines 
		INSERT INTO planning.tgProductLines
		SELECT @TargetProductLineKey AS ProductLineKey 
			, @TargetFactoryKey		 AS FactoryKey
			, @TargetProductLineID	 AS ProductLineID
			, @TargetFactoryID		 AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProductLines
		WHERE ProductLineKey = @SourceProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		--gProducts 
		INSERT INTO planning.tgProducts  
		SELECT P.ProductKey			AS ProductKey
			, @TargetProductLineKey AS ProductLineKey
			, @TargetFactoryKey		AS FactoryKey
			, gP.ProductID
			, @TargetProductLineID  AS ProductLineID
			, @TargetFactoryID		AS FactoryID
			, PropertyID
			, PropertyName
			, CommentUser
			, CommentDev
			, Unit
			, ValueText
			, ValueInt
			, Scale
			, IsROSystemProperty
			, FormatID
		FROM planning.tgProducts gP
			LEFT JOIN (
				SELECT ProductKey, ProductID 
				FROM planning.tdProducts  
				WHERE ProductLineKey = @TargetProductLineKey) P
			ON P.ProductID = gP.ProductID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		--rRights - no definition, write Rights on Factory will lead to implicit Write Rights for creator

		--rPreferences
		INSERT INTO planning.trPreferences
			SELECT 
				 UserKey				AS UserKey 
				,Username				AS UserName
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID 
				,SettingName
				,ValueInt
				,ValueText
			FROM planning.trPreferences
			WHERE 
					FactoryID		= @SourceFactoryID 
				AND ProductLineID	= @SourceProductLineID;
		
		SET @EffectedRows += @@ROWCOUNT;

		--tFiles
		INSERT INTO planning.tFiles
			SELECT 
				 @TargetFactoryKey
				,@TargetProductLineKey
				,P.ProductKey
				,@TargetFactoryID		AS FactoryID
				,@TargetProductLineID	AS ProductLineID
				,tF.ProductID
				,tF.FileID
				,tF.FileName
				,tF.FileExtension
				,tF.FileBody
				,tF.UploadedBy
				,tF.UploadedAt
				,tF.Filepath
			FROM planning.tFiles tF
				LEFT JOIN (
					SELECT ProductKey, ProductID 
					FROM planning.tdProducts  
					WHERE ProductLineKey = @TargetProductLineKey) P
				ON P.ProductID = tF.ProductID 
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID;

		SET @EffectedRows += @@ROWCOUNT;

		--tTabs
		INSERT INTO planning.tTabs
			SELECT 
				 @TargetFactoryKey
				,@TargetProductLineKey
				,COALESCE(P.ProductKey,-1)	AS ProductKey
				,@TargetFactoryID			AS FactoryID
				,@TargetProductLineID		AS ProductLineID
				,COALESCE(tT.ProductID,'')	AS ProductID
				,tT.TabID
				,tT.Orderindex
				,tT.TabName
				,tT.TabHint
				,tT.TabText
				,tT.PresentationCODE
				,tT.Datasource
				,tT.IsVisibleBOOL
				,tT.LayoutJSON
				,tT.ParameterJSON

			FROM planning.tTabs tT
				LEFT JOIN (
							SELECT ProductKey, ProductID 
							FROM planning.tdProducts  
							WHERE ProductLineKey = @TargetProductLineKey
							) P
				ON COALESCE(P.ProductID,'') = tT.ProductID 
			WHERE	
					FactoryID		= @SourceFactoryID
				AND ProductLineID	= @SourceProductLineID;


		--fStatements CopyInformation
		INSERT INTO planning.tfStatements
			SELECT ProductKey
				, @TargetProductLineKey AS ProductLineKey
				, @TargetFactoryKey		AS FactoryKey 
				, ProductID
				, @TargetProductLineID  AS ProductLineID
				, @TargetFactoryID		AS FactoryID
				, N'COPY'				AS ActionType
				, N'Product produced as a copy of Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
					+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort
					+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS Statement
				, @TransactUsername		AS UserName
				, N''					AS PCName 
				, N''					AS ProcessorCode 
				, N''					AS IPAddresses
				, GETUTCDATE()			AS Timestamp
				, 0						AS IsDeleted
				, 0						AS IsResolved
			FROM planning.tdProducts 
			WHERE ProductLineKey = @TargetProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Rights materialization 
		EXEC system.spMaterialize_trUserRights;

		SET @ResultCode = 201;

		COMMIT TRANSACTION sx_pf_COPY_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_COPY_ProductLine;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_COPY_ProductLine TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_COPY_ProductLine TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_COPY_ProductLine'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to COPY Operation for a ProductLine';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TargetProductLineNameShort';
    SET @value      = N'TargetProductLineNameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
