
DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_Factory;
GO

/*
	DELETE Operation for a Factory
	Deletes an entire Factory (include ProductLines, Products, Values)

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9

	Test call
		1. FactoryID is empty => 404 Not Found
		2. TransactionUsername not determined for User => 403 Forbidden
		3. User does not have write access to the cluster => 401 Unauthorized
		4. Factory deleted => 200 OK
		5. FactoryID not found => 204 No Content

		DECLARE  @RC		INT
				,@Username	NVARCHAR(255) = 'SQL'
				,@FactoryID	NVARCHAR(255) = 'Test';

		EXECUTE @RC = dbo.sx_pf_DELETE_Factory 
				@Username,
				@FactoryID;

		PRINT @RC;

		SELECT TOP 10 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_Factory','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_DELETE_Factory
	 @Username	NVARCHAR(255)
	,@FactoryID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'
AS 
BEGIN

	DECLARE  @FactoryKey			INT			= 0
			,@TransactUsername	NVARCHAR(255)	= N''
			,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username,		N'NULL')	+ N''',''' 
				+ ISNULL(@FactoryID,	N'NULL')	+ N''''
			,@EffectedRows		INT				= 0				
			,@ResultCode			INT			= 501			
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N'';			

	-- NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryKey	IS NULL SET @FactoryKey = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_DELETE_Factory;
		
		-- Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);	

		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID; 

		IF @FactoryKey = 0
			BEGIN
				SET @ResultCode = 204;
			END
		ELSE
			BEGIN
				-- Delete Factory items from all tables
				DELETE FROM planning.tdProductLines 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdProducts 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdValueSeries 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdTime 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tfStatements 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tfValues 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tgFactories
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tgProductLines 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tgProducts 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tdFactories 
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.trPreferences
				WHERE FactoryID = @FactoryID;

				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tFiles 
				WHERE FactoryKey = @FactoryKey;
			
				SET @EffectedRows += @@ROWCOUNT;

				DELETE FROM planning.tTabs
				WHERE FactoryKey = @FactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- Last, because of Right materialization
				DELETE FROM system.trRights					--therefore EXECUTE AS dbo needed
				WHERE FactoryID = @FactoryID;

				SET @EffectedRows += @@ROWCOUNT;
			END;

		COMMIT TRANSACTION sx_pf_DELETE_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_DELETE_Factory;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_Factory TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_Factory'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Deletes an entire Factory (including ProductLines, Products, Values)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	