
DROP PROCEDURE IF EXISTS dbo.sx_pf_DELETE_ProductDataTableValues;
GO

/*
	DELETE Operation for all Values of the ProductDateTabele

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 09/2022 for OCT 5.8

	Testcall
			DECLARE @RC							INT
			DECLARE @Username			NVARCHAR(255) = 'SQL'
			DECLARE @FactoryID			NVARCHAR(255) = 'ZT'
			DECLARE @ProductLineID		NVARCHAR(255) = 'U'
			DECLARE @ProductID			NVARCHAR(255) = '1'
			DECLARE @DeleteFormulaFlag	INT = 0

		EXECUTE @RC = dbo.sx_pf_DELETE_ProductDataTableValues 
			@Username,
			@FactoryID,
			@ProductLineID,
			@ProductID,
			@DeleteFormulaFlag
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductDataTableValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_DELETE_ProductDataTableValues','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_DELETE_ProductDataTableValues
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@DeleteFormulaFlag	INT
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @ProductKey		INT = 0;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
								+ ISNULL(@Username, N'NULL')								+ N''',''' 
								+ ISNULL(@FactoryID, N'NULL')								+ N''',''' 
								+ ISNULL(@ProductLineID, N'NULL')							+ N''',''' 
								+ ISNULL(@ProductID, N'NULL')								+ N''',''' 
								+ CAST(ISNULL(@DeleteFormulaFlag, N'NULL') AS NVARCHAR(10))	+ N'''';
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N''				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION DELETE_PDTValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@ProductID			= N'' 
			OR @ProductLineID	= N'' 
			OR @FactoryID		= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 1.2 - Determine ProductKey
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID AND ProductID	= @ProductID;

		IF @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 3 - DELETE
		-- If also formulas, we can delete everything
		IF @DeleteFormulaFlag = 1
		BEGIN
			DELETE FROM planning.tfValues 
			WHERE ProductKey = @ProductKey;

			SET @EffectedRows = @@ROWCOUNT;
		END;
		-- If not formulas, only Input Values and update Formula Values to 0/''
		IF @DeleteFormulaFlag = 0
		BEGIN
			-- alle Input Werte komplett
			DELETE fV
			FROM planning.tfValues fV
				LEFT JOIN planning.tdValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey = @ProductKey
				AND dVS.ValueSource IN ('Input','List-Strict','Link');

			-- bei Formelbasierten Werten nur den Wert, nicht die Formel
			UPDATE fV
				SET fV.ValueInt = 0
				,fV.ValueText = ''
			FROM planning.tfValues fV
				LEFT JOIN planning.tdValueSeries dVS
					ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE 
				fV.ProductKey	= @ProductKey
				AND dVS.ValueSource IN ('XLS', 'XLS-Strict');

			SET @EffectedRows = @@ROWCOUNT;
		END;

		SET @ResultCode	= IIF(@EffectedRows = 0, 204, 200);

		COMMIT TRANSACTION DELETE_PDTValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT	= ERROR_STATE();
		SET @Comment				= ERROR_MESSAGE();

		ROLLBACK TRANSACTION DELETE_PDTValues;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_DELETE_ProductDataTableValues TO pf_PlanningFactoryUser;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_DELETE_ProductDataTableValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'DELETE Operation for all Values of the ProductDateTabele';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	 SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	 SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	 SET @level2name = N'@DeleteFormulaFlag';
    SET @value      = N'DeleteFormulaFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


	