
DROP PROCEDURE IF EXISTS dbo.sx_pf_EXPORT_Product;
GO

/* Procedure to export one single product, including
	- it values
	- its dependend lists, which have to be merged in the existing lists during import
	- in two styles, one for import as sql script, one for Import over the GUI Import

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9
 
	TestCall
		DECLARE  @RC int
				,@Username			NVARCHAR(255) = 'SQL'
				,@FactoryID			NVARCHAR(255) = 'ZT'
				,@ProductLineID		NVARCHAR(255) = 'U'
				,@ProductID			NVARCHAR(255) = '1';

		EXECUTE @RC = dbo.sx_pf_EXPORT_Product
		   @Username
		  ,@FactoryID
		  ,@ProductLineID
		  ,@ProductID
		 PRINT @RC

	 Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_Product',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_Product','PARAMETER',NULL)
 */

CREATE  PROCEDURE dbo.sx_pf_EXPORT_Product
		@Username			NVARCHAR(255),
		@FactoryID			NVARCHAR(255),
		@ProductLineID		NVARCHAR(255),
		@ProductID			NVARCHAR(255),
		@OutputForSQLServer INT				= 1
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @TransactUsername	NVARCHAR(255)
			,@ProcedureName		NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR (MAX) = N'''' 
								+ ISNULL(@Username			,N'NULL')							+ N''',''' 
								+ ISNULL(@FactoryID			,N'NULL')							+ N''',''' 
								+ ISNULL(@ProductLineID		,N'NULL')							+ N''',''' 
								+ ISNULL(@ProductID			,N'NULL')							+ N''',''' 
								+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N''''
			,@EffectedRows		INTEGER = 0	
			,@ResultCode 		INTEGER = 501
			,@TimestampCall		DATETIME = GETUTCDATE()
			,@Comment			NVARCHAR (2000) = N'';		
	
	-- NULL Protection
	IF @Username			IS NULL SET @Username		= N'';
	IF @FactoryID			IS NULL SET @FactoryID		= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID	= N'';
	IF @ProductID			IS NULL SET @ProductID		= N'';

	BEGIN TRY

		DECLARE @ProductKey		BIGINT = 0;
		DECLARE @ProductLineKey BIGINT = 0;
		DECLARE @Template		NVARCHAR (255) = N'';

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);

		IF @FactoryID = N'' OR @ProductLineID = N''  OR @ProductID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR(N'Transaction user don`t exists', 16, 10);
		END;		

		-- STEP 1.2 - Determine keys
		SELECT @ProductLineKey = ProductLineKey		FROM planning.tdProductLines	 WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey				FROM planning.tdProducts		 WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF @ProductKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Keys don`t exists', 16, 10);
		END;	
	 
		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR(N'Invalid rights', 16, 10);
		END;

		-- STEP 3.1 - Determine dependend objects
		-- Determine Template
		SELECT @Template = Template FROM planning.tdProducts WHERE ProductKey = @ProductKey;

		-- Deterime all ProductLines using this Template
		DECLARE @DependendProductLinesIDs AS TABLE (ProductLineKey BIGINT);
		INSERT INTO @DependendProductLinesIDs (ProductLineKey)
			SELECT DISTINCT ProductLineKey FROM planning.tdProducts WHERE Template = @Template;

		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE ProductKey = @ProductKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT DISTINCT GlobalAttributeSource1	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource2	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource3	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource4	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource5	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource6	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource7	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource8	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource9	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource10	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource11	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource12	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource13	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource14	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource15	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource16	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource17	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource18	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource19	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource20	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource21	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource22	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource23	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource24	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
			UNION ALL SELECT DISTINCT GlobalAttributeSource25	FROM planning.tdProductLines WHERE ProductLineKey IN (SELECT ProductLineKey FROM @DependendProductLinesIDs)
		)									
		INSERT INTO @DependendListIDs (ListID)
			SELECT DISTINCT ValueListID
			FROM lists
			WHERE ValueListID != '';
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (
											 StepNumber INT NOT NULL
											,RowNumber	INT NOT NULL
											,Command	NVARCHAR(MAX));
		
		INSERT INTO @ExportTable 
		VALUES 
			 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			,(0, 1, N'DECLARE @Username			NVARCHAR(255)	= N''SQL''')
			,(0, 1, N'DECLARE @FactoryID		NVARCHAR(255)	= N''' + @FactoryID		+ N'''')
			,(0, 1, N'DECLARE @ProductLineID	NVARCHAR(255)	= N''' + @ProductLineID	+ N'''')
			,(0, 1, N'DECLARE @ProductID		NVARCHAR(255)	= N''' + @ProductID		+ N'''')
			,(0, 1, N'--This ProductID will be deleted during import, if it exists. You should be sure !');

		-- DELETE Product if exists
		INSERT INTO @ExportTable 
		VALUES (1000, 1, N'EXEC dbo.sx_pf_DELETE_Product @Username, @FactoryID, @ProductLineID, @ProductID;');
			
		-- POST Product
		INSERT INTO @ExportTable 
		SELECT 2000, 1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username, @ProductID, @ProductLineID, @FactoryID,N''' 
			+ TimeType									+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (NameLong)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (
					calc.sfUnifyLinebreaks
						(
							CommentUser
						)			
								)						+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentDev)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ResponsiblePerson)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ImageName)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL ([Status])				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (Template)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (TemplateVersion)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute1)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute2)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute3)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute4)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute5)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute6)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute7)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute8)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute9)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute10)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute13)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute14)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute15)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute16)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute17)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute18)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute19)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute20)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute23)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute24)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (GlobalAttribute25)	+ N''';'
		FROM planning.tdProducts WHERE ProductKey = @ProductKey;

		-- POST Product Properties
		INSERT INTO @ExportTable 
		SELECT 3000, 1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username, @ProductID, @ProductLineID, @FactoryID,N''' +
			+ PropertyID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (PropertyName)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (CommentDev)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (Unit)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (ValueText)			+ N''',N'''
			+ CAST(ValueInt AS NVARCHAR(255))			+ N''',N'''
			+ CAST(Scale AS NVARCHAR(255))				+ N''',N'''
			+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''',N'''
			+ FormatID									+ N''';'
		FROM planning.tgProducts
		WHERE  ProductKey = @ProductKey; 

		-- POST Statements -- statements are not exported

		-- POST all ValueSeries as NewValueSeries - N in Parameterlist  needed
		INSERT INTO @ExportTable
		SELECT 4000 
			, ROW_NUMBER()OVER (ORDER BY ProductID,ValueSeriesNo) AS RowNumber
			, N'EXEC dbo.sx_pf_POST_ValueSerie @Username, @ProductID, @ProductLineID, @FactoryID,N''' +
			+ ValueSeriesID									+ N''',N''' 
			+ CAST(ValueSeriesNo	AS NVARCHAR (255))		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (NameShort)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (NameLong)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentUser)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentDev)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ImageName)				+ N''',N''' 
			+ CAST([IsNumeric]		AS NVARCHAR (255))		+ N''',N''' 
			+ CAST(VisibilityLevel	AS NVARCHAR (255))		+ N''',N''' 
			+ ValueSource									+ N''',N''' 
			+ ValueListID									+ N''',N''' 
			+ ValueFormatID									+ N''',N''' 
			+ Unit											+ N''',N''' 
			+ CAST(Scale AS NVARCHAR (255))					+ N''',N''' 
			+ Effect										+ N''',N''' 
			+ EffectParameter								+ N''';'
		FROM planning.tdValueSeries
		WHERE ProductKey = @ProductKey
		ORDER BY
				ProductID
			,ValueSeriesNo; 
		   
		;WITH tms AS (
			SELECT (SELECT N'[(' + CAST(TimeID AS NVARCHAR(10)) + N',0,0,''''' + FormatID + N''''')],'
			FROM planning.tdTime
			WHERE  ProductKey = @ProductKey
			ORDER BY TimeID
			FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS val 
		)
		INSERT INTO @ExportTable
		SELECT 5000, 1, N'EXEC dbo.sx_pf_POST_Timeline @Username, @ProductID, @ProductLineID, @FactoryID,1,N''' + LEFT(val, LEN(val) - 1) + N''';'
		FROM tms
		WHERE NOT val IS NULL;
				  
		-- POST ProductDataTableValues - no N in Array needed !
		;WITH vals AS (
			SELECT (SELECT  N'[(''''' 
				+ fV.ValueSeriesID												+ N''''','  
				+ CAST(fV.TimeID	AS NVARCHAR(255))							+ N',''''' 
				+ fV.ValueFormula												+ N''''',''''' 
				+ CAST(fV.ValueInt	AS NVARCHAR(255))							+ N''''',''''' 
				+ dbo.sx_pf_pMaskSQL (fV.ValueText)								+ N''''',''''' 
				+ dbo.sx_pf_pMaskSQL (calc.sfUnifyLinebreaks(fV.ValueComment))	+ N''''')],'
			FROM planning.tfValues fV LEFT JOIN planning.tdValueSeries dVS 
				ON fV.ValueSeriesKey = dVS.ValueSeriesKey
			WHERE fV.ProductKey = @ProductKey 
			ORDER BY dVS.ValueSeriesNo
			FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS val 
		)
		INSERT INTO @ExportTable
		SELECT 6000, 1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username, @ProductID, @ProductLineID, @FactoryID,0,N'''
				+ LEFT(val, LEN(val) - 1) + N''';' 
		FROM vals
		WHERE NOT val IS NULL;		

		-- Create the lists
		INSERT INTO @ExportTable 
		SELECT 7001, 1, N'EXECUTE dbo.sx_pf_POST_List @Username,N'''
			+ ListID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (NameShort)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL (NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentUser)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (CommentDev)		+ N''',N''' 
			+ Datentyp								+ N''',N''' 
			+ Source								+ N''',N''' 
			+ SourceFormula							+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thLists
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);

		-- Create List Values
		INSERT INTO @ExportTable 
		SELECT 7002, 1, N'EXECUTE dbo.sx_pf_POST_ListValue @Username,0,N'''
			+ ListID								+ N''',' 
			+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
			+ CAST(Scale AS NVARCHAR(255))			+ N',N''' 
			+ dbo.sx_pf_pMaskSQL (ValueText)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL (ValueComment)		+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thListValues
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
			
		;WITH DependendFormats AS (
			-- used in depended lists from global attributes
			SELECT FormatID
			FROM planning.thListValues 
			WHERE ListID IN
				(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
			UNION ALL
			-- used to format ValueSeries 
			SELECT ValueFormatID 
			FROM planning.tdValueSeries 
			WHERE ProductKey = @ProductKey
			UNION ALL
			-- used to format Ranges
			SELECT DISTINCT FormatID
			FROM planning.tgProducts 
			WHERE 
					ProductKey = @ProductKey
				AND PropertyID LIKE N'PDTRange_%'
		) 
		INSERT INTO @ExportTable 
		SELECT 8001, 1, N'EXEC dbo.sx_pf_POST_Format @Username,N'''
			+ FormatID									+ N''',N'''
			+ BackgroundColor							+ N''',N''' 
			+ FontColor									+ N''',N''' 
			+ ValueFormat								+ N''',N''' 
			+ CAST(Bold				AS NVARCHAR(255))	+ N''',N'''
			+ CAST(Italic			AS NVARCHAR(255))	+ N''',N'''
			+ AlignmentHorizontal						+ N''',N'''
			+ AlignmentVertical							+ N''',N'''
			+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToPrevious	AS NVARCHAR(255))	+ N''';'
		FROM planning.thFormats
		WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
		INSERT INTO @ExportTable
		VALUES (9000, 1, N'GO');

		-- Final Value Output ###################################################################################################################
		SELECT Command 
		FROM @ExportTable 
		ORDER BY StepNumber, RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT N'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_EXPORT_Product TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_EXPORT_Product'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to export one single product, including values, dependend lists, which have to be merged in the existing lists during import in two styles: one for import as sql script, one for Import over the GUI Import';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@OutputForSQLServer';
    SET @value      = N'OutputForSQLServer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	