
DROP PROCEDURE IF EXISTS dbo.sx_pf_EXPORT_ProductLine;
GO

/* 
	EXPORT operation of a single productline, including
	- its properties in gtables
	- its global attributes
	- its products
	- it values
	- its dependend lists, which have to be merged in the existing lists during import
	- in two styles, one for import as sql script, one for Import over the GUI Import

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9

	Testcall Procedure direct
		DECLARE @RC INT;
		EXEC @RC = dbo.sx_pf_EXPORT_ProductLine
			 @Username				= N'SQL'
			,@FactoryID				= N'ZT'
			,@ProductLineID			= N'1';
		PRINT @RC;

	Testcall over BCP
		Results must be stored in table, if column width is to big for SSMS or SQLCMD

		0a. Activate Stepnumber for output in Line 753 around and execute prodecedure
		0b. Its difficult to send commands with german Umlaute over cmd, thats why name the database without Umlaute

		1. Execute this to store the results in the table
			IF  EXISTS (SELECT * FROM sys.objects WHERE object_id = OBJECT_ID(N'tmpout_Export') AND type in (N'U'))
			DROP TABLE tmpout_Export
			GO
			CREATE TABLE tmpout_Export (
				StepNumber INT,
				Command NVARCHAR(MAX)
				)
			INSERT INTO tmpout_Export
				EXEC dbo.sx_pf_EXPORT_ProductLine 'SQL','ZT','U',1

		2. Copy this table Content in a file over a .bat file with this command
			bcp.exe "SELECT Command FROM PlanningFactoryCars.tmpout_Export ORDER BY StepNumber" queryout import.sql -T -S localhost -n  -e error.log -c -C65001 

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_EXPORT_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_EXPORT_ProductLine
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@OutputForSQLServer INT			= 1
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @TransactUsername	AS NVARCHAR(255)
			,@ProcedureName		AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	AS NVARCHAR (MAX) = N'''' 
							+ ISNULL(@Username		, N'NULL')								+ N''',''' 
							+ ISNULL(@FactoryID		, N'NULL')								+ N''',''' 
							+ ISNULL(@ProductLineID	, N'NULL')								+ N''',''' 
							+ ISNULL(CAST(@OutputForSQLServer AS NVARCHAR(255)), N'NULL')	+ N''''
			,@EffectedRows		AS INTEGER = 0			
			,@ResultCode		AS INTEGER = 501			
			,@TimestampCall		AS DATETIME = GETUTCDATE()
			,@Comment			AS NVARCHAR (2000) = N''	
	
	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username		= N'';
	IF @FactoryID			IS NULL SET @FactoryID		= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID	= N'';

	BEGIN TRY

		DECLARE @ProductLineKey BIGINT = 0;

		-- Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID(@ProductLineID);

		IF @FactoryID = N'' OR @ProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR(N'Transaction user don`t exists', 16, 10);
		END;		

		-- Determine keys
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryID = @FactoryID AND ProductLineID = @ProductLineID;

		IF @ProductLineKey = 0 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR(N'Keys don`t exists', 16, 10);
		END;	
	
		-- Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR(N'Invalid rights', 16, 10);
		END;

		-- Determine dependend objects
		-- Determine dependend Lists and store in temporary table
		DECLARE @DependendListIDs AS TABLE (ListID NVARCHAR(255));
		
		;WITH lists AS (
			-- MERGE dependend Lists, used in ProductDataTable
			SELECT ValueListID FROM planning.tdValueSeries WHERE ProductLineKey = @ProductLineKey
			-- MERGE dependend Lists, used in Globalattributes
			UNION ALL SELECT GlobalAttributeSource1		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource2		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource3		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource4		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource5		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource6		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource7		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource8		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource9		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource10	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource11	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource12	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource13	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource14	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource15	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource16	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource17	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource18	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource19	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource20	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource21	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource22	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource23	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource24	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
			UNION ALL SELECT GlobalAttributeSource25	FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey
		)									
		INSERT INTO @DependendListIDs (ListID)
		SELECT DISTINCT ValueListID
		FROM lists;
		 
		-- Declare temporary Table for export results
		DECLARE @ExportTable AS TABLE (	 
										 StepNumber INT NOT NULL
										,RowNumber	INT NOT NULL
										,Command	NVARCHAR(MAX)
										);
	
		INSERT INTO @ExportTable 
		VALUES 
			 (0, 1, N'--CONFIG: Adjust variable names manually to fit your needs !')
			,(0, 1, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
			,(0, 1, N'DECLARE @FactoryID NVARCHAR(255)			= ''' + @FactoryID		+ N'''')
			,(0, 1, N'DECLARE @ProductLineID  NVARCHAR(255)		= ''' + @ProductLineID	+ N'''')
			,(0, 1, N'--This ProductLineID will be deleted during import, if it exists. You should be sure !');

		-- DELETE ProductLine if exists
		INSERT INTO @ExportTable 
		VALUES (1000,1, N'EXEC dbo.sx_pf_DELETE_ProductLine @Username, @FactoryID, @ProductLineID;');
				
		-- POST ProductLine
		INSERT INTO @ExportTable 
		SELECT 1001,1, N'EXEC dbo.sx_pf_POST_ProductLine @Username, @ProductLineID, @FactoryID,N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentUser)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)				+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ResponsiblePerson)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ImageName)					+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(DefaultTemplate)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource1)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias1)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource2)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias2)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource3)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias3)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource4)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias4)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource5)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias5)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource6)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias6)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource7)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias7)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource8)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias8)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource9)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias9)		+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource10)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias10)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias11)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias12)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource13)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias13)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource14)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias14)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource15)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias15)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource16)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias16)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource17)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias17)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource18)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias18)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource19)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias19)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource20)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias20)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias21)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias22)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource23)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias23)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource24)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias24)	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeSource25)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttributeAlias25)	+ N''';'
		FROM planning.tdProductLines WHERE ProductLineKey = @ProductLineKey;
					
		-- POST Products
		INSERT INTO @ExportTable
		SELECT 2000,1, N'EXEC dbo.sx_pf_POST_ProductEmpty @Username,''' + ProductID + N''', @ProductLineID, @FactoryID,N''' 
			+ TimeType								+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(
						calc.sfUnifyLinebreaks
							(
								CommentUser
							)
						)							+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ResponsiblePerson) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ImageName)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL([Status])			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(Template)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(TemplateVersion)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute1)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute2)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute3)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute4)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute5)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute6)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute7)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute8)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute9)	+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute10) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute11) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute12) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute13) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute14) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute15) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute16) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute17) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute18) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute19) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute20) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute21) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute22) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute23) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute24) + N''',N''' 
			+ dbo.sx_pf_pMaskSQL(GlobalAttribute25) + N''';'
		FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey;
							
		-- POST Statements 
			--> statements are not exported

		-- POST all ValueSeries as NewValueSeries
		INSERT INTO @ExportTable 
		SELECT 3000 AS Step
			, ROW_NUMBER()OVER (ORDER BY ProductLineID,ProductID,ValueSeriesNo) AS RowNumber
			, N'EXEC dbo.sx_pf_POST_ValueSerie @Username,N''' 
			+ ProductID									+ N''',@ProductLineID, @FactoryID,N''' 
			+ ValueSeriesID								+ N''',N'''
			+ CAST(ValueSeriesNo	AS NVARCHAR (255))	+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ImageName)				+ N''',N'''
			+ CAST([IsNumeric]		AS NVARCHAR (255))	+ N''',N'''
			+ CAST(VisibilityLevel	AS NVARCHAR (255))	+ N''',N'''
			+ ValueSource								+ N''',N'''
			+ ValueListID								+ N''',N'''
			+ ValueFormatID								+ N''',N'''
			+ Unit										+ N''',N'''
			+ CAST(Scale			AS NVARCHAR (255))	+ N''',N'''
			+ Effect									+ N''',N'''
			+ EffectParameter							+ N''';'
		FROM planning.tdValueSeries
		WHERE  ProductLineKey = @ProductLineKey
		ORDER BY 
				ProductLineID
			,ProductID
			,ValueSeriesNo;
				
		-- POST dTime 
		;WITH tms AS (
			SELECT dP.ProductID
				,(SELECT N'[(' + CAST(t.TimeID AS NVARCHAR(10)) + N',0,0,'''''+ FormatID + N''''')],' -- with DeleteFlag, AddValue,FormatID 
				FROM planning.tdTime t
				WHERE t.ProductKey = dP.ProductKey
				ORDER BY t.TimeID
				FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS vals
			FROM planning.tdProducts dP 
			WHERE dP.ProductLineKey = @ProductLineKey 
		)
		INSERT INTO @ExportTable
		SELECT 4000,1, N'EXEC dbo.sx_pf_POST_Timeline @Username, N''' + ProductID + N''', @ProductLineID, @FactoryID,1,N''' + LEFT(vals, LEN(vals) - 1) + N''';'
		FROM tms
		WHERE NOT vals IS NULL;

		-- POST ProductDataTableValues - no N in Array needed
		;WITH tms AS (
			SELECT dP.ProductID
				,(SELECT N'[('''''		+ ValueSeriesID 
						+ N''''','		+ CAST(TimeID AS NVARCHAR(255)) 
						+ N','''''		+ ValueFormula 
						+ N''''','''''	+ CAST(ValueInt AS NVARCHAR(255)) 
						+ N''''','''''	+ dbo.sx_pf_pMaskSQL(ValueText) 
						+ N''''','''''	+ dbo.sx_pf_pMaskSQL(calc.sfUnifyLinebreaks(ValueComment)) + N''''')],' 
				FROM planning.tfValues t
				WHERE t.ProductKey = dP.ProductKey
				FOR XML PATH(''), TYPE).value(N'.[1]', N'NVARCHAR(MAX)') AS vals
			FROM planning.tdProducts dP 
			WHERE dP.ProductLineKey = @ProductLineKey 
		)
		INSERT INTO @ExportTable
		SELECT 5000,1, N'EXEC dbo.sx_pf_POST_ProductDataTableValues @Username,N''' + ProductID + N''', @ProductLineID, @FactoryID,0,N''' + LEFT(vals, LEN(vals) - 1) + N''';'
		FROM tms
		WHERE NOT vals IS NULL;

		-- POST ProductLine Properties
		INSERT INTO @ExportTable 
		SELECT 6000,1, N'EXEC dbo.sx_pf_POST_ProductLineProperty @Username, @ProductLineID, @FactoryID,N''' 
				+ PropertyID								+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
				+ Unit										+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''',N'''
				+ CAST(ValueInt AS NVARCHAR(255))			+ N''',N'''
				+ CAST(Scale AS NVARCHAR(255))				+ N''',N'''
				+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''',N'''
				+ FormatID									+ N''';'
		FROM planning.tgProductLines
		WHERE ProductLineKey = @ProductLineKey; 

		-- POST ProductLine Preferences
		INSERT INTO @ExportTable 
		SELECT 6001,1, N'EXEC dbo.sx_pf_POST_Preference @Username,N'''
				+ UserName
				+ N''',@ProductLineID, @FactoryID,N''' 		
				+ SettingName									+ N''',N'''
				+ CAST(ValueInt				AS NVARCHAR(255))	+ N''',N'''
				+ dbo.sx_pf_pMaskSQL(ValueText)					+ N''';'
		FROM planning.trPreferences
		WHERE 
				FactoryID		= @FactoryID
			AND ProductLineID	= @ProductLineID; 

		-- POST Tabs
		INSERT INTO @ExportTable 
		SELECT 6003
			,Row_Number() OVER (ORDER BY ProductLineID, ProductID, Orderindex) AS Row
			,N'EXEC planning.spPOST_Tab @Username, @FactoryID, @ProductLineID, N''' 
			+ ProductID										+ N''',''' 
			+ TabID											+ N''', '
			+ CAST(Orderindex AS NVARCHAR(255))				+ N',N'''
			+ dbo.sx_pf_pMaskSQL(TabName)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(TabHint)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(TabText)					+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(PresentationCODE)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(DataSource)				+ N''',N'''
			+ CAST(IsVisibleBOOL AS NVARCHAR(255))			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(LayoutJSON)				+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ParameterJSON)				+ N'''; '

		FROM planning.tTabs
		WHERE ProductLineKey = @ProductLineKey; 

		-- POST Product Properties
		INSERT INTO @ExportTable 
		SELECT 6001,1, N'EXEC dbo.sx_pf_POST_ProductProperty @Username,N''' 
			+ ProductID									+ N''',@ProductLineID, @FactoryID,N''' 
			+ PropertyID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(PropertyName)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentUser)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(CommentDev)			+ N''',N'''
			+ Unit										+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(ValueText)				+ N''',N'''
			+ CAST(ValueInt AS NVARCHAR(255))			+ N''',N'''
			+ CAST(Scale AS NVARCHAR(255))				+ N''',N'''
			+ CAST(IsROSystemProperty AS NVARCHAR(255)) + N''',N'''
			+ FormatID									+ N''';'
		FROM planning.tgProducts
		WHERE  ProductLineKey = @ProductLineKey; 
						
		-- Create the lists
		INSERT INTO @ExportTable 
		SELECT 7001,1, N'EXEC dbo.sx_pf_POST_List @Username,N'''
			+ ListID								+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameShort)			+ N''',N'''
			+ dbo.sx_pf_pMaskSQL(NameLong)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentUser)		+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(CommentDev)		+ N''',N''' 
			+ Datentyp								+ N''',N''' 
			+ Source								+ N''',N''' 
			+ SourceFormula							+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thLists
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
					
		-- Create List Values
		INSERT INTO @ExportTable 
		SELECT 7002,1, N'EXEC dbo.sx_pf_POST_ListValue @Username,0,N'''
			+ ListID								+ N''',' 
			+ CAST(ValueInt AS NVARCHAR(255))		+ N',' 
			+ CAST(Scale AS NVARCHAR(255))			+ N',N''' 
			+ dbo.sx_pf_pMaskSQL(ValueText)			+ N''',N''' 
			+ dbo.sx_pf_pMaskSQL(ValueComment)		+ N''',N''' 
			+ FormatID								+ N''';'
		FROM planning.thListValues
		WHERE ListID IN (SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs);
						   
		-- Create Formats
		;WITH DependendFormats AS (
			-- used in depended lists from global attributes
			SELECT FormatID
			FROM planning.thListValues 
			WHERE ListID IN
				(SELECT ListID COLLATE DATABASE_DEFAULT FROM @DependendListIDs)
			UNION ALL
			-- used to format ValueSeries 
			SELECT DISTINCT ValueFormatID 
			FROM planning.tdValueSeries 
			WHERE ProductLineKey = @ProductLineKey
			UNION ALL
			-- used to format Ranges
			SELECT DISTINCT FormatID
			FROM planning.tgProducts 
			WHERE 
					ProductLineKey = @ProductLineKey
				AND PropertyID LIKE N'PDTRange_%'
		) 

		INSERT INTO @ExportTable 
		SELECT 7003,1, N'EXEC dbo.sx_pf_POST_Format @Username,N'''
			+ FormatID									+ N''',N'''
			+ BackgroundColor							+ N''',N''' 
			+ FontColor									+ N''',N''' 
			+ ValueFormat								+ N''',N''' 
			+ CAST(Bold				AS NVARCHAR(255))	+ N''',N'''
			+ CAST(Italic			AS NVARCHAR(255))	+ N''',N'''
			+ AlignmentHorizontal						+ N''',N'''
			+ AlignmentVertical							+ N''',N'''
			+ CAST(TextUnderlined	AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToNext		AS NVARCHAR(255))	+ N''',N'''
			+ CAST(BorderToPrevious AS NVARCHAR(255))	+ N''';'
		FROM planning.thFormats
		WHERE FormatID IN (SELECT DISTINCT FormatID FROM DependendFormats);
					
		INSERT INTO @ExportTable
		VALUES (8000,1, N'GO');

		-- Final Value Output ###################################################################################################################
		SELECT 
			--StepNumber, --activate only for bcp test output
			Command 
		FROM @ExportTable 
		ORDER BY StepNumber,RowNumber;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_EXPORT_ProductLine TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_EXPORT_ProductLine'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'EXPORT operation of a single productline, including,properties in gtables, global attributes, products, values, dependend lists, 
	which have to be merged in the existing lists during import in two styles: one for import as sql script, one for Import over the GUI Import';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@OutputForSQLServer';
    SET @value      = N'OutputForSQLServer - no longer in use.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
