
DROP PROCEDURE  IF EXISTS dbo.sx_pf_GET_AllProductListValues;
GO

/*
	GET Operation for all ListValues a product needs for data entry in ProductDataTable
	- contains the GUI defined lists of the products (defined by user c* or solution su*)
	- contains the dynamic system lists (sx* - which are editable by the user)
	- adds interal system lists (sx* - which are not visible on GUI)

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call - Must return Table and Code 200
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_AllProductListValues 
			@Username		 = 'SQL',
			@FactoryID		 = 'ZT',
			@ProductLineID	 = 'U',
			@ProductID		 = '1';

		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_AllProductListValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_AllProductListValues','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_GET_AllProductListValues
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		 N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	 N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,	 N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;						-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';						-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_AllProductListValues;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);

		IF	@FactoryID = N'' 
			OR @ProductLineID = N'' 
			OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16 ,10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403'  
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey = ProductKey FROM planning.tdProducts WHERE FactoryKey = @FactoryKey AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;

		IF	@ProductKey = 0 
			OR @ProductLineKey = 0 
			OR @FactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
		IF @ResultCode <> 200	
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
			
		-- Helper Table with all list values
		;WITH list AS (
			SELECT 
				hL.ListID
				,hL.Datentyp 
				,CAST(ISNULL(hLV.ValueInt	, 0) AS INT) AS ValueInt -- must return int !
                ,CAST(ISNULL(hLV.Scale		, 0) AS INT) AS Scale -- must return int !
				,ISNULL(hLV.ValueText		, N'')		 AS ValueText
				,ISNULL(hLV.ValueComment	, N'')		 AS ValueComment
				,ISNULL(hLV.FormatID		, N'')		 AS FormatID
				,ISNULL(hF.BackgroundColor	, N'')		 AS BackgroundColor
				,ISNULL(hF.FontColor		, N'')		 AS FontColor
				,ISNULL(hF.ValueFormat		, N'')		 AS ValueFormat
			FROM planning.thLists hL 
				LEFT JOIN planning.thListValues hLV	ON hL.ListID = hLV.ListID
				-- for the value
				LEFT JOIN planning.thFormats hF	ON hLV.FormatID = hF.FormatID
				-- filter the lists, which where till 4.0.57 GUI defined and are now internal system lists
			WHERE hL.ListID NOT IN (
				'sxIsNumeric'
				,'sxVisibilityLevel'
				,'sxValueLists'
				,'sxValueSources'
				,'sxValueFormats'
				,'sxScales'
				,'sxValueEffects')
		)
		-- STEP 3.1 Dynamic System Lists
		SELECT *
		FROM list
		WHERE ListID IN ('sxUnits','sxValueEffectParameters')	
		
		UNION ALL
		-- STEP 3.2 Product specific ListSet based on defined ValueLists
		SELECT *
		FROM list
		WHERE ListID IN (
			SELECT DISTINCT ValueListID
			FROM planning.tdValueSeries
			WHERE ProductKey = @ProductKey)

		-- add internal system lists ----------------------------------------------------------
		UNION ALL 
		--sxIsNumeric - static  TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES 
				 ('sxIsNumeric','String',0,1,'0','','','','','')
				,('sxIsNumeric','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL	
		--sxVisibilityLevel - static TODO: Change Datatyp to Integer with next API
		SELECT * 
		FROM (
			VALUES	
				 ('sxVisibilityLevel','String',0,1,'0','','','','','')
				,('sxVisibilityLevel','String',1,1,'1','','','','','')
		) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		UNION ALL			
		--sxValueLists - dynamic List of GUI Lists
		SELECT 
			'sxValueLists'
			,'String'
			,0		AS ValueInt 
            ,1		AS Scale 
			,ListID AS ValueText
			,''		AS ValueComment
			,''		AS FormatID
			,''		AS BackgroundColor
			,''		AS FontColor
			,''		AS ValueFormat
		FROM planning.thLists
			-- Filter the GUI Lists, which are not useful as ValueList
		WHERE ListID NOT IN ('sxValueEffectParameter')

		UNION ALL
		--sxValueSources - static (english strings for all languages
		SELECT * 
		FROM (
			VALUES	
				 ('sxValueSources','String',0,1,'Input'		 ,'','','','','')
				,('sxValueSources','String',0,1,'List-Strict','','','','','')
				,('sxValueSources','String',0,1,'XLS'		 ,'','','','','')
				,('sxValueSources','String',0,1,'XLS-Strict' ,'','','','','')
				,('sxValueSources','String',0,1,'Link'		 ,'','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueFormats - dynamic, list of defined formats
		SELECT 
			 'sxValueFormats'
			,'String'
			,0					AS ValueInt 
            ,1					AS Scale 
			,FormatID			AS ValueText
			,''					AS ValueComment
			,FormatID			AS FormatID
			,BackgroundColor	AS BackgroundColor
			,FontColor			AS FontColor
			,ValueFormat		AS ValueFormat
		FROM planning.thFormats 
		
		UNION ALL
		--sxScales - static - this List is also static defined in GET_ListValues
		SELECT * 
		FROM (
			VALUES	
				 ('sxScales','Integer',1	,1,'','','','','','')
				,('sxScales','Integer',10	,1,'','','','','','')
				,('sxScales','Integer',100	,1,'','','','','','')
				,('sxScales','Integer',1000	,1,'','','','','','')
				,('sxScales','Integer',10000,1,'','','','','','')
			) tmp (ListID, Datentyp, ValueInt, Scale, ValueText, ValueComment, FormatID, BackgroundColor, FontColor, ValueFormat)
		
		UNION ALL
		--sxValueEffects -dynamic, list of defined ValueEffects in gValueEffects
		SELECT 
			 'sxValueEffects'
			,'String'
			,0			AS ValueInt 
            ,1			AS Scale 
			,EffectID	AS ValueText
			,''			AS ValueComment
			,''			AS FormatID
			,''			AS BackgroundColor
			,''			AS FontColor
			,''			AS ValueFormat
		FROM planning.tgValueEffects

		ORDER BY ListID, ValueText, ValueInt;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_AllProductListValues;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_AllProductListValues;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_AllProductListValues TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_AllProductListValues TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_AllProductListValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for all ListValues a product needs for data entry in ProductDataTable 
	containing the GUI defined lists of the products (defined by user c* or solution su*) 
	and the dynamic system lists (sx* - which are editable by the user) 
	and adds interal system lists (sx* - which are not visible on GUI)';

    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
