
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ClusterValidation;
GO

/*
	Operation to check Cluster
	Running tests:
		Lists, which are never used
		ListMembers, which are never used
		Lists which are used but dont exists
		List members which are Used but dont exist
		Formats which are used but dont exist
		Formats which are defined but not used
		count ValueSeries per Product
		count ValuesPer Product
	User must be active to run tests

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call					   
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ClusterValidation 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterValidation',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ClusterValidation','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ClusterValidation
	@Username NVARCHAR(255) 
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @Right				NVARCHAR(255)	= N'';	
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';	
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ClusterValidation;
				
		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);
			  
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
	
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Test whether the user is Active on this cluster
		SELECT @ResultCode = IIF(Status = N'Active', 200, 401) FROM system.trUser WHERE Username = @TransactUsername;

		IF @ResultCode <> 200 
			RAISERROR('User is not active', 16, 10);
		
		-- STEP 3 - Run tests
		DECLARE @Tests AS TABLE (
			N INT IDENTITY(1,1)		NOT NULL, 
			[Name] NVARCHAR(200)	NOT NULL, 
			[Status] NVARCHAR(10)	NOT NULL,
			Result NVARCHAR(MAX)	NOT NULL,
			RowsCount BIGINT		NOT NULL);

-- Lists, which are never used			
		;WITH lst AS (
			SELECT l.ListID
				, l.ListKey
				, l.NameShort AS ListName
			FROM planning.thLists l 
				LEFT JOIN (SELECT * FROM planning.tdValueSeries WHERE ValueListID <> N'') vs ON l.ListID = vs.ValueListID
			WHERE vs.ValueSeriesKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Lists', N'Info', N'List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + ListName + N'''', 1
		FROM lst;

		SET @EffectedRows += @@ROWCOUNT;

-- Lists which are used but dont exists	
		;WITH lst AS (
			SELECT vs.ValueListID	AS ListID
				, vs.NameShort AS VSName
				, vs.ValueSeriesKey AS VSKey
				, vs.ValueSeriesID AS VSID
			FROM (SELECT * FROM planning.tdValueSeries WHERE ValueListID <> N'') vs
				LEFT JOIN planning.thLists  l ON l.ListID = vs.ValueListID
			WHERE l.ListID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists Lists', N'Error', N'ListID ' + ListID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), VSKey) + N', ' + VSID + N' ''' + VSName + N''')', 1
		FROM lst;
		
		SET @EffectedRows += @@ROWCOUNT;			  

-- ListMembers, which are never used		
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), lv.ListValueKey) + N' ''' 
				+ CASE WHEN l.Datentyp = N'INT' THEN CONVERT(NVARCHAR(MAX), lv.ValueInt) ELSE lv.ValueText END 
				+ N''' (List #' + CONVERT(NVARCHAR(MAX), l.ListKey) + N', ' + lv.ListID + N' ''' + l.NameShort + N''')' AS Result
			FROM planning.thListValues lv 
				LEFT JOIN planning.thLists l ON lv.ListID = l.ListID							
				LEFT JOIN planning.tdValueSeries vs ON l.ListID = vs.ValueListID AND ((vs.[IsNumeric] = 1 AND l.Datentyp = N'INT') OR (vs.[IsNumeric] = 0 AND l.Datentyp = N'String'))
				LEFT JOIN planning.tfValues v ON vs.ValueSeriesKey = v.ValueSeriesKey
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE v.ValueKey IS NULL
		 )
		 INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		 SELECT N'Unused List Values', N'Info', Result, 1
		 FROM mmbr;

		 SET @EffectedRows += @@ROWCOUNT;

--  List members which are Used but dont exist
		;WITH mmbr AS (
			SELECT DISTINCT N'Value #' + CONVERT(NVARCHAR(MAX), v.ValueKey) + N' ''' + CASE WHEN vs.IsNumeric = 1 THEN CONVERT(NVARCHAR(MAX), v.ValueInt) ELSE v.ValueText END
				+ N''' ListID ' + vs.ValueListID 
				+ N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), v.ValueSeriesKey) + N', ' + vs.ValueSeriesID
				+ N' ''' + vs.NameShort + N''')' AS Result
			FROM planning.tfValues v
				INNER JOIN planning.tdValueSeries vs ON vs.ValueListID <> N'' AND vs.ValueSeriesKey = v.ValueSeriesKey
				LEFT JOIN planning.thLists l ON vs.ValueListID = l.ListID AND ((vs.IsNumeric = 1 AND l.Datentyp = N'INT') OR (vs.IsNumeric = 0 AND l.Datentyp = N'String'))
				LEFT JOIN planning.thListValues lv ON lv.ListID = l.ListID
					AND ((vs.[IsNumeric] = 1 AND v.ValueInt = lv.ValueInt)
						OR (vs.[IsNumeric] = 0 AND v.ValueText = lv.ValueText))
			WHERE lv.ListValueKey IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Don`t exists List Values', N'Error', Result, 1
		FROM mmbr;

		SET @EffectedRows += @@ROWCOUNT;

		;WITH frm AS (									 
			SELECT ValueFormatID AS FormatID , N'FormatID ' + ValueFormatID + N' (ValueSerie #' + CONVERT(NVARCHAR(MAX), ValueSeriesKey) + N', ' + ValueSeriesID + N' ''' + NameShort + N''')' AS Result			 
			FROM planning.tdValueSeries
			WHERE ValueFormatID <> N''
			UNION ALL												   
			SELECT FormatID , N'FormatID ' + FormatID + N' (Cluster Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM planning.tgCluster
			WHERE FormatID <> N'' 								  
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Factory Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result								   
			FROM planning.tgFactories
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ProductLine Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result							  		
			FROM planning.tgProductLines
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (Product Property #' + CONVERT(NVARCHAR(MAX), PropertyKey) + N', ' + PropertyID + N' ''' + PropertyName + N''')' AS Result
			FROM planning.tgProducts
			WHERE FormatID <> N'' 
			UNION ALL										 
			SELECT FormatID , N'FormatID ' + FormatID + N' (List #' + CONVERT(NVARCHAR(MAX), ListKey) + N', ' + ListID + N' ''' + NameShort + N''')' AS Result
			FROM planning.thLists
			WHERE FormatID <> N'' 
			UNION ALL
			SELECT FormatID , N'FormatID ' + FormatID + N' (ListValue #' + CONVERT(NVARCHAR(MAX), ListValueKey) + N', ListID ' + ListID + N')' AS Result
			FROM planning.thListValues
			WHERE FormatID <> N'' 
		)
	-- Formats which are used but don`t exist
		, unusedFrmts AS (
			SELECT N'Format #' + CONVERT(NVARCHAR(MAX), FormatKey) + N', ' + FormatID AS Result												
			FROM planning.thFormats
			WHERE FormatID NOT IN (SELECT DISTINCT FormatID FROM frm)
		)
	-- Formats which are defined but not used	
		, notExistsFrmts AS (
			SELECT Result 
			FROM frm LEFT JOIN planning.thFormats f ON frm.FormatID	= f.FormatID
			WHERE f.FormatID IS NULL
		)
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Unused Formats', N'Info', Result, 1
		FROM unusedFrmts
		UNION ALL
		SELECT N'Don`t exists Formats', N'Error', Result, 1
		FROM notExistsFrmts;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValueSeries per Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'ValueSeries per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM planning.tdValueSeries	vs INNER JOIN planning.tdProducts p ON vs.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;

		SET @EffectedRows += @@ROWCOUNT;

--    count ValuesPer Product		
		INSERT INTO @Tests([Name], [Status], Result, RowsCount)
		SELECT N'Values per Product', N'Info', N'Product #' + CONVERT(NVARCHAR(MAX), p.ProductKey) + N', ' + p.ProductID + N' ''' + p.NameShort + N'''', COUNT(*)
		FROM planning.tfValues v INNER JOIN planning.tdProducts p ON v.ProductKey = p.ProductKey					
		GROUP BY p.ProductKey, p.ProductID, p.NameShort;								   	   
		
		SET @EffectedRows += @@ROWCOUNT;

		SELECT * 
		FROM @Tests
		ORDER BY N;

		COMMIT TRANSACTION sx_pf_GET_ClusterValidation;		
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ClusterValidation;

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ClusterValidation TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ClusterValidation TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ClusterValidation'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Operation to check Cluster
	Running tests:
    Lists, which are never used
    ListMembers, which are never used
    Lists which are used but dont exists
    List members which are Used but dont exist
    Formats which are used but dont exist
    Formats which are defined but not used
    count ValueSeries per Product
    count ValuesPer Product
	User must be active to run tests';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
   
	GO
