
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_GlobalSearch;
GO

/*
	GET Operation for receiving full text search results for a given textstring
	Search will be done in all DataFactory Elements

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC AS NVARCHAR(255)
		--should return Data and 200
		EXEC @RC = dbo.sx_pf_GET_GlobalSearch 'SQL','whixc'
		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_GlobalSearch',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_GlobalSearch','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_GlobalSearch
	 @Username				NVARCHAR(255)
	,@SearchString			NVARCHAR(255)  -- any Substring from searched list values
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @SearchDoneFlag			INT				= 0;

	-- Logging Block
	DECLARE @TransactUsername		NVARCHAR(255)	= N'';
	DECLARE @ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @EffectedRows			INT				= 0;				-- SET during Execution
	DECLARE @ResultCode				INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall			DATETIME		= GETUTCDATE();
	DECLARE @Comment				NVARCHAR(2000)  = N'';				-- SET during Execution

	DECLARE @ParameterString		NVARCHAR(MAX)	= N''''    
			+ ISNULL(@Username,					N'NULL') + N''',''' 
			+ ISNULL(@SearchString,				N'NULL') + N'''';  

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username				= N'';
	
	
	-- add wildcards to SeachString
	SET @SearchString = '%' + ISNULL(@SearchString, '') + '%';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_GlobalSearch;

		-- Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString	(@Username);
		
				
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Table for Structure Cache
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#Structure') IS NOT NULL
				DROP TABLE #Structure;

			CREATE TABLE #Structure
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryKey					BIGINT											 NOT NULL
				,ProductLineKey				BIGINT											 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,ValueSeriesKey				BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,FactoryNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductNameShort			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesNameShort		NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #Structure
			-- must also return Factories / ProductLines without Products
				SELECT
					  dF.FactoryKey
					 ,ISNULL(dPL.ProductLineKey	,-1)					AS ProductLineKey
					 ,ISNULL(dP.ProductKey		,-1)					AS ProductKey
					 ,ISNULL(dVS.ValueSeriesKey	,-1)					AS ValueSeriesKey
					 ,dF.FactoryID		
					 ,ISNULL(dPL.ProductLineID	,'-1')					AS ProductLineID
					 ,ISNULL(dP.ProductID		,'-1')					AS ProductID
					 ,ISNULL(dVS.ValueSeriesID	,'-1')					AS ValueSeriesID
					 ,dF.NameShort										AS FactoryNameShort
					 ,ISNULL(dPL.NameShort	,'<without ProductLine>')	AS ProductLineNameShort
					 ,ISNULL(dP.NameShort	,'<without Product>')		AS ProductNameShort
					 ,ISNULL(dVS.NameShort	,'<without ValueSeries>')	AS ValueSeriesNameShort

			  FROM  planning.tdFactories dF

					LEFT JOIN planning.tdProductLines dPL
						ON dF.FactoryKey = dPL.FactoryKey

					LEFT JOIN planning.tdProducts dP
						ON dPL.ProductLineKey = dP.ProductLineKey

					LEFT JOIN planning.tdValueSeries dVS
						ON dP.ProductKey = dVS.ProductKey
				
			
		-- Helper Table for Globalattributes
		-- ****************************************************************************************************************************************************
			IF OBJECT_ID('tempdb..#GA') IS NOT NULL
				DROP TABLE #GA;

			CREATE TABLE #GA
			(
				 RowKey						BIGINT IDENTITY (1,1) 							 NOT NULL
				,ProductKey					BIGINT											 NOT NULL
				,FactoryID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Globalattribute			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Content					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)

			INSERT INTO #GA
				SELECT 
						 ProductKey
						,FactoryID
						,ProductLineID
						,ProductID
						,UnpivotTable.Globalattribute
						,UnpivotTable.Content

				FROM
					planning.tdProducts
		
				UNPIVOT
					(	

						Content FOR Globalattribute IN (
													 Globalattribute1,Globalattribute2,Globalattribute3,Globalattribute4,Globalattribute5
													,Globalattribute6,Globalattribute7,Globalattribute8,Globalattribute9,Globalattribute10
													,Globalattribute11,Globalattribute12,Globalattribute13,Globalattribute14,Globalattribute15
													,Globalattribute16,Globalattribute17,Globalattribute18,Globalattribute19,Globalattribute20
													,Globalattribute21,Globalattribute22,Globalattribute23,Globalattribute24,Globalattribute25
												)
					) AS UnpivotTable

				WHERE Content <>''


		-- Table for Search Result Collection
		-- ****************************************************************************************************************************************************
		IF OBJECT_ID('tempdb..#SearchResult') IS NOT NULL
				DROP TABLE #SearchResult;

			CREATE TABLE #SearchResult
			(
				 RowKey					BIGINT IDENTITY (1,1) 							 NOT NULL
				,FactoryID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,TimeID					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resultgroup			NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,Resulttype				NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
				,DataFactoryPath		NVARCHAR (1000)			COLLATE DATABASE_DEFAULT NOT NULL
				,Result					NVARCHAR (255)			COLLATE DATABASE_DEFAULT NOT NULL
			)
		-- Getting Content
		-- ****************************************************************************************************************************************************
		
		-- Factories BEGIN #####

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'FactoryID'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.FactoryID
			FROM planning.tdFactories dF
			WHERE dF.FactoryID LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'NameShort'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.NameShort
			FROM planning.tdFactories dF
			WHERE dF.NameShort LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		INSERT INTO #SearchResult
			SELECT dF.FactoryID
			      ,''
				  ,''
				  ,''
				  ,''
				  ,'Factory'
				  ,'ResponsiblePerson'
				  ,dF.NameShort + ' (' + dF.FactoryID + ')' AS Path
				  ,dF.ResponsiblePerson
			FROM planning.tdFactories dF
			WHERE dF.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- Factories END #####

		-- ProductLines BEGIN #####
		
		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'ProductLine'
				  ,'ProductLineID'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ProductLineID
			FROM planning.tdProductLines dPL
					LEFT JOIN (
								SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
							   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ProductLineID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'ProductLine'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.NameShort
			FROM planning.tdProductLines dPL
					LEFT JOIN (
									SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
								   ) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dPL.FactoryID
			      ,dPL.ProductLineID
				  ,''
				  ,''
				  ,''
				  ,'ProductLine'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + dPL.NameShort + ' (' + dPL.FactoryID + '>'+dPL.ProductLineID +')' AS Path
				  ,dPL.ResponsiblePerson
			FROM planning.tdProductLines dPL
					LEFT JOIN (
						SELECT DISTINCT FactoryKey,FactoryNameShort FROM #Structure	
						) S ON dPL.FactoryKey = S.FactoryKey
			WHERE dPL.ResponsiblePerson LIKE @SearchString
		
		SET @EffectedRows += @@ROWCOUNT;

		-- ProductLines END #####

		-- Products BEGIN ####
		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ProductID'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ProductID
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.ProductID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.NameShort
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'ResponsiblePerson'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,dP.ResponsiblePerson
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.ResponsiblePerson LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'CommmentUser'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,LEFT(dP.CommentUser,250)+' ...'
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
			WHERE dP.CommentUser LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dP.FactoryID
			      ,dP.ProductLineID
				  ,dP.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,GA.Globalattribute
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ dP.NameShort + ' (' + dP.FactoryID + '>'+dP.ProductLineID + '>' + dP.ProductID + ')' AS Path
				  ,GA.Content
			FROM planning.tdProducts dP
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort FROM #Structure	
							   ) S ON dP.ProductLineKey = S.ProductLineKey
					LEFT JOIN #GA GA
						ON dP.ProductKey = GA.ProductKey
			WHERE GA.Content LIKE @SearchString

		-- Documents on Products
		INSERT INTO #SearchResult
			SELECT tF.FactoryID
			      ,tF.ProductLineID
				  ,tF.ProductID
				  ,''
				  ,''
				  ,'Product'
				  ,'Documents'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + ' (' + tF.FactoryID + '>'+tF.ProductLineID + '>' + tF.ProductID + ')' AS Path
				  ,tF.[FileName]
			FROM planning.tFiles tF
					LEFT JOIN (
								SELECT DISTINCT ProductLineKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON tF.ProductLineKey = S.ProductLineKey
			WHERE tf.[FileName] LIKE @SearchString


		-- Products END ####

		-- ValueSeries Start ####
		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'ValueSeriesID'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.ValueSeriesID
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.ValueSeriesID LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameShort'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameShort
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameShort LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'NameLong'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,dVS.NameLong
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.NameLong LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT dVS.FactoryID
			      ,dVS.ProductLineID
				  ,dVS.ProductID
				  ,dVS.ValueSeriesID
				  ,''
				  ,'Product'
				  ,'CommentUser'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + dVS.NameShort + ' (' + dVS.FactoryID + '>'+dVS.ProductLineID + '>' + dVS.ProductID + '>' + dVS.ValueSeriesID + ')' AS Path
				  ,LEFT(dVS.CommentUser,250)+' ...'
			FROM planning.tdValueSeries dVS
					LEFT JOIN (
								SELECT DISTINCT ProductKey,FactoryNameShort,ProductLineNameShort,ProductNameShort FROM #Structure	
							   ) S ON dVS.ProductKey = S.ProductKey
			WHERE dVS.CommentUser LIKE @SearchString

		-- ValueSeries END ####

		-- Value Start ###
		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueText'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueText,250)+' ...'
			FROM planning.tfValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductLineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueText LIKE @SearchString

		INSERT INTO #SearchResult
			SELECT fV.FactoryID
			      ,fV.ProductLineID
				  ,fV.ProductID
				  ,fV.ValueSeriesID
				  ,fV.TimeID
				  ,'Values'
				  ,'ValueComment'
				  ,S.FactoryNameShort + '>' + S.ProductLineNameShort +'>'+ S.ProductNameShort + '>' + S.ValueSeriesNameShort + '>' + CAST(fV.TimeID AS NVARCHAR(255)) +' (' + fV.FactoryID + '>'+ fV.ProductLineID + '>' + fV.ProductID + '>' + fV.ValueSeriesID + '>' + CAST(fV.TimeID AS NVARCHAR(255)) + ')' AS Path
				  ,LEFT(fV.ValueComment,250)+' ...'
			FROM planning.tfValues fV
					LEFT JOIN (
								SELECT DISTINCT ValueSeriesKey,FactoryNameShort,ProductLineNameShort,ProductNameShort,ValueSeriesNameShort FROM #Structure	
							   ) S ON fV.ValueSeriesKey = S.ValueSeriesKey
			WHERE fV.ValueComment LIKE @SearchString

			
		-- Output

		SELECT TOP (100)
			   sr.FactoryID
		      ,sr.ProductLineID
			  ,sr.ProductID
			  ,sr.ValueSeriesID
			  ,sr.Resultgroup
			  ,sr.Resulttype
			  ,sr.DataFactoryPath
			  ,sr.Result
		FROM #SearchResult sr

			INNER JOIN 
				(	
					SELECT *
					FROM system.trUserRights 
					WHERE 
						Username = @TransactUsername AND
						[Right] IN ('Write','Read') AND
						(
							FactoryID <> '' AND ProductLineID <> ''
							OR
							FactoryID <> '' AND ProductLineID = ''
						)
				) vUR

				ON 
						sr.FactoryID = vUR.FactoryID 
					AND (sr.ProductLineID = vUR.ProductLineID OR sr.ProductLineID = '-1') -- as Output for not existing ProductLine possible

		ORDER BY CASE sr.Resultgroup 
						WHEN 'Factory'		THEN 1
						WHEN 'ProductLine'	THEN 2
						WHEN 'Product'		THEN 3
						WHEN 'ValueSeries'	THEN 4
						WHEN 'Values'		THEN 5
						ELSE 99
				 END
					
				,sr.Resulttype,sr.FactoryID,sr.ProductLineID,sr.ProductID,sr.ValueSeriesID

		SET @SearchDoneFlag = 1

		SET @ResultCode = 200

		COMMIT TRANSACTION sx_pf_GET_GlobalSearch;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_GlobalSearch;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_GlobalSearch TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_GlobalSearch TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_GlobalSearch'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for receiving full text search results for a given textstring
				 Search will be done in all DataFactory Elements';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SearchString';
    SET @value      = N'SearchString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
