
DROP PROCEDURE IF EXISTS sx_pf_GET_HTMLPage;
GO

/*
	GET Operation for HTML Page
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = sx_pf_GET_HTMLPage 
			@Username		= 'W10\Admin',
			@FactoryID		= '-1', 
			@ProductLineID	= '-1', 
			@ProductID		= '-1',
			@PageID			= 'Page1'
		PRINT @RC

		SELECT * FROM planning.tgHTMLPages

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_HTMLPage',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_HTMLPage','PARAMETER',NULL)
*/

CREATE PROCEDURE sx_pf_GET_HTMLPage
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),
	@PageID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @PageKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	=  N''''
			 + ISNULL(@Username,		N'NULL') + N''','''
			 + ISNULL(@FactoryID,		N'NULL') + N''',''' 
			 + ISNULL(@ProductLineID,	N'NULL') + N''',''' 
			 + ISNULL(@ProductID,		N'NULL') + N''',''' 
			 + ISNULL(@PageID,			N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @PageID			IS NULL SET @PageID			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_HTMLPage;

		-- STEP 0.2 - Protect input parameters
		SET @Username		= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);	
		SET @ProductID		= dbo.sx_pf_pProtectID		(@ProductID);
		SET @PageID			= dbo.sx_pf_pProtectString	(@PageID);

		IF @PageID		= N'' 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterPropertiesReadRight @TransactUsername;
		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END 


		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;
		
		SELECT @PageKey = PageKey FROM planning.tgHTMLPages
		WHERE
				FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND PageID			= @PageID

		IF @PageKey = 0 	
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Keys don`t exists', 16, 10);
			END;
			
		-- Check rights on different levels 
			-- Cluster Pages are never Readprotected

		-- FactoryPage
		IF @FactoryKey > 0 AND @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_FactoryReadRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- ProductPage
		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Show public values
		SELECT  
			 FactoryID
			,ProductLineID
			,ProductID
			,PageID
			,PageName
			,PageContent
		
		FROM planning.tgHTMLPages 
		WHERE PageKey = @PageKey;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_GET_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::sx_pf_GET_HTMLPage TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::sx_pf_GET_HTMLPage TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_HTMLPage'  -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for HTML Page-only used up to Version 5.4.2, than replaced by planning.tTabs';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@PageID';
    SET @value      = N'PageID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	
