
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Info_CustomFailureReport;
GO

/*
	GET Operation for CustomFailureReport
	- Can be called as Factory Tab
	- Can be called on ZT Factory Tab for all Factories with rights
	- Returns always at least one row

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_Info_CustomFailureReport 'SQL','ZT'
		PRINT @RC
	
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_CustomFailureReport',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Info_CustomFailureReport','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Info_CustomFailureReport
	@Username NVARCHAR(255), @FactoryID NVARCHAR(255)='', @ProductLineID NVARCHAR(255)=''

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL')
														+ ISNULL(@FactoryID, N'NULL')
														+ ISNULL(@ProductLineID, N'NULL')
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';

	IF @FactoryID		= 'ZT' SET @FactoryID = ''

	BEGIN TRY
		BEGIN TRANSACTION GET_Info_CustomFailureReport;

		-- STEP 0.2 - Protect input parameters
		SET @Username = dbo.sx_pf_pProtectString (@Username);

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights via JOIN and output values
		SELECT 
             CFR.Timestamp								AS LastCheckDate
            ,CFR.FactoryID
			,COALESCE(
						 CFR.FactoryID + ' ' + dF.NameShort
						,'All Factories'
					 )									AS FactoryIDName
            ,CFR.ProductLineID
			,COALESCE(	
						 CFR.ProductLineID + ' ' + dPL.NameShort
						,'All ProductLines'
					 )
														AS ProductLineIDName
            ,CFR.ProductID
			,COALESCE(
						 CFR.ProductID + ' ' + dP.NameShort
						,'All Products'
					 )	AS ProductIDName
			,CFR.FactoryID + ' / ' + CFR.ProductLineID +' / ' + CFR.ProductID AS DataFactoryPath
            ,CFR.TestSource
            ,CFR.TestName
			,CFR.ResultType
            ,CFR.ResultText
            ,CFR.ResultNumeric
		FROM control.tCustomFailureReport CFR
			-- Right Filter with addition of dummy rows for All* Values
			INNER JOIN 
						(
							SELECT UserName, FactoryID, ProductLineID FROM system.trUserRights 
							UNION ALL 
							SELECT UserName, 'AllFID', 'AllPLID' FROM system.trUserRights GROUP BY UserName
							UNION ALL 
							SELECT UserName, 'ErrorFID', 'ErrorPLID' FROM system.trUserRights GROUP BY UserName
						) vUR 

				ON	CFR.FactoryID		= vUR.FactoryID
				AND CFR.ProductLineID	= vUR.ProductLineID
				AND vUR.UserName		= @TransactUsername -- AND Right IN (...) not necessary as only positiv Rights saved
				
			-- for Factory Name
			LEFT JOIN planning.tdFactories dF 
				ON	CFR.FactoryID		= dF.FactoryID

			-- for ProductLine Name
			LEFT JOIN planning.tdProductLines dPL
				ON	CFR.FactoryID		= dPL.FactoryID
				AND CFR.ProductLineID	= dPL.ProductLineID

			-- for Product Name
			LEFT JOIN planning.tdProducts dP
				ON	CFR.FactoryID		= dP.FactoryID
				AND CFR.ProductLineID	= dP.ProductLineID
				AND CFR.ProductID		= dP.ProductID

		WHERE
			-- only last Check Results
			CFR.Timestamp = (
								SELECT MAX(Timestamp) FROM control.tCustomFailureReport
							)
			-- called for one or all Factories
			AND (
					CFR.FactoryID = @FactoryID OR @FactoryID = '' OR CFR.FactoryID IN ('AllFID','ERRORFID')
				)

		SET @EffectedRows = @@ROWCOUNT;

		IF @EffectedRows = 0
			BEGIN
					SELECT 
					  GETUTCDATE()					AS LastCheckDate
					,'All'							AS FactoryID
					,'All Factories'				AS FactoryIDName
					,'All'							AS ProductLineID
					,'All ProductLines'				AS ProductLineIDName
					,'All'							AS ProductID
					,'All Products'					AS ProductIDName
					,'All / All / All'				AS DataFactoryPath
					,'Hint'							AS TestSource
					,'Default Entry'				AS Testname
					,'OK'							AS ResultType
					,'Keine Einträge vorhanden.'	AS ResultText
					,0								AS ResultNumeric
			END
			

		SET @ResultCode = 200;

		COMMIT TRANSACTION GET_Info_CustomFailureReport;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Info_CustomFailureReport;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Info_CustomFailureReport TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Info_CustomFailureReport'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for CustomFailureReport
- Can be called as Factory Tab
- Can be called on ZT Factory Tab for all Factories with rights
- Returns always at least one row';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO
