
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_Preference;
GO

/*
	GET Operation for Prefences
	- Preferences may exist for the Cluster, for Factories or for ProductLines - all in the same table
	- Preferences may be userdepended or not, if not they are saved for the PreferenceUserName "All"

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty FactoryID or empty ProductLineID => 404 Not Found
	2. Username non Cluster Admin => 403 Forbidden
	3. User does not have rights to read ProductLine => 401 Unauthorized
	5. No exists Factory / ProductLine > 404 Not Found
		DECLARE  @RC					INT
				,@Username				NVARCHAR(255) = 'SQL'
				,@PreferenceUserName	NVARCHAR(255) = 'W10\admin'
				,@ProductLineID			NVARCHAR(255) = 'U'
				,@FactoryID				NVARCHAR(255) = 'ZT'
				,@SettingName			NVARCHAR(255) = 'Demopref'

		EXECUTE @RC = dbo.sx_pf_GET_Preference 
				@Username,
				@PreferenceUsername,
				@ProductLineID,
				@FactoryID,
				@SettingName

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Preference',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_Preference','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_Preference
	 @Username				NVARCHAR(255)
	,@PreferenceUsername	NVARCHAR(255)
	,@ProductLineID			NVARCHAR(255)
	,@FactoryID				NVARCHAR(255)
	,@SettingName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE  @TransactUsername					NVARCHAR(255)= N''
			,@FactoryKey						INT = 0
			,@ProductLineKey					INT = 0
			,@UserExistsFlag					INT = 0
			,@PreferenceIsUserDependendFlag		INT = 0
			,@ProcedureName						NVARCHAR(255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			,@ParameterString					NVARCHAR(MAX) = N''''
							+ ISNULL(@Username, N'NULL')			+ N''',''' 
							+ ISNULL(@PreferenceUsername, N'NULL')	+ N''',''' 
							+ ISNULL(@ProductLineID, N'NULL')		+ N''',''' 
							+ ISNULL(@FactoryID, N'NULL')			+ N''','''
							+ ISNULL(@SettingName, N'NULL')			+ N''''
			
			,@EffectedRows						INT = 0					
			,@ResultCode 						INT = 501						
			,@TimestampCall						DATETIME = GETUTCDATE()
			,@Comment							NVARCHAR(2000) = N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';

	BEGIN TRY
		BEGIN TRANSACTION GET_Preference;

		-- STEP 0.2 - Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID			= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @ProductLineID		= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @PreferenceUsername = dbo.sx_pf_pProtectString	(@PreferenceUsername);
		SET @SettingName		= dbo.sx_pf_pProtectString	(@SettingName);

		IF	@PreferenceUsername = N'' 
			OR @SettingName		= N'' 
			OR @SettingName		= N'<#NV>'	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.3 - Determine, if Preferences are saved per user

		-- for Clusterpreferences
		IF @FactoryID = N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgCluster WHERE PropertyID = N'C06';
			END 
		-- for Factorypreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID = N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgFactories WHERE PropertyID = N'F06';
			END
		-- for ProductLinePreferences
		ELSE IF @FactoryID <> N'' AND @ProductLineID <> N''
			BEGIN 
				SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgProductLines WHERE PropertyID = N'P06';
			END;
		
		-- STEP 1.1 - Determine Keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		IF (@FactoryKey = 0 AND @FactoryID <> N'')
			OR (@ProductLineKey = 0 AND @ProductLineID <> N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Change PreferenceUser if Preferences User-independend
		IF @PreferenceIsUserDependendFlag <> 1
		BEGIN 
			SET @PreferenceUsername = N'All';
		END;

		--- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights - TODO: depdendend from preference type
		EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN
			RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 3. Show public values
		SELECT 
			 UserName
			,FactoryID
			,ProductLineID
			,SettingName
			,ValueInt
			,ValueText
		FROM planning.trPreferences 
		WHERE 
				FactoryID		= @FactoryID 
			AND ProductLineID	= @ProductLineID 
			AND Username		= @PreferenceUsername 
			AND SettingName		= @SettingName;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_Preference TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_Preference TO pf_PlanningFactoryUser;
GO



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_Preference'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for Prefences
				- Preferences may exist for the Cluster, for Factories or for ProductLines - all in the same table
				- Preferences may be userdepended or not, if not they are saved for the PreferenceUserName "All"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PreferenceUsername';
    SET @value      = N'PreferenceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SettingName';
    SET @value      = N'SettingName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	