
DROP PROCEDURE IF EXISTS dbo.sx_pf_GET_ProductLines;
GO

/*
	GET Operation for ProductLines 
		A: returns the ProductLines for a single Factory
		B: returns all ProductLines when Factory '' is requested

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE @RC NVARCHAR(255)
		EXEC @RC = dbo.sx_pf_GET_ProductLines 
			@Username	= 'SQL',
			@FactoryID	= 'ZT'
		PRINT @RC

		EXEC @RC = dbo.sx_pf_GET_ProductLines 
			@Username = 'SQL'
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLines',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_GET_ProductLines','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_GET_ProductLines
	@Username	NVARCHAR(255),
	@FactoryID	NVARCHAR(255) = ''
AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'' ;
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,	N'NULL')	+ N'''';
	DECLARE @EffectedRows		INT				= 0;				
	DECLARE @ResultCode			INT				= 501;				
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				

	-- STEP 0.1 - NULL Protection
	IF @Username	IS NULL SET @Username	= N'';
	IF @FactoryID	IS NULL SET @FactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_GET_ProductLines;

		-- STEP 0.2 - Protect input parameters
		SET @Username	= dbo.sx_pf_pProtectString	(@Username);
		SET @FactoryID	= dbo.sx_pf_pProtectID		(@FactoryID);
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys if single Factory is requested
		IF @FactoryID <> ''
			BEGIN
				SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

				IF @FactoryKey = 0	
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Keys don`t exists', 16, 10);
				END;
			END

		-- STEP 3. Show public values	
		;WITH vUR AS (
			SELECT 
				FactoryID, 
				ProductLineID, 
				[Right]
			FROM system.trUserRights 
			WHERE ProductLineID <> N'' 
				AND Username = @TransactUsername 
				AND	[Right] IN (N'Read', N'Write')
		)
		SELECT  
			 dPL.ProductLineID 
			,dPL.FactoryID
			,dPL.NameShort
			,dPL.NameLong
			,dPL.CommentUser
			,dPL.CommentDev
			,dPL.ResponsiblePerson
			,dPL.ImageName
			,dPL.DefaultTemplate
			,dPL.GlobalAttributeSource1
			,dPL.GlobalAttributeAlias1
			,dPL.GlobalAttributeSource2
			,dPL.GlobalAttributeAlias2
			,dPL.GlobalAttributeSource3
			,dPL.GlobalAttributeAlias3
			,dPL.GlobalAttributeSource4
			,dPL.GlobalAttributeAlias4
			,dPL.GlobalAttributeSource5
			,dPL.GlobalAttributeAlias5
			,dPL.GlobalAttributeSource6
			,dPL.GlobalAttributeAlias6
			,dPL.GlobalAttributeSource7
			,dPL.GlobalAttributeAlias7
			,dPL.GlobalAttributeSource8
			,dPL.GlobalAttributeAlias8
			,dPL.GlobalAttributeSource9
			,dPL.GlobalAttributeAlias9
			,dPL.GlobalAttributeSource10
			,dPL.GlobalAttributeAlias10

			,dPL.GlobalAttributeSource11
			,dPL.GlobalAttributeAlias11
			,dPL.GlobalAttributeSource12
			,dPL.GlobalAttributeAlias12
			,dPL.GlobalAttributeSource13
			,dPL.GlobalAttributeAlias13
			,dPL.GlobalAttributeSource14
			,dPL.GlobalAttributeAlias14
			,dPL.GlobalAttributeSource15
			,dPL.GlobalAttributeAlias15
			,dPL.GlobalAttributeSource16
			,dPL.GlobalAttributeAlias16
			,dPL.GlobalAttributeSource17
			,dPL.GlobalAttributeAlias17
			,dPL.GlobalAttributeSource18
			,dPL.GlobalAttributeAlias18
			,dPL.GlobalAttributeSource19
			,dPL.GlobalAttributeAlias19
			,dPL.GlobalAttributeSource20
			,dPL.GlobalAttributeAlias20

			,dPL.GlobalAttributeSource21
			,dPL.GlobalAttributeAlias21
			,dPL.GlobalAttributeSource22
			,dPL.GlobalAttributeAlias22
			,dPL.GlobalAttributeSource23
			,dPL.GlobalAttributeAlias23
			,dPL.GlobalAttributeSource24
			,dPL.GlobalAttributeAlias24
			,dPL.GlobalAttributeSource25
			,dPL.GlobalAttributeAlias25

			,vUR.[Right]
		FROM planning.tdProductLines dPL INNER JOIN vUR
			ON dPL.FactoryID		= vUR.FactoryID AND
			   dPL.ProductLineID	= vUR.ProductLineID
		WHERE 
				dPL.FactoryKey = @FactoryKey 
			OR @FactoryID = ''
		ORDER BY ISNULL(TRY_CAST(dPL.ProductLineID AS BIGINT), 999999999), dPL.ProductLineID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_GET_ProductLines;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_GET_ProductLines;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_GET_ProductLines TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_GET_ProductLines TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'				    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_GET_ProductLines'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for ProductLines 
				A: returns the ProductLines for a single Factory
				B: returns all ProductLines when Factory '' is requested';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
