﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_Factory;
GO

/*
	MOVE Operation for a Factory
	Factory moved by the update IDs without key change
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9

	Test call
	1. Empty source or target FactoryID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory => 404 Not Found
	5. Exists target Factory => 403 Forbidden
	6. Factory moved => 200 OK
	7. Don`t moved anything => 204 No content

	Testcall
		DECLARE  @RC INT
				,@Username			NVARCHAR(255) ='SQL'
				,@SourceFactoryID   NVARCHAR(255) ='ZT'
				,@TargetFactoryID   NVARCHAR(255) ='ZT1';

		EXECUTE @RC = dbo.sx_pf_MOVE_Factory
				   @Username
				  ,@SourceFactoryID
				  ,@TargetFactoryID;
		PRINT @RC;

		SELECT TOP 20 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_Factory
	 @Username			NVARCHAR(255)
	,@SourceFactoryID	NVARCHAR(255)
	,@TargetFactoryID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'				--cross access and data operation in system table
AS 
BEGIN

	DECLARE  @TransactUserKey			INT				= 0
			,@SourceFactoryKey			INT				= 0
			,@SourceFactoryNameShort	NVARCHAR(255)	= N''
			,@TargetFactoryKey			INT				= 0
			
			,@TransactUsername			NVARCHAR(255)	= N''
			,@ProcedureName				NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString			NVARCHAR (MAX)	= N'''' 
										+ ISNULL(@Username, N'NULL')		+ N''',''' 
										+ ISNULL(@SourceFactoryID, N'NULL') + N''',''' 
										+ ISNULL(@TargetFactoryID, N'NULL') + N''''
			,@EffectedRows				INT = 0						
			,@ResultCode				INT = 501						
			,@TimestampCall				DATETIME = GETUTCDATE()
			,@Comment					NVARCHAR (2000) = N'';				

	-- NULL Protection
	IF @Username		IS NULL SET @Username			= N'';
	IF @SourceFactoryID IS NULL SET @SourceFactoryID	= N'';
	IF @TargetFactoryID IS NULL SET @TargetFactoryID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_MOVE_Factory;
				
		-- Protect input parameters	
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID	= dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @TargetFactoryID	= dbo.sx_pf_pProtectID (@TargetFactoryID);

		IF @SourceFactoryID = N'' OR @TargetFactoryID = N''	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	

		-- Determine SourceKeys
		SELECT @SourceFactoryKey = FactoryKey, @SourceFactoryNameShort = NameShort FROM planning.tdFactories WHERE FactoryID = @SourceFactoryID;
		IF @SourceFactoryKey = 0	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- Determine TargetKeys
		SELECT @TargetFactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @TargetFactoryID;
		IF @TargetFactoryKey <> 0 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target Factory key already exists', 16, 10);
		END;
		
		-- MOVE DATA	++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		--dFactory 
		UPDATE planning.tdFactories
		SET FactoryID = @TargetFactoryID 
		WHERE FactoryKey = @SourceFactoryKey;

		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
			BEGIN 
				SET @ResultCode = 204;
			END		
		ELSE
			BEGIN
				--dProductLine  
				UPDATE planning.tdProductLines
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dProduct 
				UPDATE planning.tdProducts
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dValueSeries
				UPDATE planning.tdValueSeries
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dTime
				UPDATE planning.tdTime
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--fValues
				UPDATE planning.tfValues
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--gFactories
				UPDATE planning.tgFactories 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				--gProductLines
				UPDATE planning.tgProductLines 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;
		
				--gProducts 
				UPDATE planning.tgProducts 
				SET FactoryID  = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;
		
				SET @EffectedRows += @@ROWCOUNT;

				--rRights 
				UPDATE system.trRights				-- therefore, EXECUTE AS dbo is needed
				SET FactoryID = @TargetFactoryID
				WHERE FactoryID = @SourceFactoryID;

				SET @EffectedRows += @@ROWCOUNT;

				--rPreferences
				UPDATE planning.trPreferences 
				SET FactoryID = @TargetFactoryID
				WHERE FactoryID = @SourceFactoryID;

				SET @EffectedRows += @@ROWCOUNT;

				--fStatements
				UPDATE planning.tfStatements 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tFiles
				UPDATE planning.tFiles 
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tTabs
				UPDATE planning.tTabs
				SET FactoryID = @TargetFactoryID 
				WHERE FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				SET @ResultCode = IIF(@EffectedRows > 0, 201, 204);

				-- write action statement
				INSERT INTO planning.tfStatements   
					SELECT ProductKey
						, dp.ProductLineKey
						, @TargetFactoryKey AS FactoryKey
						, ProductID
						, dp.ProductLineID
						, @TargetFactoryID AS FactoryID
						, N'MOVE' AS ActionType
						, N'Factory is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort
							+ N'" / ProductLine <' + dp.ProductLineID + N'>"' + ISNULL(dpl.Nameshort, N'NULL') 
							+ N'" / Product <' + ProductID  + N'>"' + dp.Nameshort +  N'"' AS [Statement]
						, @TransactUsername AS UserName
						, N'' AS PCName
						, N'' AS ProcessorCode
						, N'' AS IPAddresses
						, GETUTCDATE() AS [Timestamp]
						, 0 AS IsDeleted
						, 0 AS IsResolved
					FROM planning.tdProducts dp LEFT JOIN planning.tdProductLines dpl
						ON dp.ProductLinekey = dpl.ProductLineKey
					WHERE dp.FactoryKey = @SourceFactoryKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- Rights materialization
				EXEC system.spMaterialize_trUserRights;
			END;

		COMMIT TRANSACTION sx_pf_MOVE_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_MOVE_Factory;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_Factory TO pf_PlanningFactoryService;  
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_Factory'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'MOVE Operation for a Factory
         Factory moved by the update IDs without key change';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
