﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_List;
GO

/*
	Move Operation for a List
	The Procedure will change the OldListID into the NewListID in tables:
	planning.tdValueSeries, column ValueListID
	planning.tdProductLines, columns GlobalAttributeSource 1..25
	create members from OldListID in NewListID if they don't exist there
	(for Lists of Type string compare and create ValueText, for Lists of Type Integer compare and create ValueInt)
	delete the OldListID in planning.thLists
	delete the Old List Values in planning.thListValues
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. Empty source or target ListID => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source List => 404 Not Found
	5. Not exists target List => 404 Not Found
	6. List moved => 200 OK
	7. Don`t moved anything => 204 No content

		DECLARE @RC int
		DECLARE  @Username AS NVARCHAR(255) = 'SQL'
		DECLARE  @OldListID AS NVARCHAR(255)='suEvents'
		DECLARE  @NewListID AS NVARCHAR(255)='suYesNo'

		EXECUTE @RC = dbo.sx_pf_MOVE_List
				   @Username
				  ,@OldListID
				  ,@NewListID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_List
	@Username AS NVARCHAR(255),
	@OldListID AS NVARCHAR(255),
	@NewListID AS NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @OldListKey AS INT = 0;
	DECLARE @NewListKey AS INT = 0;
	DECLARE @NewListType AS NVARCHAR(255)= N'';

	DECLARE @TransactUsername AS NVARCHAR(255)= N'';
	DECLARE @ProcedureName AS NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString AS NVARCHAR (MAX) = N'''' + ISNULL(@Username, N'NULL') + N''',''' + ISNULL(@OldListID, N'NULL') + N''',''' + ISNULL(@NewListID, N'NULL') + N'''';
	DECLARE @EffectedRows AS INT = 0;						-- SET during Execution
	DECLARE @ResultCode AS INT = 501;						-- SET during Execution
	DECLARE @TimestampCall AS DATETIME = GETUTCDATE();
	DECLARE @Comment AS NVARCHAR (2000) = N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username IS NULL SET @Username = N'';
	IF @OldListID IS NULL SET @OldListID = N'';
	IF @NewListID IS NULL SET @NewListID = N'';

	BEGIN TRY
		BEGIN TRANSACTION ONE;
				
		-- STEP 0.2 - Protect input parameters	
		SET @Username = dbo.sx_pf_pProtectString (@Username);
		SET @OldListID = dbo.sx_pf_pProtectID (@OldListID);
		SET @NewListID = dbo.sx_pf_pProtectID (@NewListID);

		IF (@OldListID = N'' OR @NewListID = N'')	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @OldListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
			OR @NewListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel')
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
 		END;;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);		

		-- STEP 1.2 - Determine SourceKeys	
		SELECT @OldListKey = ListKey FROM planning.thLists WHERE ListID = @OldListID;
		
		IF @OldListKey = 0
		BEGIN															
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);							 
		END;

		-- STEP 1.3 - Determine TargetKeys
		SELECT @NewListKey = ListKey, @NewListType = Datentyp FROM planning.thLists WHERE ListID = @NewListID;
		
		IF @NewListKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
							 
		-- STEP 3 - Move data
		-- STEP 3.1 - Move not exists UNIQUE list members

		IF @NewListType = 'Integer'
		BEGIN

			;WITH vals AS (
				SELECT v1.ListValueKey 
					, v1.ValueInt
					, v1.Scale
					, v1.ValueText
					, v1.ValueComment
					, v1.FormatID
				FROM planning.thListValues v1 LEFT JOIN  planning.thListValues v2 
					ON v2.ListID = @NewListID AND v1.ValueInt = v2.ValueInt
				WHERE v1.ListID = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES ListValueKey
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY ValueInt ORDER BY ListValueKey)
			)																				   
			INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
			SELECT @NewListID, ValueInt, Scale, ValueText, ValueComment, FormatID
			FROM vals INNER JOIN uniq ON vals.ListValueKey = uniq.ListValueKey;

			SET @EffectedRows += @@ROWCOUNT;
		END
		ELSE IF @NewListType = 'String'
		BEGIN

			;WITH vals AS (
				SELECT v1.ListValueKey 
					, v1.ValueInt
					, v1.Scale
					, v1.ValueText
					, v1.ValueComment
					, v1.FormatID
				FROM planning.thListValues v1 LEFT JOIN  planning.thListValues v2 
					ON v2.ListID = @NewListID AND v1.ValueText = v2.ValueText
				WHERE v1.ListID = @OldListID AND v2.ListID IS NULL 
			),
			uniq AS (
				SELECT TOP 1 WITH TIES ListValueKey
				FROM vals
				ORDER BY ROW_NUMBER() OVER (PARTITION BY ValueText ORDER BY ListValueKey)
			)																				   
			INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
			SELECT @NewListID, ValueInt, Scale, ValueText, ValueComment, FormatID
			FROM vals INNER JOIN uniq ON vals.ListValueKey = uniq.ListValueKey;

			SET @EffectedRows += @@ROWCOUNT;
		END;

		-- STEP 3.2 - Force all objects, which use a old list, to use a new list now
		UPDATE planning.tdValueSeries
		   SET ValueListID = @NewListID 
		WHERE ValueListID = @OldListID;
																							
		SET @EffectedRows += @@ROWCOUNT;
		
		;WITH lines AS (
			SELECT ProductLineKey 
			FROM planning.tdProductLines
			WHERE GlobalAttributeSource1 = @OldListID OR GlobalAttributeSource2 = @OldListID OR GlobalAttributeSource3 = @OldListID OR
				GlobalAttributeSource4 = @OldListID OR GlobalAttributeSource5 = @OldListID OR GlobalAttributeSource6 = @OldListID OR
				GlobalAttributeSource7 = @OldListID OR GlobalAttributeSource8 = @OldListID OR GlobalAttributeSource9 = @OldListID OR
				GlobalAttributeSource10 = @OldListID OR GlobalAttributeSource11 = @OldListID OR GlobalAttributeSource12 = @OldListID OR
				GlobalAttributeSource13 = @OldListID OR GlobalAttributeSource14 = @OldListID OR GlobalAttributeSource15 = @OldListID OR
				GlobalAttributeSource16 = @OldListID OR GlobalAttributeSource17 = @OldListID OR GlobalAttributeSource18 = @OldListID OR
				GlobalAttributeSource19 = @OldListID OR GlobalAttributeSource20 = @OldListID OR GlobalAttributeSource21 = @OldListID OR
				GlobalAttributeSource22 = @OldListID OR GlobalAttributeSource23 = @OldListID OR GlobalAttributeSource24 = @OldListID OR
				GlobalAttributeSource25 = @OldListID 
		)
		UPDATE pl
		SET GlobalAttributeSource1 = IIF(GlobalAttributeSource1 = @OldListID, @NewListID, GlobalAttributeSource1)
			, GlobalAttributeSource2 = IIF(GlobalAttributeSource2 = @OldListID, @NewListID, GlobalAttributeSource2)
			, GlobalAttributeSource3 = IIF(GlobalAttributeSource3 = @OldListID, @NewListID, GlobalAttributeSource3)
			, GlobalAttributeSource4 = IIF(GlobalAttributeSource4 = @OldListID, @NewListID, GlobalAttributeSource4)
			, GlobalAttributeSource5 = IIF(GlobalAttributeSource5 = @OldListID, @NewListID, GlobalAttributeSource5)
			, GlobalAttributeSource6 = IIF(GlobalAttributeSource6 = @OldListID, @NewListID, GlobalAttributeSource6)
			, GlobalAttributeSource7 = IIF(GlobalAttributeSource7 = @OldListID, @NewListID, GlobalAttributeSource7)
			, GlobalAttributeSource8 = IIF(GlobalAttributeSource8 = @OldListID, @NewListID, GlobalAttributeSource8)
			, GlobalAttributeSource9 = IIF(GlobalAttributeSource9 = @OldListID, @NewListID, GlobalAttributeSource9)
			, GlobalAttributeSource10 = IIF(GlobalAttributeSource10 = @OldListID, @NewListID, GlobalAttributeSource10)
			, GlobalAttributeSource11 = IIF(GlobalAttributeSource11 = @OldListID, @NewListID, GlobalAttributeSource11)
			, GlobalAttributeSource12 = IIF(GlobalAttributeSource12 = @OldListID, @NewListID, GlobalAttributeSource12)
			, GlobalAttributeSource13 = IIF(GlobalAttributeSource13 = @OldListID, @NewListID, GlobalAttributeSource13)
			, GlobalAttributeSource14 = IIF(GlobalAttributeSource14 = @OldListID, @NewListID, GlobalAttributeSource14)
			, GlobalAttributeSource15 = IIF(GlobalAttributeSource15 = @OldListID, @NewListID, GlobalAttributeSource15)
			, GlobalAttributeSource16 = IIF(GlobalAttributeSource16 = @OldListID, @NewListID, GlobalAttributeSource16)
			, GlobalAttributeSource17 = IIF(GlobalAttributeSource17 = @OldListID, @NewListID, GlobalAttributeSource17)
			, GlobalAttributeSource18 = IIF(GlobalAttributeSource18 = @OldListID, @NewListID, GlobalAttributeSource18)
			, GlobalAttributeSource19 = IIF(GlobalAttributeSource19 = @OldListID, @NewListID, GlobalAttributeSource19)
			, GlobalAttributeSource20 = IIF(GlobalAttributeSource20 = @OldListID, @NewListID, GlobalAttributeSource20)
			, GlobalAttributeSource21 = IIF(GlobalAttributeSource21 = @OldListID, @NewListID, GlobalAttributeSource21)
			, GlobalAttributeSource22 = IIF(GlobalAttributeSource22 = @OldListID, @NewListID, GlobalAttributeSource22)
			, GlobalAttributeSource23 = IIF(GlobalAttributeSource23 = @OldListID, @NewListID, GlobalAttributeSource23)
			, GlobalAttributeSource24 = IIF(GlobalAttributeSource24 = @OldListID, @NewListID, GlobalAttributeSource24)
			, GlobalAttributeSource25 = IIF(GlobalAttributeSource25 = @OldListID, @NewListID, GlobalAttributeSource25)
		FROM planning.tdProductLines pl INNER JOIN lines l ON pl.ProductLineKey = l.ProductLineKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Remove old list
				
		DELETE FROM planning.thListValues WHERE ListID = @OldListID;
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.thLists WHERE ListKey = @OldListKey;
		SET @EffectedRows += @@ROWCOUNT;
		
		IF @EffectedRows = 0
		BEGIN 
			SET @ResultCode = 204;
		END;		

		COMMIT TRANSACTION ONE;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION ONE;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_List TO pf_PlanningFactoryService;  
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_MOVE_List TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_List'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Move Operation for a List
         The Procedure will change the OldListID into the NewListID in tables:
         planning.tdValueSeries, column ValueListID
         planning.tdProductLines, columns GlobalAttributeSource 1..25
         create members from OldListID in NewListID if they dont exist there
         (for Lists of Type string compare and create ValueText, for Lists of Type Integer compare and create ValueInt)
         delete the OldListID in planning.thLists
         delete the Old List Values in planning.thListValues';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@OldListID';
    SET @value      = N'OldListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NewListID';
    SET @value      = N'NewListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
