﻿
DROP PROCEDURE IF EXISTS dbo.sx_pf_MOVE_ProductLine;
GO

/*
	MOVE Operation for a ProductLine
	ProductLine moved by the update IDs without key change

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 05/2023 for OCT 5.9

	Test call
	1. Empty source or target  FactoryID / ProductLineID  => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. User has no Rights => 401 Unauthorized
	4. Not exists source Factory / ProductLine => 404 Not Found
	5. Not exists target  Factory und existierende Target ProductLine => 404  Not Found
	6. Target ValueSerie key already exists => 403 Forbidden
	7. ProductLine moved => 200 OK
	8. Don`t moved anything => 204 No content

	Testcall
		DECLARE  @RC INT
				,@Username				NVARCHAR(255)	=N'SQL'
				,@SourceFactoryID		NVARCHAR(255)	=N'ZT'
				,@SourceProductLineID	NVARCHAR(255)	=N'U'
				,@TargetFactoryID		NVARCHAR(255)	=N'ZT'
				,@TargetProductLineID	NVARCHAR(255)	=N'U2';

		EXECUTE @RC = dbo.sx_pf_MOVE_ProductLine
				   @Username
				  ,@SourceFactoryID
				  ,@SourceProductLineID
				  ,@TargetFactoryID
				  ,@TargetProductLineID;

		PRINT @RC;

		SELECT TOP 20 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_ProductLine',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_MOVE_ProductLine','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_MOVE_ProductLine
	 @Username				NVARCHAR(255)
	,@SourceFactoryID		NVARCHAR(255)
	,@SourceProductLineID	NVARCHAR(255)
	,@TargetFactoryID		NVARCHAR(255)
	,@TargetProductLineID	NVARCHAR(255)

	WITH EXECUTE AS 'dbo'					-- due to moving the Productline in the system tables / cross action on system schema
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE  @TransactUserKey				INT = 0
			,@SourceFactoryKey 				INT = 0
			,@SourceProductLineKey			INT = 0
			,@SourceFactoryNameShort		NVARCHAR(255) = N''
			,@SourceProductLineNameShort	NVARCHAR(255) = N''
			,@TargetFactoryKey				INT = 0
			,@TargetProductLineKey			INT = 0

			,@TransactUsername				NVARCHAR(255)= N''
			,@ProcedureName					NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString				NVARCHAR (MAX) = N'''' 
					+ ISNULL(@Username				,N'NULL') + N''',''' 
					+ ISNULL(@SourceFactoryID		,N'NULL') + N''',''' 
					+ ISNULL(@SourceProductLineID	,N'NULL') + N''',''' 
					+ ISNULL(@TargetFactoryID		,N'NULL') + N''',''' 
					+ ISNULL(@TargetProductLineID	,N'NULL') + N''''
			,@EffectedRows					INT				= 0					
			,@ResultCode					INT				= 501				
			,@TimestampCall					DATETIME		= GETUTCDATE()
			,@Comment						NVARCHAR(2000)	= N'';				

	-- NULL Protection
	IF @Username			IS NULL SET @Username				= N'';
	IF @SourceFactoryID		IS NULL SET @SourceFactoryID		= N'';
	IF @SourceProductLineID IS NULL SET @SourceProductLineID	= N'';
	IF @TargetFactoryID		IS NULL SET @TargetFactoryID		= N'';
	IF @TargetProductLineID IS NULL SET @TargetProductLineID	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_MOVE_ProductLine;

		-- Protect input parameters
		SET @Username				= dbo.sx_pf_pProtectString (@Username);
		SET @SourceFactoryID		= dbo.sx_pf_pProtectID (@SourceFactoryID);
		SET @SourceProductLineID	= dbo.sx_pf_pProtectID (@SourceProductLineID);
		SET @TargetFactoryID		= dbo.sx_pf_pProtectID (@TargetFactoryID);
		SET @TargetProductLineID	= dbo.sx_pf_pProtectID (@TargetProductLineID);

		IF @SourceFactoryID = N'' OR @SourceProductLineID = N'' OR @TargetFactoryID = N''
			OR @TargetProductLineID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Check rights
		EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @TargetFactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights on target - no write rights.', 16, 10);
				
		EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactUsername, @SourceFactoryID;
		
		IF @ResultCode <> 200
			RAISERROR('Invalid rights on source - no write rights.', 16, 10);
		
		-- Determine SourceKeys	
		SELECT @SourceFactoryKey		= FactoryKey, @SourceFactoryNameShort = NameShort			FROM planning.tdFactories		WHERE FactoryID = @SourceFactoryID;
		SELECT @SourceProductLineKey	= ProductLineKey, @SourceProductLineNameShort = Nameshort	FROM planning.tdProductLines	WHERE FactoryKey = @SourceFactoryKey AND ProductLineID = @SourceProductLineID;
		
		IF @SourceFactoryKey = 0 OR @SourceProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Source keys don`t exists', 16, 10);
		END;

		-- Determine TargetKeys	
		SELECT @TargetFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @TargetFactoryID;
		SELECT @TargetProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @TargetFactoryKey AND ProductLineID = @TargetProductLineID;
		
		IF @TargetFactoryKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Target keys don`t exists', 16, 10);
		END;
		
		IF @TargetProductLineKey <> 0		
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Target ProductLine key already exists', 16, 10);
		END;
		
		-- MOVE DATA		+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		--dProductLine 
		UPDATE planning.tdProductLines 
		SET  FactoryKey		= @TargetFactoryKey
			,ProductLineID	= @TargetProductLineID
			,FactoryID		= @TargetFactoryID 
		WHERE ProductLineKey = @SourceProductLineKey;

		SET @EffectedRows += @@ROWCOUNT;

		IF @EffectedRows = 0
			BEGIN 
				SET @ResultCode = 204;
			END		
		ELSE
			BEGIN
				--dProduct
				UPDATE planning.tdProducts 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dValueSeries
				UPDATE planning.tdValueSeries 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--dTime
				UPDATE planning.tdTime 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--fValues
				UPDATE planning.tfValues 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--gProductLines
				UPDATE planning.tgProductLines 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;
		
				--gProducts 
				UPDATE planning.tgProducts 
				SET  FactoryKey			= @TargetFactoryKey
					,ProductLineID		= @TargetProductLineID
					,FactoryID			= @TargetFactoryID 
				WHERE ProductLineKey	= @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				--rRights
				UPDATE system.trRights						-- this makes execute as dbo necessary !
				SET		 FactoryID		= @TargetFactoryID
						,ProductLineID	= @TargetProductLineID
				WHERE 
						FactoryID		= @SourceFactoryID 
					AND ProductLineID	= @SourceProductLineID;

				SET @EffectedRows += @@ROWCOUNT;

				--rPreferences (no FPLP Keys a columns)
				UPDATE	 planning.trPreferences 
					SET		 ProductLineID	= @TargetProductLineID
							,FactoryID		= @TargetFactoryID

					WHERE 
							FactoryID		= @SourceFactoryID 
						AND ProductLineID	= @SourceProductLineID;

				SET @EffectedRows += @@ROWCOUNT;

				--fStatements
				UPDATE planning.tfStatements 
				SET  FactoryKey		= @TargetFactoryKey
					,ProductLineID	= @TargetProductLineID
					,FactoryID		= @TargetFactoryID 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tFiles
				UPDATE planning.tFiles 
				SET 
					 FactoryKey		= @TargetFactoryKey
					,FactoryID		= @TargetFactoryID 
					,ProductLineID	= @TargetProductLineID
				WHERE 
					ProductLineKey = @SourceProductLineKey;

				INSERT INTO planning.tfStatements
				SELECT 
						 ProductKey
						,@SourceProductLineKey	AS ProductLineKey
						,@TargetFactoryKey		AS FactoryKey
						,ProductID
						,@TargetProductLineID	AS ProductLineID
						,@TargetFactoryID		AS FactoryID
						,N'MOVE'				AS ActionType
						,N'ProductLine is moved from Factory <' + @SourceFactoryID + N'>"' + @SourceFactoryNameShort 
							+ N'"/ ProductLine <' + @SourceProductLineID + N'>"' + @SourceProductLineNameShort 
							+ N'"/ Product <' + ProductID + N'>"' + Nameshort +  N'"' AS [Statement]
						,@TransactUsername		AS UserName
						,N''					AS PCName
						,N''					AS ProcessorCode
						,N''					AS IPAddresses
						,GETUTCDATE()			[Timestamp]
						,0 AS IsDeleted
						,0 AS IsResolved
				FROM planning.tdProducts 
				WHERE ProductLineKey = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				-- tTabs
				UPDATE planning.tTabs
				SET 
					 FactoryKey		= @TargetFactoryKey
					,FactoryID		= @TargetFactoryID 
					,ProductLineID	= @TargetProductLineID
				WHERE
					ProductLineKey  = @SourceProductLineKey;

				SET @EffectedRows += @@ROWCOUNT;

				SET @ResultCode = IIF(@EffectedRows > 0, 201, 204);

				-- Rights materialization
				EXEC system.spMaterialize_trUserRights;
			END;

		COMMIT TRANSACTION sx_pf_MOVE_ProductLine;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_MOVE_ProductLine;		

		IF @Error_state <> 10 BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_MOVE_ProductLine TO pf_PlanningFactoryService;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_MOVE_ProductLine'	-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1							-- 0 = Demo object - out of shipment process
																	
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'MOVE Operation for a ProductLine
         ProductLine moved by the update IDs without key change';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFactoryID';
    SET @value      = N'SourceFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceProductLineID';
    SET @value      = N'SourceProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetFactoryID';
    SET @value      = N'TargetFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetProductLineID';
    SET @value      = N'TargetProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
