﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Cleanup;
GO

/*
	Cleanup Operation, which will clean the values in planning.tfValues for a given scope.

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE	 @RC				INT
				,@Username			NVARCHAR(255)	= 'SQL'
				,@FactoryID			NVARCHAR(255)	= 'ZT'
				,@ProductLineID		NVARCHAR(255)	= 'U'
				,@ProductID			NVARCHAR(255)	= ''
				,@ValueSeriesID		NVARCHAR(255)	= 'K3'
				,@TimeID			INT				= 20180515	

		EXECUTE	@RC = dbo.sx_pf_POST_Cleanup 
				@Username,
				@FactoryID,
				@ProductLineID,
				@ProductID,
				@ValueSeriesID,
				@TimeID

		PRINT @RC

		dbo.sx_pf_POST_Cleanup 'SQL', 'F1', 'PL1',	'P1',	'VS1',	201703012	-- will delete all values in planning.tfValues for this combination (this means one row)
		dbo.sx_pf_POST_Cleanup 'SQL', '',	'PL1',	'P1',	'VS1',	20170312	-- would delete this value in all Factories
		dbo.sx_pf_POST_Cleanup 'SQL', '',	'',		'',		'',		20170312	-- would delete all values in for this TimeID in planning.tfValues.

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Cleanup',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Cleanup','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Cleanup
	@Username		NVARCHAR(255),
	@FactoryID		NVARCHAR(255),
	@ProductLineID	NVARCHAR(255),
	@ProductID		NVARCHAR(255),	
	@ValueSeriesID	NVARCHAR(255),
	@TimeID			INT = 0
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N''''
			+ ISNULL(@Username						, N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID					, N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID						, N'NULL')	+ N''',''' 
			+ ISNULL(@ValueSeriesID					, N'NULL')	+ N''',' 
			+ ISNULL(CAST(@TimeID AS NVARCHAR(255))	, N'NULL');
	DECLARE @EffectedRows		INTEGER			= 0;					-- SET during Execution
	DECLARE @ResultCode			INTEGER			= 501;					-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';					-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL	SET @Username		= N'';
	IF @ProductID		IS NULL	SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ValueSeriesID	IS NULL SET @ValueSeriesID	= N'';
	IF @TimeID			IS NULL SET @TimeID			= 0;

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Cleanup;

		-- empty TimeID is not allowed
		IF @TimeID = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 0.2 - Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @ValueSeriesID	= dbo.sx_pf_pProtectID (@ValueSeriesID);

		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;	
		
		-- STEP 2 - Check rights
		DECLARE @Rights AS TABLE(FactoryID NVARCHAR(255) NOT NULL, ProductLineID NVARCHAR(255) NOT NULL);

		INSERT INTO @Rights (FactoryID, ProductLineID) 
			SELECT
				  FactoryID
				 ,ProductLineID
			FROM system.trUserRights 							 
			WHERE 
				Username = @TransactUsername 
				AND [Right]	= N'Write'
				AND (@FactoryID	= N'' OR FactoryID = @FactoryID)
				AND (@ProductLineID	= N'' OR ProductLineID = @ProductLineID);

		IF NOT EXISTS(SELECT 1 FROM @Rights)
		BEGIN
			SET @ResultCode = 401;
			RAISERROR('Invalid rights', 16, 10);
		END;
		
		-- STEP 3 - Delete values where user has rights
		DELETE FROM fV
		FROM planning.tfValues fV 
			INNER JOIN @Rights r ON fV.FactoryID = r.FactoryID COLLATE DATABASE_DEFAULT AND fV.ProductLineID = r.ProductLineID COLLATE DATABASE_DEFAULT
		WHERE fV.TimeID = @TimeID
			AND (@ProductID		 = N''	OR fV.ProductID		 = @ProductID)
			AND (@ValueSeriesID	 = N''	OR fV.ValueSeriesID	 = @ValueSeriesID); 

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;
		
		COMMIT TRANSACTION sx_pf_POST_Cleanup;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Cleanup;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Cleanup TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Cleanup'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Cleanup Operation, which will clean the values in planning.tfValues for a given scope.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueSeriesID';
    SET @value      = N'ValueSeriesID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeID';
    SET @value      = N'TimeID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
