﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Factory;
GO

/*
	POST Operation for a Factory
	Not existing Factories are created, existing updated
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call

	DECLARE  @RC				 INT
			,@Username			 NVARCHAR(255) = 'SQL'
			,@FactoryID			 NVARCHAR(255) = 'ZT'
			,@NameShort			 NVARCHAR(255) = ''
			,@NameLong			 NVARCHAR(255) = ''
			,@CommentUser		 NVARCHAR(255) = ''
			,@CommentDev		 NVARCHAR(255) = ''
			,@ResponsiblePerson	 NVARCHAR(255) = ''
			,@ImageName			 NVARCHAR(255) = ''

	EXECUTE @RC = dbo.sx_pf_POST_Factory 
			@Username,
			@FactoryID,
			@NameShort,
			@NameLong,
			@CommentUser,
			@CommentDev,
			@ResponsiblePerson,
			@ImageName
	PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Factory',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Factory','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Factory
	@Username			NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@NameShort			NVARCHAR(255),
	@NameLong			NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@ResponsiblePerson	NVARCHAR(255),
	@ImageName			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			 INT			= 0;
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''',''' 
			+ ISNULL(@FactoryID			, N'NULL')			 + N''','''
			+ ISNULL(@NameShort			, N'NULL')			 + N''','''
			+ ISNULL(@NameLong			, N'NULL')			 + N''','''
			+ ISNULL(@CommentUser		, N'NULL')			 + N''','''
			+ ISNULL(@CommentDev		, N'NULL')			 + N''','''
			+ ISNULL(@ResponsiblePerson	, N'NULL')			 + N''',''' 
			+ ISNULL(@ImageName			, N'NULL')			 + N'''';
	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username			 IS NULL SET @Username			 = N'';
	IF @FactoryID			 IS NULL SET @FactoryID			 = N'';
	IF @NameShort			 IS NULL SET @NameShort			 = N'';
	IF @NameLong			 IS NULL SET @NameLong			 = N'';
	IF @CommentUser			 IS NULL SET @CommentUser		 = N'';
	IF @CommentDev			 IS NULL SET @CommentDev		 = N'';
	IF @ResponsiblePerson	 IS NULL SET @ResponsiblePerson	 = N'';
	IF @ImageName			 IS NULL SET @ImageName			 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Factory;

		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			 = dbo.sx_pf_pProtectID		 (@FactoryID);
		SET @NameShort			 = dbo.sx_pf_pProtectString	 (@NameShort);
		SET @NameLong			 = dbo.sx_pf_pProtectString	 (@NameLong);
		SET @CommentUser		 = dbo.sx_pf_pProtectString	 (@CommentUser);
		SET @CommentDev			 = dbo.sx_pf_pProtectString	 (@CommentDev);
		SET @ResponsiblePerson	 = dbo.sx_pf_pProtectString	 (@ResponsiblePerson);
		SET @ImageName			 = dbo.sx_pf_pProtectString	 (@ImageName);
		
		IF @FactoryID = N'' 	
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

		-- STEP 2 - Check rights
		IF @FactoryKey = 0
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		ELSE
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3.1 - If Factory does not yet exist, it will be created in minimal configuration
		IF @FactoryKey = 0
		BEGIN
			
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);
			
			INSERT INTO planning.tdFactories (FactoryID, NameShort,NameLong, CommentUser, CommentDev, ResponsiblePerson, ImageName) 
				OUTPUT INSERTED.FactoryKey INTO @Keys([Key])
				VALUES (@FactoryID, @NameShort, '', '', '', '', '');

			SELECT TOP(1) @FactoryKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;		
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update Factory attributes
		UPDATE planning.tdFactories 
		SET  					 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	
			 NameShort			= IIF(@NameShort			= N'<#NV>', NameShort,			@NameShort)
			,NameLong			= IIF(@NameLong				= N'<#NV>', NameLong,			@NameLong)
			,CommentUser		= IIF(@CommentUser			= N'<#NV>', CommentUser,		@CommentUser)
			,CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,			@CommentDev)
			,ResponsiblePerson	= IIF(@ResponsiblePerson	= N'<#NV>', ResponsiblePerson,	@ResponsiblePerson)
			,ImageName			= IIF(@ImageName			= N'<#NV>', ImageName,			@ImageName)
		FROM planning.tdFactories 
		WHERE FactoryKey = @FactoryKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- STEP 3.3 - Rights materialize (no insert needed because only ClusterAdmin can create Factory)
		EXEC system.spMaterialize_trUserRights;	

		COMMIT TRANSACTION sx_pf_POST_Factory;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Factory;		

		IF @Error_state <> 10
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Factory TO pf_PlanningFactoryUser;
GO 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Factory'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a Factory
         Not existing Factories are created, existing updated
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ResponsiblePerson';
    SET @value      = N'ResponsiblePerson';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ImageName';
    SET @value      = N'ImageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
