﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_HTMLPage;
GO

/*
	POST Operation for an HTML Page
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE  @RC			INT
				,@Username		NVARCHAR(255) = 'SQL'
				,@ProductID		NVARCHAR(255) = ''
				,@ProductLineID	NVARCHAR(255) = ''
				,@FactoryID		NVARCHAR(255) = ''
				,@PageID		NVARCHAR(255) = 'Page1'
				,@PageName		NVARCHAR(255) = 'A great page'
				,@PageContent	NVARCHAR(255) = 'Hallo Gerd'

		EXECUTE @RC = dbo.sx_pf_POST_HTMLPage
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@PageID
				,@PageName
				,@PageContent
		PRINT @RC

		SELECT * FROM planning.tgHTMLPages

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_HTMLPage',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_HTMLPage','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_HTMLPage
	@Username			NVARCHAR(255),
	@ProductID			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PageID				NVARCHAR(255),
	@PageName			NVARCHAR(255),
	@PageContent		NVARCHAR(MAX)		

AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= -1;
	DECLARE @ProductLineKey		INT				= -1;
	DECLARE @ProductKey			INT				= -1;
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,			N'NULL')	+ N''',''' 
			+ ISNULL(@ProductID,		N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,	N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,		N'NULL')	+ N''',' 
			+ ISNULL(CAST(@PageID AS NVARCHAR(255)) + N',''' 
			+ ISNULL(@PageName,			N'NULL')	+ '', N'NULL');

	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;					
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PageID			IS NULL SET @PageID			= 0;
	IF @PageName		IS NULL SET @PageName		= '';
	IF @PageContent		IS NULL SET @PageContent	= '';


	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_HTMLPage;

		-- Page must have an ID, all other parameter may be empty
		IF @PageID = ''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Protect input parameters	
		SET @ProductID		= dbo.sx_pf_pProtectID (@ProductID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
		SET @PageID			= dbo.sx_pf_pProtectString (@PageID);
		SET @PageName		= dbo.sx_pf_pProtectString (@PageName);


		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;	

		-- Determine keys - all of them may be unused in case of Clusterpages
		SELECT @FactoryKey =		ISNULL(FactoryKey,-1)			FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey =	ISNULL(ProductLineKey,-1)		FROM planning.tdProductLines	WHERE ProductLineID = @ProductLineID	AND FactoryKey = @FactoryKey;
		SELECT @ProductKey =		ISNULL(ProductKey,-1)			FROM planning.tdProducts		WHERE ProductID		= @ProductID		AND ProductLineKey = @ProductLinekey;

		-- Check rights on different levels
		IF @FactoryKey = -1
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @FactoryKey > 0 AND @ProductLineKey = -1
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END

		IF @ProductLineKey > 0
			BEGIN
				EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
				IF @ResultCode <> 200
					RAISERROR('Invalid rights', 16, 10);
			END
		
		-- Delete old PageID
		DELETE FROM planning.tgHTMLPages 
		WHERE 
				ProductKey		= @ProductKey 
			AND ProductLineKey	= @ProductLineKey
			AND FactoryKey		= @FactoryKey
			AND PageID			= @PageID;

		IF @@RowCount = 1 
			SET @ResultCode = 200;
		ELSE 
			SET @ResultCode = 0;
		
		-- Create new Page
		INSERT INTO planning.tgHTMLPages	(
			 ProductKey
			,ProductLineKey
			,FactoryKey
			,ProductID
			,ProductLineID
			,FactoryID
			,PageID
			,PageName
			,PageContent
			)
			VALUES (
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				@PageID,
				@PageName,
				@PageContent
			);

		IF @ResultCode <> 200 SET @ResultCode = 201;
		SET @EffectedRows = @@ROWCOUNT;
		
		COMMIT TRANSACTION sx_pf_POST_HTMLPage;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_HTMLPage;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF (@ResultCode IS NULL OR @ResultCode/100 = 2) AND @ResultCode <> 204
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
    
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_HTMLPage TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_HTMLPage'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for an HTML Page';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageID';
    SET @value      = N'PageID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageName';
    SET @value      = N'PageName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PageContent';
    SET @value      = N'PageContent';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
