﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_List;
GO

/*
	POST Operation for a List
	Not existing Format will be created, existing updated.
	If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
		DECLARE  @RC			INT
				,@Username		NVARCHAR(255) = 'SQL'
				,@ListID		NVARCHAR(255) = 'sxScales'
				,@NameShort		NVARCHAR(255) = '<#NV>'
				,@NameLong		NVARCHAR(255) = '<#NV>'
				,@CommentDev	NVARCHAR(255) = '<#NV>'
				,@CommentUser	NVARCHAR(255) = '<#NV>'
				,@Datentyp		NVARCHAR(255) = '<#NV>'
				,@Source		NVARCHAR(255) = '<#NV>'
				,@SourceFormula	NVARCHAR(255) = 'Testen'
				,@FormatID		NVARCHAR(255) = '<#NV>'

		EXECUTE @RC = dbo.sx_pf_POST_List 
				@Username,
				@ListID,
				@NameShort,
				@NameLong,
				@CommentDev,
				@CommentUser,
				@Datentyp,
				@Source,
				@SourceFormula,
				@FormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_List',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_List','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_List
	 @Username		NVARCHAR(255)
	,@ListID		NVARCHAR(255)
	,@NameShort		NVARCHAR(255)
	,@NameLong		NVARCHAR(255)
	,@CommentDev	NVARCHAR(255)
	,@CommentUser	NVARCHAR(255)
	,@Datentyp		NVARCHAR(255)
	,@Source		NVARCHAR(255)
	,@SourceFormula NVARCHAR(255)
	,@FormatID		NVARCHAR(255)
AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @ListKey			INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username		,N'NULL')	+ N''',''' 
			+ ISNULL(@ListID		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameShort		,N'NULL')	+ N''',''' 
			+ ISNULL(@NameLong		,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentDev	,N'NULL')	+ N''',''' 
			+ ISNULL(@CommentUser	,N'NULL')	+ N''',''' 
			+ ISNULL(@Datentyp		,N'NULL')	+ N''',''' 
			+ ISNULL(@Source		,N'NULL')	+ N''',''' 
			+ ISNULL(@SourceFormula	,N'NULL')	+ N''',''' 
			+ ISNULL(@FormatID		,N'NULL')	+ N'''';	
	DECLARE @EffectedRows		INT				= 0;						
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR (2000) = N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ListID			IS NULL SET @ListID			= N'';
	IF @NameShort		IS NULL SET @NameShort		= N'';
	IF @NameLong		IS NULL SET @NameLong		= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Datentyp		IS NULL SET @Datentyp		= N'';
	IF @Source			IS NULL SET @Source			= N'';
	IF @SourceFormula	IS NULL SET @SourceFormula	= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_List;

		-- STEP 0.2 - Protect input parameters
		SET @ListID			= dbo.sx_pf_pProtectID (@ListID);
		SET @NameShort		= dbo.sx_pf_pProtectString (@NameShort);
		SET @NameLong		= dbo.sx_pf_pProtectString (@NameLong);
		SET @CommentUser	= dbo.sx_pf_pProtectString (@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString (@CommentDev);
		SET @Datentyp		= dbo.sx_pf_pProtectString (@Datentyp);
		SET @Source			= dbo.sx_pf_pProtectString (@Source);
		SET @SourceFormula	= dbo.sx_pf_pProtectString (@SourceFormula);
		SET @FormatID		= dbo.sx_pf_pProtectString (@FormatID); -- as it can be #<NV>

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END 
		ELSE IF @ListID IN ('sxIsNumeric', 'sxScales', 'sxValueEffects', 'sxValueLists', 'sxValueFormats', 'sxValueSources', 'sxVisibilityLevel') 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Invalid input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
				
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;

		-- STEP 3.1 - If List is does not exist, it will be created
		IF @ListKey = 0
		BEGIN

			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.thLists (ListID, NameShort, NameLong, CommentDev, CommentUser, Datentyp, Source, SourceFormula, FormatID) 
			OUTPUT INSERTED.ListKey INTO @Keys([Key])
			VALUES (@ListID, '', '', '', '', '', '', '', '');

			SELECT TOP(1) @ListKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - Update List attributes
		UPDATE planning.thLists 
		SET	 
			 NameShort		= IIF(@NameShort		= N'<#NV>', NameShort,		@NameShort)
			,NameLong		= IIF(@NameLong			= N'<#NV>', NameLong,		@NameLong)
			,CommentDev		= IIF(@CommentDev		= N'<#NV>', CommentDev,		@CommentDev)
			,CommentUser	= IIF(@CommentUser		= N'<#NV>', CommentUser,	@CommentUser)
			,Datentyp		= IIF(@Datentyp			= N'<#NV>', Datentyp,		@Datentyp)
			,Source			= IIF(@Source			= N'<#NV>', Source,			@Source)
			,SourceFormula  = IIF(@SourceFormula	= N'<#NV>', SourceFormula,	@SourceFormula)
			,FormatID		= IIF(@FormatID			= N'<#NV>', FormatID,		@FormatID)
		FROM planning.thLists
		WHERE ListKey = @ListKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_List;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_List;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_List TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_List'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a List
         Not existing Format will be created, existing updated.
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameShort';
    SET @value      = N'NameShort';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@NameLong';
    SET @value      = N'NameLong';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Datentyp';
    SET @value      = N'Datentyp';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Source';
    SET @value      = N'Source';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SourceFormula';
    SET @value      = N'SourceFormula';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
