﻿DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ListValue;
GO

/*
	POST Operation for an ListValue
	Not existing ListValue will be created (if Key = 0 is used), existing updated.
	Special: This procedure allows to pass a Key value, as there is no ID for a ListValue

	Saxess Software GmbH
	Last modified: 09/2023 for OCT 5.10

	Test call
		DECLARE  @RC			 INT
				,@Username		 NVARCHAR(255)	= 'SQL'
				,@ListValueKey	 INT			= 0
				,@ListID		 NVARCHAR(255)	= 'sxStatus'
				,@ValueInt		 INT			= 5
				,@Scale			 INT			= 1
				,@ValueText		 NVARCHAR(255)	= 'Testo'
				,@ValueComment	 NVARCHAR(255)	= ''
				,@FormatID		 NVARCHAR(255)	= ''
		
		EXECUTE @RC = dbo.sx_pf_POST_ListValue 
				@Username,
				@ListValueKey,
				@ListID,
				@ValueInt,
				@Scale,
				@ValueText,
				@ValueComment,
				@FormatID
		PRINT @RC

	Test call API Log
		SELECT TOP  5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ListValue',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ListValue','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ListValue
	 @Username		NVARCHAR(255)
	,@ListValueKey	INT
	,@ListID		NVARCHAR(255)
	,@ValueInt		INT
	,@Scale			INT
	,@ValueText		NVARCHAR(255)
	,@ValueComment	NVARCHAR(255)
	,@FormatID		NVARCHAR(255)
AS
BEGIN

	DECLARE  @ListKey			INT			= 0
			,@TransactUsername	NVARCHAR(255)  = N''
			,@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N'''' 
							+ ISNULL(@Username,								 N'NULL') + N''',' 
							+ ISNULL(CAST(@ListValueKey AS NVARCHAR(255)),	 N'NULL') + N',''' 
							+ ISNULL(@ListID,								 N'NULL') + N''',' 
							+ ISNULL(CAST(@ValueInt		AS NVARCHAR(255)),	 N'NULL') + N',' 
							+ ISNULL(CAST(@Scale		AS NVARCHAR(255)),	 N'NULL') + N',''' 
							+ ISNULL(@ValueText,							 N'NULL') + N''',''' 
							+ ISNULL(@ValueComment,							 N'NULL') + N''',''' 
							+ ISNULL(@FormatID,								 N'NULL') + N''''
			,@EffectedRows		INT			= 0			
			,@ResultCode		INT			= 501		
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000) = N''
			
			,@tmpKey			BIGINT		  = 0
			,@tmpListType		NVARCHAR(255) = '';			

	-- NULL Protection
	IF @Username		IS NULL SET @Username		 = N'';
	IF @ListValueKey	IS NULL SET @ListValueKey	 = 0;
	IF @ListID			IS NULL SET @ListID			 = N'';
	IF @ValueInt		IS NULL SET @ValueInt		 = 0;
	IF @Scale			IS NULL SET @Scale			 = 0;
	IF @ValueText		IS NULL SET @ValueText		 = N'';
	IF @ValueComment	IS NULL SET @ValueComment	 = N'';
	IF @FormatID		IS NULL SET @FormatID		 = N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ListValue;

		-- protect input parameters
		SET @ListValueKey	= dbo.sx_pf_pProtectInt		(@ListValueKey);
		SET @ListID			= dbo.sx_pf_pProtectID		(@ListID);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueComment	= dbo.sx_pf_pProtectString	(@ValueComment);
		SET @FormatID		= dbo.sx_pf_pProtectID		(@FormatID);

		IF @ListID =  N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Check rights
		EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 1.2 - Determine keys and break if not existing
		SELECT @ListKey = ListKey FROM planning.thLists WHERE ListID = @ListID;
		
		IF @ListKey = 0	BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check if Listvalue exists, when ListValueKey 0 is submitted - 0 means new entry, but API must check if entry is really new
		IF @ListValueKey = 0
			BEGIN
				
				SELECT @tmpListType = hL.Datentyp FROM planning.thLists hL WHERE hL.ListID = @ListID;

				IF @tmpListType = 'String'
					BEGIN
						SELECT @tmpKey = ListValueKey FROM planning.thListValues hLV WHERE ListID = @ListID AND hLV.ValueText = @ValueText;
					END
				ELSE
					BEGIN
						SELECT @tmpKey = ListValueKey FROM planning.thListValues hLV WHERE ListID = @ListID AND hLV.ValueInt = @ValueInt;
					END

				IF (@tmpKey IS NOT NULL AND @tmpKey != 0) SET @ListValueKey = @tmpKey;
			END

		-- If ListValueKey = 0 is submitted, create new value
		IF @ListValueKey = 0
			BEGIN
				INSERT INTO planning.thListValues (ListID, ValueInt, Scale, ValueText, ValueComment, FormatID)
				VALUES (@ListID, @ValueInt, @Scale, @ValueText, @ValueComment, @FormatID);

				SET @Resultcode = 201;
				SET @EffectedRows = 1;
			END
		ELSE
			BEGIN
			-- existing key is committed, update the Key
				UPDATE planning.thListValues 
				SET 
					 ValueInt		 = @ValueInt
					,Scale			 = @Scale
					,ValueText		 = @ValueText
					,ValueComment	 = @ValueComment
					,FormatID		 = @FormatID
				WHERE ListValueKey	 = @ListValueKey;

				SET @EffectedRows += @@ROWCOUNT;
			END;	

		COMMIT TRANSACTION sx_pf_POST_ListValue;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ListValue;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ListValue TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ListValue'			-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'POST Operation for ListValue
         Not existing ListValue will be created, existing updated.
         This procedure allows to pass a Key value, as there is no ID for a ListValue';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListValueKey';
    SET @value      = N'Use existing key to update an ListValue, use 0 to create a new ListValue';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ListID';
    SET @value      = N'ListID where this Value belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt if List of Type integer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale if List of Type integer';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText if List of Type String';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueComment';
    SET @value      = N'Comment for this value';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID for this value';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
