﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Preference;
GO

/*
	POST Operation for Preference
	Not existing Preference will be created, existing updated.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test Calls
		-- make Property UserDependend
		EXEC dbo.sx_pf_POST_ProductLineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','1','1','2',''

		-- make Property NOT UserDependend
		EXEC dbo.sx_pf_POST_ProductLineProperty 'SQL','U','ZT','P06','<#NV>','<#NV>','<#NV>','<#NV>','<#NV>','0','1','2',''

		DECLARE	 @RC					INT
				,@Username 				NVARCHAR(255) = 'SQL'
				,@PreferenceUserName	NVARCHAR(255) = 'AD-ONE\gtautenh'
				,@ProductLineID			NVARCHAR(255) = 'U'
				,@FactoryID				NVARCHAR(255) = 'ZT'
				,@SettingName			NVARCHAR(255) = 'Demopref'
				,@ValueInt				NVARCHAR(255) = '8'
				,@ValueText				NVARCHAR(255) = 'uztr'

		EXECUTE @RC = dbo.sx_pf_POST_Preference 
				@Username,
				@PreferenceUsername,
				@ProductLineID,
				@FactoryID,
				@SettingName,
				@ValueInt,
				@ValueText;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Preference',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Preference','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Preference
	@Username			NVARCHAR(255),
	@PreferenceUsername NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@SettingName		NVARCHAR(255),
	@ValueInt			INT,
	@ValueText			NVARCHAR(MAX)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername				NVARCHAR(255)	= N''
	DECLARE @FactoryKey						INT				= 0
	DECLARE @ProductLineKey					INT				= 0
	DECLARE @PreferenceIsUserDependendFlag	INT				= 0
	DECLARE @ProcedureName					NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString				NVARCHAR (MAX)	= N'''' 
			+ ISNULL(@Username,							N'NULL')	+ N''',''' 
			+ ISNULL(@PreferenceUsername,				N'NULL')	+ N''',''' 
			+ ISNULL(@ProductLineID,					N'NULL')	+ N''',''' 
			+ ISNULL(@FactoryID,						N'NULL')	+ N''',''' 
			+ ISNULL(@SettingName,						N'NULL') 	+ N''',' 
			+ ISNULL(CAST(@ValueInt AS NVARCHAR(255)),	N'NULL')	+ N',''' 
			+ ISNULL(@ValueText,						N'NULL')	+ N''''
	DECLARE @EffectedRows					INT				= 0
	DECLARE @ResultCode						INTEGER			= 501					
	DECLARE @TimestampCall					DATETIME		= GETUTCDATE()
	DECLARE @Comment						NVARCHAR(2000)	= N'';	
	DECLARE @UserKey						INT				= 0;				

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @FactoryID			IS NULL SET @FactoryID			= N'';
	IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
	IF @PreferenceUsername	IS NULL SET @PreferenceUsername = N'';
	IF @SettingName			IS NULL SET @SettingName		= N'';
	IF @ValueInt			IS NULL SET @ValueInt			= 0;
	IF @ValueText			IS NULL SET @ValueText			= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Preference

		-- STEP 0.2 - Protect input parameters
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @SettingName	= dbo.sx_pf_pProtectString	(@SettingName);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);

		IF	@SettingName = N'' 
			OR @SettingName = '<#NV>'
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine, if Preferences are saved per User
		-- for global Preferences
		IF	@FactoryID = N'' 
			AND @ProductLineID = N''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgCluster WHERE PropertyID = 'C06';
		END
		-- for Preferences per Factory
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID = ''
		BEGIN 
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgFactories WHERE PropertyID = 'F06';
		END
		-- for Preferences per ProductLine
		ELSE IF @FactoryID <> N'' 
			AND @ProductLineID <> ''
		BEGIN
			SELECT @PreferenceIsUserDependendFlag = ValueInt FROM planning.tgProductLines WHERE PropertyID = 'P06';
		END;

		-- STEP 1.2 - if Preferences are not Userdependend, only a clusteradmin can save them, else everybody with
		-- Factory / ProductLine Write Rights
		ELSE IF @PreferenceIsUserDependendFlag <> 1 
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
			
			IF @ResultCode <> 200
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- STEP 1.3 - Determine Key, they may be 0 if the preference is for Cluster / Factory
		SELECT @FactoryKey	= ISNULL(FactoryKey, 0)	FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ISNULL(ProductLineKey, 0) FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;

		-- Cancel if Factory or Product Line does not exist
		IF	(@FactoryKey = 0 AND @FactoryID <> N'')
			OR	(@ProductLineKey = 0 AND @ProductLineID <> N'')
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Determine UserKey
		SELECT @UserKey = UserKey FROM system.trUser WHERE Username = @PreferenceUsername

		-- STEP 1.4 - Set Preference User to "All" if Preferences are not Userdependend
		IF @PreferenceIsUserDependendFlag <> 1 
			BEGIN
				SET @PreferenceUsername = N'All';
				SET @UserKey			= 0 ;
			END
		ELSE
		-- STEP 1.5 - Test existence of Preferences Users if Preference is Userdepend
		IF	@UserKey = 0 AND @PreferenceIsUserDependendFlag <> 1
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('User don`t exists', 16, 10);
		END;	
		
		-- STEP 2. Rights check
		IF @ProductLineID <> N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;
		END
		ELSE IF @FactoryID <> N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername, @FactoryID;
		END
		ELSE IF @FactoryID = N''
				AND @ProductLineID = N''
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
		END

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);

		-- STEP 3. Clear Preference and edit
		-- if preferences are userdependend, they are deleted only for the specific user, else for all user
		IF @PreferenceIsUserDependendFlag = 1
			BEGIN
				DELETE FROM planning.trPreferences
				WHERE 
						FactoryID		= @FactoryID 
					AND ProductLineID	= @ProductLineID
					AND Username		= @PreferenceUsername
					AND SettingName		= @SettingName
				END
			ELSE
				BEGIN
					DELETE FROM planning.trPreferences
					WHERE 
							FactoryID		= @FactoryID 
						AND ProductLineID	= @ProductLineID
						AND SettingName		= @SettingName
				END;

		SET @EffectedRows = @@ROWCOUNT;

		-- TODO handle #NV - at the moment not handled
		SET @ValueText = IIF(@ValueText = N'<#NV>', N'', @ValueText);

		-- Save Preference
		INSERT INTO planning.trPreferences (UserKey,UserName, FactoryID, ProductLineID, SettingName, ValueInt, ValueText)
		VALUES (@UserKey,@PreferenceUsername, @FactoryID, @ProductLineID, @SettingName, @ValueInt, @ValueText);

		SET @EffectedRows += @@ROWCOUNT;
		SET @Resultcode = 201;

		COMMIT TRANSACTION sx_pf_POST_Preference;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Preference;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Preference TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Preference'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for Preference
         Not existing Preference will be created, existing updated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PreferenceUsername';
    SET @value      = N'PreferenceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SettingName';
    SET @value      = N'SettingName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
