﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_ProductLineProperty;
GO

/*
	POST Operation for ProductLineProperty
	Not existing ProductLine will be created, existing gets an Update
	If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Test call
	1. PropertyID is empty => 404 Not Found
	2. TransactionUsername not determined for User => 403 Forbidden
	3. Username does not have write access to ProductLine => 401 Unauthorized
	5. Property created => 201 Created
	6. Property updated => 200 OK

		DECLARE  @RC				 INT
				,@Username			 NVARCHAR(255) = 'SQL'
				,@ProductLineID		 NVARCHAR(255) = 'U'
				,@FactoryID			 NVARCHAR(255) = 'ZT'
				,@PropertyID		 NVARCHAR(255) = 'rreo'
				,@PropertyName		 NVARCHAR(255) = ''
				,@CommentUser		 NVARCHAR(MAX) = ''
				,@CommentDev		 NVARCHAR(MAX) = ''
				,@Unit				 NVARCHAR(255) = ''
				,@ValueText			 NVARCHAR(MAX) = ''
				,@ValueInt			 NVARCHAR(255) = '678'
				,@Scale				 NVARCHAR(255) = '<#NV>'
				,@IsROSystemProperty NVARCHAR(255) = '1'
				,@FormatID			 NVARCHAR(255) = '<#NV>'

		EXECUTE @RC = dbo.sx_pf_POST_ProductLineProperty 
				@Username,
				@ProductLineID,
				@FactoryID,
				@PropertyID,	
				@PropertyName,
				@CommentUser,
				@CommentDev,
				@Unit,
				@ValueText,
				@ValueInt,	
				@Scale,
				@IsROSystemProperty,
				@FormatID
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductLineProperty',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_ProductLineProperty','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_ProductLineProperty
	@Username			NVARCHAR(255),
	@ProductLineID		NVARCHAR(255),
	@FactoryID			NVARCHAR(255),
	@PropertyID			NVARCHAR(255),
	@PropertyName		NVARCHAR(255),
	@CommentUser		NVARCHAR(MAX),
	@CommentDev			NVARCHAR(MAX),
	@Unit 				NVARCHAR(255),
	@ValueText 			NVARCHAR(MAX),
	@ValueInt 			NVARCHAR(255),
	@Scale				NVARCHAR(255),
	@IsROSystemProperty NVARCHAR(255),
	@FormatID			NVARCHAR(255)
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @PropertyKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,		N'NULL') + N''',''' + ISNULL(@ProductLineID,	N'NULL') + N''',''' + ISNULL(@FactoryID,			N'NULL') + N''',''' 
			+ ISNULL(@PropertyID,	N'NULL') + N''',''' + ISNULL(@PropertyName,		N'NULL') + N''',''' + ISNULL(@CommentUser,			N'NULL') + N''',''' 
			+ ISNULL(@CommentDev,	N'NULL') + N''',''' + ISNULL(@Unit,				N'NULL') + N''',''' + ISNULL(@ValueText,			N'NULL') + N''',''' 
			+ ISNULL(@ValueInt,		N'NULL') + N''',''' + ISNULL(@Scale,			N'NULL') + N''',''' + ISNULL(@IsROSystemProperty,	N'NULL') + N''',''' 
			+ ISNULL(@FormatID,		N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;				-- SET during Execution
	DECLARE @ResultCode			INT				= 501;				-- SET during Execution
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';				-- SET during Execution

	-- STEP 0.1 - NULL Protection
	IF @Username		IS NULL SET @Username		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @PropertyID		IS NULL SET @PropertyID		= N'';
	IF @PropertyName	IS NULL SET @PropertyName	= N'';
	IF @CommentUser		IS NULL SET @CommentUser	= N'';
	IF @CommentDev		IS NULL SET @CommentDev		= N'';
	IF @Unit			IS NULL SET @Unit			= N'';
	IF @ValueText		IS NULL SET @ValueText		= N'';
	IF @ValueInt		IS NULL SET @ValueInt		= N'';
	IF @Scale			IS NULL SET @Scale			= N'';
	IF @FormatID		IS NULL SET @FormatID		= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_ProductLineProperty;
				
		-- STEP 0.2 - Protect input parameters	
		SET @ProductLineID	= dbo.sx_pf_pProtectID		(@ProductLineID);
		SET @FactoryID		= dbo.sx_pf_pProtectID		(@FactoryID);
		SET @PropertyID		= dbo.sx_pf_pProtectID		(@PropertyID);
		SET @PropertyName	= dbo.sx_pf_pProtectString	(@PropertyName);
		SET @CommentUser	= dbo.sx_pf_pProtectString	(@CommentUser);
		SET @CommentDev		= dbo.sx_pf_pProtectString	(@CommentDev);
		SET @Unit			= dbo.sx_pf_pProtectString	(@Unit);
		SET @ValueText		= dbo.sx_pf_pProtectString	(@ValueText);
		SET @ValueInt		= dbo.sx_pf_pProtectInt		(@ValueInt);
		SET @Scale			= dbo.sx_pf_pProtectInt		(@Scale);
		SET @FormatID		= dbo.sx_pf_pProtectString	(@FormatID); --as it can be #<NV>

		IF	@FactoryID	= N'' 
			OR @ProductLineID = N'' 
			OR @PropertyID	= N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;

		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey = FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey = ProductLineKey FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @PropertyKey = PropertyKey FROM planning.tgProductLines WHERE ProductLineKey =  @ProductLineKey AND PropertyID = @PropertyID;

		IF	@FactoryKey = 0 
			OR @ProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- STEP 1.3 - Don`t allow changing of system RO Properties - Disabled from version 4.0.63 on to enable export/import
		--SELECT @PropertyOwner = IsROSystemProperty FROM planning.tgProductLines WHERE ProductLineKey = @ProductLineKey AND PropertyID = @PropertyID;
		--IF @PropertyOwner = 1
		--BEGIN
		--	SET @ResultCode = 403;
		--	RAISERROR('Don`t allow changing of system RO Properties', 16, 10);
		--END;

		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
		
		-- STEP 3.1 - If Key does not yet exist, it will be created in minimal configuration		
		IF @PropertyKey = 0
		BEGIN
			DECLARE @Keys AS TABLE ([Key] INT NOT NULL);

			INSERT INTO planning.tgProductLines (ProductLineKey, FactoryKey, ProductLineID, FactoryID, PropertyID, PropertyName, CommentUser, CommentDev, Unit, ValueText, ValueInt, Scale, IsROSystemProperty, FormatID) 
			OUTPUT INSERTED.PropertyKey INTO @Keys([Key]) 
			VALUES (@ProductLineKey, @FactoryKey, @ProductLineID, @FactoryID, @PropertyID, '', '', '', '', '', 0, 1, 0, '');

			SELECT TOP (1) @PropertyKey = [Key] FROM @Keys ORDER BY [Key];

			SET @EffectedRows = 1;
			SET @Resultcode = 201;
		END;

		-- STEP 3.2 - The property receives the transmitted attributes undelivered be maintained
		UPDATE planning.tgProductLines 
		SET 
			PropertyName			= IIF(@PropertyName			= N'<#NV>', PropertyName,			@PropertyName)
			, CommentUser			= IIF(@CommentUser			= N'<#NV>', CommentUser,			@CommentUser)
			, CommentDev			= IIF(@CommentDev			= N'<#NV>', CommentDev,				@CommentDev)
			, Unit					= IIF(@Unit					= N'<#NV>', Unit,					@Unit)
			, ValueText				= IIF(@ValueText			= N'<#NV>', ValueText,				@ValueText)
			, ValueInt				= IIF(@ValueInt				= N'<#NV>', ValueInt,				@ValueInt)
			, Scale					= IIF(@Scale				= N'<#NV>', Scale,					@Scale)
			, IsROSystemProperty	= IIF(@IsROSystemProperty	= N'<#NV>', IsROSystemProperty,		@IsROSystemProperty)
			, FormatID				= IIF(@FormatID				= N'<#NV>', FormatID,				@FormatID)
		FROM planning.tgProductLines
		WHERE PropertyKey = @PropertyKey;

		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION sx_pf_POST_ProductLineProperty;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_ProductLineProperty;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_ProductLineProperty TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_ProductLineProperty'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for ProductLineProperty
         Not existing ProductLine will be created, existing gets an Update
         If attributes passed with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyID';
    SET @value      = N'PropertyID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PropertyName';
    SET @value      = N'PropertyName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentUser';
    SET @value      = N'CommentUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CommentDev';
    SET @value      = N'CommentDev';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Unit';
    SET @value      = N'Unit';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueText';
    SET @value      = N'ValueText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValueInt';
    SET @value      = N'ValueInt';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Scale';
    SET @value      = N'Scale';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsROSystemProperty';
    SET @value      = N'IsROSystemProperty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FormatID';
    SET @value      = N'FormatID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
