﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Template;
GO

/*
	POST Operation to apply a template (again) on a Product
	Existing Values in the Product will be saved, new/other logic will be added
	It means the Template inherits its properties again to the object

	The logical process is describe in procedure POST_TEMPLATE_ALL to which the work is delegated

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Test call
		DECLARE  @RC								INT 
				,@Username						NVARCHAR(255) = 'SQL'
				,@ProductID						NVARCHAR(255) = '2'
				,@ProductLineID					NVARCHAR(255) = 'U'
				,@FactoryID						NVARCHAR(255) = 'ZT'
		
				,@TemplateProductID				NVARCHAR(255) = '1'
				,@TemplateProductLineID			NVARCHAR(255) = 'U'
				,@TemplateFactoryID				NVARCHAR(255) = 'ZT'
				,@CompareByNameInsteadOfIDFlag	INT = 0

		EXECUTE @RC = dbo.sx_pf_POST_Template 
				 @Username
				,@ProductID
				,@ProductLineID
				,@FactoryID
				,@TemplateProductID
				,@TemplateProductLineID
				,@TemplateFactoryID
				,@CompareByNameInsteadOfIDFlag

		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template','PARAMETER',NULL)

*/

CREATE PROCEDURE dbo.sx_pf_POST_Template
	@Username						NVARCHAR(255),
	@ProductID						NVARCHAR(255),
	@ProductLineID					NVARCHAR(255),
	@FactoryID						NVARCHAR(255),
	@TemplateProductID				NVARCHAR(255),
	@TemplateProductLineID			NVARCHAR(255),
	@TemplateFactoryID				NVARCHAR(255),
	@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN
	DECLARE  @TransactUsername				NVARCHAR(255)	= N''
			,@FactoryKey						INT				= 0
			,@ProductLineKey					INT				= 0
			,@ProductKey						INT				= 0
			,@TemplateFactoryKey				INT				= 0
			,@TemplateProductLineKey			INT				= 0
			,@TemplateProductKey				INT				= 0
			,@TemplateName					NVARCHAR(255)	= N''
			,@ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString				NVARCHAR(MAX)	= N'''' 
														+ ISNULL(@Username,					N'NULL') + N''',''' 
														+ ISNULL(@ProductID,				N'NULL') + N''',''' 
														+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
														+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
														+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
														+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
														+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL')
			,@EffectedRows					INT				= 0				
			,@ResultCode					INT				= 501				
			,@TimestampCall					DATETIME		= GETUTCDATE()
			,@Comment						NVARCHAR(2000)	= N'';


	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		-- No transaction started, as delegated to other Procedure

		-- STEP 0.2 - Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductID						= dbo.sx_pf_pProtectID	(@ProductID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductLineID			= dbo.sx_pf_pProtectID	(@TemplateProductLineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);
	
		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
		
		-- STEP 0.3. - Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM planning.tdFactories		WHERE FactoryID		= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey	= @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID		= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey	= @TemplateFactoryKey AND ProductLineID = @TemplateProductLineID;

		-- Stop on invalid Keys
		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @TemplateFactoryKey = 0 OR @TemplateProductLineKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		 -- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
		BEGIN

			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineReadRight @TransactUsername, @FactoryID, @ProductLineID;

			IF @ResultCode <> 200
				RAISERROR('Invalid rights', 16, 10);
		END;

		-- STEP 2.1 - Try to identify the product keys
		SELECT @ProductKey			= ProductKey FROM planning.tdProducts WHERE ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
		SELECT @TemplateProductKey	= ProductKey FROM planning.tdProducts WHERE ProductLineKey = @TemplateProductLineKey AND ProductID = @TemplateProductID;
		
		-- Stop if Keys don't exist
		IF @ProductKey = 0 OR @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- Stop if Template is posted on itself
		IF @ProductKey = @TemplateProductKey
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Stop, as Template cant be posted on itself.', 16, 10);
		END;

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in Template
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND	EXISTS (SELECT COUNT(NameShort) FROM planning.tdValueSeries WHERE ProductKey = @TemplateProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Template', 16, 10);
		END

		-- STEP 2.3 - Stop, if compare by name and there are doubled ValueSerieNames in Product
		IF	@CompareByNameInsteadOfIDFlag = 1 
			AND EXISTS (SELECT COUNT(NameShort) FROM planning.tdValueSeries WHERE ProductKey = @ProductKey GROUP BY NameShort HAVING COUNT(NameShort) > 1)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique in Target Product', 16, 10);
		END

		-- Determine Name of SearchTempale
		SELECT @TemplateName = Template FROM planning.tdProducts WHERE ProductKey = @TemplateProductKey;
		
		-- DELEGATE Operation to POST_TEMPLATE ALL #############################################################################
		EXEC @Resultcode = dbo.sx_pf_POST_Template_All 
					 'SQL'
					 ,@ProductLineID
					 ,@FactoryID
					 ,@ProductID
					 ,@TemplateName --for @SearchTemplateName
					 ,@TemplateProductID
					 ,@TemplateProductLineID
					 ,@TemplateFactoryID
					 ,@CompareByNameInsteadOfIDFlag;
		

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO

ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryService; 
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Template'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation to apply a template (again) on a Product
         Existing Values in the Product will be saved, new/other logic will be added
         It means the Template inherits its properties again to the object';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductID';
    SET @value      = N'TemplateProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductLineID';
    SET @value      = N'TemplateProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateFactoryID';
    SET @value      = N'TemplateFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompareByNameInsteadOfIDFlag';
    SET @value      = N'CompareByNameInsteadOfIDFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
