﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Template_All;
GO

/*
	POST Operation to apply a template (again) to many Products

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9

	Function principles:
		- Determine all Products in Scope and put them on a ToDo list
		- the fact table values from the existing products are temporary stored
			- numbers with scale information
			- formulas are not stored (but their result values, as this may now be a input value series)
		- the existing products are caved out
			- values are deleted
			- timeline is deleted
			- value series are deleted
		- the template is caved in
			- timeline is inserted
			- value series are inserted
			- all values and formulas are inserted
		- the old values are partly caved in 
			- as long as they now belong of a value series of type input
			- they are also caved in for value series of type XLS/XLS Strict - to keep there values which are not formula based
			- transfers all existing data which now belongs to a column of type input
	
	Search Conditions:
		- Factory ZT is never used as target
		- empty Factory ID		-> all Factories are effected
		- empty ProductLineID	-> all ProductLines effected
		- empty Search Template -> all Products effected
 
 
	Testcall 
		DECLARE  @RC							INT
				,@Username						NVARCHAR(255) = 'SQL'
			
				,@ProductID						NVARCHAR(255) = '2'
				,@ProductLineID					NVARCHAR(255) = '1'
				,@FactoryID						NVARCHAR(255) = '1'
				,@SearchTemplateName			NVARCHAR(255) = 'Unikum_VM'
			
				,@TemplateProductID				NVARCHAR(255) = '1'
				,@TemplateProductLineID			NVARCHAR(255) = 'U' 
				,@TemplateFactoryID				NVARCHAR(255) = 'ZT'

				,@CompareByNameInsteadOfIDFlag	INT			  = 1;

		EXEC @RC = dbo.sx_pf_POST_Template_All 
			 @Username,
			 @ProductLineID,
			 @FactoryID,
			 @ProductID,
			 @SearchTemplateName,
			 @TemplateProductID,
			 @TemplateProductLineID,
			 @TemplateFactoryID,
			 @CompareByNameInsteadOfIDFlag;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template_All',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Template_All','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Template_All
	 @Username						NVARCHAR(255)
	,@ProductLineID					NVARCHAR(255)
	,@FactoryID						NVARCHAR(255)
	,@ProductID						NVARCHAR(255) = ''
	,@SearchTemplateName			NVARCHAR(255)
	,@TemplateProductID				NVARCHAR(255)
	,@TemplateProductLineID			NVARCHAR(255)
	,@TemplateFactoryID				NVARCHAR(255)
	,@CompareByNameInsteadOfIDFlag	INT
AS 
BEGIN

	DECLARE @TransactUsername				NVARCHAR(255)	= N''
			,@FactoryKey					INT				= 0
			,@ProductLineKey				INT				= 0
			,@ProductKey					INT				= 0
			,@TemplateFactoryKey			INT				= 0
			,@TemplateProductLineKey		INT				= 0
			,@TemplateProductKey			INT				= 0
			,@TemplateFactoryNameShort		NVARCHAR(255)	= N''
			,@TemplateProductLineNameShort	NVARCHAR(255)	= N''
			,@TemplateProductNameShort		NVARCHAR(255)	= N''
			,@TemplateName					NVARCHAR(255)	= N''
			,@ProductCount					BIGINT
			,@CounterSelf					INT
			
			,@ProcedureName					NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString				NVARCHAR(MAX)	= N'''' 
															+ ISNULL(@Username,					N'NULL') + N''',''' 
															+ ISNULL(@ProductLineID,			N'NULL') + N''',''' 
															+ ISNULL(@FactoryID,				N'NULL') + N''',''' 
															+ ISNULL(@SearchTemplateName,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductID,		N'NULL') + N''',''' 
															+ ISNULL(@TemplateProductLineID,	N'NULL') + N''',''' 
															+ ISNULL(@TemplateFactoryID,		N'NULL') + N''','	 
															+ ISNULL(CAST(@CompareByNameInsteadOfIDFlag AS NVARCHAR(255)), N'NULL')
			,@EffectedRows					INT				= 0			
			,@ResultCode					INT				= 501		
			,@TimestampCall					DATETIME		= GETUTCDATE()
			,@Comment						NVARCHAR(2000)	= N'';			

	-- STEP A1 - NULL Protection
	IF @Username						IS NULL SET @Username						= N'';
	IF @FactoryID						IS NULL SET @FactoryID						= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID					= N'';
	IF @ProductID						IS NULL SET @ProductID						= N'';
	IF @SearchTemplateName				IS NULL SET @SearchTemplateName				= N'';
	IF @TemplateProductID				IS NULL SET @TemplateProductID				= N'';
	IF @TemplateProductLineID			IS NULL SET @TemplateProductLineID			= N'';
	IF @TemplateFactoryID				IS NULL SET @TemplateFactoryID				= N'';
	IF @CompareByNameInsteadOfIDFlag	IS NULL SET @CompareByNameInsteadOfIDFlag	= 0;
		 
	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Template_All;

		-- ***********************************************
		-- Do Standard Protection
		-- ***********************************************

		-- Protect input parameters	
		SET @FactoryID						= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID					= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID						= dbo.sx_pf_pProtectID	(@ProductID);
		SET @SearchTemplateName				= dbo.sx_pf_pProtectID	(@SearchTemplateName);
		SET @TemplateFactoryID				= dbo.sx_pf_pProtectID	(@TemplateFactoryID);
		SET @TemplateProductLineID			= dbo.sx_pf_pProtectID	(@TemplateProductLineID);
		SET @TemplateProductID				= dbo.sx_pf_pProtectID	(@TemplateProductID);
		SET @CompareByNameInsteadOfIDFlag	= dbo.sx_pf_pProtectInt (@CompareByNameInsteadOfIDFlag);

		
		-- Exit if CompareByNameInsteadOfIDFlag not 0 or 1
		IF NOT (@CompareByNameInsteadOfIDFlag = 0 OR @CompareByNameInsteadOfIDFlag = 1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid CompareByNameInsteadOfIDFlag parameter', 16, 10);
		END
		
		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;

		-- Determine keys
		SELECT @FactoryKey				= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @FactoryID;
		SELECT @ProductLineKey			= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @FactoryKey				AND ProductLineID	= @ProductLineID;
		SELECT @ProductKey				= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey	= @ProductLineKey			AND ProductID		= @ProductID;
		SELECT @TemplateFactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID			= @TemplateFactoryID;
		SELECT @TemplateProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey		= @TemplateFactoryKey		AND ProductLineID	= @TemplateProductLineID;
		SELECT @TemplateProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey	= @TemplateProductLineKey	AND ProductID		= @TemplateProductID;
		SELECT @TemplateName			= Template			FROM planning.tdProducts 		WHERE ProductKey		= @TemplateProductKey

		-- Stop if Template is invalid (Key of Target Factory / PL may all be 0)
		IF @TemplateProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- ***********************************************
		-- Create a ToDo List of all Products in scope
		-- ***********************************************
		
		DROP TABLE IF EXISTS #ProductToDoList;

		CREATE TABLE #ProductToDoList
		(
			 FactoryKey		BIGINT										NOT NULL
			,ProductLineKey BIGINT										NOT NULL
			,ProductKey		BIGINT										NOT NULL
			,FactoryID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
		);

		-- Set all Products of scope on the ToDo List
		INSERT INTO #ProductToDoList
			SELECT 
				 dP.FactoryKey
				,dP.ProductLineKey
				,dP.ProductKey
				,dP.FactoryID
				,dP.ProductLineID 
				,dP.ProductID
			FROM planning.tdProducts dP 
			WHERE 
					 
					(dP.FactoryKey		= @FactoryKey			OR @FactoryID			= '') 
				AND (dP.ProductLineKey	= @ProductLineKey		OR @ProductLineID		= '') 
				AND (dP.ProductKey		= @ProductKey			OR @ProductID			= '')
				AND (dP.Template		= @SearchTemplateName	OR @SearchTemplateName	= '')
				AND  dP.ProductKey		<> @TemplateProductKey;									-- don't POST on yourself
			
		-- Stop if Template is inside target Products
		SELECT @CounterSelf = COUNT(ProductKey) 
		FROM #ProductToDoList 
		WHERE Productkey = @TemplateProductKey

		IF @CounterSelf > 0
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Template is in same ProductLine as Target - stopped to not post template on itself.', 16, 10);
			END;

		
		-- Clean ToDo List from Products where user don't has rights
		DELETE FROM #ProductToDoList 
		WHERE ProductKey IN	
		(
			SELECT ProductKey 
			FROM #ProductToDoList ToDo LEFT JOIN system.trUserRights	tUR
				ON ToDo.FactoryID = tUR.FactoryID AND ToDo.ProductLineID = tUR.ProductLineID
			WHERE 
				tUR.[Right] != 'Write' 
				AND tUR.UserName = @TransactUsername
		);

		SELECT @ProductCount = COUNT(ProductKey) FROM #ProductToDoList;

		IF @ProductCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No fitting products found', 16, 10);
		END

		-- STEP 2.2 - Stop, if compare by name and there are doubled ValueSerieNames in any product
		IF	@CompareByNameInsteadOfIDFlag = 1 
		AND	EXISTS (SELECT 
						 dVS.ProductKey
						,COUNT(NameShort)
					FROM planning.tdValueSeries dVS 
						-- INNER JOIN to filter on Product on ToDo List
						INNER JOIN #ProductToDoList ToDO
							ON ToDo.ProductKey = dVS.ProductKey
					 GROUP BY dVS.ProductKey, NameShort 
					 HAVING COUNT(NameShort)>1
					)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('ValueSeriesNames not unique', 16, 10);
		END;

		-- ***********************************
		-- STAGING Existing Data from Produts
		-- ***********************************
		

		-- Table for storing static Data
		CREATE TABLE #ProductDataStorage (
			 ProductKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,TimeID				INT											NOT NULL
			,ValueInt			BIGINT										NOT NULL
			,ValueText			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueComment		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,[IsNumeric]		INT											NOT NULL
			);

		-- Store Data of Products in Storage (but not formulas, they come from template)
		-- ToDo: Keep new formulas out in this step !!
		INSERT INTO #ProductDataStorage    
			SELECT
				 fV.ProductKey 
				,fV.ValueSeriesID
				,dVS.NameShort	AS ValueSeriesName
				,dVS.Scale
				,fV.TimeID
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment
				,dVS.[IsNumeric]
			FROM 
				planning.tfValues fV 
				-- for ValueSeriesNames
				LEFT JOIN planning.tdValueSeries dVS 
					ON	fV.ValueSeriesKey = dVS.ValueSeriesKey
				-- filter Products on ToDo List
				INNER JOIN	#ProductToDoList ToDo 
					ON fV.ProductKey = ToDo.ProductKey;

		-- ***********************************
		-- Cave out existing Products
		-- ***********************************
		
		-- STEP 3.2 - Delete all Values and ValueSeries of old Products (but not the dProduct, Statements...)
		DELETE FROM planning.tfValues		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdValueSeries	WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tdTime			WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		DELETE FROM planning.tgProducts		WHERE ProductKey IN (SELECT DISTINCT ProductKey FROM #ProductToDoList);
		SET @EffectedRows += @@ROWCOUNT;

		-- ***********************************
		-- Cave in Template
		-- ***********************************

		-- keep new VS Keys temporary stored
		CREATE TABLE #VSKeys(
			 ValueSeriesKey		BIGINT										NOT NULL
			,ProductKey			BIGINT										NOT NULL
			,ProductLineKey		BIGINT										NOT NULL
			,FactoryKey			BIGINT										NOT NULL
			,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,ValueSeriesName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
			,Scale				INT											NOT NULL
			,ValueSource		NVARCHAR(255)	COLLATE DATABASE_DEFAULT 	NOT NULL
		);

		-- Copy ValueSeries of Template to all Products and store their Keys
		INSERT INTO planning.tdValueSeries
				OUTPUT	 INSERTED.ValueSeriesKey
						,INSERTED.ProductKey
						,INSERTED.ProductLineKey
						,INSERTED.FactoryKey
						,INSERTED.ValueSeriesID
						,INSERTED.ProductID
						,INSERTED.ProductLineID
						,INSERTED.FactoryID
						,INSERTED.NameShort
						,INSERTED.Scale 
						,INSERTED.ValueSource
				INTO #VSKeys
			SELECT 
				ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,dVS.ValueSeriesID
				,dVS.ValueSeriesNo
				,dVS.NameShort
				,dVS.NameLong
				,dVS.CommentUser
				,dVS.CommentDev
				,dVS.ImageName
				,dVS.IsNumeric
				,dVS.VisibilityLevel
				,dVS.ValueSource
				,dVS.ValueListID
				,dVS.ValueFormatID
				,dVS.Unit
				,dVS.Scale
				,dVS.Effect
				,dVS.EffectParameter
			FROM planning.tdValueSeries dVS
					LEFT JOIN #ProductToDoList ToDO 
						ON 1=1
			WHERE dVS.ProductKey = @TemplateProductKey;

		-- Copy Timeline of Template to Product 
		INSERT INTO planning.tdTime
			SELECT 
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,dT.TimeID
				,dT.FormatID
			FROM planning.tdTime dT-- spread to all products
					LEFT JOIN #ProductToDoList ToDO 
						ON 1=1
			WHERE dT.ProductKey = @TemplateProductKey;

		-- Copy Values and Formulas of Template to Products 
		INSERT INTO planning.tfValues
			SELECT
				 vk.ValueSeriesKey
				,vk.ProductKey
				,vk.ProductLineKey
				,vk.FactoryKey
				,vk.ProductID
				,vk.ProductLineID
				,vk.FactoryID
				,fV.ValueSeriesID
				,fV.TimeID
				,fV.ValueFormula
				,fV.ValueInt
				,fV.ValueText
				,fV.ValueComment
			FROM planning.tfValues fV 
				LEFT JOIN #VSKeys vk 
					ON	fV.ValueSeriesID = vk.ValueSeriesID
			WHERE fV.ProductKey = @TemplateProductKey;

		-- Step 3.4 Copy Entrys in gProducts
		INSERT INTO planning.tgProducts
			SELECT
				 ToDo.ProductKey
				,ToDo.ProductLineKey
				,ToDo.FactoryKey
				,ToDo.ProductID
				,ToDo.ProductLineID
				,ToDo.FactoryID
				,gP.PropertyID
				,gP.PropertyName
				,gP.CommentUser
				,gP.CommentDev
				,gP.Unit
				,gP.ValueText
				,gP.ValueInt
				,gP.Scale
				,gP.IsROSystemProperty
				,gP.FormatID

			FROM planning.tgProducts gP 
			-- spread to all products
				LEFT JOIN #ProductToDoList ToDO 
					ON 1=1
                    
			WHERE gP.ProductKey = @TemplateProductKey

		--  ===============================================================
		--> Now all Products are complete fresh 1:1 copies of the Template
		--  ===============================================================

		-- *****************************************************************
		-- cave in the transfer values if they still fit to the template
		-- *****************************************************************

		-- Possible Changes:
			-- New ValueSeries where created								=>	ok, exists empty
			-- ValueSeries where deleted									=>	ok, values lost
			-- ValueSeries of type Formula where changed in its definition	=>	ok, new formulas are used
			-- The timeline was changed										=>	ok, only fitting values are transfered
			-- ValueSeriesID or Names may be changed						=>	ok, but one must keept constant
			-- Typ of Input Series has changed from Int to Text etc.		=>	ok, empty then
			-- Scale of numeric Input Series has changed					=>  ok, old values will be rescaled


		-- Delete Transfervalues, which belong to no longer existing TimeIDs
		DELETE FROM #ProductDataStorage
		WHERE TimeID NOT IN	(
			SELECT dT.TimeID
			FROM planning.tdTime dT
			WHERE dT.ProductKey = @TemplateProductKey 
		);

		-- Transfer stored Values by ID or Name
		IF @CompareByNameInsteadOfIDFlag = 0 -- compare by ID
			BEGIN
				-- Delete Transfervalues, which belong to no longer existing IDs
				DELETE FROM #ProductDataStorage 
				WHERE ValueSeriesID NOT IN	(
					SELECT dVS.ValueSeriesID 
					FROM planning.tdValueSeries dVS
					WHERE dVS.ProductKey = @TemplateProductKey 
				);	

				-- Delete Transfer Values, which belong to Value Series, which DataType has changed
				DELETE pds
				FROM #ProductDataStorage pds
				LEFT JOIN (	SELECT 
								 dVS.ValueSeriesID
								,dVS.[IsNumeric]
							FROM planning.tdValueSeries dVS
							WHERE dVS.ProductKey = @TemplateProductKey
						  ) Template
					ON pds.ValueSeriesID = Template.ValueSeriesID
				WHERE pds.[IsNumeric] <> Template.[IsNumeric];	
				
				-- Values, which belong to value series, which are formula based now are not deleted completely
					-- they are keept in case that the new cell has no formula  
					-- this happens, to keep value in value series of type XLS-Strict, where XLS-Strict just protects the cell
				DELETE FROM  #ProductDataStorage 					
				WHERE 		
					-- defined as formula in Template
					EXISTS
							(
								SELECT 
										fV.TimeID
									,fV.ValueSeriesID
								FROM planning.tfValues fV
									LEFT JOIN planning.tdValueSeries dVS
										ON fV.ValueSeriesKey = dVS.ValueSeriesKey
								WHERE 
										fV.ProductKey	= @TemplateProductKey 
									AND dVS.ValueSource IN ('XLS','XLS-Strict')
									AND fV.ValueFormula <> ''
									-- OUTER Correlation
									AND #ProductDataStorage.TimeID			= fV.TimeID
									AND #ProductDataStorage.ValueSeriesID	= fV.ValueSeriesID
							);
				
				-- Delete Values in fact table to make space for Transfervalues -> WHY DO we need this, when all Products are fresh copies of the template ?
				DELETE fV FROM planning.tfValues fV
					INNER JOIN #ProductToDoList ToDo
						ON fV.ProductKey = ToDo.ProductKey

					INNER JOIN #ProductDataStorage PDS
						ON		ToDo.ProductKey		= PDS.ProductKey
							AND fV.ValueSeriesID	= PDS.ValueSeriesID
							AND fV.TimeID			= PDS.TimeID;

				-- Transfer OVER INSERT
				INSERT INTO planning.tfValues
					SELECT 
						 vk.ValueSeriesKey
						,vk.ProductKey
						,vk.ProductLineKey
						,vk.FactoryKey
						,vk.ProductID
						,vk.ProductLineID
						,vk.FactoryID
						,pds.ValueSeriesID
						,pds.TimeID
						,'' AS ValueFormula
						,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values
						,pds.ValueText
						,pds.ValueComment
					FROM #ProductDataStorage pds LEFT JOIN #VSKeys vk 
						ON		pds.ProductKey = vk.ProductKey 
							AND pds.ValueSeriesID = vk.ValueSeriesID;

			END
		ELSE
			BEGIN -- compare by Name

				-- Delete Transfervalues, which belong to no longer existing Names
				DELETE FROM #ProductDataStorage
				WHERE ValueSeriesName NOT IN (
					SELECT dVS.NameShort
					FROM planning.tdValueSeries dVS
					WHERE dVS.ProductKey = @TemplateProductKey 
				);

				-- Delete Transfer Values, which belong to Value Series, which DataType has changed
				DELETE pds
				FROM #ProductDataStorage pds
				LEFT JOIN (	SELECT 
								 dVS.ValueSeriesName
								,dVS.[IsNumeric]
							FROM planning.tdValueSeries dVS
							WHERE dVS.ProductKey = @TemplateProductKey
						  ) Template
					ON pds.ValueSeriesName = Template.ValueSeriesName
				WHERE pds.[IsNumeric] <> Template.[IsNumeric];	

				DELETE FROM  #ProductDataStorage 					
				WHERE 		
					-- defined as formula in Template
					EXISTS
							(
								SELECT 
									 fV.TimeID
									,dVS.NameShort
								FROM planning.tfValues fV
									LEFT JOIN planning.tdValueSeries dVS
										ON fV.ValueSeriesKey = dVS.ValueSeriesKey
								WHERE 
										fV.ProductKey	= @TemplateProductKey 
									AND dVS.ValueSource IN ('XLS','XLS-Strict')
									AND fV.ValueFormula <> ''
									-- OUTER Correlation
									AND #ProductDataStorage.TimeID			= fV.TimeID
									AND #ProductDataStorage.ValueSeriesName	= dVS.NameShort
							);


				-- Delete Values in fact table to make space for Transfervalues
				DELETE fV FROM planning.tfValues fV

					LEFT JOIN planning.tdValueSeries dVS
						ON fV.ValueSeriesKey = dVS.ValueSeriesKey

					LEFT JOIN #ProductToDoList ToDo
						ON fV.ProductKey = ToDo.ProductKey

					LEFT JOIN #ProductDataStorage PDS
						ON		ToDo.ProductKey		= PDS.ProductKey
							AND dVS.NameShort		= PDS.ValueSeriesName
							AND fV.TimeID			= PDS.TimeID
				WHERE 
						PDS.TimeID			IS NOT NULL 
					AND PDS.ValueSeriesName	IS NOT NULL

				-- Insert all Transfervalues
				INSERT INTO planning.tfValues
				SELECT 
					 vk.ValueSeriesKey
					,vk.ProductKey
					,vk.ProductLineKey
					,vk.FactoryKey
					,vk.ProductID
					,vk.ProductLineID
					,vk.FactoryID
					,dVS.ValueSeriesID
					,pds.TimeID
					,'' AS ValueFormula
					,CAST(CAST(pds.ValueInt AS MONEY) / pds.Scale * vk.Scale AS BIGINT) --rescaling the int Values 
					,pds.ValueText
					,pds.ValueComment
				FROM #ProductDataStorage pds 
					-- to determine ValueSeries Key
					LEFT JOIN #VSKeys vk 
						ON		pds.ProductKey = vk.ProductKey 
							AND pds.ValueSeriesName = vk.ValueSeriesName
					-- to determine new ValueSeriesID
					LEFT JOIN 
						(
							SELECT 
								 dVS.ValueSeriesID
								,dVS.NameShort 
							FROM planning.tdValueSeries dVS 
							WHERE dVS.ProductKey = @TemplateProductKey
						) dVS 
					ON dVS.NameShort = pds.ValueSeriesName

			END;

		-- Create a System Statement
		INSERT INTO planning.tfStatements
			SELECT  
				 ProductKey
				,ProductLineKey
				,FactoryKey
				,ProductID
				,ProductLineID
				,FactoryID
				,N'POST_Template_All' AS ActionType
				,N'Product Template updated with Template Factory <'  
					+ @TemplateFactoryID			+ N'>"' 
					+ @TemplateFactoryNameShort		+ N'"/ ProductLine <' 
					+ @TemplateProductLineID		+ N'>"' 
					+ @TemplateProductLineNameShort + N'"/ Product <' 
					+ @TemplateProductID			+ N'>"' 
					+ @TemplateProductNameShort		+ '"' AS Statement
				,@TransactUsername AS UserName
				,N'' AS PCName
				,N'' AS ProcessorCode
				,N'' AS IPAddresses
				,GETUTCDATE() AS Timestamp
				,0 AS IsDeleted
				,0 AS IsResolved
				FROM #ProductToDoList 
			
			-- Update the TemplateName of the Products
			UPDATE planning.tdProducts
			SET  
				 Template = @TemplateName
				,TimeType = RIGHT(@TemplateName, 2) 
			WHERE 
				ProductKey IN (SELECT ProductKey FROM #ProductToDoList);

		SET @EffectedRows += @@ROWCOUNT;
		
		SET @ResultCode = 200;

		COMMIT TRANSACTION sx_pf_POST_Template_All;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Template_All;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
  
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryService;
GRANT EXECUTE ON OBJECT ::dbo.sx_pf_POST_Template_All TO pf_PlanningFactoryUser;
GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Template_All'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'/*
         POST Operation to apply a template (again) to many Products';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SearchTemplateName';
    SET @value      = N'SearchTemplateName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductID';
    SET @value      = N'TemplateProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateProductLineID';
    SET @value      = N'TemplateProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TemplateFactoryID';
    SET @value      = N'TemplateFactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompareByNameInsteadOfIDFlag';
    SET @value      = N'CompareByNameInsteadOfIDFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
