﻿  
DROP PROCEDURE IF EXISTS dbo.sx_pf_POST_Timeline;
GO

/*
	POST Operation for the Timeline
	The method can be switched between incremental POST and full POST
	Values with DeleteFlag will be deleted

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9

	Test case 1 - Incremental Sended
	1. One Value for an existing Value must replace them
	2. One Value for an not exiting Value must create it
	3. If all Parameter are posted empty, the value is deleted
	4. if one or more of the parameter are sended with <#NV> there Value is replaced with the existing value (or nothing if it exists not)
		DECLARE @RC int
		DECLARE @Username					NVARCHAR(255) = 'SQL'
		DECLARE @ProductID					NVARCHAR(255) = '1'
		DECLARE @ProductLineID				NVARCHAR(255) = 'U'
		DECLARE @FactoryID					NVARCHAR(255) = 'ZT'
		DECLARE @IsIncrementalValuesFlag	INT = 0

		--3 TimeIDs
		DECLARE @TimeIDsInBracketsCommaSeparated nvarchar(max) = '[(20160115,0,0,''FormatID1'')],[(20160215,0,0,''FormatID2'')],[(20160415,0,0,''FormatID3'')]'
		EXECUTE @RC = dbo.sx_pf_POST_Timeline
			 @Username
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,@IsIncrementalValuesFlag
			,@TimeIDsInBracketsCommaSeparated
		PRINT @RC

		Test case 2 - all values in a Value Series of Products:
		Additional 1. Empty FactoryID / ProductLineID / ProductID / ValueSeriesID / TimeID => 404 Not Found
		DECLARE @RC int
		DECLARE @Username							NVARCHAR(255) = 'SQL'
		DECLARE @ProductID							NVARCHAR(255) = '21'
		DECLARE @ProductLineID						NVARCHAR(255) = '1'
		DECLARE @FactoryID							NVARCHAR(255) = 'U'
		DECLARE @IsIncrementalValuesFlag			INT = 0
		DECLARE @TimeIDsInBracketsCommaSeparated	NVARCHAR(max) = ''
		DECLARE @start_time DATETIME

		-- Generate fictitious values for all TimeIDs of Products
		SELECT @TimeIDsInBracketsCommaSeparated =  @TimeIDsInBracketsCommaSeparated + '[(''' + dVS2.ValueSeriesID + ''',' + CAST(TimeID AS nvarchar(255)) +',''=A2'',1000,''Hase'',''Kommentar'')],' 
		FROM planning.tfValues fV LEFT JOIN 
			planning.tdValueSeries dVS ON fV.ValueSeriesKey = dVS.ValueSeriesKey LEFT JOIN
			(SELECT ValueSeriesID FROM planning.tdValueSeries WHERE FactoryID =  @FactoryID AND ProductLineID = @ProductLineID AND ProductID = @ProductID GROUP BY ValueSeriesID) dVS2
				ON 1=1 
		WHERE fV.FactoryID =  @FactoryID AND fV.ProductLineID = @ProductLineID AND fV.ProductID = @ProductID AND dVS.ValueSeriesNo = 1
		SET @TimeIDsInBracketsCommaSeparated = Left(@TimeIDsInBracketsCommaSeparated,Len(@TimeIDsInBracketsCommaSeparated) -1)
		PRINT @TimeIDsInBracketsCommaSeparated
		SET @start_time = GETUTCDATE()
		EXECUTE @RC = dbo.sx_pf_POST_ProductDataTableValues
			 @Username
			,@ProductID
			,@ProductLineID
			,@FactoryID
			,@IsIncrementalValuesFlag
			,@TimeIDsInBracketsCommaSeparated
		PRINT @RC
		SELECT RTRIM(CAST(DATEDIFF(MS, @start_time, GETUTCDATE()) AS CHAR(10))) AS 'TimeTaken'

		@TimeIDsInBracketsCommaSeparated format: (TimeID,DeleteFlag,AddValue,FormatID)
		if you check this in SQL Server Profiler, every String must have two inverted commas, the whole Parameter must have Single inverted comma
		this must show profiler: '[(''TimeID,DeleteFlag,AddValue,''FormatID'')],[(TimeID,DeleteFlag,AddValue,FormatID)]'
		if you check this in table system.tAPI_Log, every String must have one inverted comma, whole String has no inverted comma
		this must show Log: [(TimeID,DeleteFlag,AddValue,FormatID)],[(TimeID,DeleteFlag,AddValue,'FormatID')]

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Timeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_POST_Timeline','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_POST_Timeline
	@Username							NVARCHAR(255),
	@ProductID							NVARCHAR(255),
	@ProductLineID						NVARCHAR(255),
	@FactoryID							NVARCHAR(255),
	@IsIncrementalValuesFlag			INT,
	@TimeIDsInBracketsCommaSeparated	NVARCHAR(MAX)

AS
BEGIN
	SET NOCOUNT ON;

	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @FactoryKey			INT				= 0;
	DECLARE @ProductLineKey		INT				= 0;
	DECLARE @ProductKey			INT				= 0;

	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductID							, N'NULL')	+ N''',''' 
								+ ISNULL(@ProductLineID						, N'NULL')	+ N''',''' 
								+ ISNULL(@FactoryID							, N'NULL')	+ N''',''' 
								+ ISNULL(@TimeIDsInBracketsCommaSeparated	, N'NULL')	+ N'''';
	DECLARE @EffectedRows		INTEGER			= 0;						
	DECLARE @ResultCode			INTEGER			= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR (2000) = N'';					

	-- STEP 0.1 - NULL Protection
	IF @Username						IS NULL SET @Username							= N'';
	IF @ProductID						IS NULL SET @ProductID							= N'';
	IF @ProductLineID					IS NULL SET @ProductLineID						= N'';
	IF @FactoryID						IS NULL SET @FactoryID							= N'';
	IF @IsIncrementalValuesFlag			IS NULL SET @IsIncrementalValuesFlag			= 0;
	IF @TimeIDsInBracketsCommaSeparated IS NULL SET @TimeIDsInBracketsCommaSeparated	= N'';

	BEGIN TRY
		BEGIN TRANSACTION sx_pf_POST_Timeline;
		
		-- STEP 0.2 - Protect input parameters
		SET @FactoryID			= dbo.sx_pf_pProtectID (@FactoryID)
		SET @ProductID			= dbo.sx_pf_pProtectID (@ProductID)
		SET @ProductLineID		= dbo.sx_pf_pProtectID (@ProductLineID)

		IF @FactoryID = N'' OR @ProductLineID = N'' OR @ProductID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;
		
		-- STEP 1.1 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- STEP 1.2 - Determine keys
		SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories		WHERE FactoryID = @FactoryID;
		SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines	WHERE FactoryKey = @FactoryKey AND ProductLineID = @ProductLineID;
		SELECT @ProductKey		= ProductKey		FROM planning.tdProducts		WHERE ProductLineKey =  @ProductLineKey AND ProductID = @ProductID;

		IF @FactoryKey = 0 OR @ProductLineKey = 0 OR @ProductKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;
		
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername, @FactoryID, @ProductLineID;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);
				
		-- STEP 3.2 - Check for count of delimiter in Array

		DECLARE @OpenDelimetersCount INT = 0;
		DECLARE @DelimetersIndex INT = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount += 1;
			SET @DelimetersIndex = CHARINDEX(N'[(', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;
				
		IF @OpenDelimetersCount = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('No delimiters found in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated);

		WHILE (@DelimetersIndex > 0) 
		BEGIN
			SET @OpenDelimetersCount -= 1;
			SET @DelimetersIndex = CHARINDEX(N')]', @TimeIDsInBracketsCommaSeparated, @DelimetersIndex + 1);
		END;

		IF @OpenDelimetersCount <> 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Invalid count of delimiter in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- STEP 3.3 - Splitt the Array with the new values for the Product to a temporary table #######################################################################################################################
		-- cut leading/following brackets and ' from ValueSeriesID
		SET @TimeIDsInBracketsCommaSeparated = LEFT(@TimeIDsInBracketsCommaSeparated,  LEN(@TimeIDsInBracketsCommaSeparated) - 2);
		SET @TimeIDsInBracketsCommaSeparated = RIGHT(@TimeIDsInBracketsCommaSeparated, LEN(@TimeIDsInBracketsCommaSeparated) - 2);

		DECLARE @tblValues1 TABLE (txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues2 TABLE (TimeID		 NVARCHAR(MAX) NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues3 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, txtValuesPipe NVARCHAR(MAX) NOT NULL);
		DECLARE @tblValues4 TABLE (TimeID		 BIGINT		   NOT NULL, DeleteFlag	   INT			 NOT NULL, AddValue		 BIGINT		   NOT NULL, FormatID NVARCHAR(255) NOT NULL);

		-- Split String into Rows
		INSERT INTO @tblValues1 
			SELECT txtValues FROM dbo.sx_pf_PivotStringIntoTable(@TimeIDsInBracketsCommaSeparated, N')],[(');

		-- Split one column into two Columns (separate TimeID)
		INSERT INTO @tblValues2
			SELECT  
				 LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe)-1 ) AS TimeID
				,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)) AS Rest
			FROM @tblValues1;

		IF EXISTS(SELECT TimeID FROM @tblValues2 GROUP BY TimeID HAVING(COUNT(TimeID) > 1))
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Doubled TimeID in @TimeIDsInBracketsCommaSeparated', 16, 10);
		END;

		-- Split last columns once more (separate DeleteFlag)
		INSERT INTO @tblValues3
			SELECT  
				 TimeID
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS DeleteFlag
			,RIGHT(txtValuesPipe, LEN(txtValuesPipe) - CHARINDEX(N',', txtValuesPipe)-1) AS Rest
			FROM @tblValues2;

		-- Split last columns once more (separate AddValue)
		INSERT INTO @tblValues4
			SELECT  
				 TimeID
				,DeleteFlag
				,LEFT(txtValuesPipe, CHARINDEX(N',', txtValuesPipe) - 1) AS AddValue
			,REPLACE(Right(txtValuesPipe, LEN(txtValuesPipe) - 1), N'''','') AS FormatID
			FROM @tblValues3;

		-- STEP 3.4 - Actions #######################################################################################################################
	
		-- Delete TimeIDs which are sended as deleted in fValues and in dTime
		DELETE FROM planning.tdTime 
		WHERE 
			@ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);
	
		SET @EffectedRows = @@ROWCOUNT;

		DELETE FROM planning.tfValues 
		WHERE 
			ProductKey = @ProductKey 
			AND TimeID IN (SELECT TimeID FROM @tblValues4 WHERE DeleteFlag = 1);

		SET @EffectedRows += @@ROWCOUNT;
		-- STEP 3.5 - CASE FULL transmission or Incremental does not matter anymore #########################################
		IF @IsIncrementalValuesFlag != 1 OR  @IsIncrementalValuesFlag = 1 
		BEGIN
			-- Insert all New TimeIDs
			INSERT INTO planning.tdTime
			SELECT  
				@ProductKey,
				@ProductLineKey,
				@FactoryKey,
				@ProductID,
				@ProductLineID,
				@FactoryID,
				TimeID,
				FormatID
			FROM @tblValues4 
			WHERE TimeID NOT IN (SELECT TimeID FROM planning.tdTime WHERE ProductKey = @ProductKey);

			SET @EffectedRows += @@ROWCOUNT;
		END;
			
		COMMIT TRANSACTION sx_pf_POST_Timeline;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION sx_pf_POST_Timeline;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
ALTER AUTHORIZATION ON OBJECT ::dbo.sx_pf_POST_Timeline TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_POST_Timeline'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'POST Operation for the Timeline
         The method can be switched between incremental POST and full POST
         Values with DeleteFlag will be deleted';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@IsIncrementalValuesFlag';
    SET @value      = N'IsIncrementalValuesFlag';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TimeIDsInBracketsCommaSeparated';
    SET @value      = N'TimeIDsInBracketsCommaSeparated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
