
DROP PROCEDURE IF EXISTS dbo.sx_pf_pPOST_API_LogEntry;
GO

/*
	Internal POST Operation for one API LogEntry, always used, when a public procedure is called
	Writes an LogEntry, as long as

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.9

	Testcall
		DECLARE @RESULT AS NVARCHAR(255)
		EXEC @RESULT = dbo.sx_pf_pPOST_API_LogEntry 
			 'SQL'
			,'gerd.tautenhahn@saxess-software.com'
			,'calc.spTest'
			,'ThisAreTheParameter'
			,1
			,403
			,'20150731'
			,'Comment';
		PRINT @RESULT;

	Testcall table
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pPOST_API_LogEntry',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'dbo', 'PROCEDURE', 'sx_pf_pPOST_API_LogEntry','PARAMETER',NULL)
*/

CREATE PROCEDURE dbo.sx_pf_pPOST_API_LogEntry 
	 @Username			AS NVARCHAR(255)
	,@TransactUsername	AS NVARCHAR(255)
	,@ProcedureName		AS NVARCHAR(255)
	,@ParameterString	AS NVARCHAR (MAX)
	,@EffectedRows		AS INT
	,@ReturnCode		AS INT
	,@TimestampCall		AS Datetime
	,@Comment			AS NVARCHAR (2000)

WITH EXECUTE AS 'dbo' -- as a dbo procedure owned by OCTService writes into system table

AS

		DECLARE  @TimestampReturn		AS Datetime			= GETUTCDATE()
				,@ProcessCode			AS NVARCHAR(255)	= N''
				,@DatabaseIsReadOnly	AS INTEGER			= 0;
	
		SELECT @DatabaseIsReadOnly = is_read_only FROM sys.databases WHERE Name = DB_NAME();
		
		IF NOT @DatabaseIsReadOnly = 1
			BEGIN

				-- the functions CURRENT_User may not be avaiable for Azure in impersonation context (WITH EXECUTE AS OWNER...), depening on the object owner.
				BEGIN TRY
					INSERT INTO system.tAPI_Log VALUES (CURRENT_USER,@Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ReturnCode,@TimestampCall,@TimestampReturn,@Comment,@ProcessCode)
				END TRY
				BEGIN CATCH
				    INSERT INTO system.tAPI_Log VALUES ('Impersonated User as dbo',@Username,@TransactUsername,@ProcedureName,@ParameterString,@EffectedRows,@ReturnCode,@TimestampCall,@TimestampReturn,@Comment,@ProcessCode)
				END CATCH
			END

GO

-- Transfered Ownership (procedure must be moved in v6 to system or planning schema)
ALTER AUTHORIZATION ON OBJECT :: dbo.sx_pf_pPOST_API_LogEntry TO pf_PlanningFactoryService;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'dbo'								-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'sx_pf_pPOST_API_LogEntry'		-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'	Internal POST Operation for one API LogEntry, always caused, when a public procedure is called';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TransactUsername';
    SET @value      = N'The name of the user for whom this procedure was executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcedureName';
    SET @value      = N'Name of the stored procedure which was executed';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
   
    SET @level2name = N'@ParameterString';
    SET @value      = N'List of parameters, which were given to the called procedure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@EffectedRows';
    SET @value      = N'Count of effected rows which (returnd from the called procedure). This number my include counts from temporary tables.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@ReturnCode';
    SET @value      = N'signals the result of the called procedure (sucessful, not found objects...)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@TimestampCall';
    SET @value      = N'Timestamp the procedure was called';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
  
    SET @level2name = N'@Comment';
    SET @value      = N'Return messages from the SQL Server or from the procedure for logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
 	GO	
	