DROP PROCEDURE IF EXISTS global.spGET_Companies;
GO

/*
	Procedure to get one or all companies of an Datasource or for all Datasources
	 - use CompanyID = '' to get all companies
	 - use DatasourceID = '' to get the companies of all Datasources

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = global.spGET_Companies
				 @Username		= 'SQL'
				,@DataSourceID	= ''
				,@CompanyID		= ''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Companies',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'PROCEDURE', 'spGET_Companies','PARAMETER',NULL)
*/
 
CREATE PROCEDURE global.spGET_Companies
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50) = ''
		,@CompanyID				NVARCHAR(50) = ''
    )

WITH EXECUTE AS 'dbo'  --to access system tables

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username				,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID			,N'NULL')	+ N''','''
				+ ISNULL(@CompanyID				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@Message			NVARCHAR(2000);

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username			= N'';
		IF @DataSourceID	IS NULL SET @DataSourceID		= N'';
		IF @CompanyID		IS NULL SET @CompanyID			= N'';


		-- Content Protection for specific input parameters (e.g.IDs)
		SET @DataSourceID	= dbo.sx_pf_pProtectID	(@DataSourceID);
		SET @CompanyID		= dbo.sx_pf_pProtectString	(@CompanyID);


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of Datasource in case of specified
				IF @DataSourceID <> '' AND NOT EXISTS (
					SELECT DataSourceKey FROM system.tDatasources WHERE DataSourceKey = CAST(@DataSourceID AS INT) --temporary compatibility
					) AND @DataSourceID <> 0
					BEGIN
						SET @ResultCode = 404;
						SET @Message = CONCAT('DatsourceID ',@DataSourceID,' don`t exits');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				IF @CompanyID <> '' AND NOT EXISTS (
					SELECT CompanyID FROM global.tCompanies WHERE DataSourceKey = CAST(@DataSourceID AS INT) AND CompanyID = @CompanyID --temporary compatibility
					)
					BEGIN
						SET @ResultCode = 404;
						SET @Message = CONCAT('CompanyID ',@CompanyID,' don`t exits');
						PRINT @Message;
						RAISERROR(@Message, 16, 10);
					END

				-- Data Transaction
				PRINT CONCAT(N'Start reading',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					 CAST(DataSourceKey AS NVARCHAR(50)) AS DataSourceID --temporary compatibility
					,CompanyID
					,CompanyName
					,[Description]
					,FinancialYearBegin
					,ChartsOfAccountsID
					,AccountLength
					,Currency
					,CustomValuesJSON
					,CustomValuesJSONUser
					,OwnerCODE
				FROM global.tCompanies
				WHERE 
						(DataSourceKey	= CAST(@DataSourceID AS INT) OR @DataSourceID = '' ) --temporary compatibility
					AND (CompanyID		= @CompanyID OR @CompanyID = '');

				SET @EffectedRows = @@ROWCOUNT;
	
				SET @ResultCode = 200;
				PRINT CONCAT(N'Operation finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO


-- NO GRANT - as EXECUTE AS 'dbo'
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Companies'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all companies of a datasource';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DatasourceID';
    SET @value      = N'optional - ID of an Datasource, send '''' to get all Datasources.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CompanyID';
    SET @value      = N'optional - ID of an Company, send '''' to get all Companies.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
