
DROP PROCEDURE IF EXISTS planning.spGET_EnduserObjects;
GO

/*
	Procedure to deliver SQL Object suggestions to the enduser.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = planning.spGET_EnduserObjects
					 @Username = 'SQL'
					,@ObjectType = 'P'
					,@ShowAllBool = 1;
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_EnduserObjects',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_EnduserObjects','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spGET_EnduserObjects
    (
         @Username		NVARCHAR(255)
		,@ObjectType	NVARCHAR(255)		= 'P'	
        ,@ShowAllBool	INT					= 0
    )
AS 
	BEGIN	

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username		,N'NULL')	+ N''','''											 
				+ ISNULL(@ObjectType	,N'NULL')	+ N''','
				+ ISNULL(CAST(@ShowAllBool AS NVARCHAR(255)),N'NULL')	+ N''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username		= N'';
		IF @ObjectType				IS NULL SET @ObjectType		= N'';
		IF @ShowAllBool				IS NULL SET @ShowAllBool	= 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spGET_EnduserObjects;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				;WITH prms AS (
					SELECT DISTINCT object_id FROM (
						SELECT p1.object_id, p2.object_id other_id
						FROM [sys].[all_parameters] p1 
							LEFT JOIN [sys].[all_parameters] p2 ON p1.[object_id] = p2.[object_id] 
								AND p2.[name] NOT IN ('@Username', '@FactoryID', '@ProductLineID', '@ProductID', '@ParameterJSON')
						WHERE p1.[name] = '@Username'
					) all_p 
					WHERE other_id is null
				)			
				SELECT 
					CASE o.type
						WHEN 'P'	THEN 'Stored Procedures'
						WHEN 'U'	THEN 'Tables'
						WHEN 'V'	THEN 'Views'
						WHEN 'IF'	THEN 'Functions (Table Value Inline)'
						WHEN 'FN'	THEN 'Functions (Scalar)'
						WHEN 'TF'	THEN 'Functions (Table Value)'
						WHEN 'TR'	THEN 'Trigger'
						ELSE 'Unknown Objecttype'
						END									AS ObjectTypeName
					,o.type									AS ObjectType
					,S.name	+'.' + O.name					AS ObjectName
					,CASE
						WHEN sxo.SX_UserHint IS NOT NULL AND sxo.SX_UserHint <> '' THEN sxo.SX_UserHint
						WHEN ep.value = ''		THEN	'Description not defined'
						WHEN ep.value IS NULL	THEN	'<description defintion tag not definded>'
						WHEN ep.value <> ''		THEN	ep.value
						ELSE							'<procedure failure - this should not had happen>'
						END						AS Description

				FROM sys.all_objects O
				LEFT JOIN sys.extended_properties EP 
					ON ep.major_id = O.object_id 
				LEFT JOIN sys.schemas S 
					ON O.schema_id = S.schema_id
				-- use as separte column
				LEFT JOIN	(
								SELECT 
										major_id
									,Value AS SX_UserHint
								FROM sys.extended_properties
								WHERE Name = 'SX_UserHint'
							) sxo
							ON sxo.major_id = ep.major_id
				LEFT JOIN prms ON o.object_id = prms.object_id
				WHERE 
						O.is_ms_shipped = 0
					AND ISNULL(ep.minor_id,0) = 0				-- columns of tables and views, parameters filtered out
					AND O.type NOT IN ('PK' -- PrimaryKeys
										,'D'  -- default constraints
										)
					AND (ep.name = 'MS_Description' OR ep.name IS NULL)
					AND (o.type = @ObjectType OR @ObjectType = '')
					AND (sxo.SX_UserHint IS NOT NULL AND sxo.SX_UserHint <> '' OR @ShowAllBool = 1)
					AND (prms.object_id IS NOT NULL OR o.type <> 'P')

				SET @ResultCode = 200;
			COMMIT TRANSACTION spGET_EnduserObjects;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spGET_EnduserObjects;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT EXECUTE ON OBJECT planning.spGET_EnduserObjects TO [Username or rolename]

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_EnduserObjects'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure delivers List of all Objects, which are useful for enduser analytics.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@ObjectType';
    SET @value      = N'Allows to filter output to defined ObjectType. Default Value is P for stored procedure. Keep empty for all.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ShowAllBool';
    SET @value      = N'0 = show only objects with SX_UserHint set in Metadata, 1 = show all objects.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
