﻿
DROP PROCEDURE IF EXISTS planning.spGET_File;
GO

/*
	Procedure to Return body and file name of existing file or error if file not exist or user have not rights to read

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 05/2023 for OCT 5.9
 
	Testcall
		EXEC planning.spGET_File 
			'SQL'
			,'1'
			,'U'
			,'1000'
			,'f09e645a-3624-4497-9606-3d20f0be52f6';

		SELECT * FROM planning.tFiles;

		SELECT * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_File',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_File','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spGET_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(@FactoryID, N'NULL')+ N''', '+ ISNULL(@ProductLineID, N'NULL')
													+ N''', '+ ISNULL(@ProductID, N'NULL')+ N''', '+ ISNULL(@FileID, N'NULL')
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
	
	-- Procedure specific declarations
	DECLARE  @FactoryKey		INT			= 0
			,@ProductLineKey	INT			= 0
			,@ProductKey		INT			= 0;

	-- NULL Protection
	IF @Username		IS NULL	SET @Username = N'';
	IF @FileID			IS NULL	SET @FileID = N'';
	IF @FactoryID		IS NULL SET @FactoryID = N'';
	IF @ProductLineID	IS NULL SET @ProductLineID = N'';
	IF @ProductID		IS NULL SET @ProductID = N'';
	IF @FactoryID		= N'' SET @FactoryKey = -1;
	IF @ProductLineID	= N'' SET @ProductLineKey = -1;
	IF @ProductID		= N'' SET @ProductKey = -1;

	BEGIN TRY
		-- Protect input parameters

		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		IF @FileID IS NULL OR LEN(@FileID) = 0
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'FileID is null or empty', 100;
		END;

		-- Determine transaction user

		SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactionUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 100;
		END;

		-- error if Keys chains has gaps
		IF (@FactoryKey	= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1))
			OR	(@ProductLineKey = -1 AND @ProductKey <> -1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys combination can have only gaps at the end', 16, 10);
		END;

		IF @FactoryKey <> -1
		BEGIN
			SELECT @FactoryKey	= FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

			IF @ProductLineKey <> -1
			BEGIN
				SELECT @ProductLineKey = ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey	AND ProductLineID = @ProductLineID;
				
				IF @ProductKey <> -1
				BEGIN
					SELECT @ProductKey = ProductKey	FROM planning.tdProducts WHERE FactoryKey = @FactoryKey	AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
				END 
			END
		END

		-- check user rights for cluster level
		IF @FactoryKey = -1
		BEGIN
			-- user already have read right to cluster
			SET @ResultCode = 200;
		END
		-- check user rights for Factory level
		ELSE IF @ProductLineKey = -1
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_FactoryReadRight @TransactionUsername, @FactoryID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for Factory.', 16, 10);
			END 
		END
		-- check user rights for ProductLine / Product level
		ELSE IF @ProductLineKey > 0
		BEGIN
			EXEC @ResultCode = sx_pf_pGET_ProductLineReadRight @TransactionUsername, @FactoryID, @ProductLineID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for ProductLine.', 16, 10);
			END 
		END
				
		-- error if and Keys dont exists
		IF		@FactoryKey		= 0 
			OR	@ProductLineKey	= 0
			OR  @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
		END;

		DECLARE @FileKey BIGINT = NULL;

		SELECT TOP (1) @FileKey = FileKey 
		FROM planning.tFiles
		WHERE   FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND FileID			= @FileID;

		IF @FileKey IS NULL
		BEGIN
			SET @ResultCode = 404;
			THROW 60000, N'File not found', 100;
		END;

		SELECT
			 FactoryID
			,ProductLineID
			,ProductID
			,FileID
			,[FileName] 
			,ISNULL([Filename] + N'.' + FileExtension, [Filename]) AS 'FileNameWithExtension'
			,FileExtension
			,FileBody
			,Filepath
		FROM planning.tFiles
		WHERE FactoryKey		= @FactoryKey
			AND ProductLineKey	= @ProductLineKey
			AND ProductKey		= @ProductKey
			AND FileID			= @FileID;
		
		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	
		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

    EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
	RETURN @ResultCode;
END;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_File'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to Return body and file name of existing file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileID';
    SET @value      = N'FileID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
