
DROP PROCEDURE IF EXISTS planning.spGET_Products_JSON;
GO

/*
	Zweck
		- lesen der Globalattribute einer Liste von Products
		- die Liste der zu lesenden Products wird als JSON Array übergeben und umfasst X Products
		- die Prozedur gibt ein Recordset mit ebenfalls genau X Zeilen zurück
		- Pro Zeile wird ein Returncode zurückgegeben ob das Product gelesen werden konnte
		- Pro Zeile wird ein optionale Information mit zurückgegeben

	ToDo
		- lesende Abfrage schreiben, ähnlich planning.spGET_Values_JSON
			- prüfen, ob die angeforderten Products existieren
			- prüfen, ob der User Rechte auf den angeforderten Products hat
			- Products lesen
			- Products als Recordset zurückgeben

	Gerd Tautenhahn / Maria Rüdger for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @ProductsJSON	NVARCHAR(MAX);
		DECLARE @RC				INT;
		SET @ProductsJSON = N'{	 "connector_type":"read"
								,"version":1
								,"Values":
										{
											"Products":	
												[											
													{	 "ProductNumber": 1
														,"FactoryID"	: "ZT"
														,"ProductLineID": "U"
														,"ProductID"	: "1"
													},
													{	 "ProductNumber": 2
														,"FactoryID"	: "ZT"
														,"ProductLineID": "U"
														,"ProductID"	: "2"
													}
												]										

										}
								}
					';

		EXEC @RC = planning.spGET_Products_JSON 'SQL', @ProductsJSON;
		PRINT @RC;

	Testcall 2
		DECLARE @ProductsJSON	NVARCHAR(MAX);
		DECLARE @RC				INT;
		SET @ProductsJSON = N'{	 "connector_type":"read"
								,"version":1
								,"Values":
										{
											"Products":	
												[											
													{	 "ProductNumber": 1
														,"FactoryID"	: "ZT"
														,"ProductLineID": "*"
														,"ProductID":"*"
													}
												]										

										}
								}
					';

		EXEC @RC = planning.spGET_Products_JSON 'SQL', @ProductsJSON;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Products_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Products_JSON','PARAMETER',NULL)
*/  
 
CREATE PROCEDURE planning.spGET_Products_JSON
    (
         @Username				NVARCHAR(255)
        ,@ProductsJSON			NVARCHAR(MAX) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
			     @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName		NVARCHAR(255)		= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)		= N''''					
				+ ISNULL(@Username						,N'NULL')	+ N''','''											 
				+ ISNULL(@ProductsJSON					,N'NULL')	+ N''''

				,@EffectedRows		INT					= 0			
				,@ResultCode		INT					= 501				
				,@TimestampCall		DATETIME			= GETUTCDATE()
				,@Comment			NVARCHAR(2000)		= N''
				,@TransactUsername	NVARCHAR(255)		= N''
				,@StartTime			DATETIME2			= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ProductsJSON			IS NULL SET @ProductsJSON		= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY

				-- STEP 0.2 - Protect input parameters

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ProductsJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ProductsJSON) = 0 
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to invalid JSON', 100;
				END;
	
			-- Extract Products FROM JSON ######################################################################

			DROP TABLE IF EXISTS #RequestedValues;

			CREATE TABLE #RequestedValues
				(
					 ProductNumber	INT										 NOT NULL
					,FactoryID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					,ProductLineID	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
					,ProductID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				);

			INSERT INTO #RequestedValues
				SELECT DISTINCT
			 		 ProductNumber
					,FactoryID
					,ProductLineID
					,ProductID
				FROM OPENJSON (@ProductsJSON, N'$.Values.Products') 
				WITH (
					 ProductNumber	INT				N'$.ProductNumber'
					,FactoryID		NVARCHAR(255)	N'$.FactoryID'
					,ProductLineID	NVARCHAR(255)	N'$.ProductLineID'
					,ProductID		NVARCHAR(255)	N'$.ProductID'
				);

				IF EXISTS(
					SELECT TOP(1) 1 
					FROM #RequestedValues 
					WHERE 
						   ProductNumber	IS NULL 
						OR FactoryID		IS NULL 
						OR ProductLineID	IS NULL 
						OR ProductID		IS NULL 
						OR FactoryID		= N''
						OR ProductLineID	= N''
						OR ProductID		= N''
						)
				OR EXISTS(
					SELECT TOP(1) 1 
					FROM #RequestedValues 
					GROUP BY ProductNumber
					HAVING COUNT (ProductNumber) > 1
				)
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to invalid Request', 100;
				END;

			-- Determine transaction user ######################################################################

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;			

			-- Search ValueSeries and User`s rights ######################################################################

			-- Query all requested Products
				-- only one row if a Product is multiple times requested
				-- classified for rights, but not filterd
			DROP TABLE IF EXISTS #Products;

			CREATE TABLE #Products
				(
				 FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,TimeType				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,NameShort				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,CommentUser 			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NOT NULL
				,ResponsiblePerson		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ImageName				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,[Status]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,Template				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute1		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute2		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute3		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute4		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute5		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute6		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute7		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute8		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute9		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute10		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute11		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute12		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute13		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute14		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute15		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute16		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute17		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute18		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute19		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute20		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute21		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute22		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute23		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute24		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,GlobalAttribute25		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,[Right]				NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				);
			
			INSERT INTO #Products
				SELECT
					 dP.FactoryID		
					,dP.ProductLineID	
					,dP.ProductID		
					,dP.TimeType			
					,dP.NameShort			
					,dP.CommentUser 		
					,dP.ResponsiblePerson
					,dP.ImageName			
					,dP.[Status]			
					,dP.Template			
					,dP.GlobalAttribute1
					,dP.GlobalAttribute2
					,dP.GlobalAttribute3
					,dP.GlobalAttribute4
					,dP.GlobalAttribute5
					,dP.GlobalAttribute6
					,dP.GlobalAttribute7
					,dP.GlobalAttribute8
					,dP.GlobalAttribute9
					,dP.GlobalAttribute10
					,dP.GlobalAttribute11
					,dP.GlobalAttribute12
					,dP.GlobalAttribute13
					,dP.GlobalAttribute14
					,dP.GlobalAttribute15
					,dP.GlobalAttribute16
					,dP.GlobalAttribute17
					,dP.GlobalAttribute18
					,dP.GlobalAttribute19
					,dP.GlobalAttribute20
					,dP.GlobalAttribute21
					,dP.GlobalAttribute22
					,dP.GlobalAttribute23
					,dP.GlobalAttribute24
					,dP.GlobalAttribute25
					,UR.[Right]		AS [Right]
					
				FROM planning.tdProducts dP
					-- Filter on resquested Values
					INNER JOIN #RequestedValues RV
						ON 
								dP.FactoryID		= RV.FactoryID
							AND dP.ProductLineID	= RV.ProductLineID
							AND dP.ProductID		= RV.ProductID						

					-- right classification, but no filtering
					LEFT JOIN system.trUserRights UR
						ON 
								dP.FactoryID		= UR.FactoryID 
							AND dP.ProductLineID	= UR.ProductLineID 
							AND UR.Username			= @TransactionUsername;
--PRINT 'Done 1';

			-- Create Resultset ######################################################################

			DROP TABLE IF EXISTS #ProductSet;

			CREATE TABLE #ProductSet (
				 ProductNumber		INT				NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL

				,TimeType			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,NameShort			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,CommentUser 		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NULL
				,ResponsiblePerson	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,ImageName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,[Status]			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,Template			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute1	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute2	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute3	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute4	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute5	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute6	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute7	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute8	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute9	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute10	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute11	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute12	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute13	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute14	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute15	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute16	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute17	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute18	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute19	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute20	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute21	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute22	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute23	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute24	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,GlobalAttribute25	NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				
				,ReturnMessage		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,ReturnCode			INT				NULL
				,ProcessFlag		INT				NULL
				);

			INSERT INTO #ProductSet 
				-- Create Basic Rows
				SELECT 
					 ProductNumber		
					,FactoryID		
					,ProductLineID		
					,ProductID		
					,''		AS TimeType			
					,''		AS NameShort		
					,''		AS CommentUser 		
					,''		AS ResponsiblePerson
					,''		AS ImageName		
					,''		AS [Status]			
					,''		AS Template			
					,''		AS GlobalAttribute1	
					,''		AS GlobalAttribute2	
					,''		AS GlobalAttribute3	
					,''		AS GlobalAttribute4	
					,''		AS GlobalAttribute5	
					,''		AS GlobalAttribute6	
					,''		AS GlobalAttribute7	
					,''		AS GlobalAttribute8	
					,''		AS GlobalAttribute9	
					,''		AS GlobalAttribute10
					,''		AS GlobalAttribute11
					,''		AS GlobalAttribute12
					,''		AS GlobalAttribute13
					,''		AS GlobalAttribute14
					,''		AS GlobalAttribute15
					,''		AS GlobalAttribute16
					,''		AS GlobalAttribute17
					,''		AS GlobalAttribute18
					,''		AS GlobalAttribute19
					,''		AS GlobalAttribute20
					,''		AS GlobalAttribute21
					,''		AS GlobalAttribute22
					,''		AS GlobalAttribute23
					,''		AS GlobalAttribute24
					,''		AS GlobalAttribute25
					,''		AS ReturnMessage	
					,500	AS ReturnCode		
					,''		AS ProcessFlag	

				FROM #RequestedValues;

			-- Insert Informations for Products with Read Rights
			UPDATE PS	
				SET 
					 PS.TimeType					= P.TimeType			
					,PS.NameShort					= P.NameShort		
					,PS.CommentUser 				= P.CommentUser 		
					,PS.ResponsiblePerson			= P.ResponsiblePerson
					,PS.ImageName					= P.ImageName		
					,PS.[Status]					= P.[Status]			
					,PS.Template					= P.Template			
					,PS.GlobalAttribute1			= P.GlobalAttribute1	
					,PS.GlobalAttribute2			= P.GlobalAttribute2	
					,PS.GlobalAttribute3			= P.GlobalAttribute3	
					,PS.GlobalAttribute4			= P.GlobalAttribute4	
					,PS.GlobalAttribute5			= P.GlobalAttribute5	
					,PS.GlobalAttribute6			= P.GlobalAttribute6	
					,PS.GlobalAttribute7			= P.GlobalAttribute7	
					,PS.GlobalAttribute8			= P.GlobalAttribute8	
					,PS.GlobalAttribute9			= P.GlobalAttribute9	
					,PS.GlobalAttribute10			= P.GlobalAttribute10
					,PS.GlobalAttribute11			= P.GlobalAttribute11
					,PS.GlobalAttribute12			= P.GlobalAttribute12
					,PS.GlobalAttribute13			= P.GlobalAttribute13
					,PS.GlobalAttribute14			= P.GlobalAttribute14
					,PS.GlobalAttribute15			= P.GlobalAttribute15
					,PS.GlobalAttribute16			= P.GlobalAttribute16
					,PS.GlobalAttribute17			= P.GlobalAttribute17
					,PS.GlobalAttribute18			= P.GlobalAttribute18
					,PS.GlobalAttribute19			= P.GlobalAttribute19
					,PS.GlobalAttribute20			= P.GlobalAttribute20
					,PS.GlobalAttribute21			= P.GlobalAttribute21
					,PS.GlobalAttribute22			= P.GlobalAttribute22
					,PS.GlobalAttribute23			= P.GlobalAttribute23
					,PS.GlobalAttribute24			= P.GlobalAttribute24
					,PS.GlobalAttribute25			= P.GlobalAttribute25

					,ReturnMessage	= 'Sucessful Read'
					,ReturnCode		= 200
					,ProcessFlag	= ''
					
				FROM #ProductSet PS 
					LEFT JOIN #Products P
						ON 
							PS.FactoryID		= P.FactoryID
						AND PS.ProductLineID	= P.ProductLineID
						AND PS.ProductID		= P.ProductID

				WHERE P.[Right] IN ('Read','Write');

				-- Insert informations for Products without read Rights
				UPDATE PS	
					SET 
						 ReturnMessage	= 'Read denied'
						,ReturnCode		= 403
						,ProcessFlag	= ''
					
				FROM #ProductSet PS 
					LEFT JOIN #Products P
						ON 
							PS.FactoryID		= P.FactoryID
						AND PS.ProductLineID	= P.ProductLineID
						AND PS.ProductID		= P.ProductID

				WHERE P.[Right] IN ('Deny');

				-- Insert informations for not found Products  (still have ReturnCode 500)
				UPDATE #ProductSet
					SET 
						 ReturnMessage	= 'Product not found'
						,ReturnCode		= 404
						,ProcessFlag	= ''
					WHERE ReturnCode = 500;

			-- Output result ######################################################################
				SELECT
		
					 ProductNumber
							
					,FactoryID		
					,ProductLineID	
					,ProductID
					
					,TimeType
					,NameShort
					,CommentUser --Beschreibung
					-- keine Ausgabe von NameLong / CommentDev, da in Gui nicht sichtbar
					,ResponsiblePerson
					,ImageName
					,[Status]
					,Template
					,GlobalAttribute1
					,GlobalAttribute2
					,GlobalAttribute3
					,GlobalAttribute4
					,GlobalAttribute5
					,GlobalAttribute6
					,GlobalAttribute7
					,GlobalAttribute8
					,GlobalAttribute9
					,GlobalAttribute10
					,GlobalAttribute11
					,GlobalAttribute12
					,GlobalAttribute13
					,GlobalAttribute14
					,GlobalAttribute15
					,GlobalAttribute16
					,GlobalAttribute17
					,GlobalAttribute18
					,GlobalAttribute19
					,GlobalAttribute20
					,GlobalAttribute21
					,GlobalAttribute22
					,GlobalAttribute23
					,GlobalAttribute24
					,GlobalAttribute25
						   					 
					,ReturnMessage
					,ReturnCode
					,ProcessFlag	
				FROM #ProductSet ps;

				SET @ResultCode = 200;
				
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_Products_JSON'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure returns an in JSON  Format requested Productarray as Recordset.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Username'
    SET @value      = N'Standardparameter';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductsJSON'
    SET @value      = N'List of Products as JSON Array';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
