﻿
DROP PROCEDURE IF EXISTS planning.spGET_Values_JSON;
GO

/*
	Procedure to GET Values in JSON Format for MatrixConnector

	MatrixConnector:
		- Shows Values which are combined from a Time Axis and a Structure Axis
		- The Structure Axis has the attributes F/PL/P/VS where each can be substituted by a * to mean all
		- The Time Axis shows one TimeID (later with flex Time concept it will show Year, Month, Day where also each one can be a *)

	This GET Procedure returns for one TimeID all Values
	The Values for this TimeID are requsted as JSON 

	Retruns in a Array per Requested Number
		- Nummerischen Wert nach Operation SUM, MAX,...
		- String if single request
		- Datum
		- ValueComment if single request
		- Statistik for Aggregatfunction
			- Anzahl Treffer
			- F/PL/P/VS bei MAX, MIN

	 Aktuelle Limitierungen:
		- Es wird immer der Aggreationstyp Summe zurückgegeben - egal was angefragt wird
		- Es werden derzeit nur direkte Einzelwerte verarbeitet und zurückgegeben
		- Es werden somit alle mit * angefragten Zeilen als NULL zurückgegeben
		- Er werden nur nummerische Werte zurückgegeben, noch keine Texte
		- Es werden noch keine Rechte verarbeitet

	 TODO:
		- Nur Werte für die berechtigten Strukturinhalten zurückgeben
		- Info über Berechtigung im Text
		- Berechtigung von nicht gefunden unterscheiden
		- ValueSeries kann bei Aggregation beide mehrere Datentypen lieferen -> muss bei Aggregation Typ wählen ?
		- Werte können in unterschiedlichen ValueSeries unterschiedlich skaliert sein
		- Nach dem Schreiben von Werten nach DF kann ein Powerloading notwendig sein
		- Comments ausgeben

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.8
 
	Testcall
		DECLARE @ValuesJSON NVARCHAR(MAX)
		SET @ValuesJSON = N'{	 "connector_type":"read"
								,"version":1
								,"Values":
										{
											"ValueSeries":	
												[											
													{	 "SeriesNumber"	: 1
														,"AggregateType": "SUM"
														,"FactoryID"	: "ZT"
														,"ProductLineID": "*"
														,"ProductID":"*"
														,"ValueSeriesID": "*"
													}
												]										
											,"TimeIDs":	
												[
														{	 "TimeNumber"	: 1
															,"TimeID"		: 201902
														}, 
														{	 "TimeNumber"	: 2
															,"TimeID"		: 20190215
														}, 
														{	 "TimeNumber"	: 3
															,"TimeID"		: 20190315
														}, 
														{	 "TimeNumber"	: 4
															,"TimeID"		: 20190415
														} 
												]
										}
								}
					'
		EXEC planning.spGET_Values_JSON 'SQL', @ValuesJSON

		EXEC planning.spGET_Values_JSON 'SQL','{"connector_type":"read","version":1,"Values":{"ValueSeries":[{"SeriesNumber":1,"AggregateType":"SUM","FactoryID":"ZT","ProductLineID":"U","ProductID":1,"ValueSeriesID":"K1"}],"TimeIDs":[{"TimeNumber":1,"TimeID":20200115},{"TimeNumber":2,"TimeID":20190215},{"TimeNumber":3,"TimeID":20190315},{"TimeNumber":4,"TimeID":20190415}]}}'

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Values_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spGET_Values_JSON','PARAMETER',NULL)
*/
 
CREATE PROCEDURE planning.spGET_Values_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';						
		
		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for technical valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
				BEGIN
					SET @ResultCode = 403;
					THROW 60000, N'Exit due to technical invalid JSON', 100;
				END;
	
			-- Extract ValueSeries FROM JSON into tmp table######################################################################

			DROP TABLE IF EXISTS #RequestedValues;

			SELECT DISTINCT
			 	 SeriesNumber
				,AggregateType COLLATE DATABASE_DEFAULT AS AggregateType
				,CASE WHEN FactoryID		= N'*'	OR FactoryID LIKE N'*_%'		THEN FactoryID		ELSE dbo.sx_pf_pProtectID (FactoryID)		END COLLATE DATABASE_DEFAULT AS 'FactoryID'
				,CASE WHEN ProductLineID	= N'*'	OR ProductLineID LIKE N'*_%'	THEN ProductLineID	ELSE dbo.sx_pf_pProtectID (ProductLineID)	END COLLATE DATABASE_DEFAULT AS 'ProductLineID'
				,CASE WHEN ProductID		= N'*'	OR ProductID LIKE N'*_%'		THEN ProductID		ELSE dbo.sx_pf_pProtectID (ProductID)		END COLLATE DATABASE_DEFAULT AS 'ProductID'
				,CASE WHEN ValueSeriesID	= N'*'	OR ValueSeriesID LIKE N'*_%'	THEN ValueSeriesID	ELSE dbo.sx_pf_pProtectID (ValueSeriesID)	END COLLATE DATABASE_DEFAULT AS 'ValueSeriesID'
			INTO #RequestedValues
			FROM OPENJSON (@ValuesJSON, N'$.Values.ValueSeries') 
			WITH (
				SeriesNumber	INT				N'$.SeriesNumber'
				,AggregateType	NVARCHAR(50)	N'$.AggregateType'
				,FactoryID		NVARCHAR(255)	N'$.FactoryID'
				,ProductLineID	NVARCHAR(255)	N'$.ProductLineID'
				,ProductID		NVARCHAR(255)	N'$.ProductID'
				,ValueSeriesID	NVARCHAR(255)	N'$.ValueSeriesID'
			);
			
			-- Test on mandatory parameter completness
			IF EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				WHERE  SeriesNumber		IS NULL 
					OR AggregateType	IS NULL 
					OR FactoryID		IS NULL 
					OR ProductLineID	IS NULL 
					OR ProductID		IS NULL 
					OR ValueSeriesID	IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					OR ValueSeriesID	= N'')
			OR EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedValues 
				GROUP BY SeriesNumber
				HAVING COUNT (SeriesNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to logical failure in requested values (missing IDs, multiple SeriesNumber ..)', 100;
			END;

			-- Extract TimeIDs FROM JSON #######################################################################

			DROP TABLE IF EXISTS #RequestedTimes;

			;WITH times AS (
				SELECT DISTINCT
					 TimeNumber
					,TimeID
				FROM OPENJSON (@ValuesJSON, N'$.Values.TimeIDs') 
				WITH (
						 TimeNumber		INT		N'$.TimeNumber'
						,TimeID			INT		N'$.TimeID'
				)
			)
			SELECT *
				--determine Max/Min TimeID from cutted TimeIDs which represend Year / Month / Day
				, CASE	WHEN TimeID BETWEEN 1000		AND 9999		THEN TimeID * 10000
						WHEN TimeID BETWEEN 100000		AND 999999		THEN TimeID * 100
						WHEN TimeID BETWEEN 10000000	AND 99999999	THEN TimeID
						ELSE NULL END AS 'MinTimeID'
				, CASE	WHEN TimeID BETWEEN 1000		AND 9999		THEN TimeID * 10000 + 9999
						WHEN TimeID BETWEEN 100000		AND 999999		THEN TimeID * 100 + 99
						WHEN TimeID BETWEEN 10000000	AND 99999999	THEN TimeID
						ELSE NULL END AS 'MaxTimeID'
			INTO #RequestedTimes
			FROM times;

			IF EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedTimes 
				WHERE TimeNumber IS NULL 
					OR MinTimeID IS NULL
					OR MaxTimeID IS NULL
			) 
			OR EXISTS(
				SELECT TOP(1) 1 
				FROM #RequestedTimes 
				GROUP BY TimeNumber
				HAVING COUNT (TimeNumber) > 1
			)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to problems in processing requested TimeIDs.', 100;
			END;

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- STEP 3.0 - Search ValueSeries and User`s rights

			DROP TABLE IF EXISTS #ValueSeries;
			
			;WITH search AS (
				SELECT SeriesNumber
					,AggregateType
					,FactoryID
					,ProductLineID
					,ProductID
					,ValueSeriesID

					,CASE WHEN FactoryID		= N'*' THEN 1 WHEN FactoryID		LIKE N'*_%' THEN 2 ELSE 0 END AS FactoryIDSearchType
					,CASE WHEN ProductLineID	= N'*' THEN 1 WHEN ProductLineID	LIKE N'*_%' THEN 2 ELSE 0 END AS ProductLineIDSearchType
					,CASE WHEN ProductID		= N'*' THEN 1 WHEN ProductID		LIKE N'*_%' THEN 2 ELSE 0 END AS ProductIDSearchType
					,CASE WHEN ValueSeriesID	= N'*' THEN 1 WHEN ValueSeriesID	LIKE N'*_%' THEN 2 ELSE 0 END AS ValueSeriesIDSearchType
					
					,CASE WHEN FactoryID		LIKE N'*_%' THEN SUBSTRING(FactoryID, 2, 300)		ELSE NULL END AS FactoryIDPattern
					,CASE WHEN ProductLineID	LIKE N'*_%' THEN SUBSTRING(ProductLineID, 2, 300)	ELSE NULL END AS ProductLineIDPattern
					,CASE WHEN ProductID		LIKE N'*_%' THEN SUBSTRING(ProductID, 2, 300)		ELSE NULL END AS ProductIDPattern
					,CASE WHEN ValueSeriesID	LIKE N'*_%' THEN SUBSTRING(ValueSeriesID, 2, 300)	ELSE NULL END AS ValueSeriesIDPattern
				FROM #RequestedValues
			)
			SELECT 
				rv.SeriesNumber
				,rv.AggregateType
				,dF.FactoryID		COLLATE DATABASE_DEFAULT AS FactoryID
				,dPL.ProductLineID	COLLATE DATABASE_DEFAULT AS ProductLineID
				,dP.ProductID		COLLATE DATABASE_DEFAULT AS ProductID
				,dVS.ValueSeriesID	COLLATE DATABASE_DEFAULT AS ValueSeriesID
				,dF.FactoryKey
				,dPL.ProductLineKey
				,dP.ProductKey
				,dVS.ValueSeriesKey
				,dVS.Scale
				,dVS.[IsNumeric]
				,rF.[Row] AS 'FactoryRightRow'
				,rPL.[Row] AS 'ProductLineRightRow'
			INTO #ValueSeries
			FROM search rv
				INNER JOIN planning.tdFactories dF
					ON rv.FactoryIDSearchType = 1 
						OR (rv.FactoryIDSearchType = 2 AND df.FactoryID LIKE rv.FactoryIDPattern) 
						OR (rv.FactoryIDSearchType = 0 AND df.FactoryID = rv.FactoryID) 
				INNER JOIN planning.tdProductLines dPL
					ON dPL.FactoryKey = dF.FactoryKey
						AND (rv.ProductLineIDSearchType = 1 
							OR (rv.ProductLineIDSearchType = 2 AND dPL.ProductLineID LIKE rv.ProductLineIDPattern) 
							OR (rv.ProductLineIDSearchType = 0 AND dPL.ProductLineID = rv.ProductLineID))
				INNER JOIN planning.tdProducts dP
					ON dP.FactoryKey = dF.FactoryKey 
						AND dP.ProductLineKey = dPL.ProductLineKey
						AND (rv.ProductIDSearchType = 1 
							OR (rv.ProductIDSearchType = 2 AND dP.ProductID LIKE rv.ProductIDPattern) 
							OR (rv.ProductIDSearchType = 0 AND dP.ProductID = rv.ProductID))
				INNER JOIN planning.tdValueSeries dVS
					ON dVS.FactoryKey = dF.FactoryKey 
						AND dVS.ProductLineKey = dPL.ProductLineKey
						AND dVS.ProductKey = dP.ProductKey
						AND (rv.ValueSeriesIDSearchType = 1 
							OR (rv.ValueSeriesIDSearchType = 2 AND dVS.ValueSeriesID LIKE rv.ValueSeriesIDPattern) 
							OR (rv.ValueSeriesIDSearchType = 0 AND dVS.ValueSeriesID = rv.ValueSeriesID))
				LEFT JOIN system.trUserRights rF 
					ON df.FactoryID = rF.FactoryID 
						AND rF.ProductLineID = N''
						AND rF.Username = @TransactionUsername
						AND (rF.[Right] = N'Write' OR rF.[Right] = N'Read')
				LEFT JOIN system.trUserRights rPL 
					ON df.FactoryID = rPL.FactoryID 
						AND dPL.ProductLineID = rPL.ProductLineID 
						AND rPL.Username = @TransactionUsername
						AND (rPL.[Right] = N'Write' OR rPL.[Right] = N'Read');

			-- STEP 3.1 - Searching requested values

			DROP TABLE IF EXISTS #Values;

			SELECT
				s.SeriesNumber
				,s.FactoryKey
				,s.ProductLineKey
				,s.ProductKey
				,s.ValueSeriesKey
				,s.FactoryID      COLLATE DATABASE_DEFAULT AS FactoryID
				,s.ProductLineID  COLLATE DATABASE_DEFAULT AS ProductLineID
				,s.ProductID      COLLATE DATABASE_DEFAULT AS ProductID
				,s.ValueSeriesID  COLLATE DATABASE_DEFAULT AS ValueSeriesID
				,ISNULL(CASE WHEN s.[IsNumeric] = 1 THEN CAST(fV.ValueInt AS Money) / s.Scale ELSE 0 END, 0) AS 'ValueNumeric'
				,ISNULL(CASE WHEN s.[IsNumeric] = 0 THEN fV.ValueText ELSE N'' END, N'') COLLATE DATABASE_DEFAULT AS 'ValueString'
				,fV.ValueComment COLLATE DATABASE_DEFAULT AS ValueComment
				,t.TimeNumber
				,fV.TimeID
				,s.[IsNumeric]
			INTO #Values
			FROM #ValueSeries s
				INNER JOIN planning.tfValues fV
					ON 
							-- why join over four Key fields ?
							fV.FactoryKey		= s.FactoryKey 
						AND fV.ProductLineKey	= s.ProductLineKey
						AND fV.ProductKey		= s.ProductKey
						AND fV.ValueSeriesKey	= s.ValueSeriesKey
						AND NOT s.FactoryRightRow		IS NULL 
						AND NOT s.ProductLineRightRow	IS NULL
				INNER JOIN #RequestedTimes t
					ON fV.TimeID BETWEEN t.MinTimeID AND t.MaxTimeID;

			-- STEP 3.2 - Insert requested values into result table

			DROP TABLE IF EXISTS #ValueSet;

			CREATE TABLE #ValueSet (
				 SeriesNumber		INT				NOT NULL
				,AggregateType		NVARCHAR(50)	COLLATE DATABASE_DEFAULT NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				
				,TimeNumber			INT				NOT NULL
				,TimeID				INT				NOT NULL
				
				,FactoryKey			BIGINT			NULL
				,ProductLineKey		BIGINT			NULL
				,ProductKey			BIGINT			NULL
				,ValueSeriesKey		BIGINT			NULL

				,FactoryCount		INT				NULL
				,ProductLineCount	INT				NULL
				,ProductCount		INT				NULL
				,ValueSeriesCount	INT				NULL

				,ValueNumeric		MONEY			NULL
				,ValueString		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NULL
				,ValueDate			DATE			NULL
				,ValueComment		NVARCHAR(4000)  COLLATE DATABASE_DEFAULT NULL

				,ReturnMessage		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NULL
				,ReturnCode			INT				NULL
				,ProcessFlag		INT				NULL
				);

			INSERT INTO #ValueSet (SeriesNumber, AggregateType, FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeNumber, TimeID)
			SELECT 
				 v.SeriesNumber	
				,v.AggregateType	
				,v.FactoryID		
				,v.ProductLineID		
				,v.ProductID			
				,v.ValueSeriesID		
				,t.TimeNumber	
				,t.TimeID
			FROM #RequestedValues v, #RequestedTimes t;

			-- STEP 3.1 - Set 'No rights' comments  		
			
			;WITH series AS (
				SELECT 
					FactoryID 
					,ProductLineID
					,MAX([IsNumeric]) AS 'IsNumeric'
					,MIN([IsNumeric]) AS 'IsString'
				FROM #ValueSeries
				WHERE FactoryRightRow IS NULL 
					OR ProductLineRightRow IS NULL
				GROUP BY FactoryID, ProductLineID
			)
			UPDATE s
			SET	 ReturnMessage		= N'No rights'
				,ReturnCode			= 403
				,FactoryCount		= 0
				,ProductLineCount	= 0
				,ProductCount		= 0
				,ValueSeriesCount	= 0
				,ValueNumeric		= CASE WHEN v.[IsNumeric] = 1	THEN 0		ELSE NULL END
				,ValueString		= CASE WHEN v.IsString = 0		THEN N''	ELSE NULL END
				,ValueComment		= N''
			FROM #ValueSet s
				INNER JOIN series v
					ON s.FactoryID = v.FactoryID 
						AND s.ProductLineID = v.ProductLineID;

			---- STEP 3.2 - Fill result table with exists values, make aggregation

			;WITH vals AS (
				SELECT 
					 SeriesNumber
					,TimeNumber
					,COUNT(DISTINCT FactoryKey)		AS 'FactoryCount'
					,COUNT(DISTINCT ProductLineKey)	AS 'ProductLineCount'
					,COUNT(DISTINCT ProductKey)		AS 'ProductCount'
					,COUNT(DISTINCT ValueSeriesKey)	AS 'ValueSeriesCount'
					,MAX(FactoryKey)				AS 'FactoryKey'
					,MAX(ProductLineKey)			AS 'ProductLineKey'
					,MAX(ProductKey)				AS 'ProductKey'
					,MAX(ValueSeriesKey)			AS 'ValueSeriesKey'
					,SUM(ValueNumeric)				AS 'ValueNumeric'
				FROM #Values
				GROUP BY SeriesNumber, TimeNumber
			),
			comments AS (
				SELECT TOP (1) WITH TIES
					 SeriesNumber
					,TimeNumber
					,ValueComment
				FROM #Values
				WHERE NOT ValueComment IS NULL
				ORDER BY ROW_NUMBER() OVER(PARTITION BY SeriesNumber, TimeNumber ORDER BY FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeID DESC)
			),
			stringValues AS (
				SELECT TOP (1) WITH TIES
					 SeriesNumber
					,TimeNumber
					,ValueString
				FROM #Values
				WHERE [IsNumeric] = 0
					AND NOT ValueString IS NULL
				ORDER BY ROW_NUMBER() OVER(PARTITION BY SeriesNumber, TimeNumber ORDER BY FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeID DESC)
			)
			UPDATE s
			SET  s.FactoryKey		= CASE WHEN s.FactoryID		LIKE N'*%' THEN NULL ELSE v.FactoryKey		END
				,s.ProductLineKey	= CASE WHEN s.ProductLineID LIKE N'*%' THEN NULL ELSE v.ProductLineKey	END
				,s.ProductKey		= CASE WHEN s.ProductID		LIKE N'*%' THEN NULL ELSE v.ProductKey		END
				,s.ValueSeriesKey	= CASE WHEN s.ValueSeriesID LIKE N'*%' THEN NULL ELSE v.ValueSeriesKey	END 
				,s.FactoryCount		= v.FactoryCount
				,s.ProductLineCount = v.ProductLineCount
				,s.ProductCount		= v.ProductCount
				,s.ValueSeriesCount = v.ValueSeriesCount
				,s.ValueNumeric		= v.ValueNumeric
				,s.ValueString		= ISNULL(sv.ValueString, N'')
				,s.ValueComment		= ISNULL(c.ValueComment, N'')
			FROM #ValueSet s
				INNER JOIN vals v
					ON s.SeriesNumber = v.SeriesNumber
						AND s.TimeNumber = v.TimeNumber
				LEFT JOIN comments c
					ON s.SeriesNumber = c.SeriesNumber
						AND s.TimeNumber = c.TimeNumber
				LEFT JOIN stringValues sv
					ON s.SeriesNumber = sv.SeriesNumber
						AND s.TimeNumber = sv.TimeNumber
			WHERE s.FactoryCount IS NULL; 		

			---- STEP 3.4 - Fill ValueNumeric, ValueComment and ValueString columns and set zero-count for non exists values

			;WITH series AS (
				SELECT 
					 SeriesNumber
					,MAX([IsNumeric]) AS 'IsNumeric'
					,MIN([IsNumeric]) AS 'IsString'
				FROM #ValueSeries
				GROUP BY SeriesNumber
			)
			UPDATE s
			SET  FactoryCount		= ISNULL(FactoryCount, 0)
				,ProductLineCount	= ISNULL(ProductLineCount, 0)
				,ProductCount		= ISNULL(ProductCount, 0)
				,ValueSeriesCount	= ISNULL(ValueSeriesCount, 0)
				,ValueNumeric		= NULL
				,ValueString		= NULL
				,ValueComment		= NULL
			FROM #ValueSet s 
				INNER JOIN series v
					ON s.SeriesNumber = v.SeriesNumber
			WHERE s.FactoryCount IS NULL;

			-- STEP 3.5 - Output result

			SELECT
				 ROW_NUMBER() OVER (ORDER BY SeriesNumber, FactoryID, ProductLineID, ProductID, ValueSeriesID, TimeNumber) AS RowKey			
				,SeriesNumber
				,TimeNumber
				,AggregateType	
				,FactoryID		
				,ProductLineID	
				,ProductID		
				,ValueSeriesID	
		
				,FactoryKey		
				,ProductLineKey	
				,ProductKey		
				,ValueSeriesKey	
		
				,FactoryCount	
				,ProductLineCount
				,ProductCount	
				,ValueSeriesCount	
		
				,TimeID			
				,ValueNumeric	
				,ValueString	
				,ValueDate		
				,ValueComment	
				,IIF(ReturnMessage IS NULL,'',ReturnMessage) AS ReturnMessage
				,IIF(ReturnCode IS NULL,200,ReturnCode) AS ReturnCode
				,ProcessFlag	
			FROM #ValueSet;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;

GO
			
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_Values_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 -- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to GET Values in JSON Format for MatrixConnector';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesJSON';
    SET @value      = N'ValuesJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
