
DROP PROCEDURE IF EXISTS planning.spMOVE_Tab;
GO

/*
	Procedure to move a Tab to
		- another ID
		- another OrderIndex
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
  
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = planning.spMOVE_Tab
				 @Username			= 'W10\admin'
				,@FactoryID			= ''
				,@ProductLineID		= ''
				,@ProductID			= ''
				,@TabID				= '0'
				,@NewTabID			= 'T7'
				,@NewOrderIndex		= '188'

		PRINT @RC
		SELECT * FROM planning.tTabs ORDER BY OrderIndex

		TRUNCATE TABLE planning.tTabs

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spMOVE_Tab',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spMOVE_Tab','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spMOVE_Tab
    (
         @Username				NVARCHAR(255)
		,@FactoryID				NVARCHAR(255) = ''				
		,@ProductLineID			NVARCHAR(255) = ''
		,@ProductID				NVARCHAR(255) = ''
		,@TabID					NVARCHAR(255)
		,@NewTabID				NVARCHAR(255)
		,@NewOrderIndex			INT 
    )

AS 
	BEGIN

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N'''' 
				+ ISNULL(@Username				,N'NULL')					+ N''',''' 
				+ ISNULL(@FactoryID				,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductLineID			,N'NULL')					+ N''',''' 
				+ ISNULL(@ProductID				,N'NULL')					+ N''','''
				+ ISNULL(@TabID					,N'NULL')					+ N''','
				+ ISNULL(@NewTabID				,N'NULL')					+ N''','
				+ CAST(ISNULL(@NewOrderIndex	,N'NULL') AS NVARCHAR(255)) + N''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''

		-- Procedure specific declarations
		DECLARE  @FactoryKey		INT			= 0
				,@ProductLineKey	INT			= 0
				,@ProductKey		INT			= 0
				,@RowKey			BIGINT		= 0
				,@OldOrderIndex		INT			= 0;	

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @FactoryID			IS NULL SET @FactoryID			= N'';
		IF @ProductLineID		IS NULL SET @ProductLineID		= N'';
		IF @ProductID			IS NULL SET @ProductID			= N'';
		IF @TabID				IS NULL SET @TabID				= N'';
		IF @NewTabID			IS NULL SET @NewTabID			= N'';
		IF @NewOrderIndex		IS NULL SET @NewOrderIndex		= 0  ;

		-- dont use negative index values
		IF @NewOrderIndex		<= 0	SET @NewOrderIndex	= 1;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);
		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spMOVE_Tab;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				
				-- determine Keys for requested IDs - empty IDs keep Key -1
				IF @FactoryID		= '' SET @FactoryKey		= -1;
				IF @ProductLineID	= '' SET @ProductLineKey	= -1;
				IF @ProductID		= '' SET @ProductKey		= -1;

				SELECT @FactoryKey		= FactoryKey		FROM planning.tdFactories	 WHERE FactoryID		= @FactoryID												AND @FactoryKey <> -1;
				SELECT @ProductLineKey	= ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey		= @FactoryKey		AND ProductLineID	= @ProductLineID	AND @ProductLineKey <> -1;
				SELECT @ProductKey		= ProductKey		FROM planning.tdProducts	 WHERE ProductLineKey	= @ProductLineKey	AND ProductID		= @ProductID		AND @ProductKey <> -1;

				--PRINT @FactoryKey 
				--PRINT @ProductLineKey
				--PRINT @ProductKey

				-- error if and ID dont exists
				IF		@FactoryKey		= 0 
					OR	@ProductLineKey	= 0
					OR  @ProductKey		= 0
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
				END;

				-- check if TabID exists
				IF NOT EXISTS (
								SELECT 
									 RowKey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			= @TabID
							)
							BEGIN
								SET @ResultCode = 404;
								RAISERROR('TabID not found.', 16, 10);
							END;

				-- tab with ID 0 cant be moved
				IF @TabID = '0' OR @NewTabID = '0'
					BEGIN
							SET @ResultCode = 403;
							RAISERROR('Tab with ID 0 can''t be moved', 16, 10);
					END


				-- error if ID have gaps in the line
				IF		@FactoryKey		= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1)
					OR	@ProductLineKey	= -1 AND @ProductKey <> -1 
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('ID combination can have only gaps at the end - Keys don`t exists', 16, 10);
				END;

				-- check user rights for cluster level
				IF @FactoryKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Cluster write.', 16, 10);
								END 
					END

				-- check user rights for Factory level
				IF @FactoryKey <> - 1 AND @ProductLineKey = -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_FactoryWriteRight @TransactUsername,@FactoryID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for Factory write.', 16, 10);
								END 
					END

				-- check user rights for ProductLine / Product level
				IF @FactoryKey <> -1 AND @ProductLineKey <> -1
					BEGIN
							EXEC @ResultCode = sx_pf_pGET_ProductLineWriteRight @TransactUsername,@FactoryID,@ProductLineID;
								IF @ResultCode <> 200
								BEGIN
									RAISERROR('Invalid rights for ProductLine / Product write.', 16, 10);
								END 
					END

				-- check if new TabID is free
				IF EXISTS (
								SELECT 
									 RowKey
								FROM planning.tTabs
								WHERE 
										FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			= @NewTabID
									AND TabID			<> @TabID
							)
							BEGIN
								SET @ResultCode = 401;
								RAISERROR('New TabID already in use.', 16, 10);
							END;

				-- determine current Key and OrderIndex
				SELECT 
					 @Rowkey = RowKey
					,@OldOrderIndex = OrderIndex
				FROM planning.tTabs
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND TabID			= @TabID

				-- data operation starts	***********************************************************************************

				-- Insert new Tab
				UPDATE planning.tTabs
					SET 
						 TabID		= @NewTabID
						,OrderIndex = @NewOrderIndex
					WHERE
						RowKey		= @RowKey

				--PRINT 'OldOrderIndex ' + CAST (@OldOrderIndex AS NVARCHAR(10))
				--PRINT 'NewOrderIndex ' + CAST (@NewOrderIndex AS NVARCHAR(10))
						
				-- Update other numbers if Number is changed
				IF @OldOrderIndex <> @NewOrderIndex
					BEGIN			
						-- case if OrderIndex is bigger now and existed before - decrement all in move window
						IF @OldOrderIndex < @NewOrderIndex 
							BEGIN
								UPDATE planning.tTabs
								SET OrderIndex = OrderIndex - 1 
								WHERE 
										OrderIndex		>= @OldOrderIndex 
									AND OrderIndex		<= @NewOrderIndex
									AND	FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			<> @NewTabID
							END
						-- case if OrderIndex is smaller now and existed before - increment all in move window
						IF @OldOrderIndex > @NewOrderIndex 
							BEGIN
								UPDATE planning.tTabs
								SET OrderIndex = OrderIndex + 1 
								WHERE 
										OrderIndex		<= @OldOrderIndex 
									AND OrderIndex		>= @NewOrderIndex
									AND	FactoryKey		= @FactoryKey
									AND ProductLineKey	= @ProductLineKey
									AND ProductKey		= @ProductKey
									AND TabID			<> @NewTabID
							END

						-- eliminate all Gaps
						UPDATE planning.tTabs 
						SET OrderIndex = Rownumbers.Number
						FROM planning.tTabs tT INNER JOIN (
																	SELECT 
																		 ROW_NUMBER() OVER (ORDER BY OrderIndex) AS Number
																		,TabID
																	
																	FROM planning.tTabs
																	WHERE 
																			FactoryKey		= @FactoryKey
																		AND ProductLineKey	= @ProductLineKey
																		AND ProductKey		= @ProductKey

															) AS Rownumbers
						ON 
								tT.TabID			= Rownumbers.TabID
							AND tT.FactoryKey		= @FactoryKey
							AND tT.ProductLineKey	= @ProductLineKey
							AND tT.ProductKey		= @ProductKey

					END

				IF @ResultCode <> 201 SET @ResultCode = 200;

			COMMIT TRANSACTION spMOVE_Tab;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spMOVE_Tab;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'		-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spMOVE_Tab'		-- enter procedure name
    ,@SX_Owner          NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module	        NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1					-- 0 = not shiped from saxess
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
                                                            -- 10 = shiped from saxess as customer specific object
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module',@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag',@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to Move a Tab to a different ID and different OrderIndex. The requested TabID must be free. The requested Orderposition may be in use, reordering is done automaticly. 
	To put a tab to the end, any high order number (like 100) can be submitted. The Tab with ID 0 can''t be moved.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Name of the user this action is requested for.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'optional FactoryID, skip / NULL / empty string if MOVE is for Cluster level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'optional ProductLineID, skip / NULL / empty string if MOVE is for Cluster or Factory level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductID';
    SET @value      = N'optional ProductD, skip / NULL / empty string if MOVE is for Cluster, Factory or ProductLine level';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TabID';
    SET @value      = N'TabID which is moved. ID 0 is not accepted, its reserved for the system tab.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@NewTabID';
    SET @value      = N'TabID the tab will get. This must not be already in use.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@NewOrderIndex';
    SET @value      = N'Position in the list of Tabs. Any Number > 0 can be passed in, tabs reorder themself.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
