﻿
DROP PROCEDURE IF EXISTS planning.spPOST_File;
GO

/*
	Procedure to save new file or change exists file
	Files may be saved on Factories, Productlines, Products on Clusters - thefore the F/PL/PID is optional 
	@FileBody can be NULL - this are files which represend folders
	If @FileID is NULL or Empty then error raised

	If @FileID is '*' will create new file
		@FileExtension can be null (will be created file without extension)
		@FileName can`t be null or empty

	If @FileID is NOT null will change exists file
		If @FileExtension is NULL then don`t change extension of exists file
		If @FileExtension is empty then set to NULL extension of exists file (file without extension)
		Otherwise change extension to @FileExtension
	
		If @FileName is NULL then don`t change name of exists file
		If @FileName is empty then error raised
		Otherwise change file name to @FileName

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		DECLARE  @RC				INT
				,@ReturnedFileID	NVARCHAR (50);
		EXEC planning.spPOST_File 
			'SQL'
			,''
			,''
			,''
			,''     -- FileID or *
			,'Hallo22'
			,'pdf'
			,0x3
			,'/Rootfolder/Subfolder2'
			,@OutputID				= @ReturnedFileID OUTPUT;
		PRINT    @RC;
		PRINT	'FileID Output: '+ ISNULL(@ReturnedFileID,'no Output set');

	Testcall tables
		SELECT * FROM planning.tFiles;
		DELETE FROM planning.tFiles;

	Testcall Log
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_File',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_File','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spPOST_File
	 @Username		NVARCHAR(255)	= N''
	,@FactoryID		NVARCHAR(50)	= N''
	,@ProductLineID NVARCHAR(50)	= N''
	,@ProductID		NVARCHAR(50)	= N''
	,@FileID		NVARCHAR(50)	= N''
	,@FileName		NVARCHAR(255)	= N''
	,@FileExtension	NVARCHAR(50)	= NULL
	,@FileBody		VARBINARY(MAX)	= NULL
	,@Filepath		NVARCHAR(4000)	= N'/'
	,@OutputID		NVARCHAR(50)	= NULL OUTPUT	
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' 
													+ ISNULL(@Username					, N'NULL') + N''', '''
													+ ISNULL(@FactoryID					, N'NULL') + N''', '''
													+ ISNULL(@ProductLineID				, N'NULL') + N''', '''
													+ ISNULL(@ProductID					, N'NULL') + N''', ''' 
													+ ISNULL(@FileID					, N'NULL') + N''', ''' 
													+ ISNULL(@FileName					, N'NULL') + N''', '''	
													+ ISNULL(@FileExtension				, N'NULL') + N''', ' 
													+ ISNULL(CAST(DATALENGTH(@FileBody)   
														AS NVARCHAR(MAX)) + N' bytes'	, N'NULL') + N','''
													+ ISNULL(@Filepath,N'NULL')			+ N''''		
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N''
			,@FactoryKey			INT				= 0
			,@ProductLineKey		INT				= 0
			,@ProductKey			INT				= 0
			,@FileExistsFlag		INT				= 0;
			
	--NULL Protection
	IF @Username		IS NULL	SET @Username		= N'';
	IF @FactoryID		IS NULL SET @FactoryID		= N'';
	IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
	IF @ProductID		IS NULL SET @ProductID		= N'';
	IF @FileID			IS NULL SET @FileID			= N'';
	IF @Filepath		IS NULL SET @Filepath		= N'/';
	IF @FactoryID		= N''	SET @FactoryKey		= -1;
	IF @ProductLineID	= N''	SET @ProductLineKey = -1;
	IF @ProductID		= N''	SET @ProductKey		= -1;

	BEGIN TRY
		BEGIN TRANSACTION spPOST_File;

		-- protect IDs
		SET @Username		= dbo.sx_pf_pProtectString (@Username);
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		SET @ProductID		= dbo.sx_pf_pProtectID	(@ProductID);

		-- empty FileID will generate a new FileID
		IF @FileID = '' SET @FileID = '*';

		-- determine Transactuser
		SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactionUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			THROW 60000, N'Transaction user don`t exists', 100;
		END;
			
		-- check input parameters (NULL for FileBody is allowed to save files which represent folders)
		IF @FileName =''
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileName is empty', 100;
			END;

		-- error if Keys chains has gaps
		IF (@FactoryKey	= -1 AND (@ProductLineKey <> -1 OR @ProductKey <> -1))
			OR	(@ProductLineKey = -1 AND @ProductKey <> -1) 
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys combination can have only gaps at the end', 16, 10);
		END;

		IF @FactoryKey <> -1
		BEGIN
			SELECT @FactoryKey	= FactoryKey FROM planning.tdFactories WHERE FactoryID = @FactoryID;

			IF @ProductLineKey <> -1
			BEGIN
				SELECT @ProductLineKey = ProductLineKey	FROM planning.tdProductLines WHERE FactoryKey = @FactoryKey	AND ProductLineID = @ProductLineID;
				
				IF @ProductKey <> -1
				BEGIN
					SELECT @ProductKey = ProductKey	FROM planning.tdProducts WHERE FactoryKey = @FactoryKey	AND ProductLineKey = @ProductLineKey AND ProductID = @ProductID;
				END 
			END
		END

		-- check user rights for cluster level
		IF @FactoryKey = -1
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactionUsername;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for Cluster write.', 16, 10);
			END 
		END
		-- check user rights for Factory level
		ELSE IF @ProductLineKey = -1
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_FactoryWriteRight @TransactionUsername, @FactoryID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for Factory write.', 16, 10);
			END 
		END
		-- check user rights for ProductLine / Product level
		ELSE IF @ProductLineKey > 0
		BEGIN
			EXEC @ResultCode = dbo.sx_pf_pGET_ProductLineWriteRight @TransactionUsername, @FactoryID, @ProductLineID;
			IF @ResultCode <> 200
			BEGIN
				RAISERROR('Invalid rights for ProductLine.', 16, 10);
			END 
		END
				
		-- error if and Keys dont exists
		IF		@FactoryKey		= 0 
			OR	@ProductLineKey	= 0
			OR  @ProductKey		= 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Not existing FID / PID / PLID - Keys don`t exists', 16, 10);
		END;

		-- if FileID does not exits or FileID = '*', create new file
		SELECT @FileExistsFlag = COUNT(FileID) 
			FROM planning.tFiles 
			WHERE 
					FactoryID		= @FactoryID 
				AND ProductLineID	= @ProductLineID 
				AND ProductID		= @ProductID 
				AND FileID			= @FileID;
		
		IF @FileExistsFlag = 0 OR @FileID = '*'
			-- create new file
			BEGIN
				IF @FileID = '*'
					BEGIN
						-- Determine next free FileID - outer COALESCE FOR CASE no existing row, inner COALESCE for TRY_CAST catching
						SELECT @FileID =
						 CONCAT('F',
							COALESCE(					
									MAX(COALESCE(TRY_CAST(RIGHT(FileID,LEN(FileID)-1) AS INT),0))
									,0)+1
									)
						FROM planning.tFiles 
						WHERE	
								(@FactoryKey = -1		OR FactoryKey = @FactoryKey)
							AND (@ProductLineKey = -1	OR ProductLineKey = @ProductLineKey)
							AND (@ProductKey = -1		OR ProductKey = @ProductKey)
							AND FileID LIKE 'F%';
					END
			
				IF LEN(@FileExtension) = 0	SET @FileExtension = NULL;
			
				-- create new file
				INSERT INTO planning.tFiles (FactoryKey,  ProductLineKey,  ProductKey,  FactoryID,  ProductLineID,  ProductID,  FileID, [FileName], FileExtension,  FileBody, UploadedBy, UploadedAt,Filepath)
				VALUES						(@FactoryKey, @ProductLineKey, @ProductKey, @FactoryID, @ProductLineID, @ProductID, @FileID, @FileName, @FileExtension, @FileBody,@Username,GETUTCDATE(),@Filepath);
			END		
		ELSE 
			-- update existing file
			BEGIN
			
				DECLARE @FileKey BIGINT = NULL;

				SELECT TOP (1) @FileKey = FileKey 
				FROM planning.tFiles
				WHERE 
						FactoryKey		= @FactoryKey
					AND ProductLineKey	= @ProductLineKey
					AND ProductKey		= @ProductKey
					AND FileID			= @FileID;

				-- update existing file
				UPDATE planning.tFiles
				SET 
					 [FileName]		= ISNULL(@FileName, [FileName])
					,FileExtension	= CASE 
										WHEN @FileExtension IS NULL  THEN FileExtension
										WHEN LEN(@FileExtension) = 0 THEN NULL
										ELSE @FileExtension 
									  END
					,FileBody		= ISNULL(@FileBody,FileBody)
					,UploadedBy		= @Username
					,UploadedAt		= GETUTCDATE()
					,Filepath		= ISNULL(@Filepath,Filepath)
				WHERE 
					FileKey = @FileKey;
			END

		-- OUTPUT Parameter
		SET @OutputID = @FileID;

		SET @EffectedRows = @@ROWCOUNT;
		SET @ResultCode = 200;

		COMMIT TRANSACTION spPOST_File;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		ROLLBACK TRANSACTION spPOST_File;
			
		IF @Error_state = 100 
		BEGIN
			PRINT @Comment;
		END
		ELSE IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

    EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
	RETURN @ResultCode;
END;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'						-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_File'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to store a new file or change an existing file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID - maybe empty string to store files on Clusterlevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID - maybe empty string to store files on Factorylevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductID';
    SET @value      = N'ProductID - maybe empty string to store files on ProductLineLevel';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileID';
    SET @value      = N'FileID - use * to autocreate File ID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileName';
    SET @value      = N'FileName - username for the file';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileExtension';
    SET @value      = N'FileExtension - may be empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileBody';
    SET @value      = N'FileBody - must not be NULL';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@FilePath';
    SET @value      = N'Filepath - optional virtual file path';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
