﻿
DROP PROCEDURE IF EXISTS planning.spPOST_FileFromPath;
GO

/*
 	Procedure to Create new file or change exists file from a local SQL Server filepath or shared folder path
	File must extists and can`t be empty.
	File extensions extracted from @FilePath.

	If @FileKey is NULL will create new file
	If @FileKey is NOT NULL will change exists file
	
	Return table with columns: FileKey BIGINT, FileName NVARCHAR(MAX)

	TODO: Must be adjusted to FID;PLID;PID !!

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		EXEC planning.spPOST_FileFromPath 
			'SQL'
			,NULL						-- use NULL to send new file !
			,'c:\test.txt';

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_FileFromPath',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_FileFromPath','PARAMETER',NULL)
*/
   
CREATE PROCEDURE planning.spPOST_FileFromPath
	 @Username		NVARCHAR(255)	= N''
	,@FileKey		BIGINT			= NULL
	,@FilePath		NVARCHAR(300)	= NULL
AS
BEGIN
    SET NOCOUNT ON

	DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
			,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '+ ISNULL(CAST(@FileKey AS NVARCHAR(MAX)), N'NULL') + N', ''' + ISNULL(@FilePath, N'NULL') + N'''' 
			,@EffectedRows			INT				= 0					
			,@ResultCode			INT				= 501			
			,@TimestampCall			DATETIME		= GETUTCDATE()
			,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username IS NULL SET @Username = N'';
		IF @FilePath IS NULL SET @FilePath = N'';

		BEGIN TRY
			BEGIN TRANSACTION POST_FileFromPath;

			-- STEP 0.2 - Protect input parameters

			SET @Username = dbo.sx_pf_pProtectString (@Username);
			SET @FilePath = REPLACE(REPLACE(dbo.sx_pf_pProtectString (@FilePath), N'/*', N''), N'*\', N'');

			-- STEP 1.1 - Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;
			
			-- STEP 1.2 - Check input parameters

			IF LEN(@FilePath) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FilePath is empty', 100;
			END;

			IF NOT @FileKey IS NULL 
				AND NOT EXISTS (
					SELECT TOP (1) 1 
					FROM planning.tFiles
					WHERE FileKey = @FileKey) 
			BEGIN
				SET @ResultCode = 404;
				THROW 60000, N'File not found', 100;
			END;

			-- STEP 2.1 - Read FileBody from path

			DECLARE @FileBody VARBINARY(MAX),
				@SQLQuery NVARCHAR(MAX),
				@ParmDefinition NVARCHAR(30) = N'@Data VARBINARY(MAX) OUTPUT';

			SET @SQLQuery = N'
				SET @Data = (SELECT * FROM OPENROWSET(
				BULK N''' + @FilePath + ''', SINGLE_BLOB, MAXERRORS=0) AS InputFile)';

			EXEC sp_executesql
				@SQLQuery
				,@ParmDefinition
				,@Data = @FileBody OUTPUT;

			IF @FileBody IS NULL 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is null', 100;
			END;

			IF DATALENGTH(@FileBody) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'FileBody is empty', 100;
			END;

		    -- STEP 2.2 - Extract FileExtension from path
			DECLARE @FileExtension	NVARCHAR(50) = NULL
				,@DotPos INT;

			SET @DotPos = CHARINDEX(N'.', REVERSE(@FilePath));
			
			IF @DotPos > 1
			BEGIN
				DECLARE @PathLength INT;

				SET @PathLength = LEN(@FilePath);

				IF CHARINDEX(N'\', @FilePath, @PathLength - @DotPos + 1) = 0
					AND CHARINDEX(N'/', @FilePath, @PathLength - @DotPos + 1) = 0
					AND CHARINDEX(N':', @FilePath, @PathLength - @DotPos + 1) = 0
				BEGIN
					SET @FileExtension = RIGHT(@FilePath, @DotPos - 1);
				END;
			END;

			DECLARE @UpdatedFile AS TABLE (FileKey BIGINT, FileExtension NVARCHAR(50));

			IF NOT @FileKey IS NULL 
			BEGIN
				-- STEP 2.3 - Updating exists file. 

				UPDATE planning.tFiles
				SET FileExtension = @FileExtension
					,FileBody = @FileBody
				OUTPUT INSERTED.FileKey, INSERTED.FileExtension INTO @UpdatedFile
				WHERE FileKey = @FileKey;

			END
			ELSE
			BEGIN	
				-- STEP 2.4 - Creating new file

				INSERT planning.tFiles (FileExtension, FileBody)
				OUTPUT INSERTED.FileKey, INSERTED.FileExtension INTO @UpdatedFile
				VALUES (@FileExtension, @FileBody);

			END;

			-- STEP 3.0 - Output result

			SELECT
				 FileKey AS 'FileKey'
				,CAST(FileKey AS NVARCHAR(MAX)) + ISNULL(N'.' + FileExtension, N'') AS 'FileName'
			FROM @UpdatedFile;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;

			COMMIT TRANSACTION POST_FileFromPath;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	

			ROLLBACK TRANSACTION POST_FileFromPath;
			
			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		
		RETURN @ResultCode;
	END;
GO

GRANT EXECUTE ON OBJECT ::planning.spPOST_FileFromPath TO pf_PlanningFactoryUser;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_FileFromPath'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name; 
 
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to Create new file or change exists file from a local SQL Server filepath or shared folder path
         File must extists and can`t be empty.
         File extensions extracted from @FilePath.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FileKey';
    SET @value      = N'FileKey';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FilePath';
    SET @value      = N'FilePath';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
