﻿
DROP PROCEDURE IF EXISTS planning.spPOST_Values_JSON;
GO

/*
	The Procedure returns all sended values as recordset with the process result
 
	Procedure to POST Values in JSON Format 
	Preconditions
	- A value can be sended only on full qualified single fact -> There is no write on summary level possible
	- multiple values may be allocated to one Valueseries / TimeID - they must be aggregated
	- A value "NULL" can be sended, this means delete the existing value

	Functionality:
		- Values for not existing ValueSeriesID are thrown away and reported as ignored
		- Values for not existing TimeIDs are thrown away and reported as ignored
		- only values for elements where the user has write rights are excepted
	
		- flag values without writerights
		- delete existing values for given F/PL/P/VS in table fValues
		- Scale numeric values based on scale in dValueSeries

		- Only values of type string are sended, DB must put it in the right type
		- if in dValueSeries 
			IsNumeric = 0 -> put Value in ValueString
			IsNumeric = 1 -> put it in ValueInt, after scaling with Scale. Decimal separator is the dot "."
		- Special cases
			IsNumeric = 0 AND Value IS NULL -> DELETE, no write
			IsNumeric = 1 AND Value IS NULL -> DELETE, no write
			IsNumeric = 1 AND Value = 0 AND ValueSeriesID LIKE "_0" -> write

	Overhaul Workflow
		 - put all sended records in a temporary table (SendedRecords)
		 - create a temporary classification table with all sended F/PL/P/ID Values (SendedRecordClassification)
			- determine the ValueSeries in Classification table (Numeric, Source)
			- determine the UserRights in Classification table
			- determine additional properties
		 - create an temporary table for all values which have to be inserted in OCT (ResultSetForOCT)
		 - fill the ResultSetForOCT
			- with all Records which address Points the user has write rights
			- flag values which belong to not existing TimeIDs
			- aggregation of all records which address the same point
			- values are putted into type fitting columns
		 - flag NULL Values in ResultSetForOCT as Value to Delete
		 - Update SendedRecords to write Resultcodes / Comments about their fate in ResultSetForOCT
			- sended Summary Values not used
			- no Rights
			- not existing TimeIDs
			- aggregated and inserted
			- aggregated and deleted
			- deleted
			- inserted
			- which are sended on ValueSeries of Type XLS / XLS-String for Warning that formulas may ovewrite Result
		 - write ResultSetForOCT
		 - return SendedRecords

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC INT;
		DECLARE @ValuesJSON NVARCHAR(MAX);
			SET @ValuesJSON = N'{

				"connector_type":"write","version":1,"Values":[

				 {"ValueNumber":1	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"K1"		,"TimeID":"20210115"	,"Value":100			,"ValueComment":null}
				,{"ValueNumber":2	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"K1"		,"TimeID":"20210115"	,"Value":200			,"ValueComment":null}
				,{"ValueNumber":3	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"K3"		,"TimeID":"20210115"	,"Value":0				,"ValueComment":null}
				,{"ValueNumber":4	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"BETR"		,"TimeID":"20210115"	,"Value":770000			,"ValueComment":null}
				,{"ValueNumber":5	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"BEM"		,"TimeID":"20210115"	,"Value":null			,"ValueComment":null}
				,{"ValueNumber":6	,"FactoryID":"ZT"	,"ProductLineID":"U"	,"ProductID":"1"	,"ValueSeriesID":"STAT_BUD"	,"TimeID":"20210115"	,"Value":"Genehmigt"	,"ValueComment":null}

			]

			}';

		EXEC @RC = planning.spPOST_Values_JSON 'SQL', @ValuesJSON;
		PRINT @RC;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Values_JSON',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spPOST_Values_JSON','PARAMETER',NULL);
*/   

CREATE PROCEDURE planning.spPOST_Values_JSON
		 	 @Username		NVARCHAR(255) = ''
			,@ValuesJSON	NVARCHAR(MAX) = ''
AS

	BEGIN
		SET NOCOUNT ON;

		-- Logging Variables

		DECLARE  @TransactionUsername	NVARCHAR(255)	= N''
				,@ProcedureName			NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
				,@ParameterString		NVARCHAR(MAX)	= N'''' + ISNULL(@Username, N'NULL') + N''', '''+ ISNULL(@ValuesJSON, N'NULL')+ N''''
				,@EffectedRows			INT				= 0					
				,@ResultCode			INT				= 501			
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N'';						
		
		-- STEP 0.1 - NULL Protection

		IF @Username		IS NULL SET @Username		= N'';
		IF @ValuesJSON		IS NULL SET @ValuesJSON		= N'';

		BEGIN TRY
			BEGIN TRANSACTION spPOST_Values_JSON
			-- INPUT PROTECTION	  #################################################################################

			SET @Username		= dbo.sx_pf_pProtectString	(@Username);
			
			-- Test for valid JSON ################################################################################

			IF	@ValuesJSON			= N'' 
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

			IF ISJSON(@ValuesJSON) = 0 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid JSON (technical)', 100;
			END;
			
			-- Extract Values FROM JSON and put them into #SendedRecords #####################################

			DROP TABLE IF EXISTS #SendedRecords

			;WITH vals AS 
				(
					SELECT
						 ValueNumber							AS 'ValueNumber'
						,dbo.sx_pf_pProtectID (FactoryID)		AS 'FactoryID'
						,dbo.sx_pf_pProtectID (ProductLineID)	AS 'ProductLineID'
						,dbo.sx_pf_pProtectID (ProductID)		AS 'ProductID'
						,dbo.sx_pf_pProtectID (ValueSeriesID)	AS 'ValueSeriesID'
						,TimeID									AS 'TimeID'
						,[Value]								AS 'Value'				-- there maybe NULL inserted
						,ValueComment							AS 'ValueComment'
					FROM OPENJSON (@ValuesJSON, N'$.Values') 
					WITH (
						 ValueNumber	INT				N'$.ValueNumber'
						,FactoryID		NVARCHAR(255)	N'$.FactoryID'
						,ProductLineID	NVARCHAR(255)	N'$.ProductLineID'
						,ProductID		NVARCHAR(255)	N'$.ProductID'
						,ValueSeriesID	NVARCHAR(255)	N'$.ValueSeriesID'
						,TimeID			INT				N'$.TimeID'
						,[Value]		NVARCHAR(MAX)	N'$.Value'
						,ValueComment	NVARCHAR(MAX)	N'$.ValueComment'
						)
					)

			SELECT 	
				 ValueNumber 
				,FactoryID		COLLATE DATABASE_DEFAULT  AS FactoryID
				,ProductLineID	COLLATE DATABASE_DEFAULT  AS ProductLineID
				,ProductID		COLLATE DATABASE_DEFAULT  AS ProductID
				,ValueSeriesID	COLLATE DATABASE_DEFAULT  AS ValueSeriesID
				,TimeID		
				,[Value]	    COLLATE DATABASE_DEFAULT  AS [Value]
				,ValueComment	COLLATE DATABASE_DEFAULT  AS ValueComment
			INTO #SendedRecords	
			FROM vals;

			-- Debug: SELECT * FROM #SendedRecords

			-- Check for logical correct values #############################################
			PRINT 'Starting Check for logical correct Values'	
			-- no NULL Values contained in IDs - Value may be null due to deleted value
			IF EXISTS (
				SELECT TOP(1) 1 
				FROM #SendedRecords 
				WHERE 
					   ValueNumber		IS NULL
					OR FactoryID		IS NULL
					OR ProductLineID	IS NULL
					OR ProductID		IS NULL
					OR ValueSeriesID	IS NULL
					OR TimeID			IS NULL
					OR FactoryID		= N''
					OR ProductLineID	= N''
					OR ProductID		= N''
					OR ValueSeriesID	= N''
					OR TimeID			= 0
					)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to invalid Factory/PL/P/VS/T IDs.', 100;
			END;

			-- no doubled ValueNumbers contained
			IF EXISTS (
					SELECT TOP (1) 1
					FROM #SendedRecords
					GROUP BY ValueNumber
					HAVING COUNT(ValueNumber) > 1
				)
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Exit due to doubled ValueNumbers.', 100;
			END;

			-- User Rights ##################################################################
			PRINT 'Starting User Rights Check'
			-- Determine transaction user

			SELECT @TransactionUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactionUsername  = N'403' 
			BEGIN
				SET @ResultCode = 403;
				THROW 60000, N'Transaction user don`t exists', 100;
			END;

			-- Build classification Table for sended Records  #################################
			PRINT 'Starting classification Process'
			DROP TABLE IF EXISTS #SendedRecordsClassification;

			CREATE TABLE #SendedRecordsClassification
				(
					 ValueNumber				INT											NOT NULL 
					,FactoryID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductLineID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductID					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ValueSeriesID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,TimeID						BIGINT										NOT NULL 
					,[Value]					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	    NULL 
					,ValueComment				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	    NULL 
					,VS_IsNumeric				INT											NOT NULL 
					,Scale						INT											NOT NULL 
					,ValueSource				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ValueInt					BIGINT										NOT NULL 
					,ValueText					NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,UserRight					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,AggregationFlag			INT											NOT NULL 
					,DeleteFlag					INT											NOT NULL 
					,SaveValueZeroFlag			INT											NOT NULL 
					,TargetIDsNotExistingFlag	INT											NOT NULL 
					,ReturnCode					INT											NOT NULL 
					,StatusComment				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductKey					BIGINT										NOT NULL 
					,ValueSeriesKey				BIGINT										NOT NULL 
					,SaveValueFlag				INT											NOT NULL 
				);

			INSERT INTO #SendedRecordsClassification 
				SELECT 	
					 ValueNumber									AS ValueNumber
					,FactoryID		COLLATE DATABASE_DEFAULT		AS FactoryID
					,ProductLineID	COLLATE DATABASE_DEFAULT		AS ProductLineID
					,ProductID		COLLATE DATABASE_DEFAULT		AS ProductID
					,ValueSeriesID	COLLATE DATABASE_DEFAULT		AS ValueSeriesID
					,TimeID											AS TimeID
					,[Value]	    COLLATE DATABASE_DEFAULT		AS [Value]
					,ValueComment	COLLATE DATABASE_DEFAULT		AS ValueComment

					,0												AS VS_IsNumeric
					,0												AS Scale
					,N''			COLLATE DATABASE_DEFAULT		AS ValueSource
					,0												AS ValueInt
					,N''			COLLATE DATABASE_DEFAULT		AS ValueText
					,''												AS UserRight
					,0												AS AggregationFlag
					,0												AS DeleteFlag
					,0												AS SaveValueZeroFlag		-- for _0 ValueSeries
					,0												AS TargetIDsNotExistingFlag
					,501											AS ReturnCode
					,N'Original'		COLLATE DATABASE_DEFAULT	AS StatusComment
					,0												AS ProductKey
					,0												AS ValueSeriesKey
					,0												AS SaveValueFlag	
				FROM #SendedRecords;
			
			-- add ValuSeries Properties incl. ValueSeriesKey
			UPDATE SRC SET
				 SRC.VS_IsNumeric	= dVS.[IsNumeric]
				,SRC.Scale			= dVS.Scale
				,SRC.ValueSource	= dVS.ValueSource
				,SRC.ProductKey		= dVS.ProductKey
				,SRC.ValueSeriesKey = dVS.ValueSeriesKey
			FROM #SendedRecordsClassification SRC
				INNER JOIN planning.tdValueSeries dVS  -- only for hits
					ON 
							SRC.FactoryID		= dVS.FactoryID
						AND SRC.ProductLineID	= dVS.ProductLineID
						AND SRC.ProductID		= dVS.ProductID
						AND SRC.ValueSeriesID	= dVS.ValueSeriesID;
	
			-- Flag Target VSIDs which don't exist
			UPDATE #SendedRecordsClassification 
			SET 
				 TargetIDsNotExistingFlag	= 1
				,ReturnCode					= 404
				,StatusComment				= N'Error VSID does not exits'
				,SaveValueFlag				= -1
			WHERE ValueSeriesKey = 0;

			-- Flag Target TimeIDs which don't exist
			UPDATE SRC SET
				 SRC.TargetIDsNotExistingFlag = 1
				,SRC.ReturnCode = 404
				,SRC.StatusComment = N'Error TimeID does not exits'
				,SRC.SaveValueFlag = -1
			FROM #SendedRecordsClassification SRC
				LEFT JOIN planning.tdTime dT
				ON 
						SRC.ProductKey  = dT.ProductKey
					AND SRC.TimeID		= dT.TimeID
			WHERE 
					dT.TimeID IS NULL
				AND SRC.SaveValueFlag <> -1;

			-- Flag _0 ValueSeries
			UPDATE #SendedRecordsClassification  
				SET SaveValueZeroFlag = 1 
			WHERE 
					ValueSeriesID LIKE N'%_0'
				AND SaveValueFlag <> -1;

			-- determine UserRights
			UPDATE SRC SET
				 SRC.UserRight = tUR.[Right]	
			FROM #SendedRecordsClassification SRC
				LEFT JOIN system.trUserRights tUR
				ON 
						SRC.FactoryID		= tUR.FactoryID
					AND SRC.ProductLineID	= tUR.ProductLineID
					AND tUR.UserName		= @TransactionUsername
			WHERE  tUR.[Right] IS NOT NULL;

			UPDATE #SendedRecordsClassification 
				SET 
					 SaveValueFlag = -1 
					,ReturnCode = 403
					,StatusComment = 'No write rights.'
				WHERE 
						UserRight <> 'Write'
					AND SaveValueFlag <> -1;

			-- Set ValueInt / ValueText based on Value and ValueSeries Type
			UPDATE #SendedRecordsClassification 
				SET ValueInt = CAST(COALESCE(TRY_CAST([Value] AS MONEY),0) * SCALE AS BIGINT)
			WHERE 
					VS_IsNumeric = 1
				AND [Value] IS NOT NULL;

			UPDATE #SendedRecordsClassification 
				SET ValueText = [Value]
			WHERE 
					VS_IsNumeric = 0
				AND [Value] IS NOT NULL;
	
			-- Set a warning, if XLS / XLS-Strict values are overwritten
			UPDATE #SendedRecordsClassification 
				SET StatusComment =  StatusComment + ' Warning - XLS/XLS Strict formula field will be overwritten'
			WHERE 
					ValueSource IN ('XLS','XLS-Strict')
				
			-- Flag Values to be aggregated if they are numeric and NOT NULL and multiples are sended (from multiple sended Text Values, the last wins)
			UPDATE SRC SET
				 SRC.AggregationFlag	= 1
				,SRC.ReturnCode			= 200
				,SRC.StatusComment		= StatusComment + CONCAT(' Aggregated with ',Agg.ValueNumberCount - 1,' other values.')
				,SRC.SaveValueFlag		= 1
			FROM #SendedRecordsClassification SRC
				LEFT JOIN (
								SELECT 
									 COUNT(ValueNumber)					AS ValueNumberCount
									,SUM(TRY_CAST([Value] AS MONEY))	AS [Value]
									,FactoryID
									,ProductLineID
									,ProductID
									,ValueSeriesID
									,TimeID
								FROM #SendedRecordsClassification

								WHERE 
										VS_IsNumeric = 1
									AND SaveValueFlag <> -1
									AND [Value] IS NOT NULL				--NULLs are not marked for Aggregation in this step

								GROUP BY
									 FactoryID
									,ProductLineID
									,ProductID
									,ValueSeriesID
									,TimeID
								
								HAVING 
										COUNT(ValueNumber) > 1
									
							) AS Agg
					ON 
							Agg.FactoryID		= SRC.FactoryID
						AND Agg.ProductLineID	= SRC.ProductLineID
						AND Agg.ProductID		= SRC.ProductID
						AND Agg.ValueSeriesID	= SRC.ValueSeriesID
						AND Agg.TimeID			= SRC.TimeID

				WHERE Agg.ValueNumberCount IS NOT NULL;						

			-- Flag NULL Values as aggregated, if there are other values which write to the same poing
			UPDATE SRC
				 SET SRC.AggregationFlag	= 1
				,SRC.StatusComment			= 'Value was not deleted, as other Rows send values to this VS/TID.'
			FROM #SendedRecordsClassification SRC 
				LEFT JOIN (
								SELECT 
									 ValueSeriesKey
									,TimeID
								FROM #SendedRecordsClassification
								WHERE AggregationFlag = 1
						) Agg
					ON 
							SRC.ValueSeriesKey	= Agg.ValueSeriesKey
						AND SRC.TimeID			= Agg.TimeID
			WHERE 
				SRC.VS_IsNumeric = 1 
				AND SRC.[Value] IS NULL;			

			-- Flag Values to be deleted - but not if they are flagged for Aggregation
			UPDATE #SendedRecordsClassification 
				SET 
					 DeleteFlag		= 1 
					,ReturnCode		= 200
					,StatusComment	= 'Value deleted'
			WHERE 
					[Value] IS NULL
				AND AggregationFlag = 0
				AND SaveValueFlag <> -1;

			-- Flag Numeric Zero Values as to be deleted, except ValueSeries saving zeros
			UPDATE #SendedRecordsClassification
			SET		
				 DeleteFlag = 1 
				,ReturnCode = 200
				,StatusComment = 'Value deleted as 0 was send'
			WHERE 
					ValueInt = 0 
				AND VS_IsNumeric = 1
				AND SaveValueZeroFlag = 0
				AND SaveValueFlag <> -1;

			---- Flag Zero Values as not to be saved, except VS ValueSeries  - REPLACED by delete zero values to delete existing values
			--UPDATE #SendedRecordsClassification
			--SET		
			--	 SaveValueFlag = -1 
			--	,ReturnCode = 202
			--	,StatusComment = 'Not saved as value is zero.'
			--WHERE 
			--		ValueInt = 0 
			--	AND VS_IsNumeric = 1
			--	AND SaveValueZeroFlag = 0
			--	AND SaveValueFlag <> -1;

			-- Set Null Values for ValueComment to empty string
			UPDATE #SendedRecordsClassification
			SET		
					ValueComment = '' 
			WHERE 
					ValueComment IS NULL
				AND SaveValueFlag <> -1;

			-- Flag all remaining Value as to be saved
			UPDATE #SendedRecordsClassification 
				SET	
					 SaveValueFlag		= 1
					,ReturnCode			= 200
					,StatusComment		= 'Value saved'
				WHERE 
						SaveValueFlag	= 0
					AND DeleteFlag		= 0;

			SELECT * FROM #SendedRecordsClassification --DEBUG				
				
			-- Delete Values which have to be deleted ############################################
			PRINT 'Starting data manipulation'
			DELETE fV FROM planning.tfValues fV
				INNER JOIN  #SendedRecordsClassification SRC
					ON 
						SRC.ValueSeriesKey = fV.ValueSeriesKey 
					AND SRC.TimeID = fV.TimeID
				WHERE SRC.DeleteFlag = 1;

			-- Fill Values to #SavedRecords #######################################################
			DROP TABLE IF EXISTS #SavedRecords;

			CREATE TABLE #SavedRecords
				(
					 ValueSeriesKey	BIGINT									NOT NULL
					,TimeID			BIGINT									NOT NULL
					,ValueInt		BIGINT									NOT NULL
					,ValueText		NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL 
					,ValueComment	NVARCHAR(MAX) COLLATE DATABASE_DEFAULT	NOT NULL
				)
			
			-- the aggregated Values
			PRINT 'Filling SaveSet with aggregated values'
			INSERT INTO #SavedRecords
				SELECT
					 ValueSeriesKey
					,TimeID
					,SUM(ValueInt)		AS ValueInt
					,N''				AS ValueText
					,N''				AS ValueComment
				FROM #SendedRecordsClassification
				WHERE AggregationFlag = 1
				GROUP BY 
					 ValueSeriesKey
					,TimeID

			-- the Values to be saved
			PRINT 'Filling SaveSet with standard values'
			INSERT INTO #SavedRecords
				SELECT
					 ValueSeriesKey
					,TimeID
					,ValueInt		
					,ValueText
					,COALESCE(ValueComment,'')
				FROM #SendedRecordsClassification
				WHERE 
						SaveValueFlag	= 1
					AND DeleteFlag		= 0
					AND AggregationFlag = 0
					
			-- Check #SavedRecords for security for doubled values
			PRINT 'Start validation for SaveSet'
			IF EXISTS
				(SELECT 
					 TimeID
					,ValueSeriesKey
					,COUNT(ValueSeriesKey) AS Anzahl
				FROM #SavedRecords
				GROUP BY 
					 TimeID
					,ValueSeriesKey
				HAVING COUNT(ValueSeriesKey) > 1
				)
				BEGIN
					SET @ResultCode = 500;
					THROW 60000, N'Internal Error, Rows to save not unique.', 100;
				END

			-- Delete old Values
			PRINT 'Start deleting of old Values.'
			DELETE fV FROM planning.tfValues fV
				INNER JOIN  #SavedRecords SR
					ON 
						SR.ValueSeriesKey = fV.ValueSeriesKey 
					AND SR.TimeID = fV.TimeID
				WHERE 
							SR.ValueSeriesKey IS NOT NULL 
						AND SR.TimeID IS NOT NULL;

			-- Insert new Values
			PRINT 'Start insert new Values.'
			INSERT INTO planning.tfValues 
				SELECT 
					 SR.ValueSeriesKey
					,dV.ProductKey
					,dV.ProductLineKey
					,dV.FactoryKey
					,dV.ProductID
					,dV.ProductLineID
					,dV.FactoryID
					,dV.ValueSeriesID
					,SR.TimeID
					,'' AS ValueFormula
					,SR.ValueInt
					,SR.ValueText
					,SR.ValueComment
				FROM #SavedRecords SR
					LEFT JOIN planning.tdValueSeries dV
						ON SR.ValueSeriesKey = dV.ValueSeriesKey;

			-- Create result table to be returnd #################################################

			DROP TABLE IF EXISTS #ResultSet;

			CREATE TABLE #ResultSet (
				 ValueNumber		INT				NOT NULL
				,FactoryID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductLineID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ProductID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				,ValueSeriesID		NVARCHAR(255)	COLLATE DATABASE_DEFAULT NOT NULL
				
				,TimeID				INT				NOT NULL

				,[Value]			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT NULL
				,ValueComment		NVARCHAR(MAX)   COLLATE DATABASE_DEFAULT NULL

				,ReturnCode			INT				NULL
				,ReturnComment		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT NULL
				);
			
			-- Fill result table with requested values

			PRINT 'Start filling result set.'
			INSERT INTO #ResultSet (ValueNumber, FactoryID, ProductLineID, ProductID, ValueSeriesID, 
									TimeID, [Value], ValueComment,ReturnCode,ReturnComment)
			SELECT ValueNumber
				,FactoryID
				,ProductLineID
				,ProductID
				,ValueSeriesID
				,TimeID
				,[Value]
				,ValueComment
				,0
				,''
			FROM #SendedRecords;
		
			-- Set Return Codes an comments
			PRINT 'Setting ReturnCodes in Result Set.'
			UPDATE RS	
				SET 
					 RS.ReturnCode		= SRC.ReturnCode
					,RS.ReturnComment	= SRC.StatusComment
			FROM #ResultSet RS
				LEFT JOIN #SendedRecordsClassification SRC
					ON RS.ValueNumber = SRC.ValueNumber;

			DECLARE @ProcessTime INT;
			SET @ProcessTime = DATEDIFF(ms, @TimestampCall, GETUTCDATE());
			SET @ResultCode = 200;

			-- Return result
			SELECT  
					 *
					,@ProcessTime AS ProcessTime
			FROM #ResultSet;

			COMMIT TRANSACTION spPOST_Values_JSON;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();	
			
			ROLLBACK TRANSACTION spPOST_Values_JSON;

			IF @Error_state = 100 
			BEGIN
				PRINT @Comment;
			END
			ELSE IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH
        EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactionUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;    
		RETURN @ResultCode;
	END;
GO
		
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'planning'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_Values_JSON'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                              
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
  
-- SET documententation             *************************************************************************
  
-- SET Procedure documentation
    SET @value = N'Procedure to POST Values in JSON Format';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ValuesJSON';
    SET @value      = N'Valus which are to be saved as JSON object.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
