
DROP PROCEDURE IF EXISTS result.spDataEntryNumericValues;
GO

/*
	Procedure to deliver all numeric values from data entry products to the enduser
	Expectes an optional parameter "Year" with one or multiple Years

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username		NVARCHAR(255) = 'SQL'
		DECLARE @FactoryID		NVARCHAR(255) = ''
		DECLARE @ProductLineID	NVARCHAR(255) = ''
		DECLARE @ParameterJSON	NVARCHAR(MAX) = N'{"Year":[2020]}'

		EXECUTE @RC = result.spDataEntryNumericValues
		   @Username
		  ,@FactoryID
		  ,@ProductLineID 
		  ,@ParameterJSON

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spDataEntryNumericValues',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spDataEntryNumericValues','PARAMETER',NULL)
*/

CREATE PROCEDURE result.spDataEntryNumericValues 
	(
		 @Username		NVARCHAR(255)
		,@FactoryID		NVARCHAR(255) = N''
		,@ProductLineID NVARCHAR(255) = N''
		,@ParameterJSON NVARCHAR(MAX) = N''
	)
AS
	BEGIN
		SET NOCOUNT ON;
		DECLARE 
			@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR(MAX)	= N''''		
												+ ISNULL(@Username		,N'NULL')	+ N''','''											 
												+ ISNULL(@FactoryID		,N'NULL')	+ N''','''	
												+ ISNULL(@ProductLineID	,N'NULL')	+ N''','''	
												+ ISNULL(@ParameterJSON	,N'NULL')	+ N''''
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';
		IF @ParameterJSON	IS NULL SET @ParameterJSON	= N'';

		-- Content Protection
		SET @FactoryID		= dbo.sx_pf_pProtectID	(@FactoryID);
		SET @ProductLineID	= dbo.sx_pf_pProtectID	(@ProductLineID);
		
		DECLARE @years AS TABLE (Year INT NOT NULL);
		DECLARE @hasYears BIT = 0;

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			-- Test for valid JSON
			SET @hasYears = ISJSON(@ParameterJSON);

			IF @ParameterJSON <> N'' AND @hasYears = 0
			BEGIN 
				SET @ResultCode = 403;
				RAISERROR('Exit due to invalid JSON', 16, 10);
			END;

			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
			IF @TransactUsername = N'403' 
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- determine Years
			IF @hasYears = 1
			BEGIN 
				INSERT INTO @years
				SELECT DISTINCT value
				FROM OPENJSON(@ParameterJSON,'$.Year')
				WHERE value >= 0 AND value <= 9999;
			END;
			
			-- Helper Table for Userrights
			DROP TABLE IF EXISTS #tRights;
			CREATE TABLE #tRights
				(
					 RowKey					BIGINT IDENTITY (1,1)
					,FactoryID				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
					,ProductLineID			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL

					CONSTRAINT PK_tRights PRIMARY KEY CLUSTERED (RowKey)			

				);

		  INSERT INTO #tRights
			SELECT 
				 FactoryID
				,ProductLineID
			FROM system.trUserRights 
			WHERE 
					Username = @TransactUsername 
				AND [Right] IN (N'Write', N'Read') 
				AND ProductLineID	<> N''
				AND FactoryID		<> N''
				AND (
						-- no Filter Paramter given
						(@FactoryID = N'' AND @ProductLineID = N'') 
						OR
								
							(	-- FactoryID Filter given
								FactoryID = @FactoryID 
								-- optional also ProductLine Filter given
							AND (@ProductLineID = N'' OR ProductLineID = @ProductLineID)
							)
					);

			SELECT 
				 fV.FactoryID
				,fV.ProductLineID
				,fV.ProductID
				,fV.ValueSeriesID

				,fV.TimeID / 10000															AS [Year]
				,fV.TimeID % 10000 / 100													AS [Month]
				,fV.TimeID % 100															AS [Day]
				,TRY_CAST(CAST(fV.TimeID AS NVARCHAR) AS DATE)								AS [Date]
			
				,TRY_CAST(fV.ValueInt AS Money) / ISNULL(dVS.Scale, 1)						AS [Value]

				,fV.FactoryID + N' ' + dF.NameShort					AS FactoryIDName 
				,dF.NameShort										AS FactoryName
				,dF.ResponsiblePerson								AS FResponsiblePerson

				,fV.ProductLineID + N' ' + dPL.NameShort			AS ProductLineIDName
				,dPL.NameShort										AS ProductLineName
				,dPL.ResponsiblePerson								AS PLResponsiblePerson

				,fV.ProductID + N' ' + dP.NameShort					AS ProductIDName
				,dP.NameShort										AS ProductName
				,dP.ResponsiblePerson								AS PResponsiblePerson

				,fV.ValueSeriesID + N' ' + dVS.NameShort			AS ValueSeriesIDName
				,dVS.NameShort
				,dVS.ValueSeriesNo
				,dVS.VisibilityLevel
				,dVS.ValueSource
				,dVS.Unit
				,dVS.Effect
				,fV.ValueComment

				,dP.[Status]
				,dP.GlobalAttribute1
				,dP.GlobalAttribute2
				,dP.GlobalAttribute3
				,dP.GlobalAttribute4
				,dP.GlobalAttribute5
				,dP.GlobalAttribute6
				,dP.GlobalAttribute7
				,dP.GlobalAttribute8
				,dP.GlobalAttribute9
				,dP.GlobalAttribute10
				,dP.GlobalAttribute11
				,dP.GlobalAttribute12
				,dP.GlobalAttribute13
				,dP.GlobalAttribute14
				,dP.GlobalAttribute15
				,dP.GlobalAttribute16
				,dP.GlobalAttribute17
				,dP.GlobalAttribute18
				,dP.GlobalAttribute19
				,dP.GlobalAttribute20
				,dP.GlobalAttribute21
				,dP.GlobalAttribute22
				,dP.GlobalAttribute23
				,dP.GlobalAttribute24
				,dP.GlobalAttribute25
			FROM planning.tfValues fV
				LEFT JOIN planning.tdProducts dP
					ON fV.ProductKey = dP.ProductKey
				LEFT JOIN planning.tdProductLines dPL
					ON fV.ProductLineKey = dPL.ProductLineKey
				LEFT JOIN planning.tdFactories dF
					ON fV.FactoryKey = dF.FactoryKey
				-- Right Filter
				INNER JOIN #tRights tR 
					ON 
							fV.FactoryID		= tR.FactoryID
						AND fV.ProductLineID	= tR.ProductLineID

				-- to filter on numeric ValueSeries
				INNER JOIN planning.tdValueSeries dVS
					ON 
							fV.ValueSeriesKey = dVS.ValueSeriesKey
						AND dVS.[IsNumeric] = 1 

				WHERE 
						fV.ValueInt <> 0 
					-- Year Filter
					AND (@hasYears = 0 OR fV.TimeID / 10000 IN (SELECT Year FROM @years))
	
			SET @ResultCode = 200;
		END TRY
		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'result'			    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spDataEntryNumericValues'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'All numeric values from data entry products'		
																-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
SET @value = @SX_Owner;
EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
SET @value = @SX_Module;
EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
SET @value = @SX_ShipmentFlag;
EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

SET @value = @SX_UserHint;
EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;

  
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
SET @value = N'Procedure to deliver all numeric values of data entry products to the enduser.' ;
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

 
-- SET parameter documentation
SET @level2name = N'@Username';
SET @value      = N'Username';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
SET @level2name = N'@FactoryID';
SET @value      = N'optional: FactoryID';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

SET @level2name = N'@ProductLineID';
SET @value      = N'optional: ProductLineID';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

SET @level2name = N'@ParameterJSON';
SET @value      = N'optional: The Parameter JSON expect the "Year" parameter.';
EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
