﻿
DROP PROCEDURE IF EXISTS result.spEvents;
GO

/*
	Procedure to deliver analytic data to the enduser
	Events must have ValueSeriesID "E" and Bemerkungen ValueSeriesID "BEM"

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	Testcall
		DECLARE @RC int
		DECLARE @Username		nvarchar(255) = 'SQL'
		DECLARE @FactoryID		nvarchar(255) = ''
		DECLARE @ProductLineID	nvarchar(255) = ''

		EXECUTE @RC = result.spEvents 
		   @Username
		  ,@FactoryID
		  ,@ProductLineID

		Print @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spEvents',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'result', 'PROCEDURE', 'spEvents','PARAMETER',NULL)
*/
 
CREATE PROCEDURE result.spEvents
(@Username AS NVARCHAR(255), @FactoryID AS NVARCHAR(255)='', @ProductLineID AS NVARCHAR(255)='')

AS
	BEGIN

		DECLARE @TransactUsername	NVARCHAR(255)	= N'';
		DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
		DECLARE @ParameterString	NVARCHAR(MAX)	= N''''		+ ISNULL(@Username, N'NULL') 
													+ N''','''	+ ISNULL(@FactoryID, N'NULL') 
													+ N''','''	+ ISNULL(@ProductLineID, N'NULL') 
													+ N'''';
		DECLARE @EffectedRows		INT				= 0;			
		DECLARE @ResultCode			INT				= 501;			
		DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
		DECLARE @Comment			NVARCHAR(2000)	= N'';			

		-- STEP 0.1 - NULL Protection
		IF @Username		IS NULL SET @Username		= N'';
		IF @FactoryID		IS NULL SET @FactoryID		= N'';
		IF @ProductLineID	IS NULL SET @ProductLineID	= N'';

		BEGIN TRY
			BEGIN TRANSACTION spEvents;

			SET @Username		= dbo.sx_pf_pProtectString (@Username);
			SET @FactoryID		= dbo.sx_pf_pProtectID (@FactoryID);
			SET @ProductLineID	= dbo.sx_pf_pProtectID (@ProductLineID);

			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

			IF @TransactUsername  = N'403'
			BEGIN
				SET @ResultCode = 403;
				RAISERROR('Transaction user don`t exists', 16, 10);
			END;

			-- # Scope table for all levels access right
			DROP TABLE IF EXISTS #Scope;
			CREATE TABLE #Scope 
				(
					 RowKey			INT IDENTITY (1,1)	
					,FactoryID		NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL 
					,ProductLineID	NVARCHAR(255) COLLATE DATABASE_DEFAULT	NOT NULL
				)

			INSERT INTO #Scope
				SELECT 
					 FactoryID
					,ProductLineID
				FROM system.trUserRights rUR
				WHERE 
							Username = @TransactUsername 
						AND [Right] IN ('Write','Read') 
						AND (FactoryID		= @FactoryID		OR @FactoryID = '')
						AND (ProductLineID	= @ProductLineID	OR @ProductLineID = '')

			-- Main Query
			SELECT
					fV.FactoryID +'/'
						+ fV.ProductLineID + '/'
					 	+ fV.ProductID  						AS Path	
                    
                    ,'https://stable.factoryprovider.com/DatabaseServer/' --set servername with sql server
						+ 'OCT/' -- set databasename
                     	+ fV.FactoryID +'/' 
						+ fV.ProductLineID + '/'
					 	+ fV.ProductID  						AS Productlink_LL

					,TRY_CAST(fV.TimeID AS Integer) / 10000					AS Year
					,(TRY_CAST(fV.TimeID AS Integer) % 10000) /100			AS Month


					,Ev.ValueText								AS [Event]
					,Ev.ValueComment							AS EventComment
					,fV.ValueText								AS Beschreibung
					,fV.ValueComment							AS BeschreibungComment --Comment, which ist placed on the Beschreibung field

					,fV.FactoryID + ' ' + dF.NameShort			AS FactoryIDName 
					,dF.NameShort								AS FactoryName
					,dF.ResponsiblePerson						AS FResponsiblePerson

					,fV.ProductLineID + ' ' + dPL.NameShort		AS ProductLineIDName
					,dPL.NameShort								AS ProductLineName
					,dPL.ResponsiblePerson						AS PLResponsiblePerson

					,fV.ProductID + ' ' + dP.NameShort			AS ProductIDName
					,dP.NameShort								AS ProductName
					,dP.ResponsiblePerson						AS PResponsiblePerson

					,dP.[Status]
					,dP.GlobalAttribute1
					,dP.GlobalAttribute2
					,dP.GlobalAttribute3
					,dP.GlobalAttribute4
					,dP.GlobalAttribute5
					,dP.GlobalAttribute6
					,dP.GlobalAttribute7
					,dP.GlobalAttribute8
					,dP.GlobalAttribute9
					,dP.GlobalAttribute10

				FROM planning.tfValues fV
						-- Gruppierung nach Event
						LEFT JOIN	(
									SELECT 
											f.ProductKey
										,f.TimeID
										,f.ValueText 
										,f.ValueComment
									FROM planning.tfValues f
									INNER JOIN #Scope s
									ON		f.FactoryID		= s.FactoryID
										AND f.ProductLineID	= s.ProductLineID

									WHERE f.ValueSeriesID = 'E'
							
								) Ev ON

										fV.ProductKey	= Ev.ProductKey
									AND fV.TimeID		= Ev.TimeID

						LEFT JOIN planning.tdProducts dP
							ON fV.ProductKey = dP.ProductKey
						LEFT JOIN planning.tdProductLines dPL
							ON fV.ProductLineKey = dPL.ProductLineKey
						LEFT JOIN planning.tdFactories dF
							ON fV.FactoryKey = dF.FactoryKey
						-- Rechtefilter
						INNER JOIN #Scope Sc
							ON		fV.FactoryID	 = Sc.FactoryID
								AND fV.ProductLineID = Sc.ProductLineID
				
			WHERE fV.ValueSeriesID = 'BEM'
			SET @Resultcode = 200 

			COMMIT TRANSACTION spEvents;
		END TRY
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();

			ROLLBACK TRANSACTION spEvents;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
GO
   
GRANT EXECUTE ON OBJECT ::result.spEvents TO pf_PlanningFactoryUser;

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'result'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spEvents'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to deliver analytic data to the enduser
         Events must have ValueSeriesID "E" and Bemerkungen ValueSeriesID "BEM"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
