DROP PROCEDURE IF EXISTS system.spCOPY_Dashboard;
GO

/*
	COPY Operation for a Dashboard
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spCOPY_Dashboard
				 @Username				= 'SQL'
				,@CopyDashboardID		= 'D1'
				,@TargetDashboardname	= 'Dash2';
		PRINT @RC;

		SELECT * FROM system.tDashboards;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Dashboard',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_Dashboard','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spCOPY_Dashboard
	@Username				NVARCHAR(255),
	@CopyDashboardID		NVARCHAR(50),
	@TargetDashboardName	NVARCHAR(255)

AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @CopyDashboardKey	INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,				N'NULL') + N''',''' 
								+ ISNULL(@CopyDashboardID,		N'NULL') + N''',''' 
								+ ISNULL(@TargetDashboardName,	N'NULL') + N'''' 
	DECLARE @DashboardID		NVARCHAR(50)	= N'';
	DECLARE @LastIDNum			NVARCHAR(50)	= 0;
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username				= N'';
	IF @CopyDashboardID		IS NULL SET @CopyDashboardID		= N'';
	IF @TargetDashboardName	IS NULL SET @TargetDashboardName	= N'';


	BEGIN TRY
		BEGIN TRANSACTION spCOPY_Dashboard;

		-- STEP 0.2 - Protect input parameters	
		SET @CopyDashboardID		= dbo.sx_pf_pProtectString (@CopyDashboardID);

		IF @CopyDashboardID = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	
					
		-- STEP 2 - Get keys
		SELECT @CopyDashboardKey = DashboardKey FROM system.tDashboards WHERE DashboardID = @CopyDashboardID;

		-- If Copy user not exists, stop action
		IF @CopyDashboardKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Dashboard to copy does not exits.', 16, 10);
		END

		BEGIN
			SELECT TOP 1 
			@LastIDNum = TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT ) 
			FROM system.tDashboards
			WHERE 
				DashboardID LIKE N'D%'
				AND LEN(DashboardID) > 1
				AND TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) IS NOT NULL

				ORDER BY TRY_CAST(RIGHT(DashboardID,LEN(DashboardID)-1) AS INT) DESC

				IF @LastIDNum IS NULL SET @LastIDNum = 0;
				SET @DashboardID = 'D' + CAST(@LastIDNum +1 AS NVARCHAR(50));

			END

		--Create Target Report and output key into table
		INSERT INTO system.tDashboards(DashboardID, DashboardName, DashboardDescription, LayoutDefinition)
			SELECT 
				 @DashboardID
				,@TargetDashboardName
				,DashboardDescription
				,LayoutDefinition
			FROM system.tDashboards
			WHERE DashboardKey = @CopyDashboardKey;

		SET @EffectedRows = 1;
						
		SET @Resultcode = 201;

		COMMIT TRANSACTION spCOPY_Dashboard;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spCOPY_Dashboard;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT @Comment;
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_Dashboard TO pf_PlanningFactoryService;
  
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spCOPY_Dashboard'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Copy Operation for a Dashboard
         a new Dashboard will be created as copy of an existing Dashboard.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CopyDashboardID';
    SET @value      = N'ID of the Dashboard which is basis of the copy.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetDashboardname';
    SET @value      = N'Name of the Target Dashboard, the ID is autocreated.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

GO
