
DROP PROCEDURE IF EXISTS system.spCOPY_Pipeline;
GO

/*
	Procedure to copy a Pipeline - including its steps and schedules

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spCOPY_Pipeline
				 @Username			= 'SQL'
				,@SourcePipelineID	= 'P1'
				,@TargetPipelineID	= '*'
				,@OutputID			= @ReturnID OUTPUT;
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps;
		SELECT * FROM system.tPipelineSchedules;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spCOPY_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spCOPY_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spCOPY_Pipeline
    (
         @Username				NVARCHAR(255)
        ,@SourcePipelineID		NVARCHAR(50)
		,@TargetPipelineID		NVARCHAR(50)
		,@OutputID				NVARCHAR(50) = NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','											 
				+ ISNULL(@SourcePipelineID		,N'NULL')	+ N','''
				+ ISNULL(@TargetPipelineID		,N'NULL')	+ N''''

				,@EffectedRows			INT				= 0			
				,@ResultCode			INT				= 501				
				,@TimestampCall			DATETIME		= GETUTCDATE()
				,@Comment				NVARCHAR(2000)	= N''
				,@TransactUsername		NVARCHAR(255)	= N''
				,@StartTime				DATETIME2		= SysUTCDateTime()
				,@SourcePipelineKey		BIGINT			= 0
				,@TargetPipelineKey		BIGINT			= 0
				,@LastPipelineNum		NVARCHAR(50)	= 0
				,@TargetOrderOfDisplay	INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @SourcePipelineID		IS NULL SET @SourcePipelineID	= N'';
		IF @TargetPipelineID		IS NULL SET @TargetPipelineID	= N'';

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spCOPY_Pipeline
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @SourcePipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Source Pipeline don`t exits', 16, 10);
					END

				SELECT @SourcePipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @SourcePipelineID;

				IF EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @TargetPipelineID
					)
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Target Pipeline already exits', 16, 10);
					END

				-- determine new ID if * is sended
				IF @TargetPipelineID = '*'
							BEGIN
								SELECT TOP 1 
								@LastPipelineNum = TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT ) 
								FROM system.tPipelines 
								WHERE 
										PipelineID LIKE 'P%'
									AND LEN(PipelineID) > 1
									AND TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) DESC

								IF @LastPipelineNum IS NULL SET @LastPipelineNum = 0;
								SET @TargetPipelineID = 'P' + CAST(@LastPipelineNum +1 AS NVARCHAR(50));

							END

				-- determine TargetOrderOfDisplay 
				SELECT TOP 1 @TargetOrderOfDisplay = OrderOfDisplay + 1 FROM system.tPipelines ORDER BY OrderOfDisplay DESC;
				

				-- Data Transaction
				PRINT CONCAT(N'Starting COPY operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				-- Copy the pipeline itself
				INSERT INTO system.tPipelines
					SELECT
						 @TargetPipelineID
						,PipelineName
						,PipelineDescription
						,PipelineDescriptionIconColor
						,@TargetOrderOfDisplay
						,EmailNotificationCODE
					FROM system.tPipelines
					WHERE PipelineID = @SourcePipelineID;

					SET @TargetPipelineKey = SCOPE_IDENTITY();

				-- Copy the steps of the Pipeline
				INSERT INTO system.tPipelineSteps
					SELECT
						 @TargetPipelineKey
						,StepID
						,StepName
						,StepDescription
						,OrderOfExecution
						,StepDataSourceID
						,StepCompanyIDsJSON
						,StepDetailsJSON
						,TimeExecutionLimit
						,Active
						,ModuleID
						,StartNextStepOnError
					FROM system.tPipelineSteps
					WHERE PipelineKey = @SourcePipelineKey;

				-- Copy the schedules of the Pipeline
				INSERT INTO system.tPipelineSchedules
					SELECT
						 @TargetPipelineKey
						,ScheduleID
						,ScheduleName
						,ScheduleDescription
						,ScheduleDetailsJSON
						,Active
					FROM system.tPipelineSchedules
					WHERE PipelineKey = @SourcePipelineKey;
			
				SET @ResultCode = 201;
				SET @OutputID = @TargetPipelineID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spCOPY_Pipeline
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spCOPY_Pipeline	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_Pipeline TO pf_PlanningFactoryService;
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spCOPY_Pipeline'-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to copy a Pipeline - including its steps and schedules';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourcePipelineID';
    SET @value      = N'ID of the Pipeline which will be copied.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetPipelineID';
    SET @value      = N'ID for the pipeline which will be created as copy of the SourcePipelineID, this Pipeline must not exists at the moment.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
