﻿
DROP PROCEDURE IF EXISTS system.spCOPY_User;
GO

/*
	COPY Operation for a User
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spCOPY_User
				 @Username			= 'SQL'
				,@CopyUsername		= 'W10\admin'
				,@TargetUsername	= 'W10\Maus2';
		PRINT @RC;

		SELECT * FROM system.trUser;
		SELECT * FROM system.trRights;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'PROCEDURE', 'spCOPY_User','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spCOPY_User
	@Username			NVARCHAR(255),
	@CopyUserName		NVARCHAR(255),
	@TargetUsername		NVARCHAR(255)

AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @CopyUserKey		INT				= 0;
	DECLARE @TargetUserKey		INT				= 0;
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,			N'NULL') + N''',''' 
								+ ISNULL(@CopyUserName,		N'NULL') + N''',''' 
								+ ISNULL(@TargetUserName,	N'NULL') + N'''' 
								
	DECLARE @EffectedRows		INT				= 0;			
	DECLARE @ResultCode			INT				= 501;			
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)	= N'';			
	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);

	-- STEP 0.1 - NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @CopyUserName		IS NULL SET @CopyUserName		= N'';
	IF @TargetUserName		IS NULL SET @TargetUserName		= N'';


	BEGIN TRY
		BEGIN TRANSACTION spCOPY_User;

		-- STEP 0.2 - Protect input parameters	
		SET @CopyUserName		= dbo.sx_pf_pProtectString (@CopyUserName);
		SET @TargetUserName		= dbo.sx_pf_pProtectString (@TargetUserName);

		IF @CopyUserName = N'' OR @TargetUsername = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- STEP 1.1 - Don`t allow creation or changing of systemuser SQL
		-- the Username "FactoryService" is not allowed, as it would lead to confusion with the similar named db role
		IF @TargetUserName = N'SQL' OR @TargetUserName = N'FactoryService'	
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Forbidden creation or changing of systemuser', 16, 10);
		END;
			
		-- STEP 1.2 - Don't allow posting of members of the DB role pf_PlanningFactoryService (which are not visible in the frontend) - not over the function IS_ROLEMEMBER, its buggy
		IF EXISTS(
			SELECT r.name 
			FROM sys.database_role_members rm 
				INNER JOIN sys.database_principals r	ON rm.role_principal_id = r.principal_id
				INNER JOIN sys.database_principals m	ON rm.member_principal_id = m.principal_id
			WHERE r.name = 'pf_PlanningFactoryService' AND m.name = @TargetUserName
			)
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Don`t allow changes on the members of the DB role pf_PlanningFactoryService', 16, 10);
		END;


		-- STEP 1.2 - Determine transaction user, the transactuser 'public' is not allowed to change rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('''Public'' user is not allowed to change rights', 16, 10);
		END;
					
		-- STEP 2 - Check rights
		EXEC @ResultCode = sx_pf_pGET_ClusterWriteRight @TransactUsername;

		IF @ResultCode <> 200
			RAISERROR('Invalid rights', 16, 10);	
		
		-- STEP 2 - Get keys
		SELECT @CopyUserKey		= UserKey FROM system.trUser WHERE Username = @CopyUserName;
		SELECT @TargetUserKey	= UserKey FROM system.trUser WHERE Username = @TargetUserName;

		-- If Copy user not exists, stop action
		IF @CopyUserKey  = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('User to copy does not exits.', 16, 10);
		END 

		-- If Target user already exists, stop action
		IF @TargetUserKey  <> 0
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Overwiting existing users is not allowed', 16, 10);
		END 

		--Create Target User and output key into table
		-- the determination on the table variable is necessary because of TRIGGERS on rUser
		INSERT INTO system.trUser (UserName, PersonSurname, PersonFirstName, Email, LDAPIP, Status,DataFlowRightsCODE,VisualizationRightsCODE,IsAdministratorFlag) 
			OUTPUT INSERTED.UserKey INTO @tmpUserTable 
			SELECT 
				 @TargetUsername
				,N''
				,N''
				,N''
				,N''
				,[Status]
				,DataFlowRightsCODE
				,VisualizationRightsCODE
				,IsAdministratorFlag
			FROM system.trUser
			WHERE UserKey = @CopyUserKey;

		SET @EffectedRows = 1;
				
		SELECT TOP (1) @TargetUserKey = UserKey FROM @tmpUserTable ORDER BY UserKey;
						
		SET @Resultcode = 201;
		
		-- STEP 3.2 - Copy the rights of the copy User
		INSERT INTO system.trRights 
			SELECT
				 @TargetUserKey		AS UserKey
				,@TargetUsername	AS UserName
				,FactoryID
				,ProductLineID
				,N''					AS ProductID
				,[Right]
				,ReadCommentMandatory
				,WriteCommentMandatory
			FROM system.trRights
			WHERE Userkey = @CopyUserKey;
		
		SET @EffectedRows += @@ROWCOUNT;

		COMMIT TRANSACTION spCOPY_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spCOPY_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO

GRANT EXECUTE ON OBJECT :: system.spCOPY_User TO pf_PlanningFactoryService;
GO
  

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spCOPY_User'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Copy Operation for a User
         a new user will be created as copy of an existing user. Rights are copied, user attributes are not copied, user will be set to active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CopyUserName';
    SET @value      = N'Name of the user which is basis of the copy.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@TargetUsername';
    SET @value      = N'Name of the Target User';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

 
GO
