DROP PROCEDURE IF EXISTS system.spDELETE_Connector;
GO

/*
	Procedure to delete a connector including all queries

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spDELETE_Connector
				 @Username		= 'SQL'
				,@SourceSystemID	= 'NAVISION'
				,@SourceSystemType = 'MSSQL'
				,@ModuleID = 'FIN'
		PRINT    @RC

		SELECT * FROM system.tConnectors
		SELECT * FROM system.tConnectorQueries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Connector',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spDELETE_Connector','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spDELETE_Connector
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50) = ''
		,@SourceSystemType		NVARCHAR(50) = ''
		,@ModuleID				NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();
			

		-- NULL Protection for all Input parameters
		IF @Username		IS NULL SET @Username	= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spDELETE_Connector
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				--Right Check !!
					-- can be executed as PreSQL from any integration user
					-- must be executeable during import

				IF NOT EXISTS (
					SELECT 1 FROM system.tConnectors WHERE SourceSystemID = @SourceSystemID AND SourceSystemType = @SourceSystemType AND ModuleID = @ModuleID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Connector doesn''t exist', 16, 10);
					END;

				-- Delete Pipeline, this will also delete Steps and Schedules do to key relation
				PRINT CONCAT(N'Delete connector',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				DELETE FROM system.tConnectors WHERE SourceSystemID = @SourceSystemID AND SourceSystemType = @SourceSystemType AND ModuleID = @ModuleID;

				SET @EffectedRows = @@ROWCOUNT;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spDELETE_Connector
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spDELETE_Connector	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spDELETE_Connector TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spDELETE_Connector'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to delete a Pipeline including steps and schedules.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 

	GO
