DROP PROCEDURE IF EXISTS system.spEXPORT_Connector;
GO

/*
	Procedure to create an OCT Importscript for a selected connector - used for export  / import of connectors
		- exports the connector with its queries

	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Connector
				 @Username			= 'SQL'
				,@SourceSystemID	= 'NAVISION'
				,@SourceSystemType	= 'MSSQL'
				,@ModuleID			= 'FIN'
				,@ExportType		= 'JSON'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Connector',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Connector','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spEXPORT_Connector
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50)
		,@SourceSystemType		NVARCHAR(50)
		,@ModuleID				NVARCHAR(50)
		,@ExportType            NVARCHAR(50) = 'SQL'
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''''
			
				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@SourceClusterName NVARCHAR(255)	= N''
				,@SourceClusterAPI	NVARCHAR(255)	= N'';

		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';

		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				IF @ExportType = 'SQL'
				BEGIN
					-- Create a temporary table to store the collection of export commands
					DROP TABLE IF EXISTS #tExport;
					CREATE TABLE #tExport
						(
							 RowKey					BIGINT IDENTITY (1,1)
							,MainOrderNumber		INT						NOT NULL
							,SubOrderNumber			INT						NOT NULL
							,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

							CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

						);

					-- Determine Metadata
					SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
					SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
					SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

					-- MetaHeader
					INSERT INTO #tExport
						VALUES 
							 (0, 1, N'-- {')
							,(0, 2, N'-- "Type": "Connector",')
							,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
							,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
							,(0, 5, CONCAT(N'-- "SourceSystemID": "',@SourceSystemID,'",'))
							,(0, 6, CONCAT(N'-- "SourceSystemType": "',@SourceSystemType,'",'))
							,(0, 7, CONCAT(N'-- "ModuleID": "',@ModuleID,'"'))
							,(0, 8, N'-- }')

					INSERT INTO #tExport
						VALUES 
							 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
							,(10, 2, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
							,(10, 3, N'DECLARE @SourceSystemID NVARCHAR(255)	= ''' + @SourceSystemID+ N'''')
							,(10, 4, N'DECLARE @SourceSystemType NVARCHAR(255)	= ''' + @SourceSystemType+ N'''')
							,(10, 5, N'DECLARE @ModuleID NVARCHAR(255)			= ''' + @ModuleID+ N'''')
							,(10, 6, N'--This Connector will be deleted during import, if it exists. You should be sure !')

					-- Try_Delete existing connector
					INSERT INTO #tExport
						SELECT 
							 900	AS MainOrderNumber
							,1000	AS SubOrderNumber
							,CONCAT(
								'EXEC system.spDELETE_Connector '
								,'@Username'
								,',@SourceSystemID'
								,',@SourceSystemType'
								,',@ModuleID'
								,';'
								)	AS Command
						FROM system.tConnectors
						WHERE SourceSystemID = @SourceSystemID
						AND SourceSystemType = @SourceSystemType
						AND ModuleID = @ModuleID;

					-- Create the connector as new connector
					INSERT INTO #tExport
						SELECT 
							 1000	AS MainOrderNumber
							,1000	AS SubOrderNumber
							,CONCAT(
								'EXEC system.spPOST_Connector '
								,'@Username'
								,',@SourceSystemID'
								,',@SourceSystemType'
								,',@ModuleID'
								,','''	,dbo.sx_pf_pMaskSQL(ConnectorDescription)		,''''
								,','''	,dbo.sx_pf_pMaskSQL(ConnectorDescriptionURL)		,''''
								,','''	,dbo.sx_pf_pMaskSQL(ConnectorVersion),''''
								)	AS Command
						FROM system.tConnectors
						WHERE SourceSystemID = @SourceSystemID
						AND SourceSystemType = @SourceSystemType
						AND ModuleID = @ModuleID;
			
					-- Create the Steps
					INSERT INTO #tExport
						SELECT 
							 2000	AS MainOrderNumber
							,1000	AS SubOrderNumber
							,CONCAT(
								'EXEC system.spPOST_ConnectorQuery '
								,'@Username'
								,',@SourceSystemID'
								,',@SourceSystemType'
								,',@ModuleID'
								,','''	,cq.QueryName								,''''
								,','''	,dbo.sx_pf_pMaskSQL(cq.QuerySQL)	,''''
								,','''	,dbo.sx_pf_pMaskSQL(cq.QueryVersionComment)	,''''							
								)	AS Command
						FROM system.tConnectorQueries AS cq
							INNER JOIN system.tConnectors AS c
								ON c.ConnectorKey = cq.ConnectorKey
						LEFT JOIN system.tConnectorQueries AS cqMax
						ON cq.ConnectorKey = cqMax.ConnectorKey
						AND cq.QueryName = cqMax.QueryName
						AND cq.QueryVersion < cqMax.QueryVersion
						WHERE c.SourceSystemID = @SourceSystemID
						AND c.SourceSystemType = @SourceSystemType
						AND c.ModuleID = @ModuleID
						AND cqMax.QueryVersion IS NULL;

				
					-- final GO
					INSERT INTO #tExport
						VALUES 
							 (9999, 1, N'GO');

					-- Data Transaction

					PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
					SELECT 
						Command

					FROM #tExport
					ORDER BY 
						 MainOrderNumber
						,SubOrderNumber
				END
				
				IF @ExportType = 'JSON'
				BEGIN
					SELECT (
						SELECT
							f1.SourceSystemID,
							f1.SourceSystemType,
							f1.ModuleID,
							f1.ConnectorDescription AS ConnectorDescription,
							f1.ConnectorDescriptionURL AS ConnectorDescriptionURL,
							f1.ConnectorVersion AS ConnectorVersion,
							(
								SELECT
									f2.QueryName,
									f2.QuerySQL AS Query
								FROM system.tConnectorQueries AS f2
								WHERE f2.ConnectorKey = f1.ConnectorKey
								AND f2.QueryVersion = (
									SELECT MAX(f3.QueryVersion)
									FROM system.tConnectorQueries AS f3
									WHERE f3.ConnectorKey = f2.ConnectorKey
									AND f3.QueryName = f2.QueryName
									)
								ORDER BY CASE WHEN f2.QueryName = 'GetCompanies' THEN 1 ELSE 2 END ASC, f2.QueryName ASC
								FOR JSON PATH
							) AS 'ConnectorQueries'
						FROM system.tConnectors AS f1
						WHERE f1.SourceSystemID = @SourceSystemID
						AND f1.SourceSystemType = @SourceSystemType
						AND f1.ModuleID = @ModuleID
						FOR JSON PATH, ROOT ('Connector')
					) AS Command
				END

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spEXPORT_Connector TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Connector'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a connector with all steps in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
