
DROP PROCEDURE IF EXISTS system.spEXPORT_Pipeline;
GO

/*
	Procedure to create an OCT Importscript for an selected Pipeline - used for export  / import of pipelines
		- exports the pipeline with its steps an schedules
		- NO global parameters used by the pipeline are exported (would be possible, but complex JSON operation)
		- conditions and consequences
			- a new Pipeline with the next free ID is created
			- the user must after the import or before
				- create a datasource(s) with type fitting to the used steps
				- download / activate modules needed by the steps
				- select companies in the datasource an in the steps
				- create global parameters used in the steps

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 11/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spEXPORT_Pipeline
				 @Username		= 'SQL'
				,@PipelineID	= 'P4'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spEXPORT_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spEXPORT_Pipeline
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','											 
				+ ISNULL(CAST(@PipelineID AS NVARCHAR(255))	,N'NULL')	+ N''''
			
				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		DECLARE  @SourceClusterName		NVARCHAR(255)
				,@SourceClusterAPI		NVARCHAR(255);

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Create a temporary table to store the collection of export commands
				DROP TABLE IF EXISTS #tExport;
				CREATE TABLE #tExport
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,MainOrderNumber		INT						NOT NULL
						,SubOrderNumber			INT						NOT NULL
						,Command				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tExport PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Determine Metadata
				SELECT @SourceClusterName = ValueText FROM system.tSettings WHERE SettingID = 'Clustername'; -- May be empty if not defined
				SET @SourceClusterName = CONCAT(DB_Name(),' ',@SourceClusterName);
				SELECT @SourceClusterAPI = ValueText FROM system.tSettings WHERE SettingID = 'DBVersion';

				-- MetaHeader
				INSERT INTO #tExport
					VALUES 
						 (0, 1, N'-- {')
						,(0, 2, N'-- "Type": "Pipeline",')
						,(0, 3, CONCAT(N'-- "SourceClusterName": "', @SourceClusterName,'",'))
						,(0, 4, CONCAT(N'-- "SourceClusterAPI": "',@SourceClusterAPI,'",'))
						,(0, 5, CONCAT(N'-- "SourcePipelineID": "',@PipelineID,'"'))
						,(0, 6, N'-- }')

				INSERT INTO #tExport
					VALUES 
						 (10, 1, N'--CONFIG: Adjust variable values manually to fit your needs !')
						,(10, 2, N'DECLARE @Username NVARCHAR(255)			= ''SQL''')
						,(10, 3, N'DECLARE @PipelineID NVARCHAR(255)			= ''' + @PipelineID + N'''')
						,(10, 4, N'--This PipelneID will be deleted during import, if it exists. You should be sure !')
						,(10, 5, N'DECLARE @ReturnedPipelineID NVARCHAR (50);');

				-- Try_Delete existing Pipeline 
				INSERT INTO #tExport
					SELECT 
						 900	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spDELETE_Pipeline '
							,'''SQL'''
							,',@PipelineID'
							,';'
							)	AS Command
					FROM system.tPipelines
					WHERE PipelineID = @PipelineID;

				-- Create the Pipeline as new Pipeline and catch the ID
				INSERT INTO #tExport
					SELECT 
						 1000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_Pipeline '
							,'''SQL'''
							,',@PipelineID'
							,','''	,dbo.sx_pf_pMaskSQL(PipelineName)		,''''
							,','''	,dbo.sx_pf_pMaskSQL(PipelineDescription),''''
							,','''	,PipelineDescriptionIconColor			,''''
							,','	,'NULL'									---NULL leads to positon at the end
							,','''	,EmailNotificationCODE					, ''''
							,', @OutputID = @ReturnedPipelineID OUTPUT;'	
							)	AS Command
					FROM system.tPipelines
					WHERE PipelineID = @PipelineID;
			
				-- Create the Steps
				INSERT INTO #tExport
					SELECT 
						 2000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_PipelineStep '
							,'''SQL'''
							,','	,'@ReturnedPipelineID'		
							,',''*'''
							,','''	,StepName								,''''
							,','''	,dbo.sx_pf_pMaskSQL(StepDescription)	,''''
							,','	,OrderOfExecution										
							,','''	,StepDataSourceID						, ''''
							,','''	,StepCompanyIDsJSON						, ''''
							,','''	,dbo.sx_pf_pMaskSQL(StepDetailsJSON)	, ''''  --inverted commas inside sql statements etc. are handled by the pMaskSQL function
							,','	,TimeExecutionLimit			
							,','	,Active
							,','''	,ModuleID								, ''''
							,','	,StartNextStepOnError					,';'	
							
							)	AS Command
					FROM system.tPipelineSteps tPS
						INNER JOIN system.tPipelines tP
							ON tPS.PipelineKey = tP.PipelineKey
					WHERE tP.PipelineID = @PipelineID
					ORDER BY OrderOfExecution;

				-- Create the Schedulers

				INSERT INTO #tExport
					SELECT 
						 3000	AS MainOrderNumber
						,1000	AS SubOrderNumber
						,CONCAT(
							'EXEC system.spPOST_PipelineSchedule '
							,'''SQL'''
							,','	,'@ReturnedPipelineID'		
							,',''*'''
							,','''	,ScheduleName							,''''
							,','''	,dbo.sx_pf_pMaskSQL(ScheduleDescription),''''
							,','''	,dbo.sx_pf_pMaskSQL(ScheduleDetailsJSON),''''
							,','	,Active									,';'	
							
							)	AS Command
					FROM system.tPipelineSchedules tPS
						INNER JOIN system.tPipelines tP
							ON tPS.PipelineKey = tP.PipelineKey
					WHERE tP.PipelineID = @PipelineID;

				-- final GO
				INSERT INTO #tExport
					VALUES 
						 (9999, 1, N'GO');

				-- Data Transaction

				PRINT CONCAT(N'Output of the import script',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				SELECT 
					Command

				FROM #tExport
				ORDER BY 
					 MainOrderNumber
					,SubOrderNumber

				SET @ResultCode = 200;
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spEXPORT_Pipeline TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spEXPORT_Pipeline'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an SQL Script, which imports a pipeline with all steps in OCT.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline to be exported.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
