﻿
DROP PROCEDURE IF EXISTS system.pGET_LicenceInformation;
GO

/*
    Stored Procedure to retrieve the licence information from the database.
			
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall:
		DECLARE	@return_value int

		EXEC	 @return_value = system.pGET_LicenceInformation
				 @Username		= N'SQL'
				,@CustomerID	= N'AD032456'
				,@ServerID		= N'Server1'
				,@ServiceName	= N'OCT';

		SELECT	'Return Value' = @return_value

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_LicenceInformation',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pGET_LicenceInformation','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.pGET_LicenceInformation
	 @Username				NVARCHAR(255)
	,@CustomerID			NVARCHAR(255)
	,@ServerID				NVARCHAR(255)
	,@ServiceName			NVARCHAR(255)

AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')			 + N''','''
			+ ISNULL(@CustomerID			, N'NULL')			 + N''','''
			+ ISNULL(@ServerID		, N'NULL')			 + N'''';
			
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';
	DECLARE @WindowsServerName	 NVARCHAR(255) = @@SERVERNAME;


	BEGIN TRY
		-- STEP 1 - Check input parameters
		IF @Username			IS NULL 
		OR @CustomerID			IS NULL 
		OR @ServerID			IS NULL
		OR @ServiceName			IS NULL
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		-- STEP 2 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		-- Allow FactoryService Access
        IF EXISTS
        (
            SELECT r.name from sys.database_role_members rm
            INNER JOIN sys.database_principals r on rm.role_principal_id = r.principal_id
            INNER JOIN sys.database_principals m on rm.member_principal_id = m.principal_id
            WHERE r.name = 'pf_PlanningFactoryService' AND m.name = ORIGINAL_LOGIN()
        )
        BEGIN
            SET @TransactUsername = 'FactoryService';
        END;

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user doesn`t exist', 16, 10);
		END;		
		
		-- STEP 3 - Create licence information

		-- Declare table variable
		
		DECLARE @LicenceInformationTbl TABLE
		(
		  CustomerID		NVARCHAR(10)	NOT NULL
		 ,WindowsServerName NVARCHAR(255)	NOT NULL
		 ,ServiceName		NVARCHAR(255)	NOT	NULL
		 ,DatabaseServerID	NVARCHAR(255)	NOT NULL
		 ,DatabaseID		NVARCHAR(255)	NOT NULL
		 ,Timestamp			DATETIME		NOT NULL
		 ,PropertyName		NVARCHAR(255)	NOT NULL
		 ,PropertyValue		NVARCHAR(255)	NOT NULL
		)

		-- Insert database version property

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'DatabaseVersion', ValueText
		FROM planning.tgCluster
		WHERE PropertyID = 'C04'

		-- Insert number of data sources

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfDataSources', COUNT(*)
		FROM system.tDataSources

		-- Insert number of active companies

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfActiveCompanies', COUNT(*)
		FROM global.tCompanies;

		-- Insert number of active users

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfActiveUsers', COUNT(*)
		FROM system.trUser
		WHERE Status = 'Active';

		-- Insert number of products

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfProducts', COUNT(*)
		FROM planning.tdProducts;

		-- Insert number of productlines

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfProductLines', COUNT(*)
		FROM planning.tdProductLines;

		-- Insert number of Factories

		INSERT INTO @LicenceInformationTbl
		SELECT @CustomerID, @WindowsServerName, @ServiceName, @ServerID, DB_NAME(), GETUTCDATE(), 'NumberOfFactories', COUNT(*)
		FROM planning.tdFactories;



		SELECT * 
		FROM @LicenceInformationTbl;

		SET @Resultcode = 201;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	-- Log in API Log

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, 0, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::system.pGET_LicenceInformation TO pf_PlanningFactoryUser;
GRANT EXECUTE ON OBJECT ::system.pGET_LicenceInformation TO pf_PlanningFactoryService;
GO


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pGET_LicenceInformation'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Description: Stored Procedure to retrieve the licence information from the database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@CustomerID';
    SET @value      = N'CustomerID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ServerID';
    SET @value      = N'ServerID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ServiceName';
    SET @value      = N'ServiceName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
