DROP PROCEDURE IF EXISTS system.spGET_ConnectorQuery;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ConnectorQuery
				 @Username		= 'SQL'
				 ,@SourceSystemID = 'NAVISION'
				 ,@SourceSystemType = 'MSSQL'
				 ,@ModuleID = 'FIN'
				 ,@QueryName = 'AccountingJournal'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQuery',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQuery','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ConnectorQuery
    (
         @Username				NVARCHAR(255)
        ,@SourceSystemID		NVARCHAR(50)
		,@SourceSystemType		NVARCHAR(50)
		,@ModuleID				NVARCHAR(50)	= '' -- optional, especially for GetCompanies query which used to be not bound to a module
		,@QueryName				NVARCHAR(255)	= '' -- optional, if empty return all queries of connector		           	
	)
AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@SourceSystemType AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@QueryName AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @SourceSystemID		IS NULL SET @SourceSystemID		= N'';
		IF @SourceSystemType	IS NULL SET @SourceSystemType	= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';
		IF @QueryName			IS NULL SET @QueryName			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			IF @SourceSystemID = '' OR @SourceSystemType = '' OR (@ModuleID = '' AND @QueryName <> 'GetCompanies') OR NOT EXISTS (
				SELECT 1
				FROM system.tConnectors
				WHERE 
					SourceSystemID = @SourceSystemID
					AND SourceSystemType = @SourceSystemType
					AND (ModuleID = @ModuleID OR (@ModuleID = '' AND @QueryName = 'GetCompanies'))
			)
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Connector doesn''t exist', 16, 10);
				END;

			-- @ModuleID is empty for GetCompanies -> return first GetCompanies that is found for this SourceSystemID and SourceSystemType
			IF @ModuleID = '' AND @QueryName = 'GetCompanies'
				BEGIN
					SELECT TOP 1
						c.SourceSystemID,
						c.SourceSystemType,
						c.ModuleID,
						cq.QueryName,
						cq.QuerySQL,
						cq.QueryVersion,
						cq.QueryVersionComment,
						cq.Timestamp,
						cq.CreatedBy
					FROM system.tConnectorQueries AS cq
					JOIN system.tConnectors AS c
					ON c.ConnectorKey = cq.ConnectorKey
					LEFT JOIN system.tConnectorQueries AS cqMax
					ON cq.ConnectorKey = cqMax.ConnectorKey
					AND cq.QueryName = cqMax.QueryName
					AND cq.QueryVersion < cqMax.QueryVersion
					WHERE c.SourceSystemID = @SourceSystemID
						AND c.SourceSystemType = @SourceSystemType
						AND cq.QueryName = @QueryName
						AND cqMax.QueryVersion IS NULL;
				END
			-- QueryName specified but doesn't exist -> return empty result set
			ELSE IF @QueryName <> '' AND NOT EXISTS (
				SELECT 1
				FROM system.tConnectorQueries AS cq
					JOIN system.tConnectors   AS c
					  ON c.ConnectorKey = cq.ConnectorKey
				WHERE
					c.SourceSystemID = @SourceSystemID
					AND c.SourceSystemType = @SourceSystemType
					AND c.ModuleID = @ModuleID
					AND cq.QueryName = @QueryName
				)
				BEGIN
					SELECT
						@SourceSystemID AS SourceSystemID,
						@SourceSystemType AS SourceSystemType,
						@ModuleID AS ModuleID,
						@QueryName AS QueryName,
						'' AS QuerySQL,
						0 AS QueryVersion,
						'' AS QueryVersionComment,
						NULL AS Timestamp,
						'' AS CreatedBy
				END
			-- everything is normal -> return regular result
			ELSE
				BEGIN
					SELECT
						c.SourceSystemID,
						c.SourceSystemType,
						c.ModuleID,
						cq.QueryName,
						cq.QuerySQL,
						cq.QueryVersion,
						cq.QueryVersionComment,
						cq.Timestamp,
						cq.CreatedBy
					FROM system.tConnectorQueries AS cq
					JOIN system.tConnectors AS c
					ON c.ConnectorKey = cq.ConnectorKey
					LEFT JOIN system.tConnectorQueries AS cqMax
					ON cq.ConnectorKey = cqMax.ConnectorKey
					AND cq.QueryName = cqMax.QueryName
					AND cq.QueryVersion < cqMax.QueryVersion
					WHERE c.SourceSystemID = @SourceSystemID
						AND c.SourceSystemType = @SourceSystemType
						AND c.ModuleID = @ModuleID
						AND (cq.QueryName = @QueryName OR @QueryName = '')
						AND cqMax.QueryVersion IS NULL;
				END;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_ConnectorQuery TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ConnectorQuery'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all ConnectorQueries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@QueryName';
    SET @value      = N'QueryName - name of the query and target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
