DROP PROCEDURE IF EXISTS system.spGET_ConnectorQueryFields;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ConnectorQueryFields
				 @Username		= 'SQL'
				 ,@ModuleID = 'FIN'
				 ,@QueryName = 'AccountingJournal'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQueryFields',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorQueryFields','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ConnectorQueryFields
    (
         @Username				NVARCHAR(255)
		,@ModuleID				NVARCHAR(50)
		,@QueryName				NVARCHAR(255)	           	
	)
AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''','
				+ ISNULL(CAST(@ModuleID AS NVARCHAR(50)) ,N'NULL')	+ N''','
				+ ISNULL(CAST(@QueryName AS NVARCHAR(50)) ,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @ModuleID			IS NULL SET @ModuleID			= N'';
		IF @QueryName			IS NULL SET @QueryName			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			IF @QueryName <> 'GetCompanies' AND (@ModuleID = '' OR @QueryName = '' OR NOT EXISTS(
				SELECT 1
				FROM INFORMATION_SCHEMA.columns
				WHERE (
					(TABLE_SCHEMA = 'integration' AND TABLE_NAME = 't' + @ModuleID + '_' + @QueryName)
					OR (@QueryName = 'GetCompanies')
				)
			))
				BEGIN
					SET @ResultCode = 404;
					RAISERROR('Target table doesn''t exist', 16, 10);
				END;

			-- Data Transaction
			SELECT
				SCHEMA_NAME(t.schema_id) + '.' + t.name AS TableName,
				c.name AS ColumnName,
				UPPER(TYPE_NAME(c.system_type_id)) +
					CASE
						WHEN c.precision = 0 AND c.scale = 0 AND max_length > 0 THEN
							'(' + CAST(c.max_length / 2 AS NVARCHAR(10)) + ')'
						WHEN c.precision > 0 AND c.scale > 0 THEN
							'(' + CAST(c.precision AS NVARCHAR(10)) + ', ' + CAST(c.scale AS NVARCHAR(10)) + ')'
						ELSE ''
					END AS DataType,
				CASE WHEN c.is_nullable = 1 THEN 'NULL' ELSE 'NOT NULL' END AS NULLAble,
				p.value AS ColumnInformation,
				RANK() OVER (ORDER BY c.column_id) AS OrderNumber
			FROM sys.tables AS t
			JOIN sys.columns AS c
			ON c.object_id = t.object_id
			LEFT JOIN sys.extended_properties AS p
			ON p.major_id = t.object_id
			AND p.minor_id = c.column_id
			AND p.class = 1
			WHERE 1=1
				AND SCHEMA_NAME(t.schema_id) = 'integration'
				AND t.name = 't' + @ModuleID + '_' + @QueryName
				AND c.name <> 'RowKey'
				AND @QueryName <> 'GetCompanies'

			UNION ALL

			SELECT
				SCHEMA_NAME(t.schema_id) + '.' + t.name AS TableName,
				c.name AS ColumnName,
				UPPER(TYPE_NAME(c.system_type_id)) +
					CASE
						WHEN c.precision = 0 AND c.scale = 0 AND max_length > 0 THEN
							'(' + CAST(c.max_length / 2 AS NVARCHAR(10)) + ')'
						WHEN c.precision > 0 AND c.scale > 0 THEN
							'(' + CAST(c.precision AS NVARCHAR(10)) + ', ' + CAST(c.scale AS NVARCHAR(10)) + ')'
						ELSE ''
					END AS DataType,
				CASE WHEN c.is_nullable = 1 THEN 'NULL' ELSE 'NOT NULL' END AS NULLAble,
				p.value AS ColumnInformation,
				RANK() OVER (ORDER BY c.column_id) AS OrderNumber
			FROM sys.tables AS t
			JOIN sys.columns AS c
			ON c.object_id = t.object_id
			LEFT JOIN sys.extended_properties AS p
			ON p.major_id = t.object_id
			AND p.minor_id = c.column_id
			AND p.class = 1
			WHERE 1=1
				AND SCHEMA_NAME(t.schema_id) = 'global'
				AND t.name = 'tCompanies'
				AND c.name NOT IN ('RowKey', 'DataSourceKey', 'CompanyKey', 'CustomValuesJSONUser', 'OwnerCODE')
				AND @QueryName = 'GetCompanies'
			ORDER BY OrderNumber ASC;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_ConnectorQueryFields TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ConnectorQueryFields'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all ConnectorQueries.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@QueryName';
    SET @value      = N'QueryName - name of the query and target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
