DROP PROCEDURE IF EXISTS system.spGET_ConnectorTree;
GO

/*
	Toni August for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_ConnectorTree
				 @Username		= 'SQL'
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorTree',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ConnectorTree','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_ConnectorTree
    (
         @Username				NVARCHAR(255)
    )

AS 
	BEGIN	
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username					,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';

		BEGIN TRY
			-- check transaction user existence
			SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
				IF @TransactUsername  = N'403' 
				BEGIN
					SET @ResultCode = 403;
					RAISERROR('Transaction user doesn''t exist', 16, 10);
				END;

			-- Temp table to store all integration tables of all modules
			DROP TABLE IF EXISTS #ModuleTables;

			SELECT
				SUBSTRING(t.name, 2, CHARINDEX('_', t.name) - 2) AS ModuleID,
				SUBSTRING(t.name, CHARINDEX('_', t.name) + 1, 255) AS IntegrationTable
			INTO #ModuleTables
			FROM sys.tables t
			JOIN sys.schemas s ON s.schema_id = t.schema_id
			WHERE s.name = 'integration'
			AND t.name LIKE 't%[_]%';

			-- Add GetCompanies to all modules as default
			INSERT INTO #ModuleTables
			SELECT DISTINCT
				ModuleID,
				'GetCompanies'
			FROM #ModuleTables;
				
			-- Data output
			SELECT
				SourceSystemID,
				SourceSystemType,
				ModuleID,
				ConnectorDescription,
				ConnectorVersion,
				QueryName
			FROM (
				SELECT
					c.SourceSystemID,
					c.SourceSystemType,
					c.ModuleID,
					c.ConnectorDescription,
					c.ConnectorVersion,
					mt.IntegrationTable AS QueryName,
					CASE WHEN mt.IntegrationTable = 'GetCompanies' THEN 0 ELSE 1 END IsGetCompanies
				FROM system.tConnectors AS c
				CROSS APPLY #ModuleTables AS mt
				LEFT JOIN system.tConnectorQueries AS ct
				ON ct.ConnectorKey = c.ConnectorKey
				AND ct.QueryName = mt.IntegrationTable
				WHERE c.ModuleID = mt.ModuleID

				UNION
			
				SELECT
					c.SourceSystemID,
					c.SourceSystemType,
					c.ModuleID,
					c.ConnectorDescription,
					c.ConnectorVersion,
					ct.QueryName AS QueryName,
					CASE WHEN ct.QueryName = 'GetCompanies' THEN 0 ELSE 1 END IsGetCompanies
				FROM system.tConnectors AS c
				LEFT JOIN system.tConnectorQueries AS ct
				ON ct.ConnectorKey = c.ConnectorKey
				WHERE ct.ConnectorKey IS NOT NULL
			) Queries
			ORDER BY SourceSystemID, ModuleID, SourceSystemType, IsGetCompanies, QueryName;

			SET @EffectedRows = @@ROWCOUNT;
			SET @ResultCode = 200;
			PRINT CONCAT(N'Operation abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms), es wurden ',@EffectedRows,' Zeilen gelesen.')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_ConnectorTree TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ConnectorTree'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get one or all Connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 

	GO
