
DROP PROCEDURE IF EXISTS system.spGET_DBInfo;
GO
/*
	Procedure to get basic informations about one Database. Informations can be requested
		- from OCTService for an User
		- from OCTService for himself (passing SQL as Clientusername means check the Current SQL User)

	TODO:
		- resolve Rolemembership when assigned over Security Groups
		- check AccessRight (implicit sysadmin) over AzureAD SQL Server Administrator Group Membership

	Infolevel
		- 0 = information it clientuser can access this cluster or not
		- 1 = all the Service needs to display a tile
			- access
			- tileimage
			- number of users
			- database version

		- 2 = full Informations
			- user role
			- database size

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall Procedure
		-- Service for an user
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DBInfo
				 @Username		= 'SQL'
				,@ClientUser	= 'gerd.tautenhahn@saxess-software.com'
				,@InfoLevel		= 2;
		PRINT    @RC;

		-- Service for Service
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DBInfo
				 @Username		= 'SQL'
				,@ClientUser	= 'SQL'
				,@InfoLevel		= 2;
		PRINT    @RC;

		SELECT TOP 3 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DBInfo',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DBInfo','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_DBInfo
         @Username				NVARCHAR(255)
        ,@ClientUser			NVARCHAR(255)
		,@InfoLevel				INT		      = 1				
    
AS 
	BEGIN	

		-- SET NOCOUNT ON; -- activate this, if you read the resultset with an external source

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','''											 
				+ ISNULL(@ClientUser						,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@IsOCTService		BIT				= 0;

		-- NULL Protection for all mandatory Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @ClientUser		IS NULL SET @ClientUser		= N'';

		-- The string SQL means the octservice asks for himself
		IF @ClientUser = 'SQL' SELECT @ClientUser = CURRENT_USER; 	-- !! Clientuser will never exist in OCTUsermanagement
																	-- !! Clientuser may be 'dbo' or an AzureActive Directory Identity
																	-- passing SQL as Clientusername means check the Current SQL User
		IF EXISTS
			(																		 
			SELECT 
				DP1.[name]
			FROM sys.database_role_members DRM
				RIGHT OUTER JOIN sys.database_principals DP1
					ON DRM.role_principal_id = DP1.principal_id
				LEFT OUTER JOIN sys.database_principals DP2
					ON DRM.member_principal_id = DP2.principal_id
			WHERE
						DP1.[type] = 'R' 
					AND DP2.[name] = @ClientUser
					AND DP1.[name] IN ('pf_PlanningFactoryService','db_owner')
					)
		SET @IsOCTService = 1; -- is OCTService or can act like OCTService due to db_owner status

		PRINT CONCAT('Clientuser: ', @ClientUser,'  IsOCTService: ',@IsOCTService);

		DECLARE @ClientUserAccess_Flag INT = 0;

		-- START TRY			***********************************************************************************
		BEGIN TRY
				---- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						-- nothing, to enable Footprint Protection
						PRINT 'Unknown user - eliminate footprint'
						-- SET @ResultCode = 403;
						-- RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Caching table
				DROP TABLE IF EXISTS #tInfoCache;
				CREATE TABLE #tInfoCache
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,PropertyID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PropertyName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,InfoLevel				INT											NOT NULL
						,ValueInt				INT											NOT NULL
						,ValueMoney				MONEY										NOT NULL
						,ValueText				NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ValueDatetime			DATETIME									NULL
						,ValueBinary			VARBINARY(MAX)								NULL --max 8000 Byte to avoid Varbinary(MAX)

						CONSTRAINT PK_tInfoCache PRIMARY KEY CLUSTERED (RowKey)				
					);

				-- INFO LEVEL 0 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

				-- check AccessRight for ClientUser
				IF @IsOCTService = 1
					BEGIN 
						SET @ClientUserAccess_Flag = 1;
					END
				ELSE
					BEGIN
						SELECT @ClientUserAccess_Flag = COUNT(UserKey)
						FROM system.trUser 
						WHERE 
								Username = @ClientUser
							AND Status = 'Active';
					END

				-- check if database has public user
				IF @ClientUserAccess_Flag = 0 AND EXISTS(SELECT 1 FROM system.trUser WHERE Username = 'public' AND Status = 'Active')
				BEGIN
					SET @ClientUserAccess_Flag = 1
				END

				INSERT INTO #tInfoCache
					SELECT
						 'CU01'										AS PropertyID
						,'Flag for ClientUser is an active User'	AS PropertyName
						,0											AS InfoLevel	
						,@ClientUserAccess_Flag						AS ValueInt
						,0											AS ValueMoney	
						,''											AS ValueText	
						,NULL										AS ValueDatetime
						,NULL										AS ValueBinary;

				-- Footprint protection
				IF @ClientUserAccess_Flag = 0
					BEGIN
						SET @ParameterString = REPLACE (@ParameterString,@ClientUser,'<Request for an non Clustermember>');
						SET @Username		 = '<Non-Clustermember>';
					END

				-- INFO LEVEL 1 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

				-- Only if ClientUser has Access, further Informations are delivered (valid for all Info Levels)
				IF @ClientUserAccess_Flag = 1
					BEGIN
						-- Clustername
						INSERT INTO #tInfoCache
							SELECT
								 'C01'										AS PropertyID
								,'ClusterName'								AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,'<give me a name>'							AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache
									SET ValueText = S.ValueText
								FROM 
									system.tSettings S
								WHERE 
										S.SettingID				= 'ClusterName'
									AND #tInfoCache.PropertyID	= 'C01';

						-- Clusterdescription
						INSERT INTO #tInfoCache
							SELECT
								 'C02'										AS PropertyID
								,'ClusterDescription'								AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache
									SET ValueText = S.ValueText
								FROM 
									system.tSettings S
								WHERE 
										S.SettingID				= 'ClusterDescription'
									AND #tInfoCache.PropertyID	= 'C02';

						-- Clusterversion
						INSERT INTO #tInfoCache
								SELECT
									 'C03'										AS PropertyID
									,'OCT.core Database Version'				AS PropertyName
									,1											AS InfoLevel	
									,0											AS ValueInt
									,0											AS ValueMoney	
									,ValueText									AS ValueText	
									,NULL										AS ValueDatetime
									,NULL										AS ValueBinary
								FROM 
									system.tSettings
								WHERE SettingID = 'DBVersion';
						
						-- Cluster_Tile_Logo

						INSERT INTO #tInfoCache
							SELECT
								 'C04'										AS PropertyID
								,'Cluster_Tile_Logo'						AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache
									SET ValueBinary = FileBody
								FROM 
									planning.tFiles
								WHERE 
										FileID					= 'Cluster_Tile_Logo'
									AND #tInfoCache.PropertyID	= 'C04';

						-- Maintenance Mode -> this property exists only after first saving of database properties
						INSERT INTO #tInfoCache
							SELECT
								 'C05'										AS PropertyID
								,'Cluster is in Maintenance Mode Flag'		AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

							UPDATE #tInfoCache
									SET ValueInt = S.ValueInt
								FROM 
									system.tSettings S
								WHERE 
										S.SettingID				= 'MaintenanceMode'
									AND #tInfoCache.PropertyID	= 'C05';

						-- Scheduler Owner -> this property exists only after first saving of database properties
						INSERT INTO #tInfoCache
							SELECT
								 'C06'										AS PropertyID
								,'SchedulerSettingsOwnerServer'				AS PropertyName
								,1											AS InfoLevel	
								,0											AS ValueInt
								,0											AS ValueMoney	
								,''											AS ValueText	
								,NULL										AS ValueDatetime
								,NULL										AS ValueBinary;

						UPDATE #tInfoCache 
								SET ValueText = S.ValueText
							FROM 
								system.tSettings S
							WHERE S.SettingID  = 'SchedulerSettingsOwnerServer'
							AND #tInfoCache.PropertyID	= 'C06';

						-- Usercount for Licence information
						INSERT INTO #tInfoCache
							SELECT
									 'C10'										AS PropertyID
									,'Number of active Users'					AS PropertyName
									,1											AS InfoLevel	
									,COUNT(UserKey)								AS ValueInt
									,0											AS ValueMoney	
									,''											AS ValueText	
									,NULL										AS ValueDatetime
									,NULL										AS ValueBinary
								FROM 
									system.trUser
								WHERE Status = 'Active';

						-- Information, if the active User is an OCT Clusteradmin
						INSERT INTO #tInfoCache
							SELECT
									 'CU01'										AS PropertyID
									,'User is OCT Administrator'				AS PropertyName
									,1											AS InfoLevel	
									,COUNT(UserKey)								AS ValueInt
									,0											AS ValueMoney	
									,''											AS ValueText	
									,NULL										AS ValueDatetime
									,NULL										AS ValueBinary
								FROM 
									system.trUser
								WHERE 
										Username = @ClientUser
									AND IsAdministratorFlag = 1;

					END

				-- INFO LEVEL 2 ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				-- Informations for higher Info Levels
				IF @ClientUserAccess_Flag = 1 AND @InfoLevel >= 2
					BEGIN
						-- Databaseroles of the Clientuser
							-- Works only if the Clientuser is not the dbo
							-- returns no Serverroles
							-- returns no roles if the dabaserole is inherited from a serverrole (e.g. somebody has db_owner role due to sysadmin login etc.)
						INSERT INTO #tInfoCache	
							SELECT
								 'CU10'								AS PropertyID
								,'Database Roles of the Clientuser' AS PropertyName
								,2									AS InfoLevel
								,0									AS ValueInt
								,0									AS ValueMoney
								,COALESCE(
											STUFF( 		-- Stuffs all names in one list
														(
														SELECT 
															DP1.[name] +', '
														FROM sys.database_role_members DRM
															RIGHT OUTER JOIN sys.database_principals DP1
																ON DRM.role_principal_id = DP1.principal_id
															LEFT OUTER JOIN sys.database_principals DP2
																ON DRM.member_principal_id = DP2.principal_id
														WHERE
																	DP1.[type] = 'R' 
																AND DP2.[name] = @ClientUser
														FOR XML PATH(''), TYPE
													).value('.', 'NVARCHAR(MAX)') 
													,1,0,''
												) 	
											, 'No direct assigned Databaseroles (Serveroles,dbo status, groupmemberships or admin status over AAD may be given !)'	
										)							AS ValueText	--Databaserolename
								,NULL								AS ValueDatetime
								,NULL								AS ValueBinary;
							
						-- dbo determination - its different on Azure and on-prem
						IF CONVERT(NVARCHAR(255),SERVERPROPERTY ('Edition')) LIKE '%Azure%'	
							BEGIN
								INSERT INTO #tInfoCache
									SELECT
										 'DB01'								AS PropertyID
										,'Name or sid of dbo'				AS PropertyName
										,2									AS InfoLevel
										,0									AS ValueInt
										,0									AS ValueMoney
										,CONVERT(NVARCHAR(255),sid,2)		AS ValueText
										,NULL								AS ValueDatetime
										,NULL								AS ValueBinary
									FROM sys.sysusers where name = 'dbo'
							END
						ELSE -- on prem 
							BEGIN
								INSERT INTO #tInfoCache
									SELECT
										 'DB01'								AS PropertyID
										,'Name of dbo'				AS PropertyName
										,2									AS InfoLevel
										,0									AS ValueInt
										,0									AS ValueMoney
										,SUSER_SNAME(owner_sid)				AS ValueText	
										,NULL								AS ValueDatetime
										,NULL								AS ValueBinary
									FROM sys.databases WHERE name = DB_NAME();
							END

						-- Database Rows Size
						INSERT INTO #tInfoCache
							SELECT
								 'DB02'									AS PropertyID
								,'Database Size (Rows) in GB'			AS PropertyName
								,2										AS InfoLevel	
								,0										AS ValueInt
								,SUM(CAST(size AS MONEY) * 8/1024/1024)	AS ValueMoney	
								,''										AS ValueText	
								,NULL									AS ValueDatetime
								,NULL									AS ValueBinary
							FROM sys.database_files
							WHERE 
								type = 0;

						-- Database Log Size
						INSERT INTO #tInfoCache
							SELECT
								 'DB03'									AS PropertyID
								,'Database Size (Log) in GB'			AS PropertyName
								,2										AS InfoLevel	
								,0										AS ValueInt
								,SUM(CAST(size AS MONEY) * 8/1024/1024)	AS ValueMoney	
								,''										AS ValueText	
								,NULL									AS ValueDatetime
								,NULL									AS ValueBinary
							FROM sys.database_files
							WHERE 
								type = 1;
						
						-- Options to To:
						-- Last Database Read Access
						-- Last Database Write Access
						-- Number of scheduled Pipelines
						
					END

				-- OUTPUT

				SELECT	
					*
				FROM #tInfoCache
				WHERE InfoLevel <= @InfoLevel
				ORDER BY
					InfoLevel,
					PropertyID;
			
				SET @ResultCode = 200;
				PRINT CONCAT(N'Reading finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spGET_DBInfo TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_DBInfo'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1;					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get a collection of database informations. Pass the Clientusername SQL to request the rights of the service.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@ClientUser';
    SET @value      = N'Not logged if its not a clustermember to protect from footprinting access scans.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@InfoLevel';
    SET @value      = N'Control the amount of informations deliverd. 1 = Basic Info, 2 = extended Info.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
