
DROP PROCEDURE IF EXISTS system.spGET_DatabaseRoles;
GO

/*
	Procedure to return the database roles of the executing user

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 12/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_DatabaseRoles
				 @Username = 'SQL'				--this username is not used anywhere
		PRINT    @RC

		SELECT USER_NAME();
		SELECT ORIGINAL_LOGIN();

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DatabaseRoles',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_DatabaseRoles','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_DatabaseRoles
    (
         @Username				NVARCHAR(255)
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
			@ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

			-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
			,@ParameterString	NVARCHAR(MAX)	= N''''					
			+ ISNULL(@Username					,N'NULL')	+ N''''											 
	
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501				
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@TransactUsername	NVARCHAR(255)	= N''
			,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY

				-- Select the database role of the executing user
				PRINT CONCAT(N'Reading Roles',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #Roles;
				CREATE TABLE  #Roles
					(
						DatabaseRoleName NVARCHAR(255) COLLATE DATABASE_DEFAULT NOT NULL
					);

				-- Query all roles of the current login - this works only, if the login is not mapped to the username dbo
				INSERT INTO #Roles
					SELECT
						DP1.[name]							AS DatabaseRoleName
						-- ,ISNULL (DP2.[name], 'No members') AS DatabaseUserName
					FROM sys.database_role_members DRM
						RIGHT OUTER JOIN sys.database_principals DP1
							ON DRM.role_principal_id = DP1.principal_id
						LEFT OUTER JOIN sys.database_principals DP2
							ON DRM.member_principal_id = DP2.principal_id
					WHERE
								DP1.[type] = 'R' 
							AND DP2.[name] = ORIGINAL_LOGIN();

				-- For the case that the user is the dbo
				IF USER_NAME() = 'dbo'
				BEGIN
					INSERT INTO #Roles
						VALUES ('dbo (User is owner of the database.)');

					INSERT INTO #Roles
						SELECT
								DP1.[name]							AS DatabaseRoleName
								-- ,ISNULL (DP2.[name], 'No members') AS DatabaseUserName
							FROM sys.database_role_members DRM
								RIGHT OUTER JOIN sys.database_principals DP1
									ON DRM.role_principal_id = DP1.principal_id
								LEFT OUTER JOIN sys.database_principals DP2
									ON DRM.member_principal_id = DP2.principal_id
							WHERE
										DP1.[type] = 'R' 
									AND DP2.[name] = 'dbo';
				END

				SELECT * FROM #Roles;

				SET @ResultCode = 200;
				PRINT CONCAT(N'Reading of roles done',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spGET_DatabaseRoles TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_DatabaseRoles'    -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Returns the roles of the executing user.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Username';
    SET @value      = N'Parameter is ignored.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
