 
DROP PROCEDURE IF EXISTS system.spGET_ObjectTree;
GO
/*
	Procedure to GET the ObjectTree, which lists all database objects with its metadata attribute
	The Procedure can't list objects on which the service don't has access rights.
	To see all object, execute this Stored procedure in SSMS as DBOwner.

	It looked promising to add in this procedure the clause "WITH EXECUTE AS OWNER" - this brings up in on-prem systems all objects. 
	But it was no longer possible to excecute this stored procedure on Azure, even the owner is in both cases dbo.
	Maybe as in Azure the login behind dbo is Active Directory Account (which appears not as local login), wheras on-prem the dbo maps to a local login.
	https://docs.microsoft.com/de-de/sql/t-sql/statements/execute-as-clause-transact-sql?view=sql-server-ver15
	Executing the Procedure with "EXECUTE AS OWNER 'dbo'" also don't worked.

	The Procedure therefore works on Azure only, if the Owner of the Database is an SQL-Login
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC  @RC = system.spGET_ObjectTree 'SQL'		
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ObjectTree',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_ObjectTree','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_ObjectTree  
    (
         @Username				NVARCHAR(255)
    )

-- must be always execute as owner to show all objects - but this don't work on Azure, if the Owner is an AAD User - OPEN PROBLEM -> alternative ALTER AUTHORIZATION to dbo ?
WITH EXECUTE AS 'dbo'

AS 
	BEGIN	

		SET NOCOUNT ON;

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username					,N'NULL')	+ N''''											 

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N'';


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spGET_ObjectTree;
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

					-- Data Transaction						*************************************************************************

					SELECT
						 CASE
							WHEN tree.Owner = '<owner definition tag not defined>'		THEN		'RGB(253,138,138)'	 --red
							WHEN tree.SX_ShipmentFlag = '1' 							THEN		'RGB(160,215,119)'	 --green
							WHEN tree.SX_ShipmentFlag IN ('2','3','4')					THEN		'RGB(243,240,130)'	 --yellow
							WHEN tree.SX_ShipmentFlag IN ('10','11','12')				THEN		'RGB(130,177,243)'	 --blue
																						ELSE		'RGB(242,210,168)'	 --ocker			
						 END						AS ColorRGB
						,tree.[Owner]
						,tree.Module
						,tree.ObjectType
						,tree.ShipmentStatus
						,tree.ObjectName
						,tree.[Description]
						

					FROM (
							SELECT 
								 CASE
									WHEN sxo.SX_Owner <> ''		THEN sxo.SX_Owner
									WHEN sxo.SX_Owner = ''		THEN 'unknown Owner'
									WHEN sxo.SX_Owner IS NULL	THEN '<owner definition tag not defined>'
									ELSE							 '<procedure failure - this should not had happen>'	
								  END									AS [Owner]
								 ,CASE 
									WHEN sxm.SX_Module IS NULL	THEN '<module defintion tag not definded>'
									WHEN sxm.SX_Module = ''		THEN 'unknown Modul' 
									ELSE sxm.SX_Module
								  END									AS Module
								 ,CASE o.type
									WHEN 'P'	THEN 'Stored Procedures'
									WHEN 'U'	THEN 'Tables'
									WHEN 'V'	THEN 'Views'
									WHEN 'IF'	THEN 'Functions (Table Value Inline)'
									WHEN 'FN'	THEN 'Functions (Scalar)'
									WHEN 'TF'	THEN 'Functions (Table Value)'
									WHEN 'TR'	THEN 'Trigger'
									WHEN 'SO'	THEN 'Sequence'
									ELSE CONCAT('Unknown Objecttype ',o.type)
								 END									AS ObjectType
								,sxs.SX_ShipmentFlag					AS SX_ShipmentFlag
								,CASE 
									WHEN sxs.SX_ShipmentFlag	= '0'	THEN	'0. Demo object - out of shipment process'
									WHEN sxs.SX_ShipmentFlag	= '1'	THEN	'1. Standard - without modification'
									WHEN sxs.SX_ShipmentFlag	= '2'	THEN	'2. Standard - modified from saxess for customer'
									WHEN sxs.SX_ShipmentFlag	= '3'	THEN	'3. Standard - modified from partner for customer'
									WHEN sxs.SX_ShipmentFlag	= '4'	THEN	'4. Standard - modified from customer for himself'

									WHEN sxs.SX_ShipmentFlag	= '10'	THEN	'10. Created from Saxess for Customer'
									WHEN sxs.SX_ShipmentFlag	= '11'	THEN	'11. Created from Partner for Customer'
									WHEN sxs.SX_ShipmentFlag	= '12'	THEN	'12. Created from Customer for Customer'

									WHEN sxs.SX_ShipmentFlag	IS NULL THEN	'<module defintion tag not definded>'

									ELSE										'unknown Shipment Status'
								 END									AS ShipmentStatus
								,S.name	+'.' + O.name					AS ObjectName

								,CASE
									WHEN ep.value = ''		THEN	'Description not defined'
									WHEN ep.value IS NULL	THEN	'<description defintion tag not definded>'
									WHEN ep.value <> ''		THEN	ep.value
									ELSE							'<procedure failure - this should not had happen>'
								 END									AS Description

							FROM sys.all_objects O
							LEFT JOIN sys.extended_properties EP 
								ON ep.major_id = O.object_id 
							LEFT JOIN sys.schemas S 
								ON O.schema_id = S.schema_id
							-- use as separte column
							LEFT JOIN	(
											SELECT 
												 major_id
												,Value AS SX_Owner
											FROM sys.extended_properties
											WHERE Name = 'SX_Owner'
										) sxo
										ON sxo.major_id = ep.major_id
							-- use as separte column
							LEFT JOIN	(
											SELECT 
												 major_id
												,Value AS SX_ShipmentFlag
											FROM sys.extended_properties
											WHERE Name = 'SX_ShipmentFlag'
										) sxs
										ON sxs.major_id = ep.major_id
							LEFT JOIN	(
											SELECT 
												 major_id
												,Value AS SX_Module
											FROM sys.extended_properties
											WHERE Name = 'SX_Module'
										) sxm
										ON sxm.major_id = ep.major_id

							WHERE 
									O.is_ms_shipped = 0
								AND ISNULL(ep.minor_id,0) = 0				-- columns of tables and views, parameters filtered out
								AND O.type NOT IN ('PK' -- primary keys
												  ,'D'  -- default constraints
												  ,'UQ'	-- unique constrains
												  ,'F'	-- foreign key
												  )
								AND (ep.name = 'MS_Description' OR ep.name IS NULL)
								--AND ISNULL(ep.name,'') NOT IN ('SX_Owner','SX_ShipmentFlag','SX_Module') -- separat joined
								--AND ISNULL(ep.name,'') NOT IN ('SX_UserHint','Is')						 -- not relevant

							) tree

					ORDER BY Owner, Module, ObjectType, ObjectName

					--								*************************************************************************

			SET @ResultCode = 200;
		COMMIT TRANSACTION spGET_ObjectTree;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spGET_ObjectTree;		

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: [system].spGET_ObjectTree TO pf_PlanningFactoryService;


-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_ObjectTree'   -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'			-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'				-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1						-- 0 = Demo object - out of shipment process
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
 
																-- 10 = shiped from saxess as customer specific object
	,@SX_UserHint		NVARCHAR(2000) = N''					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to create an object tree as list of all database objects with its metadata. The owner and shipment status of the objects
					will control update processes etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 -- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username for whom the action is down.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

 
GO
