
DROP PROCEDURE IF EXISTS system.spGET_PipelineStepLog;
GO

/*
	Prozedur ot GET all or filterd StepLog Data

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_PipelineStepLog
				 @Username		= 'SQL'
				,@PipelineID	= ''
				,@StepID		= ''
				,@ExecutionID	= '';
		PRINT    @RC;

		SELECT * FROM system.tPipelineStepLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineStepLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_PipelineStepLog','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_PipelineStepLog
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50) = ''
		,@StepID				NVARCHAR(50) = ''
		,@ExecutionID			NVARCHAR(50) = ''
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username							,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID						,N'NULL')	+ N''','''
				+ ISNULL(@StepID							,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID						,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- NULL Protection for all mandatory Input parameters
		IF @Username		IS NULL SET @Username		= N'';
		IF @StepID			IS NULL SET @StepID			= N'';
		IF @ExecutionID		IS NULL SET @ExecutionID	= N'';

		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- check existence of IDs
				IF @PipelineID <> '' AND NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END;

				IF @PipelineID <> '' AND @StepID <> '' AND NOT EXISTS (
					SELECT PipelineID, StepID FROM system.tPipelineSteps	PS
						LEFT JOIN system.tPipelines P
							ON P.PipelineKey = PS.PipelineKey
							WHERE 
									PipelineID = @PipelineID
								AND StepID		= @StepID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Step don`t exits', 16, 10);
					END;

				IF @PipelineID = '' AND @StepID <> ''
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('PipelineID must be set if StepID is used', 16, 10);
					END;

				IF @ExecutionID <> '' AND NOT EXISTS (
					SELECT ExecutionID FROM system.tPipelineStepLog WHERE ExecutionID = @ExecutionID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('ExecutionID don`t exits', 16, 10);
					END;
							   
				-- Data Transaction
				PRINT CONCAT(N'Starting reading',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT
					 PipelineID
					,StepID
					,ExecutionID
					,StepName
					,StartTime
					,EndTime
					,LogMessage
					,StatusCODE
					,HandleID
				FROM system.tPipelineStepLog
				WHERE
						(PipelineID		= @PipelineID	OR @PipelineID	= '')
					AND	(StepID			= @StepID		OR @StepID		= '')
					AND (ExecutionID	= @ExecutionID	OR @ExecutionID = '')					

				SET @ResultCode = 200;
				PRINT CONCAT(N'Reading finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spGET_PipelineStepLog TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_PipelineStepLog'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Prozedur ot GET all or filterd StepLog Data';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'optional (but mandantory if StepID is used) - ID of the Pipeline to get the logs from';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'optional - ID of the Step to get the logs from';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ExecutionID';
    SET @value      = N'optional - Exectuion ID of the Pipeline to get the logs from';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
