
DROP PROCEDURE IF EXISTS system.spGET_UserRight;
GO

/*
	GET Operation for User rights in DataEntry

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE @RC INT;
		EXEC @RC = system.spGET_UserRight 
			@Username = 'SQL', 
			@RequestedUserName = 'W10\admin';
		PRINT @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRight',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRight','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spGET_UserRight
	@Username			NVARCHAR(255),
	@RequestedUserName  NVARCHAR(255) = NULL 
AS 
BEGIN
	SET NOCOUNT ON;

	DECLARE @UserKey			INT				= 0;
	DECLARE @Right				NVARCHAR(10)	= N'';
	DECLARE @TransactUsername	NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	NVARCHAR(MAX)	= N'''' 
			+ ISNULL(@Username,				N'NULL') + N''',''' 
			+ ISNULL(@RequestedUserName,	N'NULL') + N'''';
	DECLARE @EffectedRows		INT				= 0;					
	DECLARE @ResultCode			INT				= 501;						
	DECLARE @TimestampCall		DATETIME		= GETUTCDATE();
	DECLARE @Comment			NVARCHAR(2000)  = N'';				

	-- NULL Protection
	IF @Username			IS NULL SET @Username			= N'';
	IF @RequestedUserName	IS NULL SET @RequestedUserName	= N'';

	BEGIN TRY	
		BEGIN TRANSACTION GET_UserRight;

		-- Protect input parameters
		SET @Username			= dbo.sx_pf_pProtectString (@Username);
		SET @RequestedUserName	= dbo.sx_pf_pProtectString (@RequestedUserName);

		-- Check on empty name or SystemUser
		IF @RequestedUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameter <RequestedUserName>', 16, 10);
		END;	

		-- Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END;
		
		-- Determine keys
		SELECT @UserKey = UserKey FROM system.trUser WHERE UserName = @RequestedUserName;
		
		IF @UserKey = 0
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Keys don`t exists', 16, 10);
		END;

		-- Check rights 
		IF @TransactUsername = @RequestedUserName 
		BEGIN --user Must see themselves
			SET @Right		= N'Read';
			SET @ResultCode = 200;
		END 
		ELSE
		BEGIN	
			EXEC @ResultCode = dbo.sx_pf_pGET_ClusterWriteRight @TransactUsername;
		
			IF @ResultCode = 200
			BEGIN
				SET @Right = N'Write';
			END
			ELSE 
			BEGIN		
				RAISERROR('Invalid rights', 16, 10);
			END;
		END;

		-- Helper Table with Full Stucture - to Output Deny Rights too   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
			DROP TABLE #FullStructure;

		CREATE TABLE #FullStructure
		(
			RowKey			BIGINT IDENTITY (1,1)								NOT NULL
			,[Level]		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey		BIGINT												NOT NULL
			,ProductLineKey BIGINT												NOT NULL
			,FactoryID		NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
			,ProductLineID	NVARCHAR(255)			COLLATE DATABASE_DEFAULT	NOT NULL
		);
		-- C Level
		INSERT INTO #FullStructure
		SELECT 
			'Cluster'	AS 'Level'
			,0			AS FactoryKey
			,0			AS ProductLineKey
			,''			AS FactoryID
			,''			AS ProductLineID;
			 
		-- F Level
		INSERT INTO #FullStructure
		SELECT
			'Factory'		AS 'Level'
			,dF.FactoryKey
			,0				AS ProductLineKey
			,dF.FactoryID
			,''				AS ProductLineID
		FROM planning.tdFactories dF;
				
		-- PL Level
		INSERT INTO #FullStructure
		SELECT
			'ProductLine'	AS 'Level'
			,dF.FactoryKey
			,dPL.ProductLineKey
			,dF.FactoryID
			,dPL.ProductLineID
		FROM planning.tdProductLines dPL 
			LEFT JOIN planning.tdFactories dF
				ON dPL.FactoryKey = dF.FactoryKey;
		
		-- STEP 3. Show public values	
		IF @Right = N'Read' --User is allowed to see of himself only positive rights
		BEGIN

			SELECT 
				 vUR.Username				AS UserName
				,vUR.FactoryID				AS FactoryID
				,COALESCE(dF.NameShort,'')	AS FactoryName
				,vUR.ProductLineID			AS ProductLineID
				,COALESCE(dPL.NameShort,'')	AS ProductLineName
				,vUR.[Right]				AS [Right]
				,vUR.ReadCommentMandatory	AS ReadCommentMandatory
				,vUR.WriteCommentMandatory	AS WriteCommentMandatory
			FROM system.trUserRights vUR
				-- for FactoryName
				LEFT JOIN planning.tdFactories dF
					ON vUR.FactoryID = dF.FactoryID
				-- for ProductLineName
				LEFT JOIN planning.tdProductLines dPL
					ON		vUR.FactoryID = dPL.FactoryID 
						AND	vUR.ProductLineID = dPL.ProductLineID
			WHERE vUR.UserKey = @UserKey 
			ORDER BY 
				ISNULL(TRY_CAST(vUR.FactoryID AS INT), 999999999)
				,vUR.FactoryID
				,ISNULL(TRY_CAST(vUR.ProductLineID AS INT), 999999999)
				,vUR.ProductLineID;
		END
		ELSE
		IF @Right = N'Write' -- Cluster admin can see also negative rights (really don`t exist)
		BEGIN
				
			SELECT
				 @RequestedUsername						AS UserName
				,fs.FactoryID							AS FactoryID
				,COALESCE(dF.NameShort,'')				AS FactoryName
				,fs.ProductLineID						AS ProductLineID
				,COALESCE(dPL.NameShort,'')				AS ProductLineName
				,COALESCE(vUR.[Right],'Deny')			AS 'Right'
                ,COALESCE(vUR.ReadCommentMandatory,0)	AS ReadCommentMandatory
				,COALESCE(vUR.WriteCommentMandatory,0)	AS WriteCommentMandatory
			FROM #FullStructure fs
				-- for FactoryName
				LEFT JOIN planning.tdFactories dF
					ON fs.FactoryID = dF.FactoryID
				-- for ProductLineName
				LEFT JOIN planning.tdProductLines dPL
					ON		fs.FactoryID = dPL.FactoryID 
						AND	fs.ProductLineID = dPL.ProductLineID
				-- positive Rechte
				LEFT JOIN
					(
						SELECT 
							vUR.FactoryID
							,vUR.ProductLineID
							,vUR.[Right]
							,vUR.ReadCommentMandatory
							,vUR.WriteCommentMandatory
						FROM system.trUserRights vUR
						WHERE vUR.UserKey = @UserKey
					) vUR
					ON		fs.FactoryID = vUR.FactoryID
						AND fs.ProductLineID = vUR.ProductLineID
			ORDER BY  
				ISNULL(TRY_CAST(fs.FactoryID AS INT), 999999999)
				,fs.FactoryID
				,ISNULL(TRY_CAST(fs.ProductLineID AS INT), 999999999)
				,fs.ProductLineID;
		END;

		SET @EffectedRows = @@ROWCOUNT;

		COMMIT TRANSACTION GET_UserRight;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION GET_UserRight;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
 
-- GRANT
GRANT EXECUTE ON OBJECT ::system.spGET_UserRight TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spGET_UserRight'		-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'		    -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'			    -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';					-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                        
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'GET Operation for User rights';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET parameter documentation
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@RequestedUserName';
    SET @value      = N'RequestedUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
