 
DROP PROCEDURE IF EXISTS system.spGET_UserRightInformation;
GO

/*
	Procedure to get right informations for an User
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spGET_UserRightInformation
				 @Username					= 'SQL'
				,@UsernameRequestedUser		= 'W10\admin'
				,@RightAreaCODE				= ''	
				,@FactoryID					= ''
				,@ProductLineID				= ''
				,@PipelineID				= ''
				,@FullStructureBOOL			= 0

		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRightInformation',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spGET_UserRightInformation','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spGET_UserRightInformation
    (
         @Username				NVARCHAR(255)
        ,@UsernameRequestedUser	NVARCHAR(255)
		,@RightAreaCODE			NVARCHAR(50) = ''	--DATAENTRY, DATAFLOWS
		,@FactoryID				NVARCHAR(50) = ''
		,@ProductLineID			NVARCHAR(50) = ''
		,@PipelineID			NVARCHAR(50) = ''
		,@FullStructureBOOL		BIT = 0
    )

AS 
	BEGIN	

		-- SET NOCOUNT ON; -- activate this, if you read the resultset with an external source

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				-- SET PARAMETER LOG STRING -inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				,@ParameterString	NVARCHAR(MAX)	= N''''					
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@UsernameRequestedUser					,N'NULL')	+ N''','''	
				+ ISNULL(@RightAreaCODE							,N'NULL')	+ N''','''
				+ ISNULL(@FactoryID								,N'NULL')	+ N''','''	
				+ ISNULL(@ProductLineID							,N'NULL')	+ N''','''	
				+ ISNULL(@PipelineID							,N'NULL')	+ N''','
				+ ISNULL(CAST(@FullStructureBOOL AS NCHAR(1))	,N'NULL')	+ N''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime();

		-- Procedure specific declarations
		--DECLARE  @FactoryKey		INT			= 0
		--		,@ProductLineKey	INT			= 0
		--		,@ProductKey		INT			= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @UsernameRequestedUser	IS NULL SET @UsernameRequestedUser	= N'';
		IF @RightAreaCODE			IS NULL SET @RightAreaCODE			= N'';
		IF @FactoryID				IS NULL SET @FactoryID				= N'';
		IF @ProductLineID			IS NULL SET @ProductLineID			= N'';
		IF @PipelineID				IS NULL SET @PipelineID				= N'';
		IF @FullStructureBOOL		IS NULL SET @FullStructureBOOL		= 0;


		-- START TRY			***********************************************************************************
		BEGIN TRY
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- if RequestedUser doesn't exist we can use fallback to public user
				IF NOT EXISTS(SELECT 1 FROM system.trUser WHERE Username = @UsernameRequestedUser AND Status = 'Active') AND EXISTS(SELECT 1 FROM system.trUser WHERE Username = 'public' AND Status = 'Active')
				BEGIN
					SET @UsernameRequestedUser = 'public'
				END

				DROP TABLE IF EXISTS #tRightInformation;
				CREATE TABLE #tRightInformation
					(
						 RowKey					BIGINT IDENTITY (1,1)
						,RightAreaCode			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,FactoryID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ProductLineID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,UserRight				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReadCommentMandatory	BIT											NOT NULL
						,WriteCommentMandatory	BIT											NOT NULL
						,RightDescription		NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NOT NULL

						CONSTRAINT PK_tRightInformation PRIMARY KEY CLUSTERED (RowKey)			

					);

				-- Rights for DataEntry
				IF @RightAreaCode = 'DATAENTRY' OR @RightAreaCode = ''
					BEGIN
						PRINT CONCAT(N'Determining DATAENTRY Rights',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						-- For full structure requested
						IF @FullStructureBOOL = 1
							BEGIN
								INSERT INTO #tRightInformation
									SELECT 
										 'DATAENTRY'	AS RightAreaCode
										,FactoryID
										,ProductLineID
										,''				AS PipelineID
										,[Right]		AS UserRight
										,ReadCommentMandatory
										,WriteCommentMandatory
										,''				AS RightDescription

									FROM system.trUserRights
									WHERE 
										Username = @UsernameRequestedUser
										AND
										(FactoryID = @FactoryID OR @FactoryID=''  )
										AND
										(ProductLineID  = @ProductLineID OR @ProductLineID = '');
								END

						-- For single line			
						IF @FullStructureBOOL = 0
							BEGIN
								INSERT INTO #tRightInformation
									SELECT 
										 'DATAENTRY'	AS RightAreaCode
										,FactoryID
										,ProductLineID
										,''				AS PipelineID
										,[Right]		AS UserRight
										,ReadCommentMandatory
										,WriteCommentMandatory
										,''				AS RightDescription

									FROM system.trUserRights
									WHERE 
										Username = @UsernameRequestedUser
										AND
										(FactoryID = @FactoryID)
										AND
										(ProductLineID  = @ProductLineID);
								END
					END

				-- Rights for DataFlows
				IF @RightAreaCode = 'DATAFLOWS' OR @RightAreaCode = ''
					BEGIN
						PRINT CONCAT(N'Determining DATAFLOWS Rights',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						-- One Query for full structure and single object
	
						INSERT INTO #tRightInformation
								SELECT 
										'DATAFLOWS'	AS RightAreaCode
									,''				AS FactoryID
									,''				AS ProductLineID
									,PipelineID		AS PipelineID
									,'EXECUTE'		AS UserRight
									,0				AS ReadCommentMandatory
									,0				AS WriteCommentMandatory
									,'temporary'	AS RightDescription

								FROM system.tPipelines
								WHERE 
									-- Username = @UsernameRequestedUser
									-- AND
									(PipelineID = @PipelineID OR @PipelineID='')		
					END



				
				-- OUTPUT
				
				SELECT * FROM #tRightInformation;
				
				SET @ResultCode = 200;
				PRINT CONCAT(N'Determination finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spGET_UserRightInformation TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spGET_UserRightInformation'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to get right informations for an User - returns the Right as String - Read, Write, EXECUTE,DESIGN';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@Username';
    SET @value      = N'Standardparameter, executing User - use "SQL"';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@UsernameRequestedUser';
    SET @value      = N'Name of the user for which rights are requested';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@RightAreaCODE';
    SET @value      = N'CODE to specify the Rightarea, use DATAFLOWS or DATAENTRY, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@FactoryID';
    SET @value      = N'FactoryID in case of requsting Rights for DATAENTRY, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ProductLineID';
    SET @value      = N'ProductLineID in case of requsting Rights for DATAENTRY, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name

	SET @level2name = N'@PipelineID';
    SET @value      = N'PipelineID in case of requsting Rights for DATAFLOWS, keep empty to get all';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name


	SET @level2name = N'@FullStructureBOOL';
    SET @value      = N'1 or 0 to flag if the full structure under the node (downwards) is included into the output';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name


	GO
