﻿  
DROP PROCEDURE IF EXISTS system.pPOST_ProcessLogEntry;
GO

/*
	Stored Procedure to save a process log entry. For every call a new entry is created.
	Return Value according to HTTP Standard
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 01/2023 for OCT 5.8
 
	Testcall
		DECLARE	@return_value INT;

		EXEC
			@return_value = system.pPOST_ProcessLogEntry
				 @Username			= N'SQL'
				,@ModuleCode		= N'Integration'
				,@ProcessName		= N'Test'
				,@ProcessID			= 1
				,@SubProcessName	= N'SubProcessTest'
				,@SubProcessID		= 2
				,@SubSubprocessName = N'SubSubProcessTest'
				,@EventCODE			= NULL
				 -- https://stackoverflow.com/questions/12957635/sql-query-to-insert-datetime-in-sql-server
				,@Timestamp			= N'2021-01-27T15:32:21.000' --Timestamp to insert must be writen as N'20210127 15:32:21.000' or N'2021-01-27T15:32:21.000' -> N'2021-01-27 15:32:21.000' work not
				,@ReturnCODE		= 500
				,@EventMessage		= N'This is an error.';

		SELECT	'Return Value' = @return_value;

		SELECT TOP 5 * FROM system.tProcessLogs;
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pPOST_ProcessLogEntry',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'pPOST_ProcessLogEntry','PARAMETER',NULL)
*/

CREATE PROCEDURE system.pPOST_ProcessLogEntry
	 @Username				NVARCHAR(255)
	,@ModuleCode			NVARCHAR(50)
	,@ProcessName			NVARCHAR(250)
	,@ProcessID				NVARCHAR(50)
	,@SubProcessName		NVARCHAR(250)
	,@SubProcessID			NVARCHAR(50)
	,@SubSubProcessName		NVARCHAR(250)
	,@EventCODE				NVARCHAR(20)	
	,@Timestamp				DATETIME		
	,@ReturnCODE			INT
	,@EventMessage			NVARCHAR(max)

AS 
BEGIN
	SET NOCOUNT ON;
	
	DECLARE @TransactUsername	 NVARCHAR(255)	= N'';
	DECLARE @ProcedureName		 NVARCHAR (255) = OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID);
	DECLARE @ParameterString	 NVARCHAR (MAX) = N''''
			+ ISNULL(@Username			, N'NULL')					+ N''','''
			+ ISNULL(@ModuleCode		, N'NULL')					+ N''','''
			+ ISNULL(@ProcessName		, N'NULL')					+ N''','''
			+ ISNULL(@ProcessID			, N'NULL')					+ N''','''
			+ ISNULL(@SubProcessName	, N'NULL')					+ N''','''
			+ ISNULL(@SubProcessID		, N'NULL')					+ N''','''
			+ ISNULL(@SubSubProcessName	, N'NULL')					+ N''','''
			+ ISNULL(@EventCODE			, N'NULL')					+ N''',''' 
			+ ISNULL(CAST(@Timestamp AS NVARCHAR(255))	, N'NULL')	+ N''',''' 
			+ ISNULL(CAST(@ReturnCODE AS NVARCHAR(255))	, N'NULL')	+ N''','''
			+ ISNULL(@EventMessage		, N'NULL')					+ N'''';

	DECLARE @EffectedRows		 INT			= 0;					
	DECLARE @ResultCode			 INT			= 501;					
	DECLARE @TimestampCall		 DATETIME		= GETUTCDATE();
	DECLARE @Comment			 NVARCHAR(2000) = N'';		

	IF @EventCODE IS NULL SET @EventCODE = 'see last state';

	BEGIN TRY
		-- STEP 1 - Check input parameters
		IF @Username			IS NULL 
		OR @ModuleCode			IS NULL 
		OR @ProcessName			IS NULL
		OR @Timestamp			IS NULL
			BEGIN
				SET @ResultCode = 404;
				RAISERROR('Empty input parameters', 16, 10);
			END;

		-- STEP 2 - Determine transaction user
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403' 
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user doesn`t exist', 16, 10);
		END;		
		
		-- STEP 3 - Create log entry	

		INSERT INTO system.tProcessLogs(
			ModuleCODE
           ,ProcessName
           ,ProcessID
           ,SubProcessName
           ,SubProcessID
		   ,SubSubProcessName
           ,EventCODE
           ,Timestamp
           ,ReturnCODE
           ,EventMessage) 
		VALUES (
			@ModuleCODE
           ,@ProcessName
           ,@ProcessID
           ,@SubProcessName
           ,@SubProcessID
		   ,@SubSubProcessName
           ,@EventCODE
           ,@Timestamp
           ,@ReturnCODE
           ,@EventMessage);	

		SET @EffectedRows = 1;
		SET @Resultcode = 201;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();	

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	-- Log in API Log

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END
GO
   
GRANT EXECUTE ON OBJECT ::system.pPOST_ProcessLogEntry TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'pPOST_ProcessLogEntry'				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Description: Stored Procedure to save a process log entry. For every call a new entry is created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ModuleCode';
    SET @value      = N'ModuleCode';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessName';
    SET @value      = N'Name of the Process your are in.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ProcessID';
    SET @value      = N'Process ID as short ID for the process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubProcessName';
    SET @value      = N'optional - Subprocess name if Process was called from a main process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubProcessID';
    SET @value      = N'SubProcess ID as short ID for the subprocess';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@SubSubProcessName';
    SET @value      = N'optional - SubSubprocess name if Process was called from a subprocess';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@EventCODE';
    SET @value      = N'Codeword Start, End, Info. The entry "see last state" indicates the the status of process execution is unchanged since the last log entry.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Timestamp';
    SET @value      = N'Timestamp of the Event';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ReturnCODE';
    SET @value      = N'Returncode accoring to HTTP Standard (200,201,404...)';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@EventMessage';
    SET @value      = N'Free Text as event message, maybe catch from SQL Server error event message over SET @Comment = ERROR_MESSAGE() etc.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
