DROP PROCEDURE IF EXISTS system.spPOST_ConnectorQuery;
GO

/*
	POST one query of a connector

	Toni August for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
		EXEC	 @RC = system.spPOST_ConnectorQuery
				 @Username						= 'SQL'
				,@SourceSystemID				= 'NAVISION'
				,@SourceSystemType				= 'MSSQL'
				,@ModuleID						= 'FIN'
				,@QueryName						= 'AccountingJournal'
				,@QuerySQL						= 'SELECT 123 AS bla'
				,@QueryVersionComment			= 'Something'
		PRINT    @RC;

		SELECT * FROM system.tConnectorQueries;
		DELETE FROM system.tConnectorQueries

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_ConnectorQuery',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_ConnectorQuery','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_ConnectorQuery
    (
         @Username						NVARCHAR(255)
        ,@SourceSystemID				NVARCHAR(50)
		,@SourceSystemType				NVARCHAR(50)
		,@ModuleID						NVARCHAR(50)
		,@QueryName						NVARCHAR(50)
		,@QuerySQL						NVARCHAR(MAX)	= NULL
		,@QueryVersionComment			NVARCHAR(4000)  = NULL
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@SourceSystemID							,N'NULL')	+ N''','''
				+ ISNULL(@SourceSystemType							,N'NULL')	+ N''','''
				+ ISNULL(@ModuleID					,N'NULL')	+ N''','''
				+ ISNULL(@QueryName					,N'NULL')	+ N''','''
				+ ISNULL(@QuerySQL			,N'NULL')	+ N''','
				+ ISNULL(@QueryVersionComment			,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@ConnectorKey		INT				= 0
				,@QueryVersion		INT				= 1;

		-- NULL Protection for all mandatory  Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @SourceSystemID			IS NULL SET @SourceSystemID			= N'';
		IF @SourceSystemType		IS NULL SET @SourceSystemType		= N'';
		IF @ModuleID				IS NULL SET @ModuleID				= N'';
		IF @QuerySQL				IS NULL SET @QuerySQL				= N'';
		IF @QueryVersionComment		IS NULL SET @QueryVersionComment	= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @SourceSystemID		= dbo.sx_pf_pProtectID	(@SourceSystemID);
		SET @SourceSystemType		= dbo.sx_pf_pProtectID	(@SourceSystemType);
		SET @ModuleID		= dbo.sx_pf_pProtectID	(@ModuleID);
		SET @QueryName		= dbo.sx_pf_pProtectID	(@QueryName);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_ConnectorQuery
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user doesn''t exist', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Transaction started',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				-- Check if connector exists
				IF NOT EXISTS(
					SELECT 1
					FROM system.tConnectors
					WHERE SourceSystemID = @SourceSystemID
					AND SourceSystemType = @SourceSystemType
					AND ModuleID = @ModuleID
				)
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Connector doesn''t exist', 16, 10);
					END;

				-- Get ConnectorKey
				SELECT @ConnectorKey = ConnectorKey
				FROM system.tConnectors
				WHERE SourceSystemID = @SourceSystemID
				AND SourceSystemType = @SourceSystemType
				AND ModuleID = @ModuleID;

				-- Get next version number
				SELECT @QueryVersion = COALESCE(MAX(QueryVersion) + 1, 1)
				FROM system.tConnectorQueries
				WHERE ConnectorKey = @ConnectorKey
				AND QueryName = @QueryName;

				INSERT INTO tConnectorQueries (ConnectorKey, QueryName, QuerySQL, QueryVersion, QueryVersionComment, Timestamp, CreatedBy)
				VALUES (@ConnectorKey, @QueryName, @QuerySQL, @QueryVersion, @QueryVersionComment, SysUTCDateTime(), @TransactUsername);

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_ConnectorQuery
			
			SET @ResultCode = 200;
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_ConnectorQuery	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spPOST_ConnectorQuery TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_ConnectorQuery'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)                                                      
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Connector Query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@SourceSystemID';
    SET @value      = N'SourceSystemID - ID of the source ERP system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@SourceSystemType';
    SET @value      = N'SourceSystemType - type of connection to the source ERP system (MSSQL, ORACLE, ODBC).';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@ModuleID';
    SET @value      = N'ModuleID - ID of the module.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
    SET @level2name = N'@QueryName';
    SET @value      = N'QueryName - name of the query and target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@QuerySQL';
    SET @value      = N'SQL statement that is used to read data from the source system.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@QueryVersionComment';
    SET @value      = N'Comment for this specific version of the connector query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
GO
