
DROP PROCEDURE IF EXISTS system.spPOST_DataSource;
GO
/*
	Procedure to POST a DataSource
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Base tables
		SELECT * FROM system.tDataSources
 
	Testcall Procedure
		DECLARE  @RC INT;
		EXEC	 @RC = system.spPOST_DataSource
				 @Username				= N'SQL'
				,@DataSourceID			= N'0'  -- must be numeric for compatibility with modules. Full ID Support from 6.0 on
				,@DataSourceName		= N'GT1'
				,@SourceType			= N''
				,@ConnectionValuesJSON	= N''
		PRINT    @RC

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_DataSource',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_DataSource','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_DataSource
    (
         @Username				NVARCHAR(255)
        ,@DataSourceID			NVARCHAR(50)
		,@DataSourceName		NVARCHAR(255) = ''
		,@SourceType			NVARCHAR(50) = ''
		,@ConnectionValuesJSON	NVARCHAR(MAX) = ''
    )

-- must be always execute as owner as it uses IDENTITY INSERT - but this don't work on Azure, if the Owner is an AAD User - OPEN PROBLEM
WITH EXECUTE AS 'dbo'

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	=		 N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username							,N'NULL')	+ N''','''											 
				+ ISNULL(@DataSourceID						,N'NULL')	+ N''','''
				+ ISNULL(@DataSourceName					,N'NULL')	+ N''','''
				+ ISNULL(@SourceType						,N'NULL')	+ N''','''
				+ ISNULL(@ConnectionValuesJSON				,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@DataSourceKey		INT				= 0;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username				= N'';
		IF @DataSourceID			IS NULL SET @DataSourceID			= N'';
		IF @DataSourceName			IS NULL SET @DataSourceName			= N'';
		IF @SourceType				IS NULL SET @SourceType				= N'';
		IF @ConnectionValuesJSON	IS NULL SET @ConnectionValuesJSON	= N'';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_DataSource
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'Transaction user don`t exists', 16, 10);
					END;
				
				IF @DataSourceID  = ''
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'DataSourceID must not be empty.', 16, 10);
					END;

				IF @DataSourceID  = '0'
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'DataSourceID 0 is reserved for Companies without Datasource and cant be created.', 16, 10);
					END;

				IF TRY_CAST(@DataSourceID AS INT) IS NULL
					BEGIN
						SET @ResultCode = 403;
						RAISERROR(N'DataSourceID must be numeric for module compatibility.', 16, 10);
					END;

				IF @DataSourceName  = '' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('DataSourceName must not be empty.', 16, 10);
					END;

				-- KEY AND ID MUST BE THE SAME IN 5.6
				SET @DataSourceKey = CAST(@DataSourceID AS INT);

				-- Data Transactions #################################################

				-- delete / recreate DataSource with same Key
				DELETE FROM system.tDataSources WHERE DataSourceID = @DataSourceID;
				DELETE FROM system.tDataSources WHERE DataSourceKey = @DataSourceKey; -- you to be sure
	
				SET IDENTITY_INSERT system.tDatasources ON;
				INSERT INTO system.tDataSources (DatasourceKey,DataSourceID,DataSourceName,SourceType,ConnectionValuesJSON)
					VALUES (@DataSourceKey,@DataSourceID,@DataSourceName,@SourceType,@ConnectionValuesJSON)
				SET IDENTITY_INSERT system.tDatasources OFF;
	
				SET @ResultCode = 200;
			COMMIT TRANSACTION spPOST_DataSource
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_DataSource	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spPOST_DataSource TO pf_PlanningFactoryService



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_DataSource'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a DataSource.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@DataSourceID';
    SET @value      = N'The ID of the DataSource, an ID which is used to identify the datasource';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@DataSourceName';
    SET @value      = N'The name of the DataSource, free description';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceType';
    SET @value      = N'The Source type of the datasource, its a CODE Word like MSSQL and determines the way a connection is established.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ConnectionValuesJSON';
    SET @value      = N'Encrypted string containing the Connection String including credentials.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;


	GO




