
DROP PROCEDURE IF EXISTS system.spPOST_Email;
GO

/*
	Procedure to POST one Email
		- send NULL as Parametervalue to keep the existing Value
		- you can skip Parameter you would send with NULL, as NULL is the default value
		- you can pass * as EmailID to create a new Email E1..En

	This Procedure will be executed
		- from the OCT Application Edit GUI - always as User which is logged in
		- from Consultant in SSMS - as User SQL (which there must have role FactorySerivce or db_owner)
		- from a control script (create Email for exections) as User SQL

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 	Testcall Procedure
		DECLARE  @RC					INT
				,@ReturnedEmailID	NVARCHAR (50)
		EXEC	 @RC = system.spPOST_Email
				 @Username				= 'SQL'
				,@EmailID				= '*'
				,@EmailSubject			= 'Wichtig'
				,@EmailRecipients		= 'gerd.tautenhahn@saxess-software.com'
				,@EmailText				= 'Hello World'
				,@EmailAttachments		= NULL	-- List for FileIDs
				,@EmailStatusCODE		= 'PENDING'
				,@SourceTypeCODE		= 'PIPELINE'
				,@SourceID				= 'DS1-P2'
				,@CreationDate			= '20220613 08:04:02.111'
				,@PostDate				= '20220613 08:08:02.111'
				,@OutputID				= @ReturnedEmailID OUTPUT;
		PRINT    @RC;
		PRINT	'EmailID Output: '+ ISNULL(@ReturnedEmailID,'no Output set');

		SELECT * FROM system.tEmails;
		DELETE FROM system.tEmails;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Email',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Email','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Email
    (
         @Username				NVARCHAR(255)
        ,@EmailID				NVARCHAR(50)	
		,@EmailSubject			NVARCHAR(2000)	= NULL
		,@EmailRecipients		NVARCHAR(4000)	= NULL
		,@EmailText				NVARCHAR(MAX)	= NULL
		,@EmailAttachments		NVARCHAR(2000)	= NULL
		,@EmailStatusCODE		NVARCHAR(50)	= NULL
		,@SourceTypeCODE		NVARCHAR(50)	= NULL
		,@SourceID				NVARCHAR(50)	= NULL
		,@CreationDate			DATETIME		= NULL
		,@PostDate				DATETIME		= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT	
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username									,N'NULL')	+ N''','''		
				+ ISNULL(@EmailID									,N'NULL')	+ N''','''
				+ ISNULL(@EmailSubject								,N'NULL')	+ N''','''
				+ ISNULL(@EmailRecipients							,N'NULL')	+ N''','''
				+ ISNULL(@EmailText									,N'NULL')	+ N''','''
				+ ISNULL(@EmailAttachments							,N'NULL')	+ N''','''
				+ ISNULL(@EmailStatusCODE							,N'NULL')	+ N''','''
				+ ISNULL(@SourceTypeCODE							,N'NULL')	+ N''','''
				+ ISNULL(@SourceID									,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@CreationDate,104)	,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@PostDate,104)		,N'NULL')	+ N''''		

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastEmail			INT				= 0;



		-- NULL Protection for all mandatory  Input parameters
		IF @Username			IS NULL SET @Username			= N'';
		IF @EmailID				IS NULL SET @EmailID			= N'';


		-- Content Protection for specific input parameters (e.g.IDs)
		SET @EmailID		= dbo.sx_pf_pProtectID	(@EmailID);
		IF @EmailStatusCODE NOT IN ('PENDING','ERROR','SENT') SET @EmailStatusCODE = 'CODEERROR';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Email
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Starte Operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #CurrentEmail;
				CREATE TABLE #CurrentEmail
					(
						 EmailKey				BIGINT										NOT NULL
						,EmailID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,EmailSubject			NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NULL
						,EmailRecipients		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NULL
						,EmailText				NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NULL
						,EmailAttachments		NVARCHAR(2000)	COLLATE DATABASE_DEFAULT	NULL
						,EmailStatusCODE		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NULL
						,SourceTypeCODE			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NULL
						,SourceID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NULL
						,CreationDate			DATETIME		NULL
						,PostDate				DATETIME		NULL

					)

				-- Cache current values
				INSERT INTO #CurrentEmail
					SELECT 
						 EmailKey			
						,EmailID			
						,EmailSubject		
						,EmailRecipients	
						,EmailText			
						,EmailAttachments	
						,EmailStatusCODE	
						,SourceTypeCODE		
						,SourceID			
						,CreationDate		
						,PostDate			
					FROM system.tEmails
					WHERE EmailID = @EmailID;

				-- insert or update Email
				SELECT @RowCount = Count (*) FROM #CurrentEmail; 

				IF @RowCount = 0
					-- case new Email
					BEGIN
						--- determine next free ID if * is sended as ID, AutoID is generated as P1..Pn
						IF @EmailID = '*'
							BEGIN
								SELECT TOP 1 
								@LastEmail = TRY_CAST(RIGHT(EmailID,LEN(EmailID)-1) AS INT ) 
								FROM system.tEmails 
								WHERE 
										EmailID LIKE 'E%'
									AND LEN(EmailID) > 1
									AND TRY_CAST(RIGHT(EmailID,LEN(EmailID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(EmailID,LEN(EmailID)-1) AS INT) DESC

								IF @LastEmail IS NULL SET @LastEmail = 0;
								SET @EmailID = 'E' + CAST(@LastEmail +1 AS NVARCHAR(50));
							END

						-- INSERT with default values if NULL
						INSERT INTO system.tEmails VALUES
							(
								  @EmailID		
								 ,ISNULL(@EmailSubject		,'')
								 ,ISNULL(@EmailRecipients	,'')
								 ,ISNULL(@EmailText			,'')
								 ,ISNULL(@EmailAttachments	,'')
								 ,ISNULL(@EmailStatusCODE	,'')
								 ,ISNULL(@SourceTypeCODE		,'')
								 ,ISNULL(@SourceID			,'')
								 ,ISNULL(@CreationDate		,'1900-01-01 08:08:00.000')
								 ,ISNULL(@PostDate			,NULL)

							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					-- Case existing Email
					BEGIN
						-- UPDATE 
						UPDATE tE
								SET 
									  EmailSubject			= ISNULL(@EmailSubject		,cE.EmailSubject		)
									 ,EmailRecipients		= ISNULL(@EmailRecipients	,cE.EmailRecipients		)
									 ,EmailText				= ISNULL(@EmailText			,cE.EmailText			)
									 ,EmailAttachments		= ISNULL(@EmailAttachments	,cE.EmailAttachments	)
									 ,EmailStatusCODE		= ISNULL(@EmailStatusCODE	,cE.EmailStatusCODE		)
									 ,SourceTypeCODE		= ISNULL(@SourceTypeCODE	,cE.SourceTypeCODE		)	
									 ,SourceID				= ISNULL(@SourceID			,cE.SourceID			)
									 ,CreationDate			= ISNULL(@CreationDate		,cE.CreationDate		)
									 ,PostDate				= ISNULL(@PostDate			,cE.PostDate			)

							FROM system.tEmails tE
								LEFT JOIN #CurrentEmail cE
									ON tE.EmailKey = cE.EmailKey
							WHERE tE.EmailKey = cE.EmailKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- OUTPUT Parameter
				SET @OutputID = @EmailID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Email
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Email	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO


GRANT EXECUTE ON OBJECT :: system.spPOST_Email TO pf_PlanningFactoryService;
GO
 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Email'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Email.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@EmailID';
    SET @value      = N'ID of the Email, will be updated or replaced, use * to create an new Email with AutoID E1..En';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
     
	SET @level2name = N'@EmailSubject';
    SET @value      = N'Subject of the Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailRecipients';
    SET @value      = N'List of Email Recipients, semicolon separated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailText';
    SET @value      = N'EmailText';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailAttachments';
    SET @value      = N'List of FileIDs, semicolon separated';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailStatusCODE';
    SET @value      = N'Status Code for the Email, can be PENDING,ERROR,SENT';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceTypeCODE';
    SET @value      = N'Source Type (who created this mail) - can be PIPELINE, EVENT, MANUALLY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@SourceID';
    SET @value      = N'ID combination of the source oftthe mail - e.g. [DatasourceID]-[PipelineID] or [FactoryID]-[ProductLineID]-[ProductID] or someting else - only for Debugging, not technical used';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@CreationDate';
    SET @value      = N'Datetime of Email creation';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PostDate';
    SET @value      = N'Datetime of Email posting';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
