 
DROP PROCEDURE IF EXISTS system.spPOST_Pipeline;
GO

/*
	Template to POST one Pipeline
		- send NULL as Parametervalue to keep the existing Value
		- you can skip Parameter you would send with NULL, as NULL is the default value
		- you can pass * as PipelineID to create a new pipeline P1..Pn

	This Procedure will be executed
		- from the OCT Application Edit GUI - always as User which is logged in
		- from the OCT Application Module Activation - always as User which is logged in
		- from Consultant in SSMS - as User SQL (which there must have role FactorySerivce or db_owner)
		- from a control script (create Pipeline due to global management from Global DB) as User SQL

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
				,@ReturnedPipelineID	NVARCHAR (50)
		EXEC	 @RC = system.spPOST_Pipeline
				 @Username						= 'SQL'
				,@PipelineID					= '*'
				,@PipelineName					= NULL
				,@PipelineDescription			= 'Hello World'
				,@PipelineDescriptionIconColor  = '#808080'
				,@OrderOfDisplay				= NULL
				,@EmailNotificationCODE			= NULL
				,@OutputID						= @ReturnedPipelineID OUTPUT;
		PRINT    @RC;
		PRINT	'PipelineID Output: '+ ISNULL(@ReturnedPipelineID,'no Output set');

		SELECT * FROM system.tPipelines;
		DELETE FROM system.tPipelines;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Pipeline',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Pipeline','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_Pipeline
    (
         @Username						NVARCHAR(255)
        ,@PipelineID					NVARCHAR(50)	
		,@PipelineName					NVARCHAR(255)	= NULL
		,@PipelineDescription			NVARCHAR(4000)	= NULL
		,@PipelineDescriptionIconColor	NVARCHAR(50)	= NULL
		,@OrderOfDisplay				INT				= NULL
		,@EmailNotificationCODE			NVARCHAR(50)	= NULL	
		,@OutputID						NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username								,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID							,N'NULL')	+ N''','''
				+ ISNULL(@PipelineName							,N'NULL')	+ N''','''
				+ ISNULL(@PipelineDescription					,N'NULL')	+ N''','''
				+ ISNULL(@PipelineDescriptionIconColor			,N'NULL')	+ N''','
				+ ISNULL(CAST (@OrderOfDisplay AS NVARCHAR(255)),N'NULL')	+ N','''
				+ ISNULL(@EmailNotificationCODE					,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastPipeline		INT				= 0;

		-- NULL Protection for all mandatory  Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @PipelineID		= dbo.sx_pf_pProtectID	(@PipelineID);
		IF @EmailNotificationCODE NOT IN ('NEVER','ALWAYS','ONERROR') SET @EmailNotificationCODE = 'NEVER';


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Pipeline
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Data Transaction
				PRINT CONCAT(N'Starte Operation',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');

				DROP TABLE IF EXISTS #CurrentPipeline;
				CREATE TABLE #CurrentPipeline
					(
						 PipelineKey					BIGINT										NOT NULL
						,PipelineID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineDescription			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineDescriptionIconColor	NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,OrderOfDisplay					INT											NOT NULL
						,EmailNotificationCODE			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
					)

				-- Cache current values
				INSERT INTO #CurrentPipeline
					SELECT 
						 PipelineKey
						,PipelineID
						,PipelineName
						,PipelineDescription
						,PipelineDescriptionIconColor
						,OrderOfDisplay
						,EmailNotificationCODE
					FROM system.tPipelines
					WHERE PipelineID = @PipelineID;

				-- insert or update Pipeline
				SELECT @RowCount = Count (*) FROM #CurrentPipeline; 

				IF @RowCount = 0
					-- case new Pipeline
					BEGIN
						--- determine next free ID if * is sended as ID, AutoID is generated as P1..Pn
						IF @PipelineID = '*'
							BEGIN
								SELECT TOP 1 
								@LastPipeline = TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT ) 
								FROM system.tPipelines 
								WHERE 
										PipelineID LIKE 'P%'
									AND LEN(PipelineID) > 1
									AND TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(PipelineID,LEN(PipelineID)-1) AS INT) DESC

								IF @LastPipeline IS NULL SET @LastPipeline = 0;
								SET @PipelineID = 'P' + CAST(@LastPipeline +1 AS NVARCHAR(50));
							END

						-- Determine Next Ordernumber to be added at the end
						IF @OrderOfDisplay IS NULL
							BEGIN
								SELECT @OrderOfDisplay = MAX(OrderOfDisplay) + 1
								FROM system.tPipelines
							END

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelines VALUES
							(
								 @PipelineID
								,ISNULL(@PipelineName,'<set name>')
								,ISNULL(@PipelineDescription,'')
								,ISNULL(@PipelineDescriptionIconColor,'')
								,ISNULL(@OrderOfDisplay,0)
								,ISNULL(@EmailNotificationCODE,'NEVER')
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					-- Case existing Pipeline
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						UPDATE tP
								SET 
									 PipelineName					= ISNULL(@PipelineName					,cP.PipelineName)
									,PipelineDescription			= ISNULL(@PipelineDescription			,cP.PipelineDescription)
									,PipelineDescriptionIconColor	= ISNULL(@PipelineDescriptionIconColor	,cP.PipelineDescriptionIconColor)
									,OrderOfDisplay					= ISNULL(@OrderOfDisplay				,cP.OrderOfDisplay)
									,EmailNotificationCODE			= ISNULL(@EmailNotificationCODE			,cP.EmailNotificationCODE)
							FROM system.tPipelines tP
								LEFT JOIN #CurrentPipeline cP 
									ON tP.PipelineKey = cP.PipelineKey
							WHERE tP.PipelineKey = cP.PipelineKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- OUTPUT Parameter
				SET @OutputID = @PipelineID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Pipeline
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Pipeline	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights (only need if end user need direct database access to this object)
GRANT EXECUTE ON OBJECT :: system.spPOST_Pipeline TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Pipeline'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)                                                      
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline, will be updated or replaced, use * to create an new Pipeline with AutoID P1..Pn';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PipelineName';
    SET @value      = N'Name of the Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PipelineDescription';
    SET @value      = N'Description of the Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@PipelineDescriptionIconColor';
    SET @value      = N'Icon Color for the description, written as HEX Code.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OrderOfDisplay';
    SET @value      = N'Ordernumber for displaying the Pipelines';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@EmailNotificationCODE';
    SET @value      = N'EmailNotificationCODE of the Pipeline - must be NEVER, ALWAYS or ONERROR.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
