 
DROP PROCEDURE IF EXISTS system.spPOST_PipelineLog;
GO

/*
	Procedure to POST a PipelineLog
		- can be posted with ExecutionID = NULL and will return a new generated ExecutionID, which is a unique number for the database
		- optional Values can be sended as NULL or skiped, which means "keep what is"
		- the PipelineName will be materialized in the Background to be avaiable in historic logs after deleting the pipeline

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineLog
				 @Username		= 'SQL'
				,@PipelineID	= 'P1'
				,@ExecutionID	= 'EID9'
				,@StatusCODE	= 'RUNNING'			-- WAITING,RUNNING, STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
				,@StartedBy		= 'W10\admin'		-- Username or the word "Scheduler"
				,@StartTime		= NULL				-- Timestamp 
				,@EndTime		= NULL				-- Timestamp
				,@OutPutID		= @ReturnID OUTPUT
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineLog','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_PipelineLog
    (
          @Username				NVARCHAR(255)
		 ,@PipelineID			NVARCHAR(50)
		 ,@ExecutionID			NVARCHAR(50)	
		 ,@ExecutionServerName	NVARCHAR(255)	= NULL
		 ,@StatusCODE			NVARCHAR(50)	= NULL
		 ,@StartedBy			NVARCHAR(255)	= NULL
		 ,@StartTime			DATETIME		= NULL
		 ,@EndTime				DATETIME		= NULL
		 ,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	
	   PRINT 1
		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','''									 
				+ ISNULL(@PipelineID			,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID			,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionServerName	,N'NULL')	+ N''','''
				+ ISNULL(@StatusCode			,N'NULL')	+ N''','''
				+ ISNULL(@StartedBy				,N'NULL')	+ N''','''	
				+ ISNULL(CONVERT(NVARCHAR(50),@StartTime,104)		,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@EndTime,104)			,N'NULL')	+ N''''		

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTimeRun		DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@PipelineName		NVARCHAR(255)	= N'';

				 PRINT 2
		-- NULL Protection for all non-optional Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';


		SET @PipelineID		= dbo.sx_pf_pProtectID	(@PipelineID);

		
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineLog
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check exictence of IDs this procedure depends on
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('PipelineID don`t exits', 16, 10);
					END

				-- Cache Values
				PRINT CONCAT(N'Starting value caching',N' (started after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
				DROP TABLE IF EXISTS #CurrentLog;
				CREATE TABLE #CurrentLog
					(
						 LogKey					BIGINT										NOT NULL
						,PipelineID				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,PipelineName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ExecutionID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ExecutionServerName	NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StatusCODE				NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartedBy				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartTime				DATETIME									NULL
						,EndTime				DATETIME									NULL

					);
				INSERT INTO #CurrentLog
					SELECT
						 LogKey			
						,PipelineID		
						,PipelineName	
						,ExecutionID
						,ExecutionServerName
						,StatusCODE		
						,StartedBy		
						,StartTime		
						,EndTime		
					FROM system.tPipelineLog
					WHERE @ExecutionID = ExecutionID;

				SELECT @RowCount = Count (*) FROM #CurrentLog; 
				
				-- Exit on not existing ExecutionID
				IF @RowCount = 0 AND @ExecutionID <>'*'
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('ExecutionID don`t exits', 16, 10);
					END
				
				-- Data Transaction
				PRINT CONCAT(N'Start data operation',N' (started after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)')
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert new LogEntry',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
			
						-- create new ExecutionID and get PipelineName
						SELECT @ExecutionID = 'EID'+ CAST(NEXT VALUE FOR system.seqPipelineLog AS NVARCHAR(50));
						SELECT @PipelineName = PipelineName FROM system.tPipelines WHERE PipelineID = @PipelineID;
							
						PRINT 'Start Insert'
						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineLog VALUES
							(
								 @PipelineID
								,@PipelineName
								,@ExecutionID
								,ISNULL(@ExecutionServerName,'')	
								,ISNULL(@StatusCODE,'')	
								,ISNULL(@StartedBy,'')
								,ISNULL(@StartTime,NULL)
								,ISNULL(@EndTime,NULL)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
						UPDATE t
								SET 
									 ExecutionServerName	= ISNULL(@ExecutionServerName	,c.ExecutionServerName	)
									,StatusCode				= ISNULL(@StatusCode			,c.StatusCode			)
									,StartedBy				= ISNULL(@StartedBy				,c.StartedBy			)
									,StartTime				= ISNULL(@StartTime				,c.StartTime			)
									,EndTime				= ISNULL(@EndTime				,c.EndTime				)

							FROM system.tPipelineLog t
								LEFT JOIN #CurrentLog c
									ON t.LogKey = c.LogKey
							WHERE t.LogKey = c.LogKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @ExecutionID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineLog
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineLog	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

		-- additional log in the Process Log to be transferd 
		DECLARE
			 @EventMessage	NVARCHAR(2000)	= ''
			,@ProcessID		NVARCHAR(50)	= ''
			,@Timestamp		DATETIME;

		SET @EventMessage	= CONCAT(N'Execution of Pipeline "',@PipelineName, N'" is ',@StatusCODE,N' on Server ',@ExecutionServerName);  -- results in "Excecution of Pipeline DATEV is Running on Server https://..."
		SET @ProcessID		= CONCAT(@PipelineID,N'-',@ExecutionID);
		IF @EndTime IS NULL SET @Timestamp = @StartTime ELSE SET @Timestamp = @EndTime;
		IF @Timestamp	IS NULL SET @Timestamp = GETUTCDATE();


		EXEC system.pPOST_ProcessLogEntry
			 @Username			= N'SQL'
			,@ModuleCode		= N'DataFlows'
			,@ProcessName		= @PipelineName
			,@ProcessID			= @ProcessID
			,@SubProcessName	= N''
			,@SubProcessID		= ''
			,@SubSubprocessName = N''
			,@EventCODE			= @StatusCODE
			,@Timestamp			= @Timestamp
			,@ReturnCODE		= @ResultCode
			,@EventMessage		= @EventMessage;

		RETURN @ResultCode;
	END
 
GO
 

GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineLog TO pf_PlanningFactoryService;



-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineLog'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST an Logentry for an Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline this LogEntry belongs to';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ExecutionID';
    SET @value      = N'ExecutionID in case of update operation - POST as * to generate a new ExecutionID ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ExecutionServerName';
    SET @value      = N'Name of the Server, which executed this task';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StatusCODE';
    SET @value      = N'Status CODE of the PipelineLog - see list at table definiton.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StartedBy';
    SET @value      = N'Name of the user who started this pipeline, is Scheduler in case of scheduler started';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StartTime';
    SET @value      = N'Time of starting the Pipeline';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@EndTime';
    SET @value      = N'Time the pipeline was finished or setted to finish due to not succesful ending.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@OutputID';
    SET @value      = N'Returns the used or new generated ExecutionID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
	