
DROP PROCEDURE IF EXISTS system.spPOST_PipelineSchedule;
GO

/*
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 03/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC					INT
				,@ReturnedScheduleID	NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineSchedule
				 @Username					= 'SQL'
				,@PipelineID				= 'P1'			
				,@ScheduleID				= '*'
				,@ScheduleName				= 'Hallo'	
				,@ScheduleDescription		= 'Desc'
				,@ScheduleDetailsJSON		= '{}'
				,@Active					= 1
				,@OutputID					= @ReturnedScheduleID OUTPUT;			
		PRINT    @RC
		PRINT	 'ScheduleID OUT: ' + @ReturnedScheduleID;

		SELECT * FROM system.tPipelineSchedules;
		SELECT * FROM system.tPipelines;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineSchedule',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineSchedule','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_PipelineSchedule
    (
         @Username					NVARCHAR(255)
		,@PipelineID				NVARCHAR(50)
        ,@ScheduleID				NVARCHAR(50)
		,@ScheduleName				NVARCHAR(255)	= NULL
		,@ScheduleDescription		NVARCHAR(4000)	= NULL
		,@ScheduleDetailsJSON		NVARCHAR(MAX)	= NULL
		,@Active					BIT				= NULL
		,@OutputID					NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username						,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID					,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleID					,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleName					,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleDescription			,N'NULL')	+ N''','''
				+ ISNULL(@ScheduleDetailsJSON			,N'NULL')	+ N''','
				+ ISNULL(CAST(@Active AS NCHAR(1))		,N'NULL')	+ N''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT			= 0
				,@RowCount			INT				= 0
				,@LastSchedule		INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ScheduleID				IS NULL SET @ScheduleID			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @ScheduleID		= dbo.sx_pf_pProtectID	(@ScheduleID);


		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineSchedule
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check Existence of Pipeline
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;


				-- Cache
				DROP TABLE IF EXISTS #CurrentSchedule;
				CREATE TABLE #CurrentSchedule
					(
						 ScheduleKey				BIGINT										NOT NULL
						,ScheduleID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ScheduleName				NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ScheduleDescription		NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ScheduleDetailsJSON		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,Active						BIT											NOT NULL
					)

				INSERT INTO #CurrentSchedule
					SELECT
						 ScheduleKey
						,ScheduleID
						,ScheduleName
						,ScheduleDescription
						,ScheduleDetailsJSON
						,Active
					FROM system.tPipelineSchedules
					WHERE 
							PipelineKey =  @PipelineKey
						AND ScheduleID	=  @ScheduleID
				
				-- Data transaction
				SELECT @RowCount = Count (*) FROM #CurrentSchedule; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @ScheduleID = '*'
							BEGIN
								SELECT TOP 1 
								@LastSchedule = TRY_CAST(RIGHT(ScheduleID,LEN(ScheduleID)-1) AS INT ) 
								FROM system.tPipelineSchedules
								WHERE 
										ScheduleID LIKE 'J%'
									AND LEN(ScheduleID) > 1
									AND PipelineKey = @PipelineKey
									AND TRY_CAST(RIGHT(ScheduleID,LEN(ScheduleID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(ScheduleID,LEN(ScheduleID)-1) AS INT) DESC

								IF @LastSchedule IS NULL SET @LastSchedule = 0;
								SET @ScheduleID = 'J' + CAST(@LastSchedule + 1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineSchedules VALUES
							(
								 @PipelineKey
								,@ScheduleID
								,ISNULL(@ScheduleName,'<set name>')
								,ISNULL(@ScheduleDescription,'')
								,ISNULL(@ScheduleDetailsJSON,'')
								,ISNULL(@Active,1)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tPS
								SET 
									 ScheduleName				= ISNULL(@ScheduleName				,cS.ScheduleName)
									,ScheduleDescription		= ISNULL(@ScheduleDescription		,cS.ScheduleDescription)
									,ScheduleDetailsJSON		= ISNULL(@ScheduleDetailsJSON	,cS.ScheduleDetailsJSON)
									,Active						= ISNULL(@Active					,cS.Active)

						FROM system.tPipelineSchedules tPS
							LEFT JOIN #CurrentSchedule cS
								ON tPS.ScheduleKey = cS.ScheduleKey
						WHERE tPS.ScheduleKey = cS.ScheduleKey;
							
						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 
				-- Output
				SET @OutputID = @ScheduleID;
				
				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineSchedule
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineSchedule	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineSchedule TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineSchedule'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to call demo values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline which contains this schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ScheduleID';
    SET @value      = N'ID of the Schedule - POST * to create a new autogenerated ID.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ScheduleName';
    SET @value      = N'optional - name for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ScheduleDescription';
    SET @value      = N'optional - description for the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ScheduleDetailsJSON';
    SET @value      = N'optional - definition of the schedule.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Active';
    SET @value      = N'optional - Flag to set schedule to active / inactive.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
