
DROP PROCEDURE IF EXISTS system.spPOST_PipelineStep;
GO

/*
	Procedure to create a new pipeline step.
		- as StepID you can pass * to create an AutoID S1..Sn

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
	Testcall Procedure
		DECLARE  @RC INT
				,@ReturnedStepID NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineStep
				 @Username				= 'SQL'
				,@PipelineID			= 'P1'
				,@StepID				= '*'
				,@StepName				= 'Step1'
				,@StepDescription		= 'Hallo'
				,@OrderOfExecution		= 1
				,@StepDataSourceID		= '1'   -- not in use, Datasource is saved in StepDetailsJSON
				,@StepCompanyIDsJSON	= '{}'
				,@StepDetailsJSON		= '{}'
				,@TimeExecutionLimit	= 0
				,@Active				= 1
				,@ModuleID				= 'FIN'
				,@StartNextStepOnError	= 1
				,@OutputID				= @ReturnedStepID OUTPUT;
		PRINT    @RC;
		PRINT	'StepID_OUT :'+ @ReturnedStepID;

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStep',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStep','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spPOST_PipelineStep
    (
         @Username				NVARCHAR(255)
        ,@PipelineID			NVARCHAR(50)	
		,@StepID				NVARCHAR(50)
		,@StepName				NVARCHAR(255)	= NULL		
		,@StepDescription		NVARCHAR(4000)	= NULL
		,@OrderOfExecution		INT				= NULL
		,@StepDataSourceID		NVARCHAR(50)	= NULL
		,@StepCompanyIDsJSON	NVARCHAR(MAX)	= NULL
		,@StepDetailsJSON		NVARCHAR(MAX)	= NULL
		,@TimeExecutionLimit	INT				= NULL
		,@Active				BIT				= NULL
		,@ModuleID				NVARCHAR(50)	= NULL
		,@StartNextStepOnError	BIT				= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username										,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID									,N'NULL')	+ N''','''
				+ ISNULL(@StepID										,N'NULL')	+ N''','''
				+ ISNULL(@StepName										,N'NULL')	+ N''','''
				+ ISNULL(@StepDescription								,N'NULL')	+ N''','
				+ ISNULL(CAST(@OrderOfExecution AS NVARCHAR(50))		,N'NULL')	+ N','''
				+ ISNULL(@StepDataSourceID								,N'NULL')	+ N''','''
				+ ISNULL(@StepCompanyIDsJSON							,N'NULL')	+ N''','''
				+ ISNULL(@StepDetailsJSON								,N'NULL')	+ N''','
				+ ISNULL(CAST(@TimeExecutionLimit AS NVARCHAR(50))		,N'NULL')	+ N','
				+ ISNULL(CAST(@Active AS NCHAR(1))						,N'NULL')	+ N','''
				+ ISNULL(@ModuleID										,N'NULL')	+ N''','
				+ ISNULL(CAST(@StartNextStepOnError	 AS NCHAR(1))		,N'NULL')	+ N''


				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@PipelineKey		BIGINT			= 0
				,@RowCount			INT				= 0
				,@LastStep			INT				= 0;

		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @PipelineID				IS NULL SET @PipelineID			= N'';
		IF @StepID					IS NULL SET @StepID				= N'';

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineStep
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;
				-- Check existence of Pipeline
				IF NOT EXISTS (
					SELECT PipelineID FROM system.tPipelines WHERE PipelineID = @PipelineID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline don`t exits', 16, 10);
					END

				SELECT @PipelineKey = PipelineKey FROM system.tPipelines WHERE PipelineID = @PipelineID;

				-- Cache existing content
				DROP TABLE IF EXISTS #CurrentStep;
				CREATE TABLE #CurrentStep
					(
						 PipelineKey				BIGINT										NOT NULL
						,StepKey					BIGINT										NOT NULL
						,StepID						NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepName					NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepDescription			NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,OrderOfExecution			INT											NOT NULL
						,StepDataSourceID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepCompanyIDsJSON			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepDetailsJSON			NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,TimeExecutionLimit			INT											NOT NULL
						,Active						BIT											NOT NULL
						,ModuleID					NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartNextStepOnError		BIT 										NOT NULL
					);

				INSERT INTO #CurrentStep
					SELECT 
						 PipelineKey
						,StepKey
						,StepID
						,StepName
						,StepDescription
						,OrderOfExecution
						,StepDataSourceID
						,StepCompanyIDsJSON
						,StepDetailsJSON
						,TimeExecutionLimit
						,Active
						,ModuleID
						,StartNextStepOnError
					FROM system.tPipelineSteps 
					WHERE 
							PipelineKey = @PipelineKey 
						AND StepID		= @StepID;

				-- Data Transaction

				SELECT @RowCount = Count (*) FROM #CurrentStep; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @StepID = '*'
							BEGIN
								SELECT TOP 1 
								@LastStep = TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT ) 
								FROM system.tPipelineSteps 
								WHERE 
										StepID LIKE 'S%'
									AND LEN(StepID) > 1
									AND PipelineKey = @PipelineKey
									AND TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(StepID,LEN(StepID)-1) AS INT) DESC

								IF @LastStep IS NULL SET @LastStep = 0;
								SET @StepID = 'S' + CAST(@LastStep +1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineSteps VALUES
							(
								 
								 @PipelineKey
								,@StepID
								,ISNULL(@StepName,'<set name>')
								,ISNULL(@StepDescription,'')
								,ISNULL(@OrderOfExecution,0)
								,ISNULL(@StepDataSourceID,'')
								,ISNULL(@StepCompanyIDsJSON,'')
								,ISNULL(@StepDetailsJSON,'')
								,ISNULL(@TimeExecutionLimit,0)
								,ISNULL(@Active,1)
								,ISNULL(@ModuleID,'')
								,ISNULL(@StartNextStepOnError,0)
							)
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tPS
								SET 
									 StepName				= ISNULL(@StepName				,cS.StepName)
									,StepDescription		= ISNULL(@StepDescription		,cS.StepDescription)
									,OrderOfExecution		= ISNULL(@OrderOfExecution		,cS.OrderOfExecution)
									,StepDataSourceID		= ISNULL(@StepDataSourceID		,cS.StepDataSourceID)
									,StepCompanyIDsJSON		= ISNULL(@StepCompanyIDsJSON	,cS.StepCompanyIDsJSON)
									,StepDetailsJSON		= ISNULL(@StepDetailsJSON		,cS.StepDetailsJSON)
									,TimeExecutionLimit		= ISNULL(@TimeExecutionLimit	,cS.TimeExecutionLimit)
									,Active					= ISNULL(@Active				,cS.Active)
									,ModuleID				= ISNULL(@ModuleID				,cS.ModuleID)
									,StartNextStepOnError	= ISNULL(@StartNextStepOnError	,cS.StartNextStepOnError)
							FROM system.tPipelineSteps tPS
								LEFT JOIN #CurrentStep cS
									ON tPS.StepKey = cS.StepKey
							WHERE tPS.StepKey = cS.StepKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @StepID;

				PRINT CONCAT(N'Transaktion abgeschlossen',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineStep
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineStep	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO
 
GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineStep TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineStep'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST a Pipeline step, optional Parameter can be skiped, which will keep existing values.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline of the step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'ID of the Step to POST, as StepID you can pass * to create an AutoID S1..Sn';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Stepname';
    SET @value      = N'optional - name of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepDescription';
    SET @value      = N'optional - description of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OrderOfExecution';
    SET @value      = N'optional - number for order of execution';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepDataSourceID';
    SET @value      = N'not in use, Datasource is saved in StepDetailsJSON';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepCompanyIDsJSON';
    SET @value      = N'optional - JSON Array of the Company IDs to use in this Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StepDetailsJSON';
    SET @value      = N'Configuration of the step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@TimeExecutionLimit';
    SET @value      = N'Execution Time Limit in minutes';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@Active';
    SET @value      = N'Flag if the step is active';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ModuleID';
    SET @value      = N'ID of the accociated Module, for Steps which have a module parameter.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@StartNextStepOnError';
    SET @value      = N'Setting if the next step is started if this step ends with error.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO


			
		
