 
DROP PROCEDURE IF EXISTS system.spPOST_PipelineStepLog;
GO
/* 
	Procedure to POST one Step Log
		- StepLog for not existing ExecutionID will be created
		- StepLog for existing ExcecutionID will be updated
		- optional Parameters posted with NULL leads to Keep what exists
		- Step name is materialized in the background for history reasons
		- Logmessage is replaced or appended together with a line break
		- Output Parameter not in use

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC		INT
				,@ReturnID	NVARCHAR(50);
		EXEC	 @RC = system.spPOST_PipelineStepLog
				  @Username			= 'SQL'
				 ,@PipelineID		= 'P1'
				 ,@StepID			= 'S1'
				 ,@ExecutionID		= 'EID4'
				 ,@StartTime		= NULL
				 ,@EndTime			= NULL
				 ,@StatusCODE		= 'Running'
				 ,@LogMessage		= 'Hallo Gerd'
				 ,@ReplaceMessageBOOL= 1
				 ,@HandleID			= '55'
				 ,@OutputID = @ReturnID OUTPUT;
	
		PRINT	CONCAT('ReturnCode: ', @RC, ' OutputID: ', @ReturnID);

		SELECT * FROM system.tPipelines;
		SELECT * FROM system.tPipelineSteps;
		SELECT * FROM system.tPipelineLog;
		SELECT * FROM system.tPipelineStepLog;
		SELECT * FROM system.tAPI_Log;
		SELECT * FROM system.tProcessLogs;
	
		DELETE FROM system.tPipelineStepLog;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStepLog',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_PipelineStepLog','PARAMETER',NULL);
*/
 
CREATE PROCEDURE system.spPOST_PipelineStepLog
    (
         @Username				NVARCHAR(255)
		,@PipelineID			NVARCHAR(50)
		,@StepID				NVARCHAR(50)
        ,@ExecutionID			NVARCHAR(50)			-- is indexed in table as Unique
		,@StartTime				DATETIME		= NULL 
		,@EndTime				DATETIME		= NULL
		,@StatusCODE			NVARCHAR(50)	= NULL
		,@LogMessage			NVARCHAR(MAX)	= NULL
		,@ReplaceMessageBOOL	BIT				= NULL
		,@HandleID				NVARCHAR(50)	= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username									,N'NULL')	+ N''','''											 
				+ ISNULL(@PipelineID								,N'NULL')	+ N''','''
				+ ISNULL(@StepID									,N'NULL')	+ N''','''
				+ ISNULL(@ExecutionID								,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@StartTime,104)		,N'NULL')	+ N''','''
				+ ISNULL(CONVERT(NVARCHAR(50),@EndTime,104)			,N'NULL')	+ N''','''
				+ ISNULL(@StatusCODE								,N'NULL')	+ N''','''
				+ ISNULL(@LogMessage								,N'NULL')	+ N''','
				+ ISNULL(CAST(@ReplaceMessageBOOL AS NVARCHAR(255))	,N'NULL')	+ N','''
				+ ISNULL(@HandleID									,N'NULL')	+ N''','''
				+ ISNULL(@OutputID									,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTimeRun		DATETIME2		= SysUTCDateTime()
				,@StepName			NVARCHAR(255)	= ''
				,@CurrentLogMessage NVARCHAR(MAX)	= '';

		-- Procedure specific declarations
		DECLARE  @RowCount			INT				= 0
				,@PipelineName		NVARCHAR(2000)	= '';

		-- NULL Protection for all mandatory Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ExecutionID				IS NULL SET @ExecutionID		= N'';
		IF @ReplaceMessageBOOL		IS NULL SET @ReplaceMessageBOOL = 1;

		-- Content Protection for specific input parameters (e.g.IDs)
		SET @ExecutionID		= dbo.sx_pf_pProtectID	(@ExecutionID);
			
		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_PipelineStepLog
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('Transaction user don`t exists', 16, 10);
					END;

				-- Check exictence of IDs this procedure depends on
				IF NOT EXISTS (
					SELECT P.PipelineID, PS.StepID 
					FROM system.tPipelines	P
						LEFT JOIN system.tPipelineSteps PS
							ON P.PipelineKey = PS.PipelineKey
						WHERE 
								P.PipelineID	= @PipelineID 
							AND PS.StepID		= @StepID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Pipeline or Step don`t exits', 16, 10);
					END;

				IF NOT EXISTS (
					SELECT ExecutionID FROM system.tPipelineLog WHERE ExecutionID = @ExecutionID
					)
					BEGIN
						SET @ResultCode = 404;
						RAISERROR('Execution ID don`t exits', 16, 10);
					END;

				-- Cache Values 
				DROP TABLE IF EXISTS #CurrentStepLog;
				CREATE TABLE #CurrentStepLog
					(
						 LogKey			BIGINT										NOT NULL
						,PipelineID		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StepID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ExecutionID	NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StartTime		DATETIME									NULL
						,EndTime		DATETIME									NULL
						,LogMessage		NVARCHAR(MAX)	COLLATE DATABASE_DEFAULT	NOT NULL
						,StatusCODE		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,HandleID		NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
					);
				INSERT INTO #CurrentStepLog
					SELECT
						 LogKey		
						,PipelineID	
						,StepID		
						,ExecutionID
						,StartTime	
						,EndTime	
						,LogMessage	
						,StatusCODE	
						,HandleID	
					FROM system.tPipelineStepLog
					WHERE 
							StepID		= @StepID
						AND ExecutionID = @ExecutionID;

				-- Determine StepName and Pipelinename
				SELECT
						 @StepName		= PS.StepName
						,@PipelineName	= P.PipelineName
				FROM system.tPipelines P
					LEFT JOIN system.tPipelineSteps PS
						ON P.PipelineKey = PS.PipelineKey 
				WHERE 
							P.PipelineID	= @PipelineID
						AND PS.StepID		= @StepID;

				-- Data Transaction
				PRINT CONCAT(N'Start data operation',N' (started after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
				SELECT @RowCount = COUNT(*) FROM #CurrentStepLog;
				IF @RowCount = 0 
					-- Create a new Log entry
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');

						--PRINT 'PipelineID: '	+ @PipelineID;
						--PRINT 'StepID: '		+ @StepID;
						--PRINT 'ExecutionID: ' + @ExecutionID;
						--PRINT 'Stepname: '	+ @StepName;
						--PRINT 'Logmessage: '	+ @LogMessage;
						--PRINT 'StatusCode: '	+ @StatusCODE;
						--PRINT 'HandleID: '	+ @HandleID;

						-- INSERT with default values if NULL
						INSERT INTO system.tPipelineStepLog VALUES
							(
								 @PipelineID	
								,@StepID		
								,@ExecutionID
								,@StepName
								,ISNULL(@StartTime	,NULL	)
								,ISNULL(@EndTime	,NULL	)
								,ISNULL(@LogMessage	,''		)
								,ISNULL(@StatusCODE	,''		)
								,ISNULL(@HandleID	,''		)
							);
						PRINT 'Insert done'
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- decide between replace and append for log message
						IF @LogMessage IS NULL --keep the existing if not submitted
							BEGIN
								SELECT @LogMessage = LogMessage FROM #CurrentStepLog;
							END 
						IF @ReplaceMessageBOOL = 0 -- append if no replace flag
							BEGIN
								SELECT @CurrentLogMessage = LogMessage FROM #CurrentStepLog;
								SET @LogMessage = CONCAT(@CurrentLogMessage,CHAR(13),CHAR(10),@LogMessage)
							END
						-- always limit the complete length to 1M characters
						IF LEN(@LogMessage) > 1000000
							BEGIN
								SET @LogMessage		= LEFT(@LogMessage,1000000)	+ ' --- WARNING !!! Log was cutted to first 1.000k Characters from OCT API Procedure !!!'		
							END

						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)');
						UPDATE t
								SET 
									 PipelineID		= ISNULL(@PipelineID				,c.PipelineID				)
									,StepID			= ISNULL(@StepID					,c.StepID					)
									,ExecutionID	= ISNULL(@ExecutionID				,c.ExecutionID				)
									,StartTime		= ISNULL(@StartTime					,c.StartTime				)
									,EndTime		= ISNULL(@EndTime					,c.EndTime					)
									,LogMessage		= @LogMessage
									,StatusCODE		= ISNULL(@StatusCODE				,c.StatusCODE				)
									,HandleID		= ISNULL(@HandleID					,c.HandleID					)
									
							FROM system.tPipelineStepLog t
								LEFT JOIN #CurrentStepLog c
									ON t.LogKey = c.LogKey
							WHERE t.LogKey = c.LogKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @ExecutionID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTimeRun, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_PipelineStepLog
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_PipelineStepLog	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;

		-- additional log in the Process Log to be transferd 
		-- there is the Append / Replace problem - the LogEntry can be replaced or the message appended - but our process log can't handle at the moment an update of LogEntries (as they may be already synced)
		-- that why every append / replace is writen as POST to the Log - this maybe much to much and must be solvend better
		DECLARE
			 @EventMessage	NVARCHAR(2000)	= ''
			,@ProcessID		NVARCHAR(50)	= ''
			,@Timestamp		DATETIME;

		SET @EventMessage	= CONCAT(N'Execution of Pipeline "',@PipelineID, N'" Step "', @StepName,N'" is "',@StatusCODE);  -- results in "Excecution of Pipeline P1 Step DATEV Extraction is Running"
		SET @ProcessID		= CONCAT(@PipelineID,N'-',@ExecutionID);
		SET @Timestamp		= GETUTCDATE();

		EXEC system.pPOST_ProcessLogEntry
			 @Username			= N'SQL'
			,@ModuleCode		= N'DataFlows'
			,@ProcessName		= @PipelineName
			,@ProcessID			= @ProcessID
			,@SubProcessName	= @StepName
			,@SubProcessID		= @StepID
			,@SubSubprocessName = N''
			,@EventCODE			= @StatusCODE
			,@Timestamp			= @Timestamp
			,@ReturnCODE		= @ResultCode
			,@EventMessage		= @EventMessage;

		RETURN @ResultCode;
	END
 
GO
 
-- GRANT Rights, as FactoryService is not able to access system schema by default
GRANT EXECUTE ON OBJECT :: system.spPOST_PipelineStepLog TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_PipelineStepLog' -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'				-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'					-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N'';						-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure to POST Step LogEntries';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- optional SET parameter documentation (only for Core / Standardmodules)
    SET @level2name = N'@PipelineID';
    SET @value      = N'ID of the Pipeline this log belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@StepID';
    SET @value      = N'ID of the Step this log belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@ExecutionID';
    SET @value      = N'Execution - taken from the PipelineLog to keep Pipeline Log and Step Log together';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@StartTime';
    SET @value      = N'optional - Start Time of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@EndTime';
    SET @value      = N'optional - End Time of the Step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@LogMessage';
    SET @value      = N'Log Message - will always be appended.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ReplaceMessageBOOL';
    SET @value      = N'Flag if the sended Log Message will replace the existing message (1), in case of 0 the message is appended to the existing one and a line break is added.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'@StatusCODE';
    SET @value      = N'Status CODE - for definition see table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@HandleID';
    SET @value      = N'HandleID to connect Pentaho Process';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@OutputID';
    SET @value      = N'OutputID is the ExecutionID';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
