
DROP PROCEDURE IF EXISTS system.spPOST_Report;
GO
/*
	Procedure to POST an Report
		- ID will be autogenerated if posted with * as ID
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8
 
 
	Testcall Procedure
		DECLARE  @RC INT
				,@ReturnedID NVARCHAR(50);
		EXEC	 @RC = system.spPOST_Report
				 @Username			= N'SQL'
				,@ReportID			= N'*'
				,@ReportName		= N'Test'
				,@ReportDescription = N'bla'
				,@LayoutDefinition	= 0x
				,@OutputID			= @ReturnedID OUTPUT;
		PRINT    @RC
		PRINT   'Returned ID: ' + ISNULL(@ReturnedID,'<no ID returned>')

		SELECT * FROM system.tReports;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Report',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_Report','PARAMETER',NULL)
*/

CREATE PROCEDURE system.spPOST_Report
    (
         @Username				NVARCHAR(255)
        ,@ReportID				NVARCHAR(50)
		,@ReportName			NVARCHAR(255)	= NULL
		,@ReportDescription		NVARCHAR(4000)	= NULL
		,@LayoutDefinition		VARBINARY(MAX)	= NULL
		,@OutputID				NVARCHAR(50)	= NULL OUTPUT
    )

AS 
	BEGIN	

		-- Standard declaration for logging
		DECLARE 
				 @ProcedureName		NVARCHAR(255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)

				,@ParameterString	NVARCHAR(MAX)	= N''''					--set Strings inside in single quotes (N''','''), Numbers without strings inside without quotes (N','), end list with '''' in case of string or '' in case of number on last position
				+ ISNULL(@Username				,N'NULL')	+ N''','''											 
				+ ISNULL(@ReportID				,N'NULL')	+ N''','''
				+ ISNULL(@ReportName			,N'NULL')	+ N''','''
				+ ISNULL(@ReportDescription		,N'NULL')	+ N''''

				,@EffectedRows		INT				= 0			
				,@ResultCode		INT				= 501				
				,@TimestampCall		DATETIME		= GETUTCDATE()
				,@Comment			NVARCHAR(2000)	= N''
				,@TransactUsername	NVARCHAR(255)	= N''
				,@StartTime			DATETIME2		= SysUTCDateTime()
				,@RowCount			INT				= 0
				,@LastIDNum			NVARCHAR(50)	= 0;


		-- NULL Protection for all Input parameters
		IF @Username				IS NULL SET @Username			= N'';
		IF @ReportID				IS NULL SET @ReportID			= N'';



		-- Content Protection for specific input parameters (e.g.IDs)
		SET @ReportID			= dbo.sx_pf_pProtectID	(@ReportID);

		-- START TRANSACTION			***********************************************************************************
		BEGIN TRY
			BEGIN TRANSACTION spPOST_Report
				-- check transaction user existence
				SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);
					IF @TransactUsername  = N'403' 
						BEGIN
							SET @ResultCode = 403;
							RAISERROR('Transaction user don`t exists', 16, 10);
						END;

				-- Check existence of IDs of depening content

				-- Don't accept ReportID = ''
				IF @ReportID  = N'' 
					BEGIN
						SET @ResultCode = 403;
						RAISERROR('ReportID cant be empty string', 16, 10);
					END;

				-- cache existing values of this Object
				DROP TABLE IF EXISTS #CurrenctReport;
				CREATE TABLE #CurrentReport
					(
						 ReportKey			BIGINT										NOT NULL
						,ReportID			NVARCHAR(50)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReportName			NVARCHAR(255)	COLLATE DATABASE_DEFAULT	NOT NULL
						,ReportDescription	NVARCHAR(4000)	COLLATE DATABASE_DEFAULT	NOT NULL
						,LayoutDefinition	VARBINARY(MAX)								NOT NULL
					)
				INSERT INTO #CurrentReport
					SELECT
						 ReportKey
						,ReportID
						,ReportName
						,ReportDescription
						,LayoutDefinition
					FROM system.tReports
					WHERE ReportID = @ReportID;
			
				-- Data Transaction with * resolving
				PRINT CONCAT(N'Start writing Values...',N' (started after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
				
				SELECT @RowCount = Count (*) FROM #CurrentReport; 
				IF @RowCount = 0
					BEGIN
						PRINT CONCAT(N'Starting insert',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						--- determine next free ID if * is sended as ID, AutoID is generated as S1..Sn per Pipeline
						IF @ReportID = '*'
							BEGIN
								SELECT TOP 1 
								@LastIDNum = TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT ) 
								FROM system.tReports
								WHERE 
										ReportID LIKE N'R%'
									AND LEN(ReportID) > 1
									AND TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT) IS NOT NULL

								ORDER BY TRY_CAST(RIGHT(ReportID,LEN(ReportID)-1) AS INT) DESC

								IF @LastIDNum IS NULL SET @LastIDNum = 0;
								SET @ReportID = 'R' + CAST(@LastIDNum +1 AS NVARCHAR(50));

							END

						-- INSERT with default values if NULL
						INSERT INTO system.tReports VALUES
							(		 
								 @ReportID
								,ISNULL(@ReportName,N'<set name>')
								,ISNULL(@ReportDescription,'')
								,ISNULL(@LayoutDefinition,0x)
							);
						SET @EffectedRows = @@ROWCOUNT;
						SET @ResultCode = 201;
					END
				ELSE
					BEGIN
						-- UPDATE as Insert / delete not possible due to primary key constraints
						PRINT CONCAT(N'Starting update',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)');
						UPDATE tR
								SET 
									 ReportName				= ISNULL(@ReportName			,cR.ReportName)
									,ReportDescription		= ISNULL(@ReportDescription		,cR.ReportDescription)
									,LayoutDefinition		= ISNULL(@LayoutDefinition		,cR.LayoutDefinition)
								
							FROM system.tReports tR
								LEFT JOIN #CurrentReport cR
									ON tR.ReportKey = cR.ReportKey
							WHERE tR.ReportKey = cR.ReportKey

						SET @ResultCode = 200;
						SET @EffectedRows = @@ROWCOUNT;
					END 

				-- Output
				SET @OutputID = @ReportID;

				PRINT CONCAT(N'Transaction finished',N' (after ', DateDiff(millisecond, @StartTime, SysUTCDateTime()),N'ms)')
			COMMIT TRANSACTION spPOST_Report
		END TRY

		-- START CATCH				***********************************************************************************
		BEGIN CATCH
			DECLARE @Error_state INT = ERROR_STATE();
			SET @Comment = ERROR_MESSAGE();
			
			ROLLBACK TRANSACTION spPOST_Report	

			IF @Error_state <> 10 
			BEGIN
				SET @ResultCode = 500;		
				PRINT 'Rollback due to not executable command.';
			END
			ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
			BEGIN
				SET @ResultCode = 500;	
			END;
		END CATCH

		EXEC sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
		RETURN @ResultCode;
	END
 
GO

GRANT EXECUTE ON OBJECT :: system.spPOST_Report TO pf_PlanningFactoryService;
 

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'spPOST_Report'         -- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)   = N'OCT.core'       -- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)   = N'CORE'           -- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT             = 1					-- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''				-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'Procedure POST a Report Definition.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- optional SET parameter documentation (only for Core / Standardmodules)

    SET @level2name = N'@ReportID';
    SET @value      = N'BusinessID for the Report - optional, autogenerated when posted as *.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@ReportName';
    SET @value      = N'Name for the Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@ReportDescription';
    SET @value      = N'Description for the Report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@LayoutDefinition';
    SET @value      = N'Layout Definition for the Report as varbinary with images etc..';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
