﻿
DROP PROCEDURE IF EXISTS system.spPOST_User;
GO

/*
	POST Operation for a User
	Not existing User will be created, existing one updated
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 04/2023 for OCT 5.9
 
	Testcall Procedure
		DECLARE  @RC						INT
				,@Username					NVARCHAR(255) = N'SQL'
				,@PostUserName				NVARCHAR(255) = N'sxs'
				,@PersonSurname				NVARCHAR(255) = N''
				,@PersonFirstName			NVARCHAR(255) = N''
				,@Email						NVARCHAR(255) = N''
				,@LDAPIP					NVARCHAR(255) = N''
				,@Status					NVARCHAR(255) = N'Active'
				,@DataFlowRightsCODE		NVARCHAR(255) = N'DESIGNER'
				,@VisualizationRightsCODE	NVARCHAR(255) = N'DESIGNER'
				,@IsAdministratorFlag		BIT			  = 0;

		EXECUTE @RC = system.spPOST_User 
				 @Username
				,@PostUserName
				,@PersonSurname
				,@PersonFirstName
				,@Email
				,@LDAPIP
				,@Status
				,@DataFlowRightsCODE
				,@VisualizationRightsCODE
				,@IsAdministratorFlag;
		PRINT @RC;
	
	Testcall Target Table
		SELECT * FROM system.trUser;
		SELECT TOP 5 * FROM system.tAPI_Log;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_User',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spPOST_User','PARAMETER',NULL)

*/
 
CREATE PROCEDURE system.spPOST_User
	 @Username					NVARCHAR(255)
	,@PostUserName				NVARCHAR(255)
    ,@PersonSurname				NVARCHAR(255)
    ,@PersonFirstName			NVARCHAR(255)
    ,@Email						NVARCHAR(255)
    ,@LDAPIP					NVARCHAR(255)
    ,@Status					NVARCHAR(255)
	,@DataFlowRightsCODE		NVARCHAR(255)
	,@VisualizationRightsCODE	NVARCHAR(255)
	,@IsAdministratorFlag		BIT
AS
BEGIN
	SET NOCOUNT ON;
	
	DECLARE  @PostUserKey		INT				= 0
			,@TransactUsername	NVARCHAR(255)	= N''
			,@ProcedureName		NVARCHAR (255)	= OBJECT_SCHEMA_NAME(@@PROCID) + N'.' + OBJECT_NAME(@@PROCID)
			,@ParameterString	NVARCHAR (MAX)	= N'''' 
								+ ISNULL(@Username,									N'NULL') + N''',''' 
								+ ISNULL(@PostUserName,								N'NULL') + N''',''' 
								+ ISNULL(@PersonSurname,							N'NULL') + N''',''' 
								+ ISNULL(@PersonFirstName,							N'NULL') + N''',''' 
								+ ISNULL(@Email,									N'NULL') + N''',''' 
								+ ISNULL(@LDAPIP,									N'NULL') + N''',''' 
								+ ISNULL(@Status,									N'NULL') + N''','''
								+ ISNULL(@DataFlowRightsCODE,						N'NULL') + N''''
								+ ISNULL(@VisualizationRightsCODE,					N'NULL') + N''''
								+ ISNULL(CAST(@IsAdministratorFlag AS NCHAR(1)),	N'NULL') + N''''
			
			,@EffectedRows		INT				= 0			
			,@ResultCode		INT				= 501			
			,@TimestampCall		DATETIME		= GETUTCDATE()
			,@Comment			NVARCHAR(2000)	= N''
			,@Message			NVARCHAR(2000)	= N'';

	DECLARE @tmpUserTable		TABLE (UserKey INT NOT NULL);
			
	IF @Username				IS NULL SET @Username					= N'';
	IF @PostUserName			IS NULL SET @PostUserName				= N'';
	IF @PersonSurname			IS NULL SET @PersonSurname				= N'';
	IF @PersonFirstName			IS NULL SET @PersonFirstName			= N'';
	IF @Email					IS NULL SET @Email						= N'';
	IF @LDAPIP					IS NULL SET @LDAPIP						= N'';
	IF @Status					IS NULL SET @Status						= N'';
	IF @DataFlowRightsCODE		IS NULL SET @DataFlowRightsCODE			= N'';
	IF @VisualizationRightsCODE IS NULL SET @VisualizationRightsCODE	= N'';
	IF @IsAdministratorFlag		IS NULL SET @IsAdministratorFlag		= 0;

	BEGIN TRY
		BEGIN TRANSACTION spPOST_User;

		-- STEP 0.2 - Protect input parameters	
		SET @PostUserName		= dbo.sx_pf_pProtectString (@PostUserName);
		SET @PersonSurname		= dbo.sx_pf_pProtectString (@PersonSurname);
		SET @PersonFirstName	= dbo.sx_pf_pProtectString (@PersonFirstName);
		SET @Email				= dbo.sx_pf_pProtectString (@Email);
		SET @LDAPIP				= dbo.sx_pf_pProtectString (@LDAPIP);
		SET @Status				= dbo.sx_pf_pProtectString (@Status);
		-- case sensitive CODE check
		IF HASHBYTES('SHA1',@DataFlowRightsCODE)		NOT IN (HASHBYTES('SHA1',N'DESIGNER')	,HASHBYTES('SHA1',N'EXECUTOR')	,HASHBYTES('SHA1',N'NONE')) SET @DataFlowRightsCODE			= N'NONE';
		IF HASHBYTES('SHA1',@VisualizationRightsCODE)	NOT IN (HASHBYTES('SHA1',N'DESIGNER')	,HASHBYTES('SHA1',N'VIEWER')	,HASHBYTES('SHA1',N'NONE')) SET @VisualizationRightsCODE	= N'NONE';

		IF @PostUserName = N''
		BEGIN
			SET @ResultCode = 404;
			RAISERROR('Empty input parameters', 16, 10);
		END;	

		-- Don`t allow creation of an User "SQL" - this is a reserverd word
		IF @PostUserName = N'SQL'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('A username "SQL" is not an valid username.', 16, 10);
		END;

		-- Determine transaction user, the transactuser 'public' is not allowed to change rights
		SELECT @TransactUsername = dbo.sx_pf_Determine_TransactionUsername (@Username);

		IF @TransactUsername  = N'403'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('Transaction user don`t exists', 16, 10);
		END 
		ELSE IF @TransactUsername = N'public'
		BEGIN
			SET @ResultCode = 403;
			RAISERROR('User "public" is not allowed to change rights', 16, 10);
		END;

		-- Only OCT Administrators can manage users
		IF NOT EXISTS
			(SELECT 
				Userkey
			FROM system.trUser
			WHERE 
					Username			= @TransactUsername 
				AND IsAdministratorFlag	= 1
				AND Status				= N'Active'
			)
		BEGIN
			SET @Message = CONCAT('Invalid rights, User "',@Transactusername,'" is not an OCT Administrator.');
			RAISERROR(@Message, 16, 10);
		END;

		-- The Clusteradmin can`t set himself to Inactive or remove his designer rights
		IF @TransactUsername = @PostUsername 
			BEGIN
				SET @Status						= N'Active';
				SET @DataFlowRightsCODE			= N'DESIGNER';
				SET @VisualizationRightsCODE	= N'DESIGNER';
				SET @IsAdministratorFlag		= 1;
			END;

		-- Administrators have always full rights
		IF @IsAdministratorFlag = 1
			BEGIN
				SET @Status						= N'Active';
				SET @DataFlowRightsCODE			= N'DESIGNER';
				SET @VisualizationRightsCODE	= N'DESIGNER';
			END;

		-- Get keys
		SELECT @PostUserKey = UserKey FROM system.trUser WHERE Username = @PostUserName;
				
		-- If user does not yet exist, it will be created
		IF @PostUserKey = 0
			BEGIN
				INSERT INTO system.trUser (UserName, PersonSurname, PersonFirstName, Email, LDAPIP, Status,DataFlowRightsCODE,VisualizationRightsCODE,IsAdministratorFlag) 
				OUTPUT INSERTED.UserKey INTO @tmpUserTable 
				VALUES (@PostUserName, '', '', '', '', '','','',0);

				SET @EffectedRows = 1;
				
				-- the determination on the table variable is necessary because of TRIGGERS on system.trUser
				SELECT TOP (1) @PostUserKey = UserKey FROM @tmpUserTable ORDER BY UserKey;
						
				SET @Resultcode = 201;
			END
		ELSE 
			-- existing User case
			BEGIN
				SET @ResultCode = 200;
			END

		-- Update the User with the commited attributes
		UPDATE system.trUser
		SET
			 PersonSurname				= IIF(@PersonSurname	= N'<#NV>', PersonSurname,		@PersonSurname)
			,PersonFirstName			= IIF(@PersonFirstName	= N'<#NV>', PersonFirstName,	@PersonFirstName)
			,Email						= IIF(@Email			= N'<#NV>', Email,				@Email)
			,LDAPIP						= IIF(@LDAPIP			= N'<#NV>', LDAPIP,				@LDAPIP)
			,Status						= IIF(@Status			= N'<#NV>', Status,				@Status)
			,DataFlowRightsCODE			= @DataFlowRightsCODE 
			,VisualizationRightsCODE	= @VisualizationRightsCODE	
			,IsAdministratorFlag		= @IsAdministratorFlag
		WHERE UserKey = @PostUserKey;

		SET @EffectedRows += @@ROWCOUNT;

		-- Administrators get full DataEntry Rights
		IF @IsAdministratorFlag = 1
			BEGIN
				DELETE FROM system.trRights WHERE UserName = @PostUserName;
				INSERT INTO system.trRights VALUES (@PostUserKey,@PostUserName,'','','','Write',0,0)
			END;

		SET @ResultCode = 200;

		COMMIT TRANSACTION spPOST_User;
	END TRY
	BEGIN CATCH
		DECLARE @Error_state INT = ERROR_STATE();
		SET @Comment = ERROR_MESSAGE();

		ROLLBACK TRANSACTION spPOST_User;		

		IF @Error_state <> 10 
		BEGIN
			SET @ResultCode = 500;		
			PRINT 'Rollback due to not executable command.';
		END
		ELSE IF @ResultCode IS NULL OR @ResultCode/100 = 2
		BEGIN
			SET @ResultCode = 500;	
		END;
	END CATCH

	EXEC dbo.sx_pf_pPOST_API_LogEntry @Username, @TransactUsername, @ProcedureName, @ParameterString, @EffectedRows, @ResultCode, @TimestampCall, @Comment;
	RETURN @ResultCode;
END

GO
 
-- GRANT
GRANT EXECUTE ON OBJECT ::system.spPOST_User TO pf_PlanningFactoryService;

GO
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'							-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spPOST_User'						-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET Procedure documentation
    SET @value = N'POST Operation for a User
         Not existing User will be created, existing one updated
         If attributes passed  with value <#NV>, it will be ignored during the update and maintain the existing value, during insert the attribute is created empty';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @level2name = N'@Username';
    SET @value      = N'Username';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PostUserName';
    SET @value      = N'PostUserName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PersonSurname';
    SET @value      = N'PersonSurname';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@PersonFirstName';
    SET @value      = N'PersonFirstName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Email';
    SET @value      = N'Email';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@LDAPIP';
    SET @value      = N'LDAPIP';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'@Status';
    SET @value      = N'Status';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@DataFlowRightsCODE';
    SET @value      = N'CODE Word for the DataFlow Rights of this user - DESIGNER, EXECUTOR, NONE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@VisualizationRightsCODE';
    SET @value      = N'CODE Word for the Visulalitzation Rights of this user - DESIGNER, VIEWER, NONE';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'@IsAdministratorFlag';
    SET @value      = N'Flag 0/1 so set this user as an Administrator of this database.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
GO
