﻿DROP PROCEDURE IF EXISTS system.spMaterialize_trUserRights ;
GO

/*
	Procedure for materialization of UserRights
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.8

	CUSTOM: Performanceverbesserung durch temporäre Tabelle in Schritt 2a

	Testcall
		EXEC system.spMaterialize_trUserRights
		SELECT * FROM system.trUserRights order by Username, FactoryID, ProductLineID
 
	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spMaterialize_trUserRights ',NULL,NULL)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'PROCEDURE', 'spMaterialize_trUserRights ','PARAMETER',NULL)
*/
 
CREATE PROCEDURE system.spMaterialize_trUserRights 
AS 

BEGIN
	BEGIN TRY
		BEGIN TRANSACTION spMaterialize_trUserRights


		-- Table to store the calculated Rights
		IF OBJECT_ID('tempdb..#RightsTemp') IS NOT NULL
		DROP TABLE #RightsTemp;
		
		CREATE TABLE #RightsTemp 
		(
			 Source					NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL							
			,UserKey				BIGINT									NOT NULL
			,Username				NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,FactoryKey				BIGINT									NOT NULL
			,ProductLineKey			BIGINT									NOT NULL
			,FactoryID				NVARCHAR (255) COLLATE DATABASE_DEFAULT NOT NULL
			,ProductLineID			NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightExplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,[RightImplicit]		NVARCHAR (255) COLLATE DATABASE_DEFAULT	NOT NULL
			,ReadCommentMandatory	INT 									NOT NULL
			,WriteCommentMandatory	INT										NOT NULL
		);

		-- Concept for saving only positive Rights, no Denys

		-- ClusterLevel			********************************************************************************************************************
		-- this are Rights for FactoryID = '' AND ProductLineID = ''

		-- Clusteradmins		*****************************
		-- Has Write Rights on Clusterlevel, no limitation possible
		-- Insert Write Rights on ClusterLevel
		-- Insert Write Rights on FactoryLevel
		-- Insert Write Rights on ProductLine Level

		-- ClusterReader		******************************
		-- Has Read Rights on Clusterlevel, may have additional Write Rights on Factory / ProductLine Level
		-- Insert Read Rights on ClusterLevel
		-- Insert Read Rights on FactoryLevel
		-- Insert Read Rights on ProductLine Level

		-- FactoryLevel			********************************************************************************************************************
		-- Rights defined on FactoryLevel with FactoryID <> 0 AND ProductLineID = ''
		-- Write Rights on Factories 
		-- Has Write Rights on a Factory - no limitation under this Factory possible
		-- Read Rights on Factories
		-- Has Read Rights on Factory Level, may have additional Write Rights on ProductLineLevel

		-- ProductLineLevel		*************************************************************************
		-- Rights defined on ProductLineLevel with FactoryID <> 0 AND ProductLineID <> 0
		-- Write Rights on ProductLines
		-- Read Rights on ProductLines

		-- Preparation			**************************************************************************
		-- Create Table with all rows, this are:
			-- for User with Rights on ClusterLevel		Linecount = 1 + FactoryCount				+ PLCount 
			-- for User with Rights on FactoryLevel		Linecount = 1 + FactoryWithRightCount		+ PLCount of FactoriesWith Rights
			-- for User with Rights on ProductLineLevel Linecount = 1 + FactoryCountPLWithRights	+ PLCount with Rights

		-- Helper Table with Stucture   *******************************************************************
		IF OBJECT_ID('tempdb..#FullStructure') IS NOT NULL
		DROP TABLE #FullStructure;
				CREATE TABLE #FullStructure
						(
							 RowKey			BIGINT	IDENTITY (1,1)								NOT NULL
							,[Level]		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,FactoryKey		BIGINT												NOT NULL
							,ProductLineKey BIGINT												NOT NULL
							,FactoryID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductLineID	NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
						);

				-- C Level
				INSERT INTO #FullStructure
					SELECT 
						 'Cluster'	AS Level
						,0			AS FactoryKey
						,0			AS ProductLineKey
						,''			AS FactoryID
						,''			ASProductLineID;
			 
				-- F Level
				INSERT INTO #FullStructure
					SELECT
						 'Factory'		AS Level
						,dF.FactoryKey
						,0				AS ProductLineKey
						,dF.FactoryID
						,''				AS ProductLineID
					FROM planning.tdFactories dF;
				
				-- PL Level
				INSERT INTO #FullStructure
					SELECT
						 'ProductLine'	AS Level
						,dF.FactoryKey
						,dPL.ProductLineKey
						,dF.FactoryID
						,dPL.ProductLineID
					FROM planning.tdProductLines dPL 
						LEFT JOIN planning.tdFactories dF
							ON dPL.FactoryKey = dF.FactoryKey;

		-- SELECT * FROM #FullStructure

		-- Fill UserRight Table with fitting RowCount and set all to read ************************************************************************

		-- 1a. The Full Structure Rows for User with Clusterrights  -> no additional Rows for Rights on Factory / ProductLine Level need
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ClusterRight' AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1				AS ReadCommentMandatory
					,1				AS WriteCommentMandatory
			FROM #FullStructure fs 
				LEFT JOIN system.trUser rU
					ON 1=1
			WHERE 
					rU.Status = 'Active'
				-- Clusterrights
				AND rU.UserKey IN (SELECT UserKey FROM system.trRights WHERE FactoryID = '' AND ProductLineID = '');

		-- 1b. Cluster Read right Row for Transfer for alle user with any right except the already done users with clusterrights
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-TransferCluser'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'				AS [RightExplicit]
					,'Read'				AS [RightImplizit]
					,1					AS ReadCommentMandatory
					,1					AS WriteCommentMandatory
			FROM #FullStructure fs
				
				LEFT JOIN system.trUser rU
					ON 1=1

			WHERE 
					rU.Status = 'Active'
				AND fs.Level = 'Cluster'
				-- anybody who has any set right 
				AND rU.UserKey IN (SELECT DISTINCT UserKey FROM system.trRights)
				-- and is not already ClusterRight Owner
				AND rU.Userkey NOT IN (SELECT DISTINCT UserKey FROM system.trRights WHERE FactoryID = '');
			
		-- 1c. Rows for User with direct Factory Rights 
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-FactoryRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID FROM system.trRights WHERE FactoryID <> '' AND ProductLineID = ''
					) rFactoryDirect
					
					ON FS.FactoryID = rFactoryDirect.FactoryID COLLATE DATABASE_DEFAULT
				
				INNER JOIN system.trUser rU
					ON rFactoryDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					AND rFactoryDirect.FactoryID IS NOT NULL
					-- and is not already ClusterRight Owner
					AND rU.UserKey NOT IN (SELECT DISTINCT UserKey FROM system.trRights WHERE FactoryID = '');

		--1d. Rows for Factory Transfer Rights for User with direct ProduclineRights Rights (but not already direct Factory / Clusterrights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-Transfer Factory'	AS [Source]
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM system.trRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID  COLLATE DATABASE_DEFAULT-- Join only over Factory
				
				INNER JOIN system.trUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status	= 'Active'
					AND fs.Level	= 'Factory'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM system.trRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM system.trRights WHERE FactoryID = '' )

			GROUP BY 
                 rU.UserKey
				,rU.UserName
				,fs.FactoryKey
				,fs.ProductLineKey
				,fs.FactoryID
				,fs.ProductLineID;

		--1e. Rows Rights for User with direct ProduclineRights Rights (which not already have this due to FactoryRights)
		INSERT INTO #RightsTemp
			SELECT  
					 'Row-ProductLineRights'	AS Source
					,rU.UserKey
					,rU.UserName
					,fs.FactoryKey
					,fs.ProductLineKey
					,fs.FactoryID
					,fs.ProductLineID
					,'-'			AS [RightExplicit]
					,'Read'			AS [RightImplizit]
					,1 AS ReadCommentMandatory
					,1 AS WriteCommentMandatory
			FROM #FullStructure fs
				
				INNER JOIN 
					(
						SELECT Userkey,FactoryID,ProductLineID FROM system.trRights WHERE FactoryID <> '' AND ProductLineID <> ''
					) rPLDirect
					
					ON		FS.FactoryID = rPLDirect.FactoryID COLLATE DATABASE_DEFAULT
						AND FS.ProductLineID = rPLDirect.ProductLineID COLLATE DATABASE_DEFAULT
				
				INNER JOIN system.trUser rU
					ON rPLDirect.UserKey = rU.UserKey

				WHERE 
						rU.Status = 'Active'
					-- exclude direct Factory Rights
					AND CAST(rU.UserKey AS NVARCHAR(255)) +'_'+fs.FactoryID COLLATE DATABASE_DEFAULT NOT IN 
									(
										SELECT CAST(UserKey AS NVARCHAR(255)) +'_'+FactoryID COLLATE DATABASE_DEFAULT FROM system.trRights WHERE FactoryID <> '' AND ProductLineID = ''
									)
					-- exclude ClusterRight owner
					AND rU.UserKey NOT IN (SELECT Userkey FROM system.trRights WHERE FactoryID = '' );

			--SELECT * FROM #RightsTemp rt ORDER BY Username,FactoryID,rt.ProductLineID;

	

			--2. Evaluate Rights to give write Rights and set Commentproperties#########################################################################

			-- 2a. SET Clusterrights
			DROP TABLE IF EXISTS  #Rights;
				CREATE TABLE #Rights
						(
							 RightKey		INT													NOT NULL
							,UserKey		INT													NOT NULL
							,UserName		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,FactoryID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductLineID	NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ProductID		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,[Right]		NVARCHAR (255)			COLLATE DATABASE_DEFAULT	NOT NULL
							,ReadCommentMandatory		INT										NOT NULL
							,WriteCommentMandatory		INT										NOT NULL
						);


				INSERT INTO #Rights
					SELECT * 
					FROM system.trRights 

			UPDATE #RightsTemp 
		
			SET	 [Source]				= 'ClusterRight'		
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt 
					INNER JOIN 	#Rights rR
						ON rt.UserKey = rR.UserKey
					WHERE	rR.FactoryID = '' 
						AND rR.ProductLineID = '';

			-- 2a. Set Factory Rights with downwards inheritance
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'FactoryRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt 
					INNER JOIN #Rights rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT
					WHERE	rR.FactoryID <> '' 
						AND rR.ProductLineID = '';

			-- 2a. Set ProductLine Rights 
			UPDATE #RightsTemp 
		
			SET  
				 [Source]				= 'ProductLineRight'
				,RightExplicit			= rR.[Right] 
				,ReadCommentMandatory	= rR.ReadCommentMandatory
				,WriteCommentMandatory	= rR.WriteCommentMandatory
				
					FROM #RightsTemp rt 
					INNER JOIN #Rights rR
						ON		rt.UserKey = rR.UserKey 
							AND rt.FactoryID = rR.FactoryID COLLATE DATABASE_DEFAULT
							AND rt.ProductLineID = rR.ProductLineID COLLATE DATABASE_DEFAULT
					WHERE	rR.FactoryID <> '' 
						AND rR.ProductLineID <> '';
					
		
			--SELECT * FROM	#RightsTemp ORDER BY Username,FactoryID,ProductLineID


			-- OUTPUT
			-- TRUNCATE TABLE system.trUserRights -- deactivated, because a 
			/*	The minimum permission required is ALTER on table_name. TRUNCATE TABLE permissions default to the table owner,
				members of the sysadmin fixed server role, and the db_owner and db_ddladmin fixed database roles, 
				and are not transferable. However, you can incorporate the TRUNCATE TABLE statement within a module,
				such as a stored procedure, and grant appropriate permissions to the module using the EXECUTE AS clause.
			
				The EXECUTE AS Clause I did not master till now in foreign environments. 
			 */
			DELETE FROM system.trUserRights; 

			INSERT INTO system.trUserRights 
			SELECT 
				 UserKey
				,Username
				,FactoryID
				,ProductLineID
				,IIF(RightExplicit='-',RightImplicit,RightExplicit) AS [Right]
				,ReadCommentMandatory
				,WriteCommentMandatory

			FROM #RightsTemp

			COMMIT TRANSACTION spMaterialize_trUserRights
		END TRY
		BEGIN CATCH
			Print 'Da ging was schief'
			ROLLBACK TRANSACTION spMaterialize_trUserRights
		END CATCH
	END
GO

GRANT EXECUTE ON OBJECT :: system.spMaterialize_trUserRights TO pf_PlanningFactoryService;

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)  = N'system'					-- enter schema name of the table
    ,@level1name        NVARCHAR(255)  = N'spMaterialize_trUserRights '				-- enter procedure name
    ,@SX_Owner	        NVARCHAR(255)  = N'OCT.core'						-- enter owner name of the procedure from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)  = N'CORE'							-- enter module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag	INT            = 1									-- 0 = Demo object - out of shipment process
																			
																			-- STANDARD OBJECTS
																			-- 1 = shiped from saxess standard without modification
																			-- 2 = shiped from saxess standard modified FOR customer from saxess
																			-- 3 = shiped from saxess standard modified FOR customer from partner
																			-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																			-- CUSTOM OBJECTS
																			-- 10 = shiped from saxess as customer specific object
																			-- 11 = shiped from partner	as customer specific object
																			-- 12 = shiped from customer as own specific object

	,@SX_UserHint		NVARCHAR(2000) = N''								-- optional, fill if Procedure shall be offerend for end user (e.g. for Pivot / Datagrid usage)
                                                             
 
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'PROCEDURE'
    ,@level2type    NVARCHAR(255)   = N'PARAMETER'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty N'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty N'SX_Module'		,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty N'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;

	SET @value = @SX_UserHint;
    EXEC sys.sp_addextendedproperty N'SX_UserHint'		,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
 
-- SET Procedure documentation
    SET @value = N'Procedure to materialize effectiv Userrights on all levels from trUser and trRights, which save only hightest level.
	Procedure will be triggered from any action on tables trUser and trRights.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
GO