 DROP TABLE IF EXISTS control.tCustomFailureReport; 
 GO
/*
	Table for failure reporting
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM control.tCustomFailureReport

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tCustomFailureReport',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'control', 'TABLE', 'tCustomFailureReport','COLUMN',default)
*/

CREATE TABLE control.tCustomFailureReport
(
	 RowKey				BIGINT IDENTITY (1,1) 
	,Timestamp			DATETIME		NOT NULL
	,FactoryID			NVARCHAR(255)	NOT NULL
	,ProductLineID		NVARCHAR(255)	NOT NULL
	,ProductID			NVARCHAR(255)	NOT NULL
	,TestSource			NVARCHAR(255)	NOT NULL
	,TestName			NVARCHAR(255)	NOT NULL
	,ResultType			NVARCHAR(255)	NOT NULL -- OK, ERROR, WARNING
	,ResultText			NVARCHAR(MAX)	NOT NULL
	,ResultNumeric		MONEY			NOT NULL

	,CONSTRAINT PK_tCustomFailureReport PRIMARY KEY NONCLUSTERED (RowKey)
)

GO

-- Table is clustered by Timestamp for faster query the last failure report
CREATE CLUSTERED INDEX IX_tCustomFailureReport_TimeFID ON control.tCustomFailureReport (Timestamp DESC, FactoryID ASC);

GO 

-- Hint Entry on Startup - will be deleted due to self cleaning in control procedure
INSERT INTO control.tCustomFailureReport 
	SELECT
		 GETUTCDATE() - 14	AS Timestamp
		,'AllFID'			AS FactoryID
		,'AllPLID'			AS ProductLineID
		,'AllPID'			AS ProductID
		,'Startup'			AS TestSoure
		,'Startup Hint'		AS TestName
		,'HINT'				AS ResultType
		,'Execute (everyday by agent) Stored Procedure control.spCustomFailureReport to see test results here.'	AS ResultText
		,0					AS ResultNumeric

GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT control.tCustomFailureReport TO [Username or rolename]
 
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'control'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCustomFailureReport'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'				-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'					-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1							-- 0 = Demo object - out of shipment process
																	-- STANDARD OBJECTS
																	-- 1 = shiped from saxess standard without modification
																	-- 2 = shiped from saxess standard modified FOR customer from saxess
																	-- 3 = shiped from saxess standard modified FOR customer from partner
																	-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																	-- CUSTOM OBJECTS
																	-- 10 = shiped from saxess as customer specific object
																	-- 11 = shiped from partner	as customer specific object
																	-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table for failure reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Timestamp';
    SET @value      = N'The timestamp of the failure.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FactoryID';
    SET @value      = N'Business Key for the factory.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProductLineID';
    SET @value      = N'Business Key for the productline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ProductID';
    SET @value      = N'Business key for the product.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TestSource';
    SET @value      = N'The test source.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'TestName';
    SET @value      = N'The test name.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultType';
    SET @value      = N'The result type of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ResultText';
    SET @value      = N'The content of the failure report.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ResultNumeric';
    SET @value      = N'The numeric result.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
