DROP TABLE IF EXISTS global.tCodes;
GO

/*
	Table to store informations about CODEs
	these are Upper Letter Keywords to define domains or static keywords for the code which will be translated into languages later in result.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tCodes

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCodes',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tCodes','COLUMN',default)
*/

CREATE TABLE global.tCodes (
    RowKey              BIGINT        IDENTITY(1,1)     NOT NULL     
	,ModuleID			NVARCHAR(255)					NOT NULL
	,LanguageID			NVARCHAR(10)                    NOT NULL 
    ,CODE               NVARCHAR(50)                    NOT NULL
	,Scope				NVARCHAR(255)					NOT NULL
    ,[Description]		NVARCHAR(255)                   NOT NULL
	
    ,CONSTRAINT PK_global_tCodes PRIMARY KEY CLUSTERED (RowKey)
);

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tCodes TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tCodes'         -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';

    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
  
 	SET @value = N'Table to store informations about CODEs - these are Upper Letter Keywords to define domains or static keywords for the code which will be translated into languages later in result.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;

-- Extended property Default Content Flag
	SET @name		= N'IsDefaultContent';
	SET @value		= N'1';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
	SET @name = N'MS_Description';

 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value		= N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value		= N'ID for the Modul which created this Code. Part of combined Business Key ModuleID, LanguageID, CODE. For System Codes use ''system'' as ModuleID.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LanguageID';
    SET @value		= N'Language identifier for the code description. Part of combined Business Key ModuleID, LanguageID, CODE. Identifiers are stored in ISO 639-1 standard (e.g. DE, EN, FR, ...).';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'CODE';
    SET @value		= N'Code written in uppercase letters, the CODE must be unique for the modul. Part of combined Business Key ModuleID, LanguageID, CODE.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Scope';
    SET @value		= N'Scope for the CODE - can be used to allocate Codes to groups or objects. Use ''default'' if no scope is needed.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	       
	SET @level2name = N'Description';
    SET @value		= N'Description of CODE in specific language.';
	EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO

