DROP TABLE IF EXISTS global.tPeriods;
PRINT N'Creating table global.tPeriods'
GO
/*
	Table to store informations about time periods.
	Each Year contains 16 Accounting periods, the periods 12...16 are mapped to month 12
	Each Year can be a Financal Year starting with calender month 1 (Financal Year = Calender Year),
	or an Financial Year starting in calender month <> 1.
	 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 06/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM global.tPeriods;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tPeriods',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'global', 'TABLE', 'tPeriods','COLUMN',default)
*/

CREATE TABLE global.tPeriods (
     RowKey                   INT        IDENTITY(1,1)     NOT NULL
	,PeriodKey  AS 
		  (FinancialYear * 10000) 
		+ (FinancialYearBegin * 100) 
		+ [Period]
    ,FinancialYear            INT							NOT NULL -- 2023
    ,[Period]                 INT							NOT NULL -- 1..16
    ,FinancialYearBegin       INT							NOT NULL -- 1..12
    ,FinancialYearName        NVARCHAR(50)					NOT NULL -- 2023 or 2023/2024	 
    ,CalendarYear             INT							NOT NULL -- 2023
    ,CalendarMonth            INT							NOT NULL -- 1..12
    ,CalendarQuarterCODE      NVARCHAR(50)					NOT NULL -- Q1...Q4
    ,CalendarHalfYearCODE     NVARCHAR(50)					NOT NULL -- HY1, HY2
	,YYYYMM					  INT							NOT NULL -- 202301
	,[YYYY-MM]				  NVARCHAR(50)					NOT NULL -- 2023-01						
	,YYYYQ					  INT							NOT NULL -- 20231...20234
	,[YYYY QQ]				  NVARCHAR(50)					NOT NULL --	2023 Q1..2023 Q4
    
	,CONSTRAINT PK_global_tPeriods PRIMARY KEY CLUSTERED (RowKey)
);

-- Default Content Flag
EXEC sp_addextendedproperty
    @name		= N'IsDefaultContent',
    @value		= N'1',
    @level0type = N'Schema',
    @level0name = N'global',
    @level1type = N'Table',
    @level1name = N'tPeriods'
GO

-- GRANT Rights (only need if end user need direct database access to this object)
-- GRANT SELECT ON OBJECT global.tPeriods TO [Username or rolename]
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'global'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPeriods'       -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store informations about accounting time periods. Each Finanancial Year contains 16 periods.
				Financial years are defined for each possible start monat. FinancialYearBegin = 1 describes the Financial Year which is equal the calender year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'RowKey';
    SET @value      = N'Technical Primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PeriodKey';
    SET @value      = N'Key to identify a period, its automaticly calculated as (FinancialYear * 10000) + (FinancialYearBegin * 100) + [Period].';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FinancialYear';
    SET @value      = N'The financial Year YYYY';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Period';
    SET @value      = N'The periods 1..16.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FinancialYearBegin';
    SET @value      = N'Number indicating the beginning of the financial year. 1.. 12';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'FinancialYearName';
    SET @value      = N'The name of the financial year. 2023 or 2023/2024';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CalendarYear';
    SET @value      = N'The calendar year.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CalendarMonth';
    SET @value      = N'The calendar month.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'CalendarQuarterCODE';
    SET @value      = N'The CODE for the calendar quarter. Q1..Q4';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'CalendarHalfYearCODE';
    SET @value      = N'The CODE for the calendar half year. HY1..HY2';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYYMM';
    SET @value      = N'Year and Month formated as YYYYMM';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYY-MM';
    SET @value      = N'Year and Month formated as YYYY-MM, mostly used for print reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYYQ';
    SET @value      = N'Year and Quarter formated as YYYYQ';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'YYYY QQ';
    SET @value      = N'Year and Quarter formated as YYYY QQ, mostly used for print reporting.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
