
DROP TABLE IF EXISTS planning.thFormats;
GO
/*
	Table for format definitions. They are applied on:
		- ValueSeries
		- TimeIDs (only borders at the moment)
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM planning.thFormats

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thFormats',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'planning', 'TABLE', 'thFormats','COLUMN',default)
*/

CREATE TABLE planning.thFormats 

	(
	 FormatKey				INT				NOT NULL IDENTITY (1,1)
	,FormatID				NVARCHAR(255)	NOT NULL
	,BackgroundColor		NVARCHAR(255)	NOT NULL
	,FontColor				NVARCHAR(255)	NOT NULL
	,ValueFormat			NVARCHAR(255)	NOT NULL
	,Bold					INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,Italic					INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,AlignmentHorizontal	NVARCHAR(255)	NOT NULL -- left, center, right
	,AlignmentVertical		NVARCHAR(255)	NOT NULL -- top, center, bottom
	,TextUnderlined			INT				NOT NULL -- TryState -1,0,1 = Not set, false, true
	,BorderToNext			INT				NOT NULL -- Boolean
	,BorderToPrevious		INT				NOT NULL -- Boolean

	,CONSTRAINT PK_hFormats_PK PRIMARY KEY NONCLUSTERED (FormatKey)
	);

CREATE CLUSTERED INDEX IX_hFormats_FormatID ON planning.thFormats (FormatID ASC);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'planning'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'thFormats'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store Format Definitions, which are applied on ValueSeries, GlobalAttributes and TimeIDs.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'FormatKey';
    SET @value      = N'Technical Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FormatID';
    SET @value      = N'Business Primary Key.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'BackgroundColor';
    SET @value      = N'RGB Color Definition, which is applied for Background eg. 233,233,233.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'FontColor';
    SET @value      = N'RGB Color Definition, which is applied for Fonts eg. 233,233,233.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'ValueFormat';
    SET @value      = N'Format Definition in englisch XLS Style, which is applied to format values, e.g. #,###0.00 for Thousand Separator and two decimal places.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Bold';
    SET @value      = N'Font is bold Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'Italic';
    SET @value      = N'Font is italic Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'AlignmentHorizontal';
    SET @value      = N'Definition of horizontal font alignment, allowed values are left, center, right.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'AlignmentVertical';
    SET @value      = N'Definition of vertical font alignment, allowed values are top, center, bottom.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TextUnderlined';
    SET @value      = N'Font is underlined Definition, as try state -1,0,1 = Not set, false, true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'BorderToNext';
    SET @value      = N'Definition Flag if a border to the next (ValueSeries, TimeID) is painted, allowed values are 0 = false, 1 = true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'BorderToPrevious';
    SET @value      = N'Definition Flag if a border to the previous (ValueSeries, TimeID) is painted, allowed values are 0 = false, 1 = true.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
