DROP TABLE IF EXISTS system.tConnectorQueries;

/*
	Table to store connector queries.
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tConnectorQueries

		INSERT INTO system.tConnectorQueries VALUES
			 (1,'GetCompanies','SELECT 1 AS CompanyID, ''Company A'' AS CompanyName, 1 AS FinancialYearBegin',1,'',GETUTCDATE(),ORIGINAL_LOGIN()),
			 (1,'Accounts','SELECT 1 AS CompanyID, ''SKTO'' AS AccountGroupCODE, ''1234'' AS AccountID, ''BILANZ'' AS AccoutTypeCode, ''Account 1234'' AS AccountName, ''{}'' AS CustomValuesJSON',1,'',GETUTCDATE(),ORIGINAL_LOGIN())

		DELETE FROM system.tConnectorQueries WHERE ConnectorQueryKey = 1;

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectorQueries',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tConnectorQueries','COLUMN',default)
*/

CREATE TABLE system.tConnectorQueries
    (
       ConnectorQueryKey   INT            IDENTITY(1, 1) NOT NULL
      ,ConnectorKey        INT            NOT NULL
      ,QueryName           NVARCHAR(255)  NOT NULL
      ,QuerySQL            NVARCHAR(MAX)  NOT NULL
      ,QueryVersion        INT            NOT NULL
      ,QueryVersionComment NVARCHAR(4000) NOT NULL
      ,Timestamp           DATETIME       NOT NULL
      ,CreatedBy           NVARCHAR(255)  NOT NULL
       
	  ,CONSTRAINT PK_tConnectorQueries_ConnectorQueryKey PRIMARY KEY CLUSTERED (ConnectorQueryKey)
      ,CONSTRAINT FK_tConnectorQueries_ConnectorKey
            FOREIGN KEY (ConnectorKey)
            REFERENCES system.tConnectors (ConnectorKey)
			ON DELETE CASCADE
			ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
    );

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tConnectorQueries'	-- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'			-- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'				-- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1						-- 0 = Demo object - out of shipment process
																-- STANDARD OBJECTS
																-- 1 = shiped from saxess standard without modification
																-- 2 = shiped from saxess standard modified FOR customer from saxess
																-- 3 = shiped from saxess standard modified FOR customer from partner
																-- 4 = shiped from saxess standard modified FROM customer themself for own needs
																-- CUSTOM OBJECTS
																-- 10 = shiped from saxess as customer specific object
																-- 11 = shiped from partner	as customer specific object
																-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store connectors.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'ConnectorQueryKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ConnectorKey';
    SET @value      = N'Foreign Key to system.tConnectors.ConnectorKey.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'QueryName';
    SET @value      = N'Name of the query which also determines the target integration table.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'QuerySQL';
    SET @value      = N'SQL query which is executed by the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'QueryVersion';
    SET @value      = N'Version of the query. Default = 1. If the query is edited the version goes up in a separate entry. Only MAX(Version) is used by the connector.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
	SET @level2name = N'QueryVersionComment';
    SET @value      = N'If query is edited it should include a comment about what was changed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'Timestamp';
    SET @value      = N'Timestamp when the query was created.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	
	SET @level2name = N'CreatedBy';
    SET @value      = N'User who created or edited the query.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
