
-- Foreign Key dependend tables must be deleted first, disabling constraints does not allow drop / delete or truncate
DROP TABLE IF EXISTS system.tPipelineStepLog;
 
DROP TABLE IF EXISTS system.tPipelineLog;
GO
/*
	Table for storing pipeline executions. Table materialises all details, to show full informations also in case of deleted / modified pipelines
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineLog;

		DELETE FROM system.tPipelineLog WHERE LogKey = 2; --testing delete cascade

	TestData
		INSERT INTO system.tPipelineLog VALUES
			('1','Pipe','E1','Server','RUNNING','Hase','20221230 08:08:08.888','20221230 08:08:08.888')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineLog',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineLog','COLUMN',default)
*/
 
CREATE TABLE system.tPipelineLog
    (
         LogKey					BIGINT					IDENTITY(1,1)
		,PipelineID				NVARCHAR(50)			NOT NULL
		,PipelineName			NVARCHAR(255)			NOT NULL
		,ExecutionID			NVARCHAR(50)			NOT NULL
		,ExecutionServerName	NVARCHAR(255)			NOT NULL
        ,StatusCODE				NVARCHAR(50)			NOT NULL
		,StartedBy				NVARCHAR(255)			NOT NULL
		,StartTime				DATETIME				NULL
		,EndTime				DATETIME				NULL

		,CONSTRAINT PK_tPipelineLog_LogKey PRIMARY KEY CLUSTERED (LogKey DESC)
		,CONSTRAINT AK_tPipelines_PID_EID  UNIQUE (PipelineID,ExecutionID)
    );

GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineLog'   -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;               -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to Logs per Pipeline Execution. Step Logs are saved separatly in system.StepLog';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineID';
    SET @value      = N'This column just holds a sample text.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'PipelineName';
    SET @value      = N'Name of the pipeline - materialized to be historic correct.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionID';
    SET @value      = N'Generated Key on first call to Kep Pipeline and Steps together.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionServerName';
    SET @value      = N'URL of the Server, which executed this pipeline.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StatusCODE';
    SET @value      = N'CODE of the current pipeline status - may be RUNNING, STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartedBy';
    SET @value      = N'Name of the user who started this pipeline, or the word scheduler if it was scheduler started.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartTime';
    SET @value      = N'Datetime showing start of the pipeline run.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EndTime';
    SET @value      = N'Datetime showing end of the pipeline run.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
