
DROP TABLE IF EXISTS system.tPipelineStepLog;
GO
/*
	Table for storing Log events per Step
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelineStepLog

	Dependend Tables
		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps
		SELECT * FROM system.tPipelineLog

	TestData
		INSERT INTO system.tPipelineStepLog VALUES
			 ('1','1','E1','Step1','20221230 08:08:08.888','20221230 08:08:08.888','Hallo Log','END','Handle2')
			,('1','2','E1','Step1','20221230 08:08:08.888','20221230 08:08:08.888','Hallo Log','END','Handle2')

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineStepLog',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineStepLog','COLUMN',default)
*/
 
CREATE TABLE system.tPipelineStepLog
    (
         LogKey			BIGINT					IDENTITY(1,1)
		,PipelineID		NVARCHAR(50)			NOT NULL
		,StepID			NVARCHAR(50)			NOT NULL
		,ExecutionID	NVARCHAR(50)			NOT NULL
		,StepName		NVARCHAR(255)			NOT NULL
		,StartTime		DATETIME				NULL
		,EndTime		DATETIME				NULL
		,LogMessage		NVARCHAR(MAX)			NOT NULL
		,StatusCODE		NVARCHAR(50)			NOT NULL
		,HandleID		NVARCHAR(50)			NOT NULL

		,CONSTRAINT PK_tPipelineStepLog_LogKey PRIMARY KEY CLUSTERED (LogKey DESC) 
		,CONSTRAINT AK_tPipelineStepLog_StepIDExecutionID UNIQUE (StepID,ExecutionID)
		-- Foreign Key relation to PipelineLog to delete Step Logs, when deleting the PipelineLog ExecutionID
		,CONSTRAINT FK_tPipelineLog_LogKey FOREIGN KEY (PipelineID,ExecutionID)
			REFERENCES system.tPipelineLog (PipelineID,ExecutionID)
			ON DELETE CASCADE
			ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);
 
-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'    -- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineStepLog'          -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1 ;                -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store logs per exectuted step'
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'LogKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineID';
    SET @value      = N'ID of the pipeline this step belongs to.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'StepID';
    SET @value      = N'ID of the step';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ExecutionID';
    SET @value      = N'Relation to the pipeline log.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepName';
    SET @value      = N'Name of the Step to keep history.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StartTime';
    SET @value      = N'Datetime the step was started.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'EndTime';
    SET @value      = N'Datetime the step was finished.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'LogMessage';
    SET @value      = N'Messages from logging.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StatusCODE';
    SET @value      = N'CODE of the current pipeline status - may be RUNNING, WAITING (Pipeline was started and step queued), STOPPED (from User), DIED (computer restarted etc.), FINISHED (succesful), FINISHEDWITHERROR';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'HandleID';
    SET @value      = N'ID for Penatho Job handling.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	GO
	