DROP TABLE IF EXISTS system.tPipelineSteps;
GO
/*
	Table to store pipeline steps

	They have a foreign key constraint to the the system.tPipelinesTable and are deleted automaticly, if the Pipeline is deleted
 
	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 02/2023 for OCT 5.9
 
	Testcall
		SELECT * FROM system.tPipelines
		SELECT * FROM system.tPipelineSteps

		INSERT INTO system.tPipelineSteps VALUES
			 (1,'1','Step 1','Hallo Step',1,'1','{}','{}',30,1,'FIN',0)
			,(1,'2','Step 2','Hallo Step',2,'1','{}','{}',30,1,'FIN',0)
			,(1,'3','Step 3','Hallo Step',3,'1','{}','{}',30,1,'FIN',0)

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSteps',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'TABLE', 'tPipelineSteps','COLUMN',default)
*/

CREATE TABLE system.tPipelineSteps (
     StepKey					INT        IDENTITY(1,1)    NOT NULL		
	,PipelineKey				INT                         NOT NULL
	,StepID						NVARCHAR(50)				NOT NULL
    ,StepName					NVARCHAR(255)               NOT NULL		
	,StepDescription			NVARCHAR(4000)				NOT NULL
	,OrderOfExecution			INT                         NOT NULL
	,StepDataSourceID			NVARCHAR(50)				NOT NULL
	,StepCompanyIDsJSON			NVARCHAR(MAX)  				NOT NULL
	,StepDetailsJSON			NVARCHAR(MAX)  				NOT NULL		
	,TimeExecutionLimit			INT							NOT NULL		
	,Active						BIT							NOT NULL
	,ModuleID					NVARCHAR(255)				NOT NULL
	,StartNextStepOnError		BIT							NOT NULL
	-- Primary Key
    ,CONSTRAINT PK_tPipelineSteps_StepKey PRIMARY KEY CLUSTERED (StepKey)
	-- Foreign Key to system.tPipelines (with Delete Cascade)
	,CONSTRAINT AK_tPipelineSteps_PipelineKeyStepID  UNIQUE (PipelineKey,StepID)
	,CONSTRAINT FK_tPipelines_PipelineKey FOREIGN KEY (PipelineKey)
        REFERENCES system.tPipelines (PipelineKey)
        ON DELETE CASCADE
		ON UPDATE CASCADE -- this never happens, as the Key is an Identity column (?)
	);

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'			-- enter schema name of the table
    ,@level1name        NVARCHAR(255)   = N'tPipelineSteps'  -- enter table name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the table from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'TABLE'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET documententation             *************************************************************************
 
-- SET table documentation
    SET @value = N'Table to store process steps.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
    SET @level2name = N'StepKey';
    SET @value      = N'Technical primary Key for row identity.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'PipelineKey';
    SET @value      = N'Foreign key to tPipelines.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'StepName';
    SET @value      = N'Name of the Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'StepDescription';
    SET @value      = N'Description of the Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

    SET @level2name = N'OrderOfExecution';
    SET @value      = N'Stores at which position of the process pipeline the step should be executed.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepDataSourceID';
    SET @value      = N'ID of the Datasource which is used from this Step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'StepCompanyIDsJSON';
    SET @value      = N'JSON Array with the IDs of the companies activated for this step.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
	 
    SET @level2name = N'StepDetailsJSON';
    SET @value      = N'The details of this step, content depending from the step type CODE.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'TimeExecutionLimit';
    SET @value      = N'The time limit for the execution in minutes, step is skiped after this limit, use 0 for no limit.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'Active';
    SET @value      = N'Flag if the step is active.';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	SET @level2name = N'ModuleID';
    SET @value      = N'ID of the Module, if the Step has the property module, else empty string';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;

	GO
