
DROP VIEW IF  EXISTS system.vLogEntrys;
GO

/*
	View for analysing the LogEntrys of PlanningFactory API 
		- shows the execution time per LogEntry in Milliseconds and Seconds
	Only for use by administrators, as the log shows all activities of all users

	Gerd Tautenhahn for Saxess Software GmbH
	Last modified: 10/2022 for OCT 5.9
 
	Testcall
		SELECT * FROM system.vLogEntrys

	Testcall Documentation
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'VIEW', 'vLogEntrys',NULL,default)
		UNION ALL
		SELECT * FROM ::fn_listextendedproperty (NULL, 'SCHEMA', 'system', 'VIEW', 'vLogEntrys','COLUMN',default);
*/

CREATE VIEW system.vLogEntrys 

AS 

	SELECT 
		 @@Servername									AS Servername
		,DB_NAME()										AS ClusterName
		,SQLSystemUser
		,Username										AS ServiceUsername
		,TransactUsername
		,ProcedureName
		,ParameterString
		,EffectedRows
		,ReturnCode
		,TimestampCall
		,TimestampReturn
		,CASE
			WHEN DATEDIFF(SECOND,TimestampCall,TimestampReturn) < 10 THEN
				DATEDIFF(MILLISECOND,TimestampCall,TimestampReturn)
			ELSE
				10000 -- to avoid to big numbers
		 END											AS DurationInMilliseconds
		,DATEDIFF(SECOND,TimestampCall,TimestampReturn) AS DurationInSeconds
		,Comment
		,ProcessCode
	FROM 
		system.tAPI_Log;

GO

GRANT SELECT ON OBJECT::system.vLogEntrys TO pf_PlanningFactoryService;
GO

-- SET documentation variables      ***********************************************************************
DECLARE
     @level0name        NVARCHAR(255)   = N'system'				-- enter schema name of the view
    ,@level1name        NVARCHAR(255)   = N'vLogEntrys'   -- enter view name
    ,@SX_Owner			NVARCHAR(255)	= N'OCT.core'       -- enter owner name of the view from list (OCT.core, OCT.modules, Custom)
    ,@SX_Module			NVARCHAR(255)	= N'CORE'           -- module name as free text  (CORE,FIN, DEBKRED, HR, ...)
    ,@SX_ShipmentFlag   INT             = 1                 -- 0 = Demo object - out of shipment process
                                                            -- STANDARD OBJECTS
                                                            -- 1 = shiped from saxess standard without modification
                                                            -- 2 = shiped from saxess standard modified FOR customer from saxess
                                                            -- 3 = shiped from saxess standard modified FOR customer from partner
                                                            -- 4 = shiped from saxess standard modified FROM customer themself for own needs
															-- CUSTOM OBJECTS
                                                            -- 10 = shiped from saxess as customer specific object
															-- 11 = shiped from partner	as customer specific object
															-- 12 = shiped from customer as own specific object
                                                             
-- KEEP this default constants      *************************************************************************
DECLARE
     @name          NVARCHAR(255)   = N'MS_Description'
    ,@level0type    NVARCHAR(255)   = N'SCHEMA'
    ,@level1type    NVARCHAR(255)   = N'VIEW'
    ,@level2type    NVARCHAR(255)   = N'COLUMN'
    ,@level2name    NVARCHAR(255)   = N''
    ,@value         NVARCHAR(1000)  = N'';
 
    SET @value = @SX_Owner;
    EXEC sys.sp_addextendedproperty 'SX_Owner'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_Module;
    EXEC sys.sp_addextendedproperty 'SX_Module'			,@value,@level0type,@level0name,@level1type,@level1name;
 
    SET @value = @SX_ShipmentFlag;
    EXEC sys.sp_addextendedproperty 'SX_ShipmentFlag'	,@value,@level0type,@level0name,@level1type,@level1name;
 
 
-- SET documententation             *************************************************************************
 
-- SET view documentation
    SET @value = N'View for analysing the LogEntrys of PlanningFactory API';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name;
 
-- SET columns documentation
 
        SET @level2name = N'Servername';
    SET @value      = N'Servername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ClusterName';
    SET @value      = N'ClusterName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'SQLSystemUser';
    SET @value      = N'SQLSystemUser';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ServiceUsername';
    SET @value      = N'ServiceUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TransactUsername';
    SET @value      = N'TransactUsername';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ProcedureName';
    SET @value      = N'ProcedureName';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ParameterString';
    SET @value      = N'ParameterString';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'EffectedRows';
    SET @value      = N'EffectedRows';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'ReturnCode';
    SET @value      = N'ReturnCode';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimestampCall';
    SET @value      = N'TimestampCall';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'TimestampReturn';
    SET @value      = N'TimestampReturn';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
    SET @level2name = N'DurationInMilliseconds';
    SET @value      = N'DurationInMilliseconds, display limited to 10.000';
    EXEC sys.sp_addextendedproperty @name,@value,@level0type,@level0name,@level1type,@level1name,@level2type,@level2name;
 
	GO
